/* BasedefichesLib_API - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.album;

import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import java.text.ParseException;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import java.util.regex.Matcher;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;


/**
 *
 * @author Vincent Calame
 */
public class AlbumUtils implements AlbumConstants {

    private static final Pattern ALBUMDIMNAME_PATTERN;
    private static final Pattern IMAGENAME_PATTERN;

    static {
        try {
            ALBUMDIMNAME_PATTERN = Pattern.compile("^[a-z][a-z0-9_]*$");
            IMAGENAME_PATTERN = Pattern.compile("^[-_a-zA-Z0-9.]+$");
        } catch (PatternSyntaxException pse) {
            throw new ShouldNotOccurException(pse);
        }
    }

    private AlbumUtils() {
    }

    public static String getAlbumIntitule(Album album, int langInteger) {
        Intitule albumIntitule = album.getAlbumMetadata().getIntituleByName(ALBUM);
        if (albumIntitule == null) {
            return album.getSubsetKey().toString();
        }
        return albumIntitule.getLibelleString(langInteger, true);
    }

    public static String toFileName(Illustration illustration) {
        return illustration.getAlbum().getSubsetKey().getSubsetName() + "-" + illustration.getIdalbum() + "." + formatTypeToString(illustration.getFormatType());
    }

    public static short formatTypeToShort(String typeString) {
        typeString = typeString.toLowerCase();
        if (typeString.equals("png")) {
            return PNG_FORMATTYPE;
        }
        if (typeString.equals("jpg")) {
            return JPG_FORMATTYPE;
        }
        if (typeString.equals("jpeg")) {
            return JPG_FORMATTYPE;
        }
        throw new IllegalArgumentException();
    }

    public static String formatTypeToString(short type) {
        switch (type) {
            case PNG_FORMATTYPE:
                return "png";
            case JPG_FORMATTYPE:
                return "jpg";
            default:
                throw new IllegalArgumentException("unknown type " + type);
        }
    }

    public static short dimTypeToShort(String typeString) {
        typeString = typeString.toLowerCase();
        if (typeString.equals("fixed-width")) {
            return FIXEDWIDTH_DIMTYPE;
        }
        if (typeString.equals("fixed-height")) {
            return FIXEDHEIGHT_DIMTYPE;
        }
        if (typeString.equals("max-width")) {
            return MAXWIDTH_DIMTYPE;
        }
        if (typeString.equals("max-height")) {
            return MAXHEIGHT_DIMTYPE;
        }
        if (typeString.equals("max-dim")) {
            return MAXDIM_DIMTYPE;
        }
        throw new IllegalArgumentException();
    }

    public static String dimTypeToString(short type) {
        switch (type) {
            case FIXEDWIDTH_DIMTYPE:
                return "fixed-width";
            case FIXEDHEIGHT_DIMTYPE:
                return "fixed-height";
            case MAXWIDTH_DIMTYPE:
                return "max-width";
            case MAXHEIGHT_DIMTYPE:
                return "max-height";
            case MAXDIM_DIMTYPE:
                return "max-dim";
            default:
                throw new IllegalArgumentException("unknown type " + type);
        }
    }

    public static String getMimeType(short type) {
        switch (type) {
            case PNG_FORMATTYPE:
                return "image/png";
            case JPG_FORMATTYPE:
                return "image/jpeg";
            default:
                throw new IllegalArgumentException("unknown type " + type);
        }
    }

    public static boolean isValidImageType(short type) {
        switch (type) {
            case PNG_FORMATTYPE:
            case JPG_FORMATTYPE:
                return true;
            default:
                return false;
        }
    }

    public static boolean isValidShortType(short type) {
        switch (type) {
            case FIXEDWIDTH_DIMTYPE:
            case FIXEDHEIGHT_DIMTYPE:
            case MAXWIDTH_DIMTYPE:
            case MAXHEIGHT_DIMTYPE:
            case MAXDIM_DIMTYPE:
                return true;
            default:
                return false;
        }
    }

    public static boolean needWidth(short type) {
        switch (type) {
            case FIXEDWIDTH_DIMTYPE:
            case MAXWIDTH_DIMTYPE:
            case MAXDIM_DIMTYPE:
                return true;
            default:
                return false;
        }
    }

    public static boolean needHeight(short type) {
        switch (type) {
            case FIXEDHEIGHT_DIMTYPE:
            case MAXHEIGHT_DIMTYPE:
            case MAXDIM_DIMTYPE:
                return true;
            default:
                return false;
        }
    }

    public static boolean isValidImageName(String s) {
        Matcher matcher = IMAGENAME_PATTERN.matcher(s);
        return matcher.matches();
    }

    public static void testAlbumDimName(String albumDimName) throws ParseException {
        Matcher matcher = ALBUMDIMNAME_PATTERN.matcher(albumDimName);
        if (!matcher.matches()) {
            throw new ParseException(albumDimName, 0);
        }
    }

    public static boolean isValidAlbumDimName(String albumDimName) {
        Matcher matcher = ALBUMDIMNAME_PATTERN.matcher(albumDimName);
        return matcher.matches();
    }

    public static boolean isRemoveable(Illustration illustration) {
        SubsetKey albumSubsetKey = illustration.getAlbum().getSubsetKey();
        Basedefiches basedefiches = illustration.getAlbum().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        int illustrationTicket = illustration.getTicket();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), albumSubsetKey);
            if (croisement.getCroisementItem(albumSubsetKey, illustrationTicket).getCount() > 0) {
                return false;
            }
        }
        return true;
    }

    public static String specialDimToString(short specialDim) {
        switch (specialDim) {
            case ORIGINAL_SPECIALDIM:
                return AlbumConstants.ORIGINAL_DIMNAME;
            case MINI_SPECIALDIM:
                return AlbumConstants.MINI_DIMNAME;
            default:
                throw new IllegalArgumentException("Unknown specialDim : " + specialDim);
        }
    }

}
