/* BasedefichesLib_API - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.association;

import java.io.Serializable;


/**
 * Indique un nom de version d'un fichier. Un nom de version est composé d'un radical (basename) et d'une extension.
 * Lorsqu'on construit une instance de VersionName à partir d'une chaîne, le radical et l'extension doivent être valides
 * (voir AssociationUtils.validateBasename et AssociationUtils.validateExtension).
 *<p> Il y a quelques exceptions à cette règle :
 *<ul>
 *<li>En construisant une nom de version à partir d'un entier positif ou nul, le radical commence par un trait de soulignement '_' suivi du nombre indiqué.
 *<li>En utilisant la méthode statique getExtensionEmptyInstance, on obtient une instance de VersionName avec une extension vide. Cette instance sert à être dérivée.
 * </ul>
 *
 * @author Vincent Calame
 */
public final class VersionName implements Serializable {

    private static final long serialVersionUID = 1L;
    private String basename;
    private String extension;

    VersionName(String basename, String extension) {
        this.basename = basename;
        this.extension = extension;
    }

    /**
     * Retourne une instance de VersionName dont l'extension est de longueur nulle.
     *
     *@throws IllegalArgument si basename n'est pas valide
     */
    public static VersionName getExtensionEmptyInstance(String basename) {
        if (!AssociationUtils.validateBasename(basename)) {
            throw new IllegalArgumentException("basename is not valid : " + basename);
        }
        return new VersionName(basename, "");
    }

    /**
     *
     *@throws IllegalArgumentException si id < 0
     */
    public static VersionName getExtensionEmptyInstance(int id) {
        if (id < 0) {
            throw new IllegalArgumentException("id = " + id + " < 0");
        }
        return new VersionName("_" + String.valueOf(id), "");
    }

    /**
     *
     *@throws IllegalArgumentException si basename ou extension ne sont pas valide.
     */
    public static VersionName getInstance(String basename, String extension) {
        if (!AssociationUtils.validateBasename(basename)) {
            throw new IllegalArgumentException("basename is not valid : " + basename);
        }
        if (!AssociationUtils.validateExtension(extension)) {
            throw new IllegalArgumentException("extension is not valid : " + extension);
        }
        return new VersionName(basename, extension);
    }

    public VersionName(VersionName other) {
        this.basename = other.basename;
        this.extension = other.extension;
    }

    /**
     *
     *@throws IllegalArgumentException si extension est incorrect
     */
    public VersionName derive(String extension) {
        if (!AssociationUtils.validateExtension(extension)) {
            throw new IllegalArgumentException("extension is not valid : " + extension);
        }
        VersionName derive = new VersionName(this);
        derive.extension = extension;
        return derive;
    }

    /**
     * Dérive en utilisant l'extension de versionName
     *
     *@throws IllegalArgumentException si versionName.getExtension().length() == 0
     */
    public VersionName derive(VersionName versionName) {
        String newextension = versionName.getExtension();
        if (newextension.length() == 0) {
            throw new IllegalArgumentException("versionName.getExtension().length() == 0");
        }
        VersionName derive = new VersionName(this);
        derive.extension = newextension;
        return derive;
    }

    public String getBasename() {
        return basename;
    }

    public String getExtension() {
        return extension;
    }

    @Override
    public String toString() {
        if (extension.length() == 0) {
            return basename;
        }
        return basename + "." + extension;
    }

    @Override
    public int hashCode() {
        return basename.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof VersionName)) {
            return false;
        }
        VersionName other = (VersionName) obj;
        return (this.basename.equals(other.basename)) && (this.extension.equals(other.extension));
    }

    /**
     * Retourne null si filename est nul ou vide.
     */
    public static ParseResult parse(String fileName, boolean ignoreCase) {
        if (fileName == null) {
            return null;
        }
        if (fileName.length() == 0) {
            return null;
        }
        String basename = null;
        String extension = null;
        int index = fileName.lastIndexOf('.');
        if ((index < 1) || (index == fileName.length() - 1)) {
            basename = fileName;
            extension = "";
        } else {
            basename = fileName.substring(0, index);
            extension = fileName.substring(index + 1);
        }
        if (ignoreCase) {
            basename = basename.toLowerCase();
            extension = extension.toLowerCase();
        }
        return new ParseResult(basename, extension);
    }


    public static class ParseResult {

        String basename;
        String extension;

        ParseResult(String basename, String extension) {
            this.basename = basename;
            this.extension = extension;
        }

        public String getBasename() {
            return basename;
        }

        /**
         * Peut être de longueur nulle.
         */
        public String getExtension() {
            return extension;
        }

        /**
         *Équivalent de getExtension().length() != 0
         */
        public boolean hasExtension() {
            return (extension.length() != 0);
        }

    }

}
