/* BasedefichesLib_API - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.croisement;

import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class CroisementUtils {

    public static CroisementItem EMPTY_CROISEMENTITEM = new EmptyCroisementItem();
    private static ConcurrentMap<SubsetKey, CroisementList> croisementListMap = new ConcurrentHashMap<SubsetKey, CroisementList>();

    private CroisementUtils() {
    }

    public static Croisement[] getCroisementWithCorpusArray(Thesaurus thesaurus) {
        Basedefiches basedefiches = thesaurus.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        Croisement[] croisements = new Croisement[corpusCount];
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurus.getSubsetKey());
            croisements[i] = croisement;
        }
        return croisements;
    }

    public static CroisementItem getSingletonCroisementItem(int ticket, int poids) {
        return new SingletonCroisementItem(ticket, poids);
    }

    public static boolean hasAssociation(Document document) {
        Basedefiches basedefiches = document.getAddenda().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            SubsetKey corpusKey = corpus.getSubsetKey();
            Croisement croisement = basedefiches.getCroisement(corpusKey, SubsetKey.ADDENDA);
            int count = croisement.getCroisementItem(SubsetKey.ADDENDA, document.getTicket()).getCount();
            if (count != 0) {
                return true;
            }
        }
        return false;
    }

    public static boolean areCrossed(AbregeFiche abregeFiche, Document document) {
        Basedefiches basedefiches = abregeFiche.getCorpus().getBasedefiches();
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        Croisement croisement = basedefiches.getCroisement(corpusKey, SubsetKey.ADDENDA);
        int crossing = croisement.getPoids(corpusKey, abregeFiche.getTicket(), SubsetKey.ADDENDA, document.getTicket());
        return (crossing != 0);
    }

    public static boolean isUniqueCroisement(AbregeFiche abregeFiche, Document document) {
        Basedefiches basedefiches = abregeFiche.getCorpus().getBasedefiches();
        SubsetKey documentKey = SubsetKey.ADDENDA;
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        Croisement croisement = basedefiches.getCroisement(corpusKey, documentKey);
        CroisementItem croisementItem = croisement.getCroisementItem(documentKey, document.getTicket());
        if (croisementItem.getCount() != 1) {
            return false;
        }
        int ticket = croisementItem.getTicket(0);
        return (ticket == abregeFiche.getTicket());
    }

    public static CroisementItem sortByPoids(CroisementItem croisementItem) {
        int count = croisementItem.getCount();
        if (count < 2) {
            return croisementItem;
        }
        SortedSet<SortBundle> treeSet = new TreeSet<SortBundle>();
        for (int i = 0; i < count; i++) {
            SortBundle sortBundle = new SortBundle(croisementItem.getTicket(i), croisementItem.getPoids(i), i);
            treeSet.add(sortBundle);
        }
        int p = 0;
        int[] poidsArray = new int[count];
        int[] ticketArray = new int[count];
        for (SortBundle sortBundle : treeSet) {
            poidsArray[p] = sortBundle.getPoids();
            ticketArray[p] = sortBundle.getTicket();
            p++;
        }
        return new ArrayCroisementItem(ticketArray, poidsArray);
    }

    public static CroisementList getEmptyCroisementList(SubsetKey subsetKey) {
        CroisementList croisementList = croisementListMap.get(subsetKey);
        if (croisementList == null) {
            if (subsetKey.isCorpusSubset()) {
                croisementList = new EmptyFicheCroisementList(subsetKey);
            } else if (subsetKey.isThesaurusSubset()) {
                croisementList = new EmptyMotcleCroisementList(subsetKey);
            } else if (subsetKey.isAlbumSubset()) {
                croisementList = new EmptyIllustrationCroisementList(subsetKey);
            } else if (subsetKey.equals(SubsetKey.ADDENDA)) {
                croisementList = new EmptyDocumentCroisementList();
            } else {
                throw new SwitchException("Not managed subset: " + subsetKey.toString());
            }
            croisementListMap.put(subsetKey, croisementList);
        }
        return croisementList;
    }

    public static CroisementList toCroisementList(SubsetKey subsetKey, CroisementItem croisementItem) {
        return new CroisementItemToCroisementList(subsetKey, croisementItem);
    }

    public static CroisementItem toCroisementItem(List<Integer> ticketList, List<Integer> poidsList) {
        int size = ticketList.size();
        if (poidsList.size() != size) {
            throw new IllegalArgumentException("ticketList and poidsList have different sizes");
        }
        int[] ticketArray = new int[size];
        int[] poidsArray = new int[size];
        for (int i = 0; i < size; i++) {
            ticketArray[i] = ticketList.get(i);
            poidsArray[i] = poidsList.get(i);
        }
        return new ArrayCroisementItem(ticketArray, poidsArray);
    }

    public static CroisementList toCroisementList(SubsetKey subsetKey, List<Integer> ticketList, List<Integer> poidsList) {
        CroisementItem croisementItem = toCroisementItem(ticketList, poidsList);
        return toCroisementList(subsetKey, croisementItem);
    }

    public static FicheCroisementList toFicheCroisementList(CroisementItem croisementItem, Corpus corpus) {
        int length = croisementItem.getCount();
        InternalFicheCroisementList ficheCroisementList = new InternalFicheCroisementList(corpus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int poids = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ticket);
            if (abregeFiche != null) {
                ficheCroisementList.add(abregeFiche, poids);
            }
        }
        return ficheCroisementList;
    }

    public static MotcleCroisementList toMotcleCroisementList(CroisementItem croisementItem, Thesaurus thesaurus) {
        int length = croisementItem.getCount();
        InternalMotcleCroisementList motcleCroisementList = new InternalMotcleCroisementList(thesaurus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int poids = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket);
            if (motcle != null) {
                motcleCroisementList.add(motcle, poids);
            }
        }
        return motcleCroisementList;
    }

    public static IllustrationCroisementList toIllustrationCroisementList(CroisementItem croisementItem, Album album) {
        int length = croisementItem.getCount();
        InternalIllustrationCroisementList motcleCroisementList = new InternalIllustrationCroisementList(album.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            int poids = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            Illustration motcle = (Illustration) album.getSubsetItemByTicket(ticket);
            if (motcle != null) {
                motcleCroisementList.add(motcle, poids);
            }
        }
        return motcleCroisementList;
    }

    public static DocumentCroisementList toDocumentCroisementList(CroisementItem croisementItem, Addenda addenda) {
        int length = croisementItem.getCount();
        InternalDocumentCroisementList documentCroisementList = new InternalDocumentCroisementList(length);
        for (int i = 0; i < length; i++) {
            int poids = croisementItem.getPoids(i);
            int ticket = croisementItem.getTicket(i);
            Document motcle = (Document) addenda.getSubsetItemByTicket(ticket);
            if (motcle != null) {
                documentCroisementList.add(motcle, poids);
            }
        }
        return documentCroisementList;
    }

    public static FicheCroisementList poidsFilter(FicheCroisementList ficheCroisementList, int poidsFilter) {
        int count = ficheCroisementList.getSubsetItemCount();
        InternalFicheCroisementList newFicheCroisementList = new InternalFicheCroisementList(ficheCroisementList.getSubsetKey(), count);
        for (int i = 0; i < count; i++) {
            int poids = ficheCroisementList.getPoids(i);
            if (poids == poidsFilter) {
                newFicheCroisementList.add(ficheCroisementList.getAbregeFiche(i), poids);
            }
        }
        return newFicheCroisementList;
    }

    public static FicheCroisementList excludePoids(FicheCroisementList ficheCroisementList, RangeList excludePoids) {
        int count = ficheCroisementList.getSubsetItemCount();
        InternalFicheCroisementList newFicheCroisementList = new InternalFicheCroisementList(ficheCroisementList.getSubsetKey(), count);
        for (int i = 0; i < count; i++) {
            int poids = ficheCroisementList.getPoids(i);
            if (!excludePoids.contains(poids)) {
                newFicheCroisementList.add(ficheCroisementList.getAbregeFiche(i), poids);
            }
        }
        return newFicheCroisementList;
    }

    public static MotcleCroisementList poidsFilter(MotcleCroisementList motcleCroisementList, int poidsFilter) {
        int count = motcleCroisementList.getSubsetItemCount();
        InternalMotcleCroisementList newMotcleCroisementList = new InternalMotcleCroisementList(motcleCroisementList.getSubsetKey(), count);
        for (int i = 0; i < count; i++) {
            int poids = motcleCroisementList.getPoids(i);
            if (poids == poidsFilter) {
                newMotcleCroisementList.add(motcleCroisementList.getMotcle(i), poids);
            }
        }
        return newMotcleCroisementList;
    }

    public static MotcleCroisementList excludePoids(MotcleCroisementList motcleCroisementList, RangeList excludePoids) {
        int count = motcleCroisementList.getSubsetItemCount();
        InternalMotcleCroisementList newMotcleCroisementList = new InternalMotcleCroisementList(motcleCroisementList.getSubsetKey(), count);
        for (int i = 0; i < count; i++) {
            int poids = motcleCroisementList.getPoids(i);
            if (!excludePoids.contains(poids)) {
                newMotcleCroisementList.add(motcleCroisementList.getMotcle(i), poids);
            }
        }
        return newMotcleCroisementList;
    }

    public static IllustrationCroisementList poidsFilter(IllustrationCroisementList illustrationCroisementList, int poidsFilter) {
        int count = illustrationCroisementList.getSubsetItemCount();
        InternalIllustrationCroisementList newIllustrationCroisementList = new InternalIllustrationCroisementList(illustrationCroisementList.getSubsetKey(), count);
        for (int i = 0; i < count; i++) {
            int poids = illustrationCroisementList.getPoids(i);
            if (poids == poidsFilter) {
                newIllustrationCroisementList.add(illustrationCroisementList.getIllustration(i), poids);
            }
        }
        return newIllustrationCroisementList;
    }

    public static IllustrationCroisementList excludePoids(IllustrationCroisementList illustrationCroisementList, RangeList excludePoids) {
        int count = illustrationCroisementList.getSubsetItemCount();
        InternalIllustrationCroisementList newIllustrationCroisementList = new InternalIllustrationCroisementList(illustrationCroisementList.getSubsetKey(), count);
        for (int i = 0; i < count; i++) {
            int poids = illustrationCroisementList.getPoids(i);
            if (!excludePoids.contains(poids)) {
                newIllustrationCroisementList.add(illustrationCroisementList.getIllustration(i), poids);
            }
        }
        return newIllustrationCroisementList;
    }

    public static Croisement toEmptyCroisement(Basedefiches basedefiches, SubsetCouple subsetCouple) {
        return new EmptyCroisement(basedefiches, subsetCouple);
    }

    public static boolean isEmpty(Croisement croisement) {
        SubsetCouple couple = croisement.getSubsetCouple();
        SubsetKey subsetKey1 = couple.getSubsetKey1();
        SubsetKey subsetKey2 = couple.getSubsetKey2();
        Subset subset1 = croisement.getBasedefiches().getSubset(subsetKey1);
        Subset subset2 = croisement.getBasedefiches().getSubset(subsetKey2);
        if ((subset1 == null) || (subset2 == null)) {
            return true;
        }
        int length1 = subset1.getTicketLength();
        int length2 = subset2.getTicketLength();
        Subset subset = null;
        SubsetKey subsetKey = null;
        int length = 0;
        if (length1 < length2) {
            subsetKey = subsetKey1;
            length = length1;
            subset = subset1;
        } else {
            subsetKey = subsetKey2;
            length = length2;
            subset = subset2;
        }
        for (int i = 0; i < length; i++) {
            SubsetItem subsetItem = subset.getSubsetItemByTicket(i);
            if (subsetItem != null) {
                CroisementItem croisementItem = croisement.getCroisementItem(subsetKey, subsetItem.getTicket());
                int count = croisementItem.getCount();
                if (count > 0) {
                    return false;
                }
            }
        }
        return true;
    }


    private static class EmptyCroisementItem implements CroisementItem {

        public int getCount() {
            return 0;
        }

        public int getPoids(int i) {
            throw new IndexOutOfBoundsException();
        }

        public int getTicket(int i) {
            throw new IndexOutOfBoundsException();
        }

        public int indexOfTicket(int ticket) {
            return -1;
        }

    }


    private static class SingletonCroisementItem implements CroisementItem {

        int ticket;
        int poids;

        SingletonCroisementItem(int ticket, int poids) {
            this.ticket = ticket;
            this.poids = poids;
        }

        public int getCount() {
            return 1;
        }

        public int getPoids(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException("i != 0");
            }
            return poids;
        }

        public int getTicket(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException("i != 0");
            }
            return ticket;
        }

        public int indexOfTicket(int ticket) {
            if (this.ticket == ticket) {
                return 0;
            }
            return -1;
        }

    }


    private static class ArrayCroisementItem implements CroisementItem {

        private int[] poidsArray;
        private int[] ticketArray;

        ArrayCroisementItem(int[] ticketArray, int[] poidsArray) {
            this.ticketArray = ticketArray;
            this.poidsArray = poidsArray;
        }

        public int getCount() {
            return ticketArray.length;
        }

        public int getPoids(int i) {
            return poidsArray[i];
        }

        public int getTicket(int i) {
            return ticketArray[i];
        }

        public int indexOfTicket(int ticket) {
            for (int i = 0; i < ticketArray[i]; i++) {
                if (ticketArray[i] == ticket) {
                    return i;
                }
            }
            return -1;
        }

    }


    private static class SortBundle implements Comparable {

        private int poids;
        private int position;
        private int ticket;

        SortBundle(int ticket, int poids, int position) {
            this.ticket = ticket;
            this.position = position;
            this.poids = poids;
        }

        @Override
        public int hashCode() {
            return ticket;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (!(obj instanceof SortBundle)) {
                return false;
            }
            SortBundle other = (SortBundle) obj;
            return (other.ticket == this.ticket);
        }

        public int compareTo(Object obj) {
            SortBundle other = (SortBundle) obj;
            if (other.poids > this.poids) {
                return -1;
            }
            if (other.poids < this.poids) {
                return 1;
            }
            if (other.position > this.position) {
                return -1;
            }
            if (other.position < this.position) {
                return 1;
            }
            if (other.ticket > this.ticket) {
                return -1;
            }
            if (other.ticket < this.ticket) {
                return 1;
            }
            return 0;
        }

        public int getPoids() {
            return poids;
        }

        public int getTicket() {
            return ticket;
        }

    }


    private static class CroisementItemToCroisementList implements CroisementList {

        private SubsetKey subsetKey;
        private CroisementItem croisementItem;

        private CroisementItemToCroisementList(SubsetKey subsetKey, CroisementItem croisementItem) {
            this.subsetKey = subsetKey;
            this.croisementItem = croisementItem;
        }

        public int getSubsetItemCount() {
            return croisementItem.getCount();
        }

        public int getTicket(int index) {
            return croisementItem.getTicket(index);
        }

        public int getPoids(int index) {
            return croisementItem.getPoids(index);
        }

        public SubsetKey getSubsetKey() {
            return subsetKey;
        }

    }


    private static class InternalFicheCroisementList implements FicheCroisementList {

        private SubsetKey corpusKey;
        private AbregeFiche[] abregeArray;
        private int[] poidsArray;
        private int size = 0;

        private InternalFicheCroisementList(SubsetKey corpusKey, int initialCapacity) {
            this.corpusKey = corpusKey;
            abregeArray = new AbregeFiche[initialCapacity];
            poidsArray = new int[initialCapacity];
        }

        private void add(AbregeFiche abregeFiche, int poids) {
            abregeArray[size] = abregeFiche;
            poidsArray[size] = poids;
            size++;
        }

        public SubsetKey getSubsetKey() {
            return corpusKey;
        }

        public int getTicket(int index) {
            return getAbregeFiche(index).getTicket();
        }

        public int getSubsetItemCount() {
            return size;
        }

        public AbregeFiche getAbregeFiche(int index) {
            if (index >= size) {
                throw new IndexOutOfBoundsException();
            }
            return abregeArray[index];
        }

        public int getPoids(int index) {
            if (index >= size) {
                throw new IndexOutOfBoundsException();
            }
            return poidsArray[index];
        }

    }


    private static class InternalMotcleCroisementList implements MotcleCroisementList {

        private SubsetKey theaurusSubsetKey;
        private Motcle[] motcleArray;
        private int[] poidsArray;
        private int size = 0;

        private InternalMotcleCroisementList(SubsetKey theaurusSubsetKey, int initialCapacity) {
            this.theaurusSubsetKey = theaurusSubsetKey;
            motcleArray = new Motcle[initialCapacity];
            poidsArray = new int[initialCapacity];
        }

        private void add(Motcle motcle, int poids) {
            motcleArray[size] = motcle;
            poidsArray[size] = poids;
            size++;
        }

        public SubsetKey getSubsetKey() {
            return theaurusSubsetKey;
        }

        public int getTicket(int index) {
            return getMotcle(index).getTicket();
        }

        public int getSubsetItemCount() {
            return size;
        }

        public Motcle getMotcle(int index) {
            if (index >= size) {
                throw new IndexOutOfBoundsException();
            }
            return motcleArray[index];
        }

        public int getPoids(int index) {
            if (index >= size) {
                throw new IndexOutOfBoundsException();
            }
            return poidsArray[index];
        }

    }


    private static class InternalIllustrationCroisementList implements IllustrationCroisementList {

        private SubsetKey theaurusSubsetKey;
        private Illustration[] illustrationArray;
        private int[] poidsArray;
        private int size = 0;

        private InternalIllustrationCroisementList(SubsetKey illustrationSubsetKey, int initialCapacity) {
            this.theaurusSubsetKey = illustrationSubsetKey;
            illustrationArray = new Illustration[initialCapacity];
            poidsArray = new int[initialCapacity];
        }

        private void add(Illustration motcle, int poids) {
            illustrationArray[size] = motcle;
            poidsArray[size] = poids;
            size++;
        }

        public SubsetKey getSubsetKey() {
            return theaurusSubsetKey;
        }

        public int getTicket(int index) {
            return getIllustration(index).getTicket();
        }

        public int getSubsetItemCount() {
            return size;
        }

        public Illustration getIllustration(int index) {
            if (index >= size) {
                throw new IndexOutOfBoundsException();
            }
            return illustrationArray[index];
        }

        public int getPoids(int index) {
            if (index >= size) {
                throw new IndexOutOfBoundsException();
            }
            return poidsArray[index];
        }

    }


    private static class InternalDocumentCroisementList implements DocumentCroisementList {

        private Document[] documentArray;
        private int size = 0;

        private InternalDocumentCroisementList(int initialCapacity) {
            documentArray = new Document[initialCapacity];
        }

        private void add(Document motcle, int poids) {
            documentArray[size] = motcle;
            size++;
        }

        public SubsetKey getSubsetKey() {
            return SubsetKey.ADDENDA;
        }

        public int getTicket(int index) {
            return getDocument(index).getTicket();
        }

        public int getSubsetItemCount() {
            return size;
        }

        public Document getDocument(int index) {
            if (index >= size) {
                throw new IndexOutOfBoundsException();
            }
            return documentArray[index];
        }

        public int getPoids(int index) {
            if (index >= size) {
                throw new IndexOutOfBoundsException();
            }
            return 1;
        }

    }


    private static class EmptyFicheCroisementList implements FicheCroisementList {

        private SubsetKey corpusSubsetKey;

        private EmptyFicheCroisementList(SubsetKey corpusSubsetKey) {
            this.corpusSubsetKey = corpusSubsetKey;
        }

        public SubsetKey getSubsetKey() {
            return corpusSubsetKey;
        }

        public int getTicket(int index) {
            throw new IndexOutOfBoundsException();
        }

        public int getSubsetItemCount() {
            return 0;
        }

        public AbregeFiche getAbregeFiche(int index) {
            throw new IndexOutOfBoundsException();
        }

        public int getPoids(int index) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyMotcleCroisementList implements MotcleCroisementList {

        private SubsetKey theaurusSubsetKey;

        private EmptyMotcleCroisementList(SubsetKey theaurusSubsetKey) {
            this.theaurusSubsetKey = theaurusSubsetKey;
        }

        public SubsetKey getSubsetKey() {
            return theaurusSubsetKey;
        }

        public int getTicket(int index) {
            throw new IndexOutOfBoundsException();
        }

        public int getSubsetItemCount() {
            return 0;
        }

        public Motcle getMotcle(int index) {
            throw new IndexOutOfBoundsException();

        }

        public int getPoids(int index) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyIllustrationCroisementList implements IllustrationCroisementList {

        private SubsetKey albumSubsetKey;

        private EmptyIllustrationCroisementList(SubsetKey albumSubsetKey) {
            this.albumSubsetKey = albumSubsetKey;
        }

        public SubsetKey getSubsetKey() {
            return albumSubsetKey;
        }

        public int getTicket(int index) {
            throw new IndexOutOfBoundsException();
        }

        public int getSubsetItemCount() {
            return 0;
        }

        public Illustration getIllustration(int index) {
            throw new IndexOutOfBoundsException();

        }

        public int getPoids(int index) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyDocumentCroisementList implements DocumentCroisementList {

        private EmptyDocumentCroisementList() {
        }

        public SubsetKey getSubsetKey() {
            return SubsetKey.ADDENDA;
        }

        public int getTicket(int index) {
            throw new IndexOutOfBoundsException();
        }

        public int getSubsetItemCount() {
            return 0;
        }

        public Document getDocument(int index) {
            throw new IndexOutOfBoundsException();
        }

        public int getPoids(int index) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyCroisement implements Croisement {

        private Basedefiches basedefiches;
        private SubsetCouple subsetCouple;

        private EmptyCroisement(Basedefiches basedefiches, SubsetCouple subsetCouple) {
            this.basedefiches = basedefiches;
            this.subsetCouple = subsetCouple;
        }

        public Basedefiches getBasedefiches() {
            return basedefiches;
        }

        public SubsetCouple getSubsetCouple() {
            return subsetCouple;
        }

        public CroisementItem getCroisementItem(SubsetKey subsetKey, int ticket) {
            return EMPTY_CROISEMENTITEM;
        }

        public int getPoids(SubsetKey subsetKey1, int ticket1, SubsetKey subsetKey2, int ticket2) {
            return 0;
        }

    }

}
