/* BasedefichesLib_API - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.croisement;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.util.primitives.PrimUtils;
import java.util.Arrays;


/**
 *
 * @author Vincent Calame
 */
public class DefaultFicheCroisementList implements FicheCroisementList {

    private SubsetKey corpusKey;
    private AbregeFiche[] abregeArray;
    private int[] poidsArray;
    private int size;

    public DefaultFicheCroisementList(SubsetKey corpusKey, int initialCapacity) {
        this.corpusKey = corpusKey;
        abregeArray = new AbregeFiche[initialCapacity];
        poidsArray = new int[initialCapacity];
    }

    /**
     * Construit une liste avec un seul élément.

     *@throws IllegalArgumentException si poids < 1
     */
    public DefaultFicheCroisementList(AbregeFiche abregeFiche, int poids) {
        if (poids < 1) {
            throw new IllegalArgumentException("poids < 1");
        }
        corpusKey = abregeFiche.getCorpus().getSubsetKey();
        abregeArray = new AbregeFiche[1];
        poidsArray = new int[1];
        size = 1;
        abregeArray[0] = abregeFiche;
        poidsArray[0] = poids;
    }

    public int getSubsetItemCount() {
        return size;
    }

    public AbregeFiche getAbregeFiche(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return abregeArray[index];
    }

    public int getPoids(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return poidsArray[index];
    }

    /**
     *
     *@throws IllegalArgumentException si poids < 1
     */
    public void add(AbregeFiche abregeFiche, int poids) {
        if (!abregeFiche.getCorpus().getSubsetKey().equals(corpusKey)) {
            throw new IllegalArgumentException("subsetKey must be the same");
        }
        if (poids < 1) {
            throw new IllegalArgumentException("poids < 1");
        }
        for (int i = 0; i < size; i++) {
            if (abregeArray[i].equals(abregeFiche)) {
                poidsArray[i] = Math.max(poidsArray[i], poids);
                return;
            }
        }
        checkLength();
        abregeArray[size] = abregeFiche;
        poidsArray[size] = poids;
        size++;
    }

    public void append(FicheCroisementList otherFicheCroisementList) {
        if (!otherFicheCroisementList.getSubsetKey().equals(corpusKey)) {
            throw new IllegalArgumentException("subsetKey must be the same");
        }
        int currentSize = size;
        int otherSize = otherFicheCroisementList.getSubsetItemCount();
        int newsize = currentSize + otherSize;
        if (newsize > abregeArray.length) {
            extendsLength(newsize);
        }
        for (int i = 0; i < otherSize; i++) {
            AbregeFiche otherAbregeFiche = otherFicheCroisementList.getAbregeFiche(i);
            int otherPoids = otherFicheCroisementList.getPoids(i);
            boolean here = false;
            for (int j = 0; j < currentSize; j++) {
                if (abregeArray[j].equals(otherAbregeFiche)) {
                    poidsArray[j] = Math.max(poidsArray[j], otherPoids);
                    here = true;
                    break;
                }
            }
            if (!here) {
                abregeArray[size] = otherAbregeFiche;
                poidsArray[size] = otherPoids;
                size++;
            }
        }
    }

    public SubsetKey getSubsetKey() {
        return corpusKey;
    }

    public int getTicket(int index) {
        return getAbregeFiche(index).getTicket();
    }

    private void checkLength() {
        if (size == abregeArray.length) {
            extendsLength(Math.max(size * 2, 4));
        }
    }

    private void extendsLength(int newLength) {
        AbregeFiche[] nv = new AbregeFiche[newLength];
        System.arraycopy(abregeArray, 0, nv, 0, size);
        abregeArray = nv;
        int[] nvi = new int[newLength];
        System.arraycopy(poidsArray, 0, nvi, 0, size);
        poidsArray = nvi;
    }

    public void sortByIdcorpus() {
        if (size < 2) {
            return;
        }
        long[] longArray = new long[size];
        for (int i = 0; i < size; i++) {
            longArray[i] = PrimUtils.toLong(abregeArray[i].getIdcorpus(), i);
        }
        Arrays.sort(longArray);
        AbregeFiche[] newAbregeArray = new AbregeFiche[size];
        int[] newPoidsArray = new int[size];
        for (int i = 0; i < size; i++) {
            int oldIndex = PrimUtils.getInt2(longArray[i]);
            newAbregeArray[i] = abregeArray[oldIndex];
            newPoidsArray[i] = poidsArray[oldIndex];
        }
        abregeArray = newAbregeArray;
        poidsArray = newPoidsArray;
    }

}
