/* BasedefichesLib_API - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.format.Calcul;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.impl.InstructionParser;
import net.mapeadores.util.text.StringUtils;
import java.text.ParseException;
import java.util.List;
import net.mapeadores.util.format.FormatConstants;
import net.mapeadores.util.format.FormatUtils;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesFormatDefParser {

    private ErrorHandler errorHandler;
    private BasedefichesFormatDefBuilder formatDefBuilder = null;

    public static BasedefichesFormatDef parse(String formatString, ErrorHandler errorHandler) {
        formatString = formatString.trim();
        if (formatString.length() == 0) {
            return null;
        }
        String[] tokens = StringUtils.getLineTokens(formatString, StringUtils.EMPTY_EXCLUDE);
        return parse(tokens, errorHandler);
    }

    public static BasedefichesFormatDef parse(String[] formatArray, ErrorHandler errorHandler) {
        int length = formatArray.length;
        if (length == 0) {
            throw new IllegalArgumentException("formatArray.length = 0");
        }
        String formatSource = formatArray[0];
        String formatPattern = (length > 1) ? formatArray[1] : null;
        String formatInstruction = (length > 2) ? formatArray[2] : null;
        return parse(formatSource, formatPattern, formatInstruction, errorHandler);
    }

    public static BasedefichesFormatDef parse(List<String> formatList, ErrorHandler errorHandler) {
        int length = formatList.size();
        if (length == 0) {
            throw new IllegalArgumentException("formatArray.length = 0");
        }
        String formatSource = formatList.get(0);
        String formatPattern = (length > 1) ? formatList.get(1) : null;
        String formatInstruction = (length > 2) ? formatList.get(2) : null;
        return parse(formatSource, formatPattern, formatInstruction, errorHandler);
    }

    public static BasedefichesFormatDef parse(String formatSource, String formatPattern, String formatInstruction, ErrorHandler errorHandler) {
        BasedefichesFormatDefParser ficheFormatDefParser = new BasedefichesFormatDefParser(errorHandler);
        return ficheFormatDefParser.parseBasedefichesFormatDef(formatSource, formatPattern, formatInstruction);
    }

    private BasedefichesFormatDefParser(ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
    }

    private BasedefichesFormatDef parseBasedefichesFormatDef(String formatSource, String formatPattern, String formatInstruction) {
        boolean done = initBasedefichesFormatDef(formatSource);
        if (!done) {
            return null;
        }
        initFormatPattern(formatDefBuilder, formatPattern);
        Instruction instruction = getInstruction(formatInstruction);
        if (instruction != null) {
            int length = instruction.getLength();
            for (int i = 0; i < length; i++) {
                Argument arg = instruction.getArgument(i);
                initArgument(arg);
            }
        }
        return formatDefBuilder.toBasedefichesFormatDef();

    }

    private void initFormatPattern(BasedefichesFormatDefBuilder formatDefBuilder, String formatPattern) {
        if (formatPattern == null) {
            return;
        }
        int idx = formatPattern.indexOf("||");
        if (idx == -1) {
            formatDefBuilder.addFormatPattern(formatPattern);
        } else {
            formatDefBuilder.addFormatPattern(formatPattern.substring(0, idx));
            initFormatPattern(formatDefBuilder, formatPattern.substring(idx + 2));
        }

    }

    private void initArgument(Argument arg) {
        String key = arg.getKey();
        String value = arg.getValue();
        boolean done = false;
        try {
            done = initCommonArgument(key, value);
        } catch (IllegalArgumentException iae) {
            errorHandler.wrongArgumentValueInstructionError(key, value);
            return;
        }
        if (done) {
            return;
        }
        if (key.equals("defprop")) {
            if (value == null) {
                errorHandler.missingArgumentValueInstructionError(key);
            }
            try {
                FieldKey propFieldKey = FieldKey.parse(value);
                try {
                    formatDefBuilder.setDefaultProprieteKey(propFieldKey);
                } catch (IllegalArgumentException iae) {
                    errorHandler.wrongArgumentValueInstructionError(key, value);
                }
            } catch (ParseException pe) {
                errorHandler.wrongArgumentValueInstructionError(key, value);
            }
        } else if (key.equals("noitem")) {
            formatDefBuilder.setNoItem(true);
        } else {
            errorHandler.unknownArgumentKeyInstructionWarning(key);
        }
    }

    private boolean initCommonArgument(String key, String value) throws NumberFormatException, IllegalArgumentException {
        if (key.equals("cast")) {
            if (testValue(key, value, false)) {
                if ((value == null) || (value.length() == 0)) {
                    value = "no";
                }
                short castType = FormatUtils.castTypeToShort(value);
                formatDefBuilder.setCastType(castType);
            }
        } else if (key.equals("limit")) {
            if (testValue(key, value, false)) {
                int limit = Integer.parseInt(value);
                formatDefBuilder.setLimit(limit);
            }
        } else if (key.equals("pos")) {
            if (testValue(key, value, false)) {
                if (value.equals("last")) {
                    formatDefBuilder.setPosition(FormatConstants.MAX_POSITION);
                } else {
                    int position = Integer.parseInt(value);
                    formatDefBuilder.setPosition(position - 1);
                }
            }
        } else if (key.equals("defval")) {
            if (testValue(key, value, true)) {
                formatDefBuilder.setDefaultString(value);
            }
        } else if (key.equals("last")) {
            formatDefBuilder.setFillByLast(true);
        } else if (key.equals("null")) {
            formatDefBuilder.setConvertEmptyToNull(true);
        } else if (key.equals("calc")) {
            if (testValue(key, value, false)) {
                try {
                    Calcul calcul = Calcul.parse(value);
                    formatDefBuilder.setCalcul(calcul);
                } catch (java.text.ParseException pe) {
                    errorHandler.wrongArgumentValueInstructionError(key, value);
                }
            }
        } else if (key.startsWith("sep")) {
            if (testValue(key, value, true)) {
                int length = key.length();
                if (length == 3) {
                    if (value.equals("NL")) {
                        value = "\n";
                    }
                    formatDefBuilder.setSeparator(value);
                } else {
                    int idx = key.indexOf('_');
                    if (idx == -1) {
                        try {
                            int pos = Integer.parseInt(key.substring(3));
                            formatDefBuilder.setInternalSeparator(pos - 1, value);
                        } catch (NumberFormatException nfe) {
                            return false;
                        }
                    } else {
                        try {
                            int pos1 = Integer.parseInt(key.substring(3, idx));
                            int pos2 = Integer.parseInt(key.substring(idx + 1));
                            formatDefBuilder.setSourceSeparator(pos1 - 1, pos2 - 1, value);
                        } catch (NumberFormatException nfe) {
                            return false;
                        }
                    }
                }
            }
        } else {
            return false;
        }
        return true;
    }

    private boolean testValue(String key, String value, boolean allowEmpty) {
        if ((value == null) || ((!allowEmpty) && (value.length() == 0))) {
            errorHandler.missingArgumentValueInstructionError(key);
            return false;
        } else {
            return true;
        }
    }

    private Instruction getInstruction(String formatInstruction) {
        if (formatInstruction == null) {
            return null;
        }
        formatInstruction = formatInstruction.trim();
        if (formatInstruction.length() == 0) {
            return null;
        }
        InstructionErrorHandler eieh = new InstructionErrorHandler();
        return InstructionParser.parse(formatInstruction, eieh);
    }

    private boolean initBasedefichesFormatDef(String formatSource) {
        formatSource = checkFormatSource(formatSource);
        if (formatSource == null) {
            return false;
        }
        String[] tokens = StringUtils.getTechnicalTokens(formatSource);
        int length = tokens.length;
        if (length == 0) {
            errorHandler.nullFormatSourceError();
            return false;
        }
        formatDefBuilder = new BasedefichesFormatDefBuilder();
        if (length == 1) {
            boolean done = parseFormatSource(tokens[0], false);
            return done;
        }
        boolean withError = false;
        for (int i = 0; i < length; i++) {
            boolean done = parseFormatSource(tokens[i], true);
            if (!done) {
                withError = true;
            }
        }
        return !withError;
    }

    private boolean parseFormatSource(String formatSource, boolean part) {
        SubsetKey slaveCorpusKey = null;
        int idx = formatSource.indexOf('/');
        if (idx != -1) {
            try {
                slaveCorpusKey = SubsetKey.parse(formatSource.substring(0, idx));
                if (!slaveCorpusKey.isCorpusSubset()) {
                    slaveCorpusKey = null;
                }
            } catch (java.text.ParseException pe) {
            }
            if (slaveCorpusKey == null) {
                if (part) {
                    errorHandler.wrongFormatSourcePartError(formatSource);
                } else {
                    errorHandler.wrongFormatSourceError();
                }
                return false;
            }
            formatSource = formatSource.substring(idx + 1);
        }
        if (formatSource.endsWith("*")) {
            try {
                FieldKey typePrefixFieldKey = FieldKey.parse(formatSource.substring(0, formatSource.length() - 1));
                formatDefBuilder.addSource(FormatSourceKey.newTypePrefixFieldKeyInstance(typePrefixFieldKey, slaveCorpusKey));
            } catch (ParseException pe1) {
                if (part) {
                    errorHandler.wrongFormatSourcePartError(formatSource);
                } else {
                    errorHandler.wrongFormatSourceError();
                }
            } catch (IllegalArgumentException iae) {
                if (part) {
                    errorHandler.wrongFormatSourcePartError(formatSource);
                } else {
                    errorHandler.wrongFormatSourceError();
                }
            }
        }
        try {
            FieldKey fk = FieldKey.parse(formatSource);
            formatDefBuilder.addSource(FormatSourceKey.newFieldKeyInstance(fk, slaveCorpusKey));
        } catch (ParseException pe1) {
            try {
                SubfieldKey subfieldKey = SubfieldKey.parse(formatSource);
                formatDefBuilder.addSource(FormatSourceKey.newSubfieldKeyInstance(subfieldKey, slaveCorpusKey));
            } catch (ParseException pe2) {
                try {
                    IncludeKey includeKey = IncludeKey.parse(formatSource);
                    formatDefBuilder.addSource(FormatSourceKey.newIncludeKeyInstance(includeKey, slaveCorpusKey));
                } catch (ParseException pe3) {
                    try {
                        ThesaurusFieldKey thesaurusFieldKey = ThesaurusFieldKey.parse(formatSource);
                        formatDefBuilder.addSource(FormatSourceKey.newThesaurusFieldKeyInstance(thesaurusFieldKey));
                    } catch (ParseException pe4) {
                        if (part) {
                            errorHandler.wrongFormatSourcePartError(formatSource);
                        } else {
                            errorHandler.wrongFormatSourceError();
                        }
                        return false;
                    }
                }
            }
        }
        return true;
    }

    private String checkFormatSource(String formatSource) {
        if (formatSource == null) {
            errorHandler.nullFormatSourceError();
        } else {
            formatSource = formatSource.trim();
            if (formatSource.length() == 0) {
                errorHandler.nullFormatSourceError();
                formatSource = null;
            }
        }
        return formatSource;
    }


    private class InstructionErrorHandler implements InstructionParser.ErrorHandler {

        public void invalidAsciiCharacterError(String part, int row, int col) {
            errorHandler.invalidAsciiCharacterInstructionError(part);
        }

        public void invalidEndCharacterError(String part, int row, int col) {
            errorHandler.invalidEndCharacterInstructionError(part);
        }

        public void invalidSeparatorCharacterError(String part, int row, int col) {
            errorHandler.invalidSeparatorCharacterInstructionError(part);
        }

    }


    public static interface ErrorHandler {

        public void nullFormatSourceError();

        public void wrongFormatSourceError();

        public void wrongFormatSourcePartError(String formatSourcePart);

        public void invalidAsciiCharacterInstructionError(String formatInstructionPart);

        public void invalidEndCharacterInstructionError(String formatInstructionPart);

        public void invalidSeparatorCharacterInstructionError(String formatInstructionPart);

        public void unknownArgumentKeyInstructionWarning(String argumentKey);

        public void missingArgumentValueInstructionError(String argumentKey);

        public void wrongArgumentValueInstructionError(String argumentKey, String argumentValue);

    }

}







