/* BasedefichesLib_API - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.Montant;
import net.mapeadores.basedefiches.corpus.fiche.Nombre;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.util.Locale;
import java.io.IOException;
import java.text.DecimalFormatSymbols;


/**
 *
 * @author Vincent Calame
 */
public class MontantFormat extends FicheItemFormat {

    private static final int CUR_SUBTYPE = 1;
    private static final int LIB_SUBTYPE = 2;
    private static final int VALUE_SUBTYPE = 3;
    private static final int CODE_SUBTYPE = 4;

    public MontantFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Montant) {
            return formatMontant((Montant) obj, localisationInfo);
        } else if (obj instanceof Nombre) {
            return formatNombre((Nombre) obj, localisationInfo);
        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatMontant(Montant montant, LocalisationInfo localisationInfo) {
        if (montant == null) {
            return null;
        }
        return processFormat(new MontantResolver(montant, localisationInfo));
    }

    public String formatNombre(Nombre nombre, LocalisationInfo localisationInfo) {
        if (nombre == null) {
            return null;
        }
        return processFormat(new NombreResolver(nombre, localisationInfo));
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("cur")) {
            return new FormatPart(CUR_SUBTYPE, null);
        } else if (key.equals("lib")) {
            return new FormatPart(LIB_SUBTYPE, new LibParameters(instructionArray));
        } else if (key.equals("value")) {
            return new FormatPart(VALUE_SUBTYPE, null);
        } else if (key.equals("code")) {
            return new FormatPart(CODE_SUBTYPE, null);
        } else {
            return null;
        }
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }


    private class MontantResolver implements ObjectFormatPartResolver {

        Montant nombre;
        LocalisationInfo localisationInfo;

        MontantResolver(Montant montant, LocalisationInfo localisationInfo) {
            this.nombre = montant;
            this.localisationInfo = localisationInfo;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            Decimal decimal = nombre.getDecimal();
            int currencyInteger = nombre.getCurrencyInteger();
            int subType = formatPart.getSubType();
            switch (subType) {
                case CODE_SUBTYPE:
                    buf.append(CurrencyInteger.toString(decimal, currencyInteger));
                    break;
                case CUR_SUBTYPE:
                    buf.append(CurrencyInteger.toString(currencyInteger));
                    break;
                case VALUE_SUBTYPE:
                    buf.append(decimal.toString());
                    break;
                case LIB_SUBTYPE:
                    LibParameters libParameters = (LibParameters) formatPart.getObject();
                    int[] array = libParameters.getlangIntegerArray();
                    if (array == null) {
                        appendLocale(buf, decimal, currencyInteger, getFormatLocale(localisationInfo));
                    } else {
                        for (int i = 0; i < array.length; i++) {
                            if (i > 0) {
                                buf.append(libParameters.getSeparator());
                            }
                            appendLocale(buf, decimal, currencyInteger, LangInteger.getLocale(array[i]));
                        }
                    }
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

        private void appendLocale(Appendable buf, Decimal decimal, int currencyInteger, Locale locale) throws IOException {
            DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
            buf.append(CurrencyInteger.toString(decimal, currencyInteger, symbols));
        }

    }


    private class NombreResolver implements ObjectFormatPartResolver {

        Nombre nombre;
        LocalisationInfo localisationInfo;

        NombreResolver(Nombre nombre, LocalisationInfo localisationInfo) {
            this.nombre = nombre;
            this.localisationInfo = localisationInfo;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            Decimal decimal = nombre.getDecimal();
            int subType = formatPart.getSubType();
            switch (subType) {
                case CUR_SUBTYPE:
                case VALUE_SUBTYPE:
                case CODE_SUBTYPE:
                    buf.append(decimal.toString());
                    break;
                case LIB_SUBTYPE:
                    LibParameters libParameters = (LibParameters) formatPart.getObject();
                    int[] array = libParameters.getlangIntegerArray();
                    if (array == null) {
                        appendLocale(buf, decimal, getFormatLocale(localisationInfo));
                    } else {
                        for (int i = 0; i < array.length; i++) {
                            if (i > 0) {
                                buf.append(libParameters.getSeparator());
                            }
                            appendLocale(buf, decimal, LangInteger.getLocale(array[i]));
                        }
                    }
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

        private void appendLocale(Appendable buf, Decimal decimal, Locale locale) throws IOException {
            DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
            buf.append(decimal.toString(symbols));
        }

    }

}
