/* BasedefichesLib_API - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.include;

import java.text.ParseException;
import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public final class IncludeKey {

    public static final short SPECIAL_CATEGORY = 10;
    public static final short THESAURUS_CATEGORY = 11;
    public static final short CHRONO_CATEGORY = 12;
    public static final short CORPUS_CATEGORY = 13;
    public static final short ALBUM_CATEGORY = 14;
    public final static IncludeKey ASSOCIATION = new IncludeKey("association", SPECIAL_CATEGORY);
    public final static IncludeKey LIAGE = new IncludeKey("liage", SPECIAL_CATEGORY);
    public final static IncludeKey RATTACHEMENT = new IncludeKey("rattachement", SPECIAL_CATEGORY);
    public final static IncludeKey DATE_CREATION = new IncludeKey("date_creation", CHRONO_CATEGORY);
    public final static IncludeKey DATE_MODIFICATION = new IncludeKey("date_modification", CHRONO_CATEGORY);
    private String includeKey;
    private SubsetKey subsetKey;
    private short category;
    private int poidsFilter = -1;

    private IncludeKey(String includeKey, short category) {
        this.includeKey = includeKey;
        this.subsetKey = null;
        this.category = category;
    }

    private IncludeKey(short category, SubsetKey subsetKey, int poidsFilter) {
        this.subsetKey = subsetKey;
        this.includeKey = subsetKey.toString();
        this.poidsFilter = poidsFilter;
        this.category = category;
    }

    public short getCategory() {
        return category;
    }

    public SubsetKey getSubsetKey() {
        return subsetKey;
    }

    public boolean isSubsetInclude() {
        return (getSubsetKey() != null);
    }

    public boolean isSpecialInclude() {
        return (getCategory() == SPECIAL_CATEGORY);
    }

    public boolean isChronoInclude() {
        return (getCategory() == CHRONO_CATEGORY);
    }

    public boolean isThesaurusInclude() {
        return (getCategory() == THESAURUS_CATEGORY);
    }

    public boolean isCorpusInclude() {
        return (getCategory() == CORPUS_CATEGORY);
    }

    public boolean isAlbumInclude() {
        return (getCategory() == ALBUM_CATEGORY);
    }

    public int getPoidsFilter() {
        return poidsFilter;
    }

    public boolean hasPoidsFilter() {
        return (poidsFilter != -1);
    }

    @Override
    public String toString() {
        if (poidsFilter < 1) {
            return includeKey;
        }
        return includeKey + "_" + poidsFilter;
    }

    public static IncludeKey parse(String key) throws ParseException {
        int lastIdx = -1;
        int firstIdx = - 1;
        int length = key.length();
        for (int i = 0; i < length; i++) {
            char carac = key.charAt(i);
            switch (carac) {
                case ':':
                    return oldParse(key, i);
                case '_':
                    if (lastIdx == -1) {
                        lastIdx = i;
                    } else if (firstIdx == -1) {
                        firstIdx = lastIdx;
                        lastIdx = i;
                    } else {
                        throw new ParseException("Too many undescore", i);
                    }
            }
        }
        int poidsFilter = -1;
        String trueKey = key;
        if (lastIdx != -1) {
            try {
                poidsFilter = Integer.parseInt(key.substring(lastIdx + 1));
                if (poidsFilter < 1) {
                    throw new ParseException("negative integer", lastIdx + 1);
                }
                trueKey = key.substring(0, lastIdx);
            } catch (NumberFormatException nfe) {
            }
        }
        return parse(trueKey, poidsFilter);
    }

    private static IncludeKey oldParse(String key, int idx) throws ParseException {
        String trueKey = key.substring(0, idx);
        String part = key.substring(idx + 1);
        try {
            int poidsFilter = Integer.parseInt(part) + 1;
            if (poidsFilter < 1) {
                throw new ParseException(key + " has bad poidsFilter part value", idx);
            }
            return parse(trueKey, poidsFilter);
        } catch (NumberFormatException nfe) {
            throw new ParseException(key + " has bad poidsFilter part value", idx);
        }
    }

    private static IncludeKey parse(String s, int poidsFilter) throws ParseException {
        if (s.equals("association")) {
            testPoidsFilter(poidsFilter);
            return ASSOCIATION;
        }
        if (s.equals("liage")) {
            testPoidsFilter(poidsFilter);
            return LIAGE;
        }
        if (s.equals("rattachement")) {
            testPoidsFilter(poidsFilter);
            return RATTACHEMENT;
        }
        if (s.equals("date_creation")) {
            testPoidsFilter(poidsFilter);
            return DATE_CREATION;
        }
        if (s.equals("date_modification")) {
            testPoidsFilter(poidsFilter);
            return DATE_MODIFICATION;
        }
        SubsetKey subsetKey = SubsetKey.parse(s);
        try {
            short category = getCategory(subsetKey);
            return new IncludeKey(category, subsetKey, poidsFilter);
        } catch (IllegalArgumentException iae) {
            throw new ParseException("unkown subsetkey", 0);
        }
    }

    private static void testPoidsFilter(int poidsFilter) throws ParseException {
        if (poidsFilter != -1) {
            throw new ParseException("unexpected poids filter", 0);
        }
    }

    /**
     * Équivalent de parse() mais en envoyant IllegalArgumentException plutôt que ParseException.
     * à utiliser quand on est sûr de la syntaxe et permet d'éviter un try {} catch {}
     */
    public static IncludeKey build(String key) {
        try {
            return parse(key);
        } catch (ParseException pe) {
            throw new IllegalArgumentException(pe.getMessage());
        }
    }

    @Override
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        IncludeKey otherKey = (IncludeKey) other;
        if (otherKey.category != this.category) {
            return false;
        }
        if (otherKey.poidsFilter != this.poidsFilter) {
            return false;
        }
        return otherKey.includeKey.equals(this.includeKey);
    }

    @Override
    public int hashCode() {
        return includeKey.hashCode() + poidsFilter;
    }

    /**
     *
     *@throws IllegalArgumentException si subsetKey est illégal
     */
    public static IncludeKey newInstance(SubsetKey subsetKey) {
        return newInstance(subsetKey, -1);
    }

    /**
     *
     *@throws IllegalArgumentException si subsetKey est illégal
     */
    public static IncludeKey newInstance(SubsetKey subsetKey, int poidsFilter) {
        if ((poidsFilter == 0) || (poidsFilter < -1)) {
            throw new IllegalArgumentException();
        }
        short category = getCategory(subsetKey);
        return new IncludeKey(category, subsetKey, poidsFilter);
    }

    private static short getCategory(SubsetKey subsetKey) {
        if (subsetKey.isCorpusSubset()) {
            return CORPUS_CATEGORY;
        } else if (subsetKey.isThesaurusSubset()) {
            return THESAURUS_CATEGORY;
        } else if (subsetKey.isAlbumSubset()) {
            return ALBUM_CATEGORY;
        } else {
            throw new IllegalArgumentException();
        }
    }

}
