/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.sphere;

import net.mapeadores.basedefiches.SubsetKey;
import java.util.Map;
import java.util.HashMap;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public final class RedacteurKey implements Comparable {

    /**
     * Indique que le RedacteurBaseKey doit être représenté sous la forme {nomsphere}_{idsphere}
     */
    public final static short UNDERSCORE_STYLE = 1;
    /**
     * Indique que le RedacteurBaseKey doit être représenté sous la forme {idsphere}[{nomsphere}]
     */
    public final static short BRACKET_STYLE = 2;
    private static final Map<String, RedacteurKey> internMap = new HashMap<String, RedacteurKey>();
    private String redacteurKeyString;
    private int idsphere;
    private SubsetKey sphereKey;

    private RedacteurKey(String redacteurKeyString, SubsetKey sphereKey, int idsphere) {
        this.redacteurKeyString = redacteurKeyString.intern();
        this.sphereKey = sphereKey;
        this.idsphere = idsphere;
        intern(this);
    }

    private static RedacteurKey initRedacteurKey(SubsetKey sphereKey, int idsphere) {
        String key = toKeyString(sphereKey, idsphere);
        RedacteurKey current = internMap.get(key);
        if (current != null) {
            return current;
        }
        return new RedacteurKey(key, sphereKey, idsphere);
    }

    private static RedacteurKey initRedacteurKey(String sphereSubId, int idsphere) throws ParseException {
        String key = sphereSubId + "_" + Idsphere.toString(idsphere);
        RedacteurKey current = internMap.get(key);
        if (current != null) {
            return current;
        }
        return new RedacteurKey(key, SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereSubId), idsphere);
    }

    private synchronized static void intern(RedacteurKey redacteurKey) {
        internMap.put(redacteurKey.redacteurKeyString, redacteurKey);
    }

    public int getIdsphere() {
        return idsphere;
    }

    public String getIdsphereString() {
        return Idsphere.toString(idsphere);
    }

    public SubsetKey getSphereKey() {
        return sphereKey;
    }

    public int compareTo(Object o) {
        RedacteurKey autreid = (RedacteurKey) o;
        if (idsphere > autreid.idsphere) {
            return 1;
        }
        if (idsphere < autreid.idsphere) {
            return -1;
        }
        if (sphereKey == null) {
            if (autreid.sphereKey == null) {
                return 0;
            } else {
                return -1;
            }
        } else if (autreid.sphereKey == null) {
            return 1;
        }
        return sphereKey.compareTo(autreid.sphereKey);
    }

    @Override
    public int hashCode() {
        return idsphere;
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof RedacteurKey)) {
            return false;
        }
        RedacteurKey autreid = (RedacteurKey) obj;
        if (idsphere != autreid.idsphere) {
            return false;
        }
        if (sphereKey == null) {
            if (autreid.sphereKey == null) {
                return true;
            } else {
                return false;
            }
        } else if (autreid.sphereKey == null) {
            return false;
        }
        return sphereKey.equals(autreid.sphereKey);
    }

    @Override
    public String toString() {
        return redacteurKeyString;
    }

    public String toString(short style) {
        if (sphereKey == null) {
            return Idsphere.toString(idsphere);
        }
        switch (style) {
            case UNDERSCORE_STYLE:
                return redacteurKeyString;
            default:
                return Idsphere.toString(idsphere) + "[" + sphereKey.getSubsetName() + "]";
        }
    }

    private static String toKeyString(SubsetKey subsetKey, int idsphere) {
        return subsetKey.getSubsetName() + "_" + Idsphere.toString(idsphere);
    }

    public static RedacteurKey parse(String s) throws ParseException {
        RedacteurKey current = internMap.get(s);
        if (current != null) {
            return current;
        }
        int lastIndex = s.length() - 1;
        int idx1 = s.indexOf('_');
        if (idx1 > 0) {
            if (idx1 == lastIndex) {
                throw new ParseException("End with _", idx1);
            }
            SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, s.substring(0, idx1));
            int idsphere = Idsphere.parse(s.substring(idx1 + 1));
            return new RedacteurKey(s, sphereKey, idsphere);
        } else {
            int idx2 = s.indexOf("[");
            if (idx2 == -1) {
                throw new ParseException("Missing _ or [", 0);
            }
            if (s.charAt(lastIndex) != ']') {
                throw new ParseException("]", lastIndex);
            }
            SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, s.substring(idx2 + 1, lastIndex));
            int idsphere = Idsphere.parse(s.substring(0, idx2));
            String keyString = toKeyString(sphereKey, idsphere);
            current = internMap.get(keyString);
            if (current != null) {
                return current;
            }
            return new RedacteurKey(keyString, sphereKey, idsphere);
        }
    }

    /**
     * Parse un idcomplet sous la forme {nomsphere}_{idsphere} ou {idsphere}[{nomsphere}]. Parseur laxiste qui ne tient pas compte des blancs en début et en fin.
     * Si defaultSphereKey != null, il prend également sous la form {idsphere}.
     */
    public static RedacteurKey parseFromUserString(String keyString, SubsetKey defaultSphereKey) {
        if ((keyString == null) || (keyString.length() == 0)) {
            return null;
        }
        StringBuilder bufid = new StringBuilder();
        int taille = keyString.length();
        int idsphere = 0;
        boolean debut = true;
        boolean avecblanc = false;
        int debutnomsphere = -1;
        int posSouligne = -1;
        for (int i = 0; i < taille; i++) {
            char carac = keyString.charAt(i);
            if (carac <= '\u0020') {
                if (debut) {
                    continue;
                } else {
                    avecblanc = true;
                    continue;
                }
            }
            if (carac == '[') {
                debutnomsphere = i + 1;
                break;
            }
            if (carac == '_') {
                posSouligne = i;
            } else if (Idsphere.toSixbits(carac) == -1) {
                return null;
            }
            debut = false;
            if (avecblanc) {
                return null;
            }
            bufid.append(carac);
        }
        if (debutnomsphere == -1) {
            if (posSouligne == 0) {
                return null;
            }
            if (posSouligne > 0) {
                try {
                    idsphere = Idsphere.parse(bufid.substring(posSouligne + 1));
                    String nomsphere = bufid.substring(0, posSouligne);
                    return initRedacteurKey(nomsphere, idsphere);
                } catch (ParseException pe) {
                    return null;
                }
            } else {
                if (defaultSphereKey == null) {
                    return null;
                }
                try {
                    idsphere = Idsphere.parse(bufid.toString());
                    return initRedacteurKey(defaultSphereKey, idsphere);
                } catch (ParseException pe) {
                    return null;
                }
            }
        } else {
            try {
                idsphere = Idsphere.parse(bufid.toString());
            } catch (ParseException pe) {
                return null;
            }
        }
        StringBuilder bufnom = new StringBuilder();
        debut = true;
        avecblanc = false;
        int finnom = -1;
        for (int i = debutnomsphere; i < taille; i++) {
            char carac = keyString.charAt(i);
            if (carac <= '\u0020') {
                if (debut) {
                    continue;
                } else {
                    avecblanc = true;
                    continue;
                }
            }
            if (carac == ']') {
                finnom = i + 1;
                break;
            }
            debut = false;
            if (avecblanc) {
                return null;
            }
            bufnom.append(carac);
            finnom = i + 1;
        }
        if (finnom < taille) {
            for (int i = finnom; i < taille; i++) {
                char carac = keyString.charAt(i);
                if (carac <= '\u0020') {
                    continue;
                } else {
                    return null;
                }
            }
        }
        try {
            return initRedacteurKey(bufnom.toString(), idsphere);
        } catch (ParseException pe) {
            return null;
        }
    }

}


