 /* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.util.primitives.DateInteger;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.TreeMap;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author  Vincent Calame
 */
public class AbregeFicheSortedList implements BufferedFicheEligibility, SelectionConstants {

    private short sortType;
    private Collator collator;
    private Map<AbregeFiche, AbregeFicheBundle> abregeFicheBundleMap = new HashMap<AbregeFiche, AbregeFicheBundle>();
    private Comparator<AbregeFiche> abregeFicheComparator;
    private TriParCorpus[] trisParCorpus;

    private AbregeFicheSortedList(short sortType, Comparator<AbregeFiche> abregeFicheComparator) {
        this.sortType = sortType;
        this.abregeFicheComparator = abregeFicheComparator;
    }

    public static AbregeFicheSortedList noSort(Collection<AbregeFiche> abregeFicheCollection) {
        AbregeFicheSortedList afsl = new AbregeFicheSortedList(NONE_SORTTYPE, null);
        afsl.initCollection(abregeFicheCollection);
        return afsl;
    }

    public static AbregeFicheSortedList sortByCreationDate(Collection<AbregeFiche> abregeFicheCollection) {
        Comparator<AbregeFiche> abregeFicheComparator = new ComparateurParDateSaisie();
        AbregeFicheSortedList afsl = new AbregeFicheSortedList(CREATIONDATE_SORTTYPE, abregeFicheComparator);
        afsl.initCollection(abregeFicheCollection);
        return afsl;
    }

    public static AbregeFicheSortedList sortByModificationDate(Collection<AbregeFiche> abregeFicheCollection) {
        Comparator<AbregeFiche> abregeFicheComparator = new ComparateurParDateModif();
        AbregeFicheSortedList afsl = new AbregeFicheSortedList(MODIFICATIONDATE_SORTTYPE, abregeFicheComparator);
        afsl.initCollection(abregeFicheCollection);
        return afsl;
    }

    public static AbregeFicheSortedList sortByIdcorpus(Collection<AbregeFiche> abregeFicheCollection) {
        Comparator<AbregeFiche> abregeFicheComparator = new ComparateurParIdcorpus();
        AbregeFicheSortedList afsl = new AbregeFicheSortedList(IDCORPUS_SORTTYPE, abregeFicheComparator);
        afsl.initCollection(abregeFicheCollection);
        return afsl;
    }

    public static AbregeFicheSortedList sortByTitre(Collection<AbregeFiche> abregeFicheCollection, int langInteger) {
        Collator collator = Collator.getInstance(LangInteger.getLocale(langInteger));
        collator.setStrength(Collator.PRIMARY);
        return sortByTitre(abregeFicheCollection, collator);
    }

    public static AbregeFicheSortedList sortByTitre(Collection<AbregeFiche> abregeFicheCollection, Collator collator) {
        Comparator<AbregeFiche> abregeFicheComparator = new ComparateurParTitre(collator);
        AbregeFicheSortedList afsl = new AbregeFicheSortedList(TITRE_SORTTYPE, abregeFicheComparator);
        afsl.collator = collator;
        afsl.initCollection(abregeFicheCollection);
        return afsl;
    }

    private void initCollection(Collection<AbregeFiche> abregeFicheCollection) {
        TreeMap<SubsetKey, TriParCorpus> corpusMap = new TreeMap<SubsetKey, TriParCorpus>();
        for (AbregeFiche abregeFiche : abregeFicheCollection) {
            if (abregeFicheBundleMap.containsKey(abregeFiche)) {
                continue;
            }
            Corpus corpus = abregeFiche.getCorpus();
            TriParCorpus tri = (TriParCorpus) corpusMap.get(corpus.getSubsetKey());
            if (tri == null) {
                tri = new TriParCorpus(corpus);
                corpusMap.put(corpus.getSubsetKey(), tri);
            }
            AbregeFicheBundle bundle = tri.addAbregeFiche(abregeFiche);
            abregeFicheBundleMap.put(abregeFiche, bundle);
        }
        trisParCorpus = new TriParCorpus[corpusMap.size()];
        int p = 0;
        for (TriParCorpus tri : corpusMap.values()) {
            trisParCorpus[p] = tri;
            tri.setPosition(p);
            if (sortType != NONE_SORTTYPE) {
                tri.sort();
            }
            p++;
        }
    }

    /**
     * Retourne le type de tri appliqué.
     */
    public short getSortType() {
        return sortType;
    }

    /**
     * Retourne l'instance de Collator éventuellement utilisée.
     */
    public Collator getCollator() {
        return collator;
    }

    /**
     * Retourne le nombre de corpus auxquels des fiches de la liste appartiennent.
     */
    public int getCorpusCount() {
        return trisParCorpus.length;
    }

    /**
     * Retourne le i-ème corpus ( i est compris entre 0 et getCorpusCount()).
     */
    public Corpus getCorpus(int i) {
        return trisParCorpus[i].corpus;
    }

    /**
     * Retourne la liste des instances d'AbregeFiche pour le i-ème corpus
     */
    public SubsetItemList getCorpusAbregeFicheList(int i) {
        return trisParCorpus[i];
    }

    /**
     * Implémentation de l'interface FicheEgibility. Une fiche est éligible si elle appartient à la liste.
     * Autrement dit, abregeSortedList.accept(abregeFiche) == (AbregeSortedList.indexOf(abregeFiche) == -1).
     * La méthode <em>accept()</em> est néanmoins beaucoup plus rapide.
     */
    public boolean accept(AbregeFiche abregeFiche) {
        return abregeFicheBundleMap.containsKey(abregeFiche);
    }

    public int getAbregeFicheCount() {
        return abregeFicheBundleMap.size();
    }

    /**
     * Retourne l'abrégé précédent à la celui indiqué dans l'ordre de tri.
     * Retourne <em>null</em> si la fiche n'appartient pas à la liste ou si c'est la première.
     **/
    public AbregeFiche getPrevious(AbregeFiche abregeFiche) {
        AbregeFicheBundle bundle = abregeFicheBundleMap.get(abregeFiche);
        if (bundle == null) {
            return null;
        }
        TriParCorpus tri = bundle.getTriCorpus();
        if (bundle.getPosition() == 0) {
            if (tri.getPosition() == 0) {
                return null;
            }
            TriParCorpus precedent = trisParCorpus[tri.getPosition() - 1];
            return precedent.bundleArray[precedent.size - 1].getAbregeFiche();
        }
        return tri.bundleArray[bundle.getPosition() - 1].getAbregeFiche();
    }

    /**
     * Retourne l'abrégé suivant dans la liste.
     * Retourne <em>null</em> si la fiche n'appartient pas à la liste ou si c'est la dernière.
     */
    public AbregeFiche getNext(AbregeFiche abregeFiche) {
        AbregeFicheBundle bundle = abregeFicheBundleMap.get(abregeFiche);
        if (bundle == null) {
            return null;
        }
        TriParCorpus tri = bundle.getTriCorpus();
        if (bundle.getPosition() == tri.size - 1) {
            if (tri.getPosition() == (trisParCorpus.length - 1)) {
                return null;
            }
            TriParCorpus suivant = trisParCorpus[tri.getPosition() + 1];
            return suivant.bundleArray[0].getAbregeFiche();
        }
        return tri.bundleArray[bundle.getPosition() + 1].getAbregeFiche();
    }

    /*
     * retourne la position de la fiche dans l'ensemble des fiches du même corpus.
     * Retourne -1 si la fiche n'est pas présente.
     */
    public int getRelativePosition(AbregeFiche abregeFiche) {
        AbregeFicheBundle bundle = abregeFicheBundleMap.get(abregeFiche);
        if (bundle == null) {
            return -1;
        }
        return bundle.getPosition();
    }

    /**
     * Retourne la position de la fiche dans l'ensemble de la liste.
     * Retourne -1 si la fiche n'est pas présente.
     */
    public int getAbsolutePosition(AbregeFiche abregeFiche) {
        Corpus corpus = abregeFiche.getCorpus();
        int posrelat = getRelativePosition(abregeFiche);
        if (posrelat == -1) {
            return -1;
        }
        int cumul = 0;
        for (int i = 0; i < trisParCorpus.length; i++) {
            if (trisParCorpus[i].corpus.equals(corpus)) {
                break;
            } else {
                cumul = cumul + trisParCorpus[i].size;
            }
        }
        return cumul + posrelat;
    }

    public List<AbregeFiche> toList() {
        List<AbregeFiche> list = new ArrayList<AbregeFiche>();
        int length = trisParCorpus.length;
        for (int k = 0; k < length; k++) {
            TriParCorpus triParCorpus = trisParCorpus[k];
            AbregeFicheBundle[] array = triParCorpus.bundleArray;
            int size = triParCorpus.size;
            for (int i = 0; i < size; i++) {
                list.add(array[i].getAbregeFiche());
            }
        }
        return list;
    }

    public SubsetItemList getAbregeFicheListByCorpus(Corpus corpus) {
        for (int k = 0; k < trisParCorpus.length; k++) {
            if (trisParCorpus[k].corpus.equals(corpus)) {
                return trisParCorpus[k];
            }
        }
        return null;
    }


    private class TriParCorpus implements SubsetItemList {

        Corpus corpus;
        AbregeFicheBundle[] bundleArray = new AbregeFicheBundle[32];
        int size = 0;
        int positioncorpus;

        TriParCorpus(Corpus corpus) {
            this.corpus = corpus;
        }

        public int getPosition() {
            return positioncorpus;
        }

        private void setPosition(int position) {
            this.positioncorpus = position;
        }

        public int getSubsetItemCount() {
            return size;
        }

        public SubsetItem getSubsetItem(int i) {
            if (i >= size) {
                throw new IndexOutOfBoundsException();
            }
            return bundleArray[i].getAbregeFiche();
        }

        AbregeFicheBundle addAbregeFiche(AbregeFiche abregeFiche) {
            if (size == bundleArray.length) {
                AbregeFicheBundle[] nv = new AbregeFicheBundle[size * 2];
                System.arraycopy(bundleArray, 0, nv, 0, size);
                bundleArray = nv;
            }
            AbregeFicheBundle bundle = new AbregeFicheBundle(this, abregeFiche, size);
            bundleArray[size] = bundle;
            size++;
            return bundle;
        }

        void sort() {
            BundleComparator bundleComparator = new BundleComparator(abregeFicheComparator);
            Arrays.sort(bundleArray, 0, size, bundleComparator);
            for (int i = 0; i < size; i++) {
                bundleArray[i].setPosition(i);
            }
        }

    }


    private class AbregeFicheBundle {

        private TriParCorpus triParCorpus;
        private int position;
        private AbregeFiche abregeFiche;

        AbregeFicheBundle(TriParCorpus triParCorpus, AbregeFiche abregeFiche, int position) {
            this.triParCorpus = triParCorpus;
            this.position = position;
            this.abregeFiche = abregeFiche;
        }

        TriParCorpus getTriCorpus() {
            return triParCorpus;
        }

        AbregeFiche getAbregeFiche() {
            return abregeFiche;
        }

        void setPosition(int position) {
            this.position = position;
        }

        int getPosition() {
            return position;
        }

    }


    static private class BundleComparator implements Comparator<AbregeFicheBundle> {

        Comparator<AbregeFiche> abregeFicheComparator;

        BundleComparator(Comparator<AbregeFiche> abregeFicheComparator) {
            this.abregeFicheComparator = abregeFicheComparator;
        }

        public int compare(AbregeFicheBundle bundle1, AbregeFicheBundle bundle2) {
            return abregeFicheComparator.compare(bundle1.getAbregeFiche(), bundle2.getAbregeFiche());
        }

    }


    static private class ComparateurParTitre implements Comparator<AbregeFiche> {

        Collator collator;

        ComparateurParTitre(Collator collator) {
            this.collator = collator;
        }

        public int compare(AbregeFiche abrg1, AbregeFiche abrg2) {
            int resultat = collator.compare(abrg1.getTitre(), abrg2.getTitre());
            if (resultat == 0) {
                resultat = compareInt(abrg1.getIdcorpus(), abrg2.getIdcorpus());
                if (resultat == 0) {
                    resultat = (new Integer(abrg1.hashCode())).compareTo(new Integer(abrg2.hashCode()));
                }
            }
            return resultat;
        }

    }


    static private class ComparateurParIdcorpus implements Comparator<AbregeFiche> {

        public int compare(AbregeFiche abrg1, AbregeFiche abrg2) {
            int resultat = compareInt(abrg1.getIdcorpus(), abrg2.getIdcorpus());
            if (resultat == 0) {
                resultat = abrg1.getTitre().compareTo(abrg2.getTitre());
                if (resultat == 0) {
                    resultat = (new Integer(abrg1.hashCode())).compareTo(new Integer(abrg2.hashCode()));
                }
            }
            return resultat;
        }

    }


    static private class ComparateurParDateSaisie implements Comparator<AbregeFiche> {

        public int compare(AbregeFiche abrg1, AbregeFiche abrg2) {
            int resultat = -DateInteger.compare(abrg1.getCreationDate(), abrg2.getCreationDate());
            if (resultat == 0) {
                resultat = -compareInt(abrg1.getIdcorpus(), abrg2.getIdcorpus());
                if (resultat == 0) {
                    resultat = (new Integer(abrg1.hashCode())).compareTo(new Integer(abrg2.hashCode()));
                }
            }
            return resultat;
        }

    }


    static private class ComparateurParDateModif implements Comparator<AbregeFiche> {

        public int compare(AbregeFiche abrg1, AbregeFiche abrg2) {
            int resultat = -DateInteger.compare(abrg1.getModificationDate(), abrg2.getModificationDate());
            if (resultat == 0) {
                resultat = -DateInteger.compare(abrg1.getCreationDate(), abrg2.getCreationDate());
                if (resultat == 0) {
                    resultat = -compareInt(abrg1.getIdcorpus(), abrg2.getIdcorpus());
                    if (resultat == 0) {
                        resultat = (new Integer(abrg1.hashCode())).compareTo(new Integer(abrg2.hashCode()));
                    }
                }
            }
            return resultat;
        }

    }

    static int compareInt(int i1, int i2) {
        if (i1 < i2) {
            return -1;
        }
        if (i1 > i2) {
            return 1;
        }
        return 0;
    }

}
