/* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.pointeurs.PointeurUtils;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.conditions.TextTestEngine;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class FicheSelectUnit {

    private final static FieldKey[] titreScopeArray = {FieldKey.TITRE};
    private FichePointeur fichePointeur;
    private IndexationSelector indexationSelector;
    private RangeList rangeList;
    private RedacteurKey[] redacteurKeyArray;
    private SubsetKey[] sphereKeyArray;
    private FieldTest fieldTest;
    private PoidsCondition poidsCondition;

    public FicheSelectUnit(FichePointeur fichePointeur) {
        this.fichePointeur = fichePointeur;
    }

    public void setFicheByMotcleSelector(IndexationSelector indexationSelector) {
        this.indexationSelector = indexationSelector;
    }

    public void setFieldCondition(Condition fieldCondition, int langInteger, FieldKey[] fieldKeyArray) {
        if (fieldCondition == null) {
            fieldTest = null;
        } else {
            fieldTest = initFieldTest(fieldCondition, langInteger);
            fieldTest.setFieldKeyArray(fieldKeyArray);
        }

    }

    public void setRedacteurKeyArray(RedacteurKey[] redacteurKeyArray) {
        this.redacteurKeyArray = redacteurKeyArray;
    }

    public void setSphereKeyArray(SubsetKey[] sphereKeyArray) {
        this.sphereKeyArray = sphereKeyArray;
    }

    public void setRangeList(RangeList rangeList) {
        this.rangeList = rangeList;
    }

    public void setPoidsCondition(PoidsCondition poidsCondition) {
        this.poidsCondition = poidsCondition;
    }

    public boolean isCurrentAbregeFicheSelected(int poids) {
        if (poidsCondition != null) {
            boolean inRange = poidsCondition.getPoidsRangeList().contains(poids);
            if (poidsCondition.isExclusionRangeList()) {
                if (inRange) {
                    return false;
                }
            } else {
                if (!inRange) {
                    return false;
                }
            }
        }
        AbregeFiche abregeFiche = (AbregeFiche) fichePointeur.getCurrentSubsetItem();
        if (rangeList != null) {
            if (!rangeList.contains(abregeFiche.getIdcorpus())) {
                return false;
            }
        }
        if ((redacteurKeyArray != null) || (sphereKeyArray != null)) {
            if (!isRedacteurSelected(abregeFiche)) {
                return false;
            }
        }
        if (indexationSelector != null) {
            if (!indexationSelector.isSelected(abregeFiche)) {
                return false;
            }
        }
        if (fieldTest != null) {
            if (!fieldTest.isSelected()) {
                return false;
            }
        }
        return true;
    }

    public static FicheSelectUnit newInstance(FichePointeur fichePointeur, FicheSelectOptions ficheSelectOptions, int langInteger) {
        FicheSelectUnit ficheSelectUnit = new FicheSelectUnit(fichePointeur);
        ficheSelectUnit.setRangeList(ficheSelectOptions.getRangeList());
        ficheSelectUnit.setRedacteurKeyArray(ficheSelectOptions.getRedacteurKeyArray());
        ficheSelectUnit.setSphereKeyArray(ficheSelectOptions.getSphereKeyArray());
        ficheSelectUnit.setPoidsCondition(ficheSelectOptions.getPoidsCondition());
        Condition fieldCondition = ficheSelectOptions.getFieldCondition();
        if (fieldCondition != null) {
            short scope = ficheSelectOptions.getFieldConditionScope();
            FieldKey[] fieldKeyArray = null;
            switch (scope) {
                case FicheSelectOptions.TITRE_SCOPE:
                    fieldKeyArray = titreScopeArray;
                    break;
                case FicheSelectOptions.ENTETE_SCOPE:
                    fieldKeyArray = CorpusMetadataUtils.getEnteteFieldKeyArray(((Corpus) fichePointeur.getSubset()).getCorpusMetadata());
                    break;
                case FicheSelectOptions.FICHE_SCOPE:
                    fieldKeyArray = CorpusMetadataUtils.getFicheFieldKeyArray(((Corpus) fichePointeur.getSubset()).getCorpusMetadata());
                    break;
                case FicheSelectOptions.SELECTION_SCOPE:
                    fieldKeyArray = ficheSelectOptions.getFieldKeyArray();
                    break;
                default:
                    throw new SwitchException("scope: " + scope);
            }
            ficheSelectUnit.setFieldCondition(fieldCondition, langInteger, fieldKeyArray);
        }
        return ficheSelectUnit;
    }

    private boolean isRedacteurSelected(AbregeFiche abregeFiche) {
        if (sphereKeyArray != null) {
            int length = sphereKeyArray.length;
            for (int i = 0; i < length; i++) {
                SubsetKey subsetKey = sphereKeyArray[i];
                if (abregeFiche.hasRedacteurInSphere(subsetKey)) {
                    return true;
                }
            }
        }
        if (redacteurKeyArray != null) {
            int length = redacteurKeyArray.length;
            for (int i = 0; i < length; i++) {
                RedacteurKey redacteurKey = redacteurKeyArray[i];
                if (abregeFiche.hasRedacteur(redacteurKey)) {
                    return true;
                }
            }
        }
        return false;
    }

    private FieldTest initFieldTest(Condition fieldCondition, int langInteger) {
        switch (fieldCondition.getState()) {
            case ConditionConstants.X:
                return new ConstantFieldTest(false);
            case ConditionConstants.N:
            case ConditionConstants.U:
                return new ConstantFieldTest(true);
            case ConditionConstants.T:
                return new NotEmptyFieldTest();
            case ConditionConstants.A:
                return new EmptyFieldTest();
            case ConditionConstants.P:
            case ConditionConstants.PA:
                return new ContentFieldTest(fieldCondition, langInteger);
            default:
                throw new SwitchException("state = " + fieldCondition.getState());
        }
    }


    private abstract class FieldTest {

        protected FieldKey[] fieldKeyArray;

        public abstract boolean isSelected();

        public void setFieldKeyArray(FieldKey[] fieldKeyArray) {
            this.fieldKeyArray = fieldKeyArray;
        }

    }


    private class ConstantFieldTest extends FieldTest {

        private boolean constantAnswer;

        ConstantFieldTest(boolean constantAnswer) {
            this.constantAnswer = constantAnswer;
        }

        public boolean isSelected() {
            return constantAnswer;
        }

    }


    private class EmptyFieldTest extends FieldTest {

        EmptyFieldTest() {
        }

        public boolean isSelected() {
            int length = fieldKeyArray.length;
            for (int i = 0; i < length; i++) {
                FieldKey fieldKey = fieldKeyArray[i];
                Object obj = fichePointeur.getValue(fieldKey);
                if (obj != null) {
                    return false;
                }
            }
            return true;
        }

    }


    private class NotEmptyFieldTest extends FieldTest {

        NotEmptyFieldTest() {
        }

        public boolean isSelected() {
            int length = fieldKeyArray.length;
            for (int i = 0; i < length; i++) {
                FieldKey fieldKey = fieldKeyArray[i];
                Object obj = fichePointeur.getValue(fieldKey);
                if (obj != null) {
                    return true;
                }
            }
            return false;
        }

    }


    private class ContentFieldTest extends FieldTest {

        private TextTestEngine textTestEngine;
        private boolean withEmpty;

        private ContentFieldTest(Condition contentCondition, int langInteger) {
            withEmpty = (contentCondition.getState() == ConditionConstants.PA);
            textTestEngine = TextTestEngine.newInstance(contentCondition, langInteger);
        }

        public boolean isSelected() {
            int length = fieldKeyArray.length;
            boolean empty = true;
            textTestEngine.start();
            for (int i = 0; i < length; i++) {
                FieldKey fieldKey = fieldKeyArray[i];
                Object obj = fichePointeur.getValue(fieldKey);
                if (obj != null) {
                    empty = false;
                    PointeurUtils.testContent(textTestEngine, obj);
                    if (textTestEngine.canStop()) {
                        break;
                    }
                }
            }
            if (empty) {
                if (withEmpty) {
                    return true;

                } else {
                    return false;

                }
            }
            return textTestEngine.getResult();
        }

    }

}
