/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class IndexationSelectUnit {

    private IndexationTest indexationTest;
    private Thesaurus[] thesaurusArray;
    private Croisement[] croisementArray;
    private int arrayLength;

    private IndexationSelectUnit() {
    }

    public static IndexationSelectUnit newInstance(Corpus corpus, Thesaurus[] thesaurusArray, short conditionState, MotcleTestEngine motcleTestEngine, MotcleRangeTestCache rangeTestCache) {
        IndexationSelectUnit indexationSelectUnit = new IndexationSelectUnit();
        indexationSelectUnit.setThesaurusArray(corpus, thesaurusArray);
        indexationSelectUnit.initIndexationTest(conditionState, motcleTestEngine, rangeTestCache);
        return indexationSelectUnit;
    }

    private void setThesaurusArray(Corpus corpus, Thesaurus[] thesaurusArray) {
        SubsetKey corpusKey = corpus.getSubsetKey();
        Basedefiches basedefiches = corpus.getBasedefiches();
        int length = thesaurusArray.length;
        croisementArray = new Croisement[length];
        for (int i = 0; i < length; i++) {
            Thesaurus thesaurus = thesaurusArray[i];
            croisementArray[i] = basedefiches.getCroisement(corpusKey, thesaurus.getSubsetKey());
        }
        this.arrayLength = length;
        this.thesaurusArray = thesaurusArray;
    }

    private void initIndexationTest(short state, MotcleTestEngine motcleTestEngine, MotcleRangeTestCache rangeTestCache) {
        if (state == ConditionConstants.X) {
            this.indexationTest = new ConstantIndexationTest(false);
            return;
        }
        if ((state == ConditionConstants.N) || (state == ConditionConstants.U)) {
            if (rangeTestCache != null) {
                this.indexationTest = new RangeOnlyIndexationTest(rangeTestCache);
            } else {
                this.indexationTest = new ConstantIndexationTest(true);
            }
            return;
        }
        if (state == ConditionConstants.T) {
            this.indexationTest = new NotEmptyIndexationTest(true, rangeTestCache);
            return;
        }
        if (state == ConditionConstants.A) {
            this.indexationTest = new NotEmptyIndexationTest(false, rangeTestCache);
            return;
        }
        if (state == ConditionConstants.P) {
            this.indexationTest = new ContentIndexationTest(motcleTestEngine, false, rangeTestCache);
            return;
        }
        if (state == ConditionConstants.PA) {
            this.indexationTest = new ContentIndexationTest(motcleTestEngine, true, rangeTestCache);
            return;
        }
        throw new SwitchException("condition.getState():" + state);
    }

    public boolean isSelected(AbregeFiche abregeFiche) {
        return indexationTest.isSelected(abregeFiche);
    }


    private abstract class IndexationTest {

        public abstract boolean isSelected(AbregeFiche abregeFiche);

    }


    private class ConstantIndexationTest extends IndexationTest {

        private boolean constantAnswer;

        private ConstantIndexationTest(boolean constantAnswer) {
            this.constantAnswer = constantAnswer;
        }

        public boolean isSelected(AbregeFiche abregeFiche) {
            return constantAnswer;
        }

    }


    private class RangeOnlyIndexationTest extends IndexationTest {

        private MotcleRangeTestCache rangeTestCache;

        private RangeOnlyIndexationTest(MotcleRangeTestCache rangeTestCache) {
            this.rangeTestCache = rangeTestCache;
        }

        public boolean isSelected(AbregeFiche abregeFiche) {
            int length = arrayLength;
            int ficheTicket = abregeFiche.getTicket();
            SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
            for (int i = 0; i < length; i++) {
                Croisement croisement = croisementArray[i];
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, ficheTicket);
                int count = croisementItem.getCount();
                if (count > 0) {
                    rangeTestCache.setCurrentThesaurus(thesaurusArray[i]);
                    for (int j = 0; j < count; j++) {
                        int ticket = croisementItem.getTicket(j);
                        if (rangeTestCache.isSelected(ticket)) {
                            return true;
                        }
                    }
                }
            }
            return false;
        }

    }


    private class NotEmptyIndexationTest extends IndexationTest {

        private MotcleRangeTestCache rangeTestCache;
        private boolean notempty;

        private NotEmptyIndexationTest(boolean notempty, MotcleRangeTestCache rangeTestCache) {
            this.notempty = notempty;
            this.rangeTestCache = rangeTestCache;
        }

        public boolean isSelected(AbregeFiche abregeFiche) {
            int length = arrayLength;
            int ficheTicket = abregeFiche.getTicket();
            SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
            for (int i = 0; i < length; i++) {
                Croisement croisement = croisementArray[i];
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, ficheTicket);
                int count = croisementItem.getCount();
                if (count > 0) {
                    if (rangeTestCache == null) {
                        return notempty;
                    }
                    rangeTestCache.setCurrentThesaurus(thesaurusArray[i]);
                    for (int j = 0; j < count; j++) {
                        int ticket = croisementItem.getTicket(j);
                        if (rangeTestCache.isSelected(ticket)) {
                            return notempty;
                        }
                    }
                }
            }
            return (!notempty);
        }

    }


    private class ContentIndexationTest extends IndexationTest {

        private MotcleTestEngine motcleTestEngine;
        private MotcleRangeTestCache rangeTestCache;
        private boolean emptyAllowed;

        private ContentIndexationTest(MotcleTestEngine motcleTestEngine, boolean emptyAllowed, MotcleRangeTestCache rangeTestCache) {
            this.motcleTestEngine = motcleTestEngine;
            this.rangeTestCache = rangeTestCache;
            this.emptyAllowed = emptyAllowed;
        }

        public boolean isSelected(AbregeFiche abregeFiche) {
            int length = arrayLength;
            int ficheTicket = abregeFiche.getTicket();
            SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
            motcleTestEngine.start();
            boolean indexationEmpty = true;
            for (int i = 0; i < length; i++) {
                Croisement croisement = croisementArray[i];
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, ficheTicket);
                int count = croisementItem.getCount();
                if (count > 0) {
                    Thesaurus thesaurus = thesaurusArray[i];
                    if (rangeTestCache != null) {
                        rangeTestCache.setCurrentThesaurus(thesaurus);
                    }
                    motcleTestEngine.setCurrentThesaurus(thesaurus);
                    for (int j = 0; j < count; j++) {
                        int ticket = croisementItem.getTicket(j);
                        if (rangeTestCache != null) {
                            if (!rangeTestCache.isSelected(ticket)) {
                                continue;
                            }
                        }
                        indexationEmpty = false;
                        motcleTestEngine.addMotcleTicket(ticket);
                        if (motcleTestEngine.canStop()) {
                            break;

                        }
                    }
                }
                if (motcleTestEngine.canStop()) {
                    break;

                }
            }
            if (indexationEmpty) {
                return emptyAllowed;
            }
            return motcleTestEngine.getResult();
        }

    }

}
