/* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.pointeurs.MotclePointeur;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.TextTestEngine;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class MotcleSelectUnit {

    private MotclePointeur motclePointeur;
    private RangeList idthsRangeList;
    private RangeList levelRangeList;
    private boolean idalphaOnly;
    private int langInteger;
    private FieldTest fieldTest;
    private PoidsCondition poidsCondition;

    public MotcleSelectUnit(MotclePointeur motclePointeur, int langInteger) {
        this.motclePointeur = motclePointeur;
        this.langInteger = langInteger;
    }

    public static MotcleSelectUnit newInstance(MotclePointeur motclePointeur, MotcleSelectOptions motcleSelectOptions, int langInteger) {
        MotcleSelectUnit motcleSelectUnit = new MotcleSelectUnit(motclePointeur, langInteger);
        motcleSelectUnit.setLevelRangeList(motcleSelectOptions.getLevelRangeList());
        motcleSelectUnit.setIdthsRangeList(motcleSelectOptions.getIdthsRangeList());
        motcleSelectUnit.setIdalphaOnly(motcleSelectOptions.isIdalphaOnly());
        motcleSelectUnit.setFieldCondition(motcleSelectOptions.getFieldCondition(), langInteger);
        motcleSelectUnit.setPoidsCondition(motcleSelectOptions.getPoidsCondition());
        return motcleSelectUnit;
    }

    private void setIdthsRangeList(RangeList idthsRangeList) {
        this.idthsRangeList = idthsRangeList;
    }

    private void setLevelRangeList(RangeList levelRangeList) {
        this.levelRangeList = levelRangeList;
    }

    public void setIdalphaOnly(boolean idalphaOnly) {
        this.idalphaOnly = idalphaOnly;
    }

    public void setFieldCondition(Condition fieldCondition, int langInteger) {
        if (fieldCondition == null) {
            fieldTest = null;
        } else {
            fieldTest = initFieldTest(fieldCondition, langInteger);
        }

    }

    public void setPoidsCondition(PoidsCondition poidsCondition) {
        this.poidsCondition = poidsCondition;
    }

    private FieldTest initFieldTest(Condition fieldCondition, int langInteger) {
        switch (fieldCondition.getState()) {
            case ConditionConstants.X:
                return new ConstantFieldTest(false);
            case ConditionConstants.N:
            case ConditionConstants.U:
                return new ConstantFieldTest(true);
            case ConditionConstants.T:
                return new NotEmptyFieldTest(true);
            case ConditionConstants.A:
                return new NotEmptyFieldTest(false);
            case ConditionConstants.P:
            case ConditionConstants.PA:
                return new ContentFieldTest(fieldCondition, langInteger);
            default:
                throw new SwitchException("state = " + fieldCondition.getState());
        }
    }

    public boolean isCurrentMotcleSelected(int poids) {
        if (poidsCondition != null) {
            boolean inRange = poidsCondition.getPoidsRangeList().contains(poids);
            if (poidsCondition.isExclusionRangeList()) {
                if (inRange) {
                    return false;
                }
            } else {
                if (!inRange) {
                    return false;
                }
            }
        }
        Motcle motcle = (Motcle) motclePointeur.getCurrentSubsetItem();
        if (levelRangeList != null) {
            if (!levelRangeList.contains(motcle.getLevel())) {
                return false;
            }
        }
        if (idthsRangeList != null) {
            if (!idthsRangeList.contains(motcle.getIdths())) {
                return false;
            }
        }
        if (fieldTest != null) {
            if (!fieldTest.isSelected()) {
                return false;
            }
        }
        return true;
    }


    private abstract class FieldTest {

        public abstract boolean isSelected();

    }


    private class ConstantFieldTest extends FieldTest {

        private boolean constantAnswer;

        ConstantFieldTest(boolean constantAnswer) {
            this.constantAnswer = constantAnswer;
        }

        public boolean isSelected() {
            return constantAnswer;
        }

    }


    private class NotEmptyFieldTest extends FieldTest {

        private boolean notEmptyWanted;

        NotEmptyFieldTest(boolean notEmpty) {
            this.notEmptyWanted = notEmpty;
        }

        public boolean isSelected() {
            String lib = ((Motcle) motclePointeur.getCurrentSubsetItem()).getLibelleString(langInteger, false);
            if (lib != null) {
                return notEmptyWanted;
            } else {
                return !notEmptyWanted;
            }
        }

    }


    private class ContentFieldTest extends FieldTest {

        private TextTestEngine textTestEngine;
        private boolean withEmpty;

        ContentFieldTest(Condition fieldCondition, int langInteger) {
            withEmpty = (fieldCondition.getState() == ConditionConstants.PA);
            textTestEngine = TextTestEngine.newInstance(fieldCondition, langInteger);

        }

        public boolean isSelected() {
            Motcle motcle = (Motcle) motclePointeur.getCurrentSubsetItem();
            textTestEngine.start();
            String idalpha = motcle.getIdalpha();
            if (idalpha != null) {
                textTestEngine.addString(idalpha);
                if (idalphaOnly) {
                    return textTestEngine.getResult();
                }
            }
            if (!textTestEngine.canStop()) {
                String lib = motcle.getLibelleString(langInteger, false);
                if (lib != null) {
                    textTestEngine.addString(lib);
                } else {
                    if (withEmpty) {
                        return true;

                    } else {
                        return false;

                    }
                }
            }

            return textTestEngine.getResult();
        }

    }

}
