/* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.ThesaurusList;
import net.mapeadores.basedefiches.pointeurs.MotclePointeur;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.conditions.Condition;


/**
 * Cette classe permet la sélection de mots-clés d'une base de fiches répondant à certains critères.
 * Ces critères sont définis par des instances de MotcleFilterWrapper ou de MotcleSelectOptions.
 *
 * @author Vincent Calame
 */
public class MotcleSelector {

    private Basedefiches basedefiches;
    private Map<SubsetKey, SelectorByThesaurus> selectorByThesaurusMap = new HashMap<SubsetKey, SelectorByThesaurus>();
    private List<SelectorByThesaurus> selectorByThesaurusList = new ArrayList<SelectorByThesaurus>();
    private int langInteger;
    private int aloneCompletelyMask = -1;

    /**
     * Construit une nouvelle instance sélectionnant des mots-clés sur la base de fiches
     * en argument.
     */
    public MotcleSelector(Basedefiches basedefiches, int langInteger) {
        this.basedefiches = basedefiches;
        this.langInteger = langInteger;
    }

    /**
     * Retourne l'instance de Basedefiches sur laquelle se fait la sélection.
     */
    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public int getLangInteger() {
        return langInteger;
    }

    public boolean containsThesaurus(SubsetKey thesaurusKey) {
        return selectorByThesaurusMap.containsKey(thesaurusKey);
    }

    public int getThesaurusCount() {
        return selectorByThesaurusList.size();
    }

    public Thesaurus getThesaurus(int i) {
        SelectorByThesaurus selectorByThesaurus = selectorByThesaurusList.get(i);
        return selectorByThesaurus.getThesaurus();
    }

    public boolean isSelected(Motcle motcle) {
        return isSelected(motcle, -1);
    }

    public boolean isSelected(Motcle motcle, int poids) {
        SelectorByThesaurus selectorByThesaurus = selectorByThesaurusMap.get(motcle.getThesaurus().getSubsetKey());
        if (selectorByThesaurus == null) {
            return false;
        }
        return selectorByThesaurus.isSelected(motcle, poids);
    }

    public void addMotcleSelectOptions(MotcleSelectOptions selectOptions) {
        add(selectOptions);
    }

    public void addMotcleSelectOptionsList(MotcleSelectOptionsList selectOptionsList) {
        int count = selectOptionsList.getMotcleSelectOptionsCount();
        if (count == 0) {
            add(new MotcleSelectOptions());
        } else {
            for (int i = 0; i < count; i++) {
                add(selectOptionsList.getMotcleSelectOptions(i));
            }
        }
    }

    private void add(MotcleSelectOptions motcleSelectOptions) {
        ThesaurusList thesaurusList = SelectionUtils.getThesaurusList(basedefiches, motcleSelectOptions);
        int thesaurusCount = thesaurusList.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = thesaurusList.getThesaurus(i);
            SelectorByThesaurus selectorByThesaurus = getSelectorByThesaurus(thesaurus);
            selectorByThesaurus.add(motcleSelectOptions);
        }
        if (aloneCompletelyMask == -1) {
            aloneCompletelyMask = 1;
            Condition condition = motcleSelectOptions.getFieldCondition();
            if (condition != null) {
                int count = condition.getIncludingTextTestCount();
                if (count > 0) {
                    aloneCompletelyMask = ((int) Math.pow(2, count) - 1);
                }
            }
        }
    }

    private SelectorByThesaurus getSelectorByThesaurus(Thesaurus thesaurus) {
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        SelectorByThesaurus selectorByThesaurus = (SelectorByThesaurus) selectorByThesaurusMap.get(thesaurusKey);
        if (selectorByThesaurus == null) {
            selectorByThesaurus = new SelectorByThesaurus(thesaurus);
            selectorByThesaurusMap.put(thesaurusKey, selectorByThesaurus);
            selectorByThesaurusList.add(selectorByThesaurus);
        }
        return selectorByThesaurus;
    }


    private class SelectorByThesaurus {

        private List<MotcleSelectUnit> motcleSelectUnitList = new ArrayList<MotcleSelectUnit>();
        private MotclePointeur motclePointeur;

        private SelectorByThesaurus(Thesaurus thesaurus) {
            motclePointeur = new MotclePointeur(thesaurus);
        }

        private Thesaurus getThesaurus() {
            return (Thesaurus) motclePointeur.getSubset();
        }

        private void add(MotcleSelectOptions motcleSelectOptions) {
            MotcleSelectUnit motcleSelectUnit = MotcleSelectUnit.newInstance(motclePointeur, motcleSelectOptions, langInteger);
            motcleSelectUnitList.add(motcleSelectUnit);
        }

        private boolean isSelected(Motcle motcle, int poids) {
            motclePointeur.setCurrentSubsetItem(motcle);
            int size = motcleSelectUnitList.size();
            for (int i = 0; i < size; i++) {
                if (motcleSelectUnitList.get(i).isCurrentMotcleSelected(poids)) {
                    return true;
                }
            }
            return false;
        }

    }

}
