/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.text.Collator;
import java.text.RuleBasedCollator;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.conditions.CollatedStringTestUnit;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.conditions.ConditionUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.collation.CollationUnit;


/**
 *
 * @author Vincent Calame
 */
public class MotcleTestEngine {

    private CollatedStringTestUnit[] excludingTestUnitArray;
    private CollatedStringTestUnit[] includingTestUnitArray;
    private int allExcludedMask = 0;
    private int allIncludedMask = 0;
    private RuleBasedCollator collator;
    private short mode;
    private Locale locale;
    private Thesaurus currentThesaurus;
    private int langInteger;
    private Map<SubsetKey, Map<Integer, Masks>> cacheMap = new HashMap<SubsetKey, Map<Integer, Masks>>();
    private Map<Integer, Masks> currentMasksMap = new HashMap<Integer, Masks>();
    private int currentIncludedMask;
    private int currentExcludedMask;
    private boolean canStop;
    private boolean success;
    private boolean failure;

    public MotcleTestEngine(Condition condition, int langInteger) {
        this.langInteger = langInteger;
        locale = LangInteger.getLocale(langInteger);
        collator = (RuleBasedCollator) Collator.getInstance(locale);
        collator.setStrength(Collator.PRIMARY);
        int excludingCount = condition.getExcludingTextTestCount();
        if (excludingCount > 0) {
            excludingTestUnitArray = new CollatedStringTestUnit[excludingCount];
            for (int i = 0; i < excludingCount; i++) {
                excludingTestUnitArray[i] = CollatedStringTestUnit.newInstance(condition.getExcludingTextTest(i), collator);
            }
            allExcludedMask = getMask(excludingCount);
        }
        int includingCount = condition.getIncludingTextTestCount();
        if (includingCount > 0) {
            includingTestUnitArray = new CollatedStringTestUnit[includingCount];
            for (int i = 0; i < includingCount; i++) {
                includingTestUnitArray[i] = CollatedStringTestUnit.newInstance(condition.getIncludingTextTest(i), collator);
            }
            allIncludedMask = getMask(includingCount);
        }
        mode = condition.getMode();
    }

    public Locale getLocale() {
        return locale;
    }

    public void setCurrentThesaurus(Thesaurus thesaurus) {
        currentMasksMap = cacheMap.get(thesaurus.getSubsetKey());
        if (currentMasksMap == null) {
            currentMasksMap = new HashMap<Integer, Masks>();
            cacheMap.put(thesaurus.getSubsetKey(), currentMasksMap);
        }
        currentThesaurus = thesaurus;
    }

    public void start() {
        currentIncludedMask = 0;
        currentExcludedMask = 0;
        success = false;
        failure = false;
        canStop = false;
    }

    public boolean canStop() {
        return canStop;
    }

    public void addMotcleTicket(int motcleTicket) {
        if (canStop) {
            return;
        }
        Masks masks = getMasks(motcleTicket);
        if (mode == ConditionConstants.MODE_AND) {
            if (masks.getExcludeMask() > 0) {
                failure = true;
                canStop = true;
            } else {
                currentIncludedMask = currentIncludedMask | masks.getIncludeMask();
            }

        } else {
            if (masks.getIncludeMask() > 0) {
                success = true;
                canStop = true;
            } else {
                currentExcludedMask = currentExcludedMask | masks.getExcludeMask();
            }
        }
    }

    public boolean getResult() {
        if (mode == ConditionConstants.MODE_AND) {
            if (failure) {
                return false;
            }
            if (allIncludedMask > 0) {
                if (currentIncludedMask == allIncludedMask) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return true;
            }
        } else {
            if (success) {
                return true;
            }
            if (allExcludedMask > 0) {
                if (currentExcludedMask == allExcludedMask) {
                    return false;
                } else {
                    return true;
                }
            } else {
                return false;
            }
        }

    }

    private Masks getMasks(int motcleTicket) {
        Masks masks = currentMasksMap.get(motcleTicket);
        if (masks == null) {
            Motcle motcle = (Motcle) currentThesaurus.getSubsetItemByTicket(motcleTicket);
            if (motcle == null) {
                return null;
            }
            masks = testMotcle(motcle);
            currentMasksMap.put(motcleTicket, masks);
        }
        return masks;
    }

    private Masks testMotcle(Motcle motcle) {
        int includeMask = 0;
        if (allIncludedMask > 0) {
            int length = includingTestUnitArray.length;
            for (int i = 0; i < length; i++) {
                int pow = getPow(i);
                CollatedStringTestUnit testUnit = includingTestUnitArray[i];
                String idalpha = motcle.getIdalpha();
                if (idalpha != null) {
                    if (ConditionUtils.accept(testUnit.getTextTest(), idalpha)) {
                        includeMask = includeMask | pow;
                    }
                } else {
                    String text = motcle.getLibelleString(langInteger, false);
                    if (text != null) {
                        String collatedString = CollationUnit.collate(text, collator);
                        if (testUnit.accept(collatedString)) {
                            includeMask = includeMask | pow;
                        }
                    }
                }
            }
        }
        int excludeMask = 0;
        if (allExcludedMask > 0) {
            int length = excludingTestUnitArray.length;
            for (int i = 0; i < length; i++) {
                int pow = getPow(i);
                CollatedStringTestUnit testUnit = excludingTestUnitArray[i];
                String idalpha = motcle.getIdalpha();
                if (idalpha != null) {
                    if (!ConditionUtils.accept(testUnit.getTextTest(), idalpha)) {
                        excludeMask = excludeMask | pow;
                    }
                } else {
                    String text = motcle.getLibelleString(langInteger, false);
                    if (text != null) {
                        String collatedString = CollationUnit.collate(text, collator);
                        if (!testUnit.accept(collatedString)) {
                            excludeMask = excludeMask | pow;
                        }
                    }
                }
            }
        }
        return new Masks(includeMask, excludeMask);
    }

    private static int getPow(int i) {
        return ((int) Math.pow(2, i));
    }

    private static int getMask(int count) {
        return ((int) Math.pow(2, count) - 1);
    }


    private static class Masks {

        private int includeMask;
        private int excludeMask;

        Masks(int includeMask, int excludeMask) {
            this.includeMask = includeMask;
            this.excludeMask = excludeMask;
        }

        public int getIncludeMask() {
            return includeMask;
        }

        public int getExcludeMask() {
            return excludeMask;
        }

    }

}
