/* BasedefichesLib_Srz - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.srz;

import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;
import java.text.ParseException;
import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public class FichePrimitivesStorage {

    private FichePrimitivesStorage() {
    }

    public static void writeField(PrimitivesWriter primitivesWriter, Fiche fiche, FieldKey fieldKey) throws IOException {
        if (fieldKey.equals(FieldKey.TITRE)) {
            primitivesWriter.writeString(fiche.getTitre());
        } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            writeFicheItem(primitivesWriter, fiche.getSoustitre());
        } else if (fieldKey.isProprieteFieldKey()) {
            writeProprieteFicheItem(primitivesWriter, fiche.getPropriete(fieldKey));
        } else if (fieldKey.isInformationFieldKey()) {
            writeFicheItemList(primitivesWriter, fiche.getInformation(fieldKey));
        } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
            writeFicheItemList(primitivesWriter, fiche.getRedacteurs());
        } else if (fieldKey.equals(FieldKey.LANG)) {
            writeFicheItem(primitivesWriter, new Langue(fiche.getLangInteger()));
        } else {
            throw new SwitchException("fieldKey = " + fieldKey.toString());
        }
    }

    public static void writeProprieteFicheItem(PrimitivesWriter primitivesWriter, Propriete propriete) throws IOException {
        if (propriete == null) {
            primitivesWriter.writeChar('x');
            return;
        }
        writeFicheItem(primitivesWriter, propriete.getFicheItem());
    }

    public static void writeFicheItemList(PrimitivesWriter primitivesWriter, FicheItemList ficheItemList) throws IOException {
        if (ficheItemList == null) {
            primitivesWriter.writeInt(0);
            return;
        }
        int size = ficheItemList.getFicheItemCount();
        primitivesWriter.writeInt(size);
        for (int i = 0; i < size; i++) {
            writeFicheItem(primitivesWriter, ficheItemList.getFicheItem(i));
        }
    }

    public static void writeFicheBlockList(PrimitivesWriter primitivesWriter, FicheBlockList ficheBlockList) throws IOException {
        if (ficheBlockList == null) {
            primitivesWriter.writeInt(0);
            return;
        }
        int size = ficheBlockList.getFicheBlockCount();
        primitivesWriter.writeInt(size);
        for (int i = 0; i < size; i++) {
            writeFicheBlock(primitivesWriter, ficheBlockList.getFicheBlock(i));
        }
    }

    public static Object readField(PrimitivesReader primitivesReader, FieldKey fieldKey) throws IOException {
        if (fieldKey.equals(FieldKey.TITRE)) {
            return primitivesReader.readString();
        } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            Object obj = readFicheItem(primitivesReader);
            if (obj != null) {
                if (!(obj instanceof Para)) {
                    throw new IOException("Not a Para ficheItem : " + obj.getClass().getName());
                }
            }
            return obj;
        } else if (fieldKey.isProprieteFieldKey()) {
            return readFicheItem(primitivesReader);
        } else if (fieldKey.isInformationFieldKey()) {
            return readFicheItemList(primitivesReader);
        } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return readFicheItemList(primitivesReader);
        } else if (fieldKey.equals(FieldKey.LANG)) {
            Object obj = readFicheItem(primitivesReader);
            if (obj != null) {
                if (!(obj instanceof Langue)) {
                    throw new IOException("Not a Langue ficheItem : " + obj.getClass().getName());
                }
            }
            return obj;
        }
        throw new SwitchException("fieldKey = " + fieldKey.toString());
    }

    public static FicheItemList readFicheItemList(PrimitivesReader primitivesReader) throws IOException {
        int size = primitivesReader.readInt();
        if (size == 0) {
            return null;
        }
        MutableFicheItemList ficheItemList = new MutableFicheItemList(size);
        for (int i = 0; i < size; i++) {
            FicheItem ficheItem = readFicheItem(primitivesReader);
            if (ficheItem != null) {
                ficheItemList.addFicheItem(ficheItem);
            }
        }
        if (ficheItemList.getFicheItemCount() == 0) {
            return null;
        }
        return ficheItemList;
    }

    public static FicheItem readFicheItem(PrimitivesReader primitivesReader) throws IOException {
        char carac = primitivesReader.readChar();
        switch (carac) {
            case 'x':
                return null;
            case 'i':
                String value = primitivesReader.readString();
                return new Item(value);
            case 'l':
                int langInteger = primitivesReader.readInt();
                return new Langue(langInteger);
            case 'c':
                String countryCode = primitivesReader.readString();
                return new Pays(countryCode);
            case 'd':
                int dateInteger = primitivesReader.readInt();
                return new Datation(dateInteger);
            case 'a':
                String href = primitivesReader.readString();
                String title = primitivesReader.readString();
                String comment = primitivesReader.readString();
                return new Link(href, title, comment);
            case 'e':
                String addrspec = primitivesReader.readString();
                String nomcomplet = primitivesReader.readString();
                CourrielBean courrielBean = new CourrielBean();
                courrielBean.setAddrSpec(addrspec);
                courrielBean.setNomComplet(nomcomplet);
                return new Courriel(courrielBean);
            case 'n':
                Decimal decimale1 = readDecimal(primitivesReader);
                return new Nombre(decimale1);
            case 'm':
                Decimal decimale2 = readDecimal(primitivesReader);
                int devise = primitivesReader.readInt();
                return new Montant(decimale2, devise);
            case 'g':
                DegreDecimal latitude = readDegreDecimal(primitivesReader);
                DegreDecimal longitude = readDegreDecimal(primitivesReader);
                return new Geopoint(latitude, longitude);
            case 'r':
                try {
                    RedacteurKey redacteurKey = RedacteurKey.parse(primitivesReader.readString());
                    return new Personne(redacteurKey.toString());
                } catch (java.text.ParseException pe) {
                    return null;
                }
            case 'p':
                String nom = primitivesReader.readString();
                boolean nomavant = primitivesReader.readBoolean();
                String prenom = primitivesReader.readString();
                String original = primitivesReader.readString();
                String organisme = primitivesReader.readString();
                PersonneBean personneBean = new PersonneBean();
                personneBean.setNom(nom);
                personneBean.setNomAvant(nomavant);
                personneBean.setPrenom(prenom);
                personneBean.setOriginal(original);
                return new Personne(personneBean, organisme);
            case 'b':
                return readPara(primitivesReader);
            case 's':
                String src = primitivesReader.readString();
                String alt = primitivesReader.readString();
                String imageTitle = primitivesReader.readString();
                return new Image(src, alt, imageTitle);
            default:
                throw new SwitchException("initiale = " + carac);
        }
    }

    public static FicheBlockList readFicheBlockList(PrimitivesReader primitivesReader) throws IOException {
        int size = primitivesReader.readInt();
        if (size == 0) {
            return null;
        }
        MutableFicheBlockList ficheBlockList = new MutableFicheBlockList(size);
        for (int i = 0; i < size; i++) {
            FicheBlock ficheBlock = readFicheBlock(primitivesReader);
            if (ficheBlock != null) {
                ficheBlockList.addFicheBlock(ficheBlock);
            }
        }
        if (ficheBlockList.getFicheBlockCount() == 0) {
            return null;
        }
        return ficheBlockList;
    }

    private static Decimal readDecimal(PrimitivesReader primitivesReader) throws IOException {
        long partieEntiere = primitivesReader.readLong();
        byte zeroLength = primitivesReader.readByte();
        int partieDecimale = primitivesReader.readInt();
        return new Decimal(partieEntiere, zeroLength, partieDecimale);
    }

    private static DegreDecimal readDegreDecimal(PrimitivesReader primitivesReader) throws IOException {
        int partieEntiere = primitivesReader.readInt();
        byte zeroLength = primitivesReader.readByte();
        int partieDecimale = primitivesReader.readInt();
        return new DegreDecimal(partieEntiere, zeroLength, partieDecimale);
    }

    public static void writeFicheItem(PrimitivesWriter primitivesWriter, FicheItem ficheItem) throws IOException {
        if (ficheItem == null) {
            primitivesWriter.writeChar('x');
            return;
        }
        if (ficheItem instanceof Item) {
            primitivesWriter.writeChar('i');
            primitivesWriter.writeString(((Item) ficheItem).getValue());
        } else if (ficheItem instanceof Langue) {
            primitivesWriter.writeChar('l');
            primitivesWriter.writeInt(((Langue) ficheItem).getLangInteger());
        } else if (ficheItem instanceof Pays) {
            primitivesWriter.writeChar('c');
            primitivesWriter.writeString(((Pays) ficheItem).getCountryCode());
        } else if (ficheItem instanceof Datation) {
            primitivesWriter.writeChar('d');
            primitivesWriter.writeInt(((Datation) ficheItem).getDateInteger());
        } else if (ficheItem instanceof Link) {
            primitivesWriter.writeChar('a');
            Link link = (Link) ficheItem;
            primitivesWriter.writeString(link.getHref());
            primitivesWriter.writeString(link.getTitle());
            primitivesWriter.writeString(link.getComment());
        } else if (ficheItem instanceof Nombre) {
            primitivesWriter.writeChar('n');
            Nombre nombre = (Nombre) ficheItem;
            writeDecimal(primitivesWriter, nombre.getDecimal());
        } else if (ficheItem instanceof Montant) {
            primitivesWriter.writeChar('m');
            Montant montant = (Montant) ficheItem;
            writeDecimal(primitivesWriter, montant.getDecimal());
            primitivesWriter.writeInt(montant.getCurrencyInteger());
        } else if (ficheItem instanceof Courriel) {
            primitivesWriter.writeChar('e');
            Courriel courriel = (Courriel) ficheItem;
            primitivesWriter.writeString(courriel.getAddrSpec());
            primitivesWriter.writeString(courriel.getNomComplet());
        } else if (ficheItem instanceof Personne) {
            Personne personne = (Personne) ficheItem;
            if (personne.isRedacteurSphere()) {
                primitivesWriter.writeChar('r');
                primitivesWriter.writeString(personne.getRedacteurKeyString());
            } else {
                primitivesWriter.writeChar('p');
                primitivesWriter.writeString(personne.getNom());
                primitivesWriter.writeBoolean(personne.isNomAvant());
                primitivesWriter.writeString(personne.getPrenom());
                primitivesWriter.writeString(personne.getOriginal());
                String organisme = personne.getOrganisme();
                if (organisme == null) {
                    organisme = "";
                }
                primitivesWriter.writeString(organisme);
            }
        } else if (ficheItem instanceof Geopoint) {
            primitivesWriter.writeChar('g');
            Geopoint geopoint = (Geopoint) ficheItem;
            writeDegreDecimal(primitivesWriter, geopoint.getLatitude());
            writeDegreDecimal(primitivesWriter, geopoint.getLongitude());
        } else if (ficheItem instanceof Para) {
            primitivesWriter.writeChar('b');
            Para para = (Para) ficheItem;
            writeTextContentList(primitivesWriter, para);
        } else if (ficheItem instanceof Image) {
            primitivesWriter.writeChar('s');
            Image image = (Image) ficheItem;
            primitivesWriter.writeString(image.getSrc());
            primitivesWriter.writeString(image.getAlt());
            primitivesWriter.writeString(image.getTitle());
        } else {
            throw new SwitchException("class = " + ficheItem.getClass().getName());
        }

    }

    public static void writeFicheBlock(PrimitivesWriter primitivesWriter, FicheBlock ficheBlock) throws IOException {
        if (ficheBlock == null) {
            primitivesWriter.writeChar('x');
            return;
        }
        if (ficheBlock instanceof CdataDiv) {
            primitivesWriter.writeChar('v');
            CdataDiv cdataDiv = (CdataDiv) ficheBlock;
            writeCommonZoneBlock(primitivesWriter, cdataDiv);
            primitivesWriter.writeString(cdataDiv.getCdata());
        } else if (ficheBlock instanceof Code) {
            primitivesWriter.writeChar('c');
            Code code = (Code) ficheBlock;
            primitivesWriter.writeShort(code.getType());
            writeCommonZoneBlock(primitivesWriter, code);
            int lnCount = code.getFicheBlockCount();
            primitivesWriter.writeInt(lnCount);
            for (int i = 0; i < lnCount; i++) {
                Ln ln = code.getLn(i);
                writeFicheBlock(primitivesWriter, ln);
            }
        } else if (ficheBlock instanceof Div) {
            primitivesWriter.writeChar('d');
            Div div = (Div) ficheBlock;
            writeCommonZoneBlock(primitivesWriter, div);
            writeFicheBlockList(primitivesWriter, div.getFicheBlockList());
        } else if (ficheBlock instanceof H) {
            primitivesWriter.writeChar('h');
            H h = (H) ficheBlock;
            primitivesWriter.writeInt(h.getLevel());
            writeCommon(primitivesWriter, h);
            writeTextContentList(primitivesWriter, h);
        } else if (ficheBlock instanceof Insert) {
            primitivesWriter.writeChar('i');
            Insert insert = (Insert) ficheBlock;
            primitivesWriter.writeShort(insert.getType());
            writeCommonZoneBlock(primitivesWriter, insert);
            primitivesWriter.writeString(insert.getSrc());
            primitivesWriter.writeString(insert.getRef());
            primitivesWriter.writeString(insert.getCredit());
            primitivesWriter.writeInt(insert.getWidth());
            primitivesWriter.writeInt(insert.getHeight());
            primitivesWriter.writeShort(insert.getPosition());
            SubsetKey albumSubsetKey = insert.getAlbumSubsetKey();
            if (albumSubsetKey != null) {
                primitivesWriter.writeString(albumSubsetKey.getSubsetName());
                primitivesWriter.writeInt(insert.getIdalbum());
                primitivesWriter.writeString(insert.getAlbumDimName());
            } else {
                primitivesWriter.writeString("");
                primitivesWriter.writeInt(0);
                primitivesWriter.writeString("");
            }
        } else if (ficheBlock instanceof Ln) {
            primitivesWriter.writeChar('l');
            Ln ln = (Ln) ficheBlock;
            primitivesWriter.writeString(ln.getValue());
            primitivesWriter.writeInt(ln.getIndentation());
            writeCommon(primitivesWriter, ln);
        } else if (ficheBlock instanceof P) {
            primitivesWriter.writeChar('p');
            P p = (P) ficheBlock;
            primitivesWriter.writeShort(p.getType());
            primitivesWriter.writeString(p.getSource());
            writeCommon(primitivesWriter, p);
            writeTextContentList(primitivesWriter, p);
        } else if (ficheBlock instanceof Table) {
            primitivesWriter.writeChar('t');
            Table table = (Table) ficheBlock;
            writeCommonZoneBlock(primitivesWriter, table);
            int trCount = table.getTrCount();
            primitivesWriter.writeInt(trCount);
            for (int i = 0; i < trCount; i++) {
                Tr tr = table.getTr(i);
                primitivesWriter.writeString(tr.getClassName());
                primitivesWriter.writeString(tr.getHtmlId());
                writeFicheBlockList(primitivesWriter, tr);
            }
        } else if (ficheBlock instanceof Td) {
            primitivesWriter.writeChar('e');
            Td td = (Td) ficheBlock;
            primitivesWriter.writeShort(td.getType());
            primitivesWriter.writeInt(td.getRowSpan());
            primitivesWriter.writeInt(td.getColSpan());
            writeCommon(primitivesWriter, td);
            writeTextContentList(primitivesWriter, td);
        } else if (ficheBlock instanceof Ul) {
            primitivesWriter.writeChar('u');
            Ul ul = (Ul) ficheBlock;
            writeCommon(primitivesWriter, ul);
            int liCount = ul.getLiCount();
            primitivesWriter.writeInt(liCount);
            for (int i = 0; i < liCount; i++) {
                Li li = ul.getLi(i);
                primitivesWriter.writeString(li.getClassName());
                primitivesWriter.writeString(li.getHtmlId());
                writeFicheBlockList(primitivesWriter, li);
            }
        } else {
            throw new SwitchException("class = " + ficheBlock.getClass().getName());
        }

    }

    public static FicheBlock readFicheBlock(PrimitivesReader primitivesReader) throws IOException {
        char carac = primitivesReader.readChar();
        switch (carac) {
            case 'x':
                return null;
            case 'v':
                CdataDiv cdataDiv = new CdataDiv();
                readCommonZoneBlock(primitivesReader, cdataDiv);
                String cdata = primitivesReader.readString();
                cdataDiv.setCdata(cdata);
                return cdataDiv;
            case 'c':
                short codeType = primitivesReader.readShort();
                Code code = new Code(codeType);
                readCommonZoneBlock(primitivesReader, code);
                int lnSize = primitivesReader.readInt();
                for (int i = 0; i < lnSize; i++) {
                    Ln ln = (Ln) readFicheBlock(primitivesReader);
                    code.addLn(ln);
                }
                return code;
            case 'd':
                Div div = new Div();
                readCommonZoneBlock(primitivesReader, div);
                int divFBCount = primitivesReader.readInt();
                for (int i = 0; i < divFBCount; i++) {
                    div.getFicheBlockList().addFicheBlock(readFicheBlock(primitivesReader));
                }
                return div;
            case 'h':
                int hLevel = primitivesReader.readInt();
                H h = new H(hLevel);
                readCommon(primitivesReader, h);
                readTextContent(primitivesReader, h);
                return h;
            case 'i':
                short insertType = primitivesReader.readShort();
                Insert insert = new Insert(insertType);
                readCommonZoneBlock(primitivesReader, insert);
                insert.setSrc(primitivesReader.readString());
                insert.setRef(primitivesReader.readString());
                insert.setCredit(primitivesReader.readString());
                insert.setWidth(primitivesReader.readInt());
                insert.setHeight(primitivesReader.readInt());
                insert.setPosition(primitivesReader.readShort());
                String albumSubsetName = primitivesReader.readString();
                int idalbum = primitivesReader.readInt();
                String albumDimName = primitivesReader.readString();
                if (albumSubsetName.length() > 0) {
                    try {
                        SubsetKey albumSubsetKey = SubsetKey.parse(SubsetKey.CATEGORY_ALBUM, albumSubsetName);
                        insert.setIllustration(albumSubsetKey, idalbum, albumDimName);
                    } catch (ParseException pe) {
                    }
                }
                return insert;
            case 'l':
                String lnValue = primitivesReader.readString();
                Ln ln = new Ln(lnValue);
                ln.setIndentation(primitivesReader.readInt());
                readCommon(primitivesReader, ln);
                return ln;
            case 'p':
                short pType = primitivesReader.readShort();
                P p = new P(pType);
                p.setSource(primitivesReader.readString());
                readCommon(primitivesReader, p);
                readTextContent(primitivesReader, p);
                return p;
            case 't':
                Table table = new Table();
                readCommonZoneBlock(primitivesReader, table);
                int trCount = primitivesReader.readInt();
                for (int i = 0; i < trCount; i++) {
                    Tr tr = new Tr();
                    tr.setClassName(primitivesReader.readString());
                    tr.setHtmlId(primitivesReader.readString());
                    int trSize = primitivesReader.readInt();
                    for (int j = 0; j < trSize; j++) {
                        Td td = (Td) readFicheBlock(primitivesReader);
                        tr.addTd(td);
                    }
                    table.addTr(tr);
                }
                return table;
            case 'e':
                short tdType = primitivesReader.readShort();
                Td td = null;
                if (tdType == Td.HEADER_TYPE) {
                    td = Td.getHeaderTdInstance();
                } else {
                    td = new Td();
                }
                td.setRowSpan(primitivesReader.readInt());
                td.setColSpan(primitivesReader.readInt());
                readCommon(primitivesReader, td);
                readTextContent(primitivesReader, td);
                return td;
            case 'u':
                Ul ul = new Ul();
                ul.setClassName(primitivesReader.readString());
                ul.setHtmlId(primitivesReader.readString());
                int liCount = primitivesReader.readInt();
                for (int i = 0; i < liCount; i++) {
                    Li li = new Li();
                    li.setClassName(primitivesReader.readString());
                    li.setHtmlId(primitivesReader.readString());
                    int liFBCount = primitivesReader.readInt();
                    for (int j = 0; j < liFBCount; j++) {
                        li.addFicheBlock(readFicheBlock(primitivesReader));
                    }
                    ul.addLi(li);
                }
                return ul;
            default:
                throw new SwitchException("initiale = " + carac);
        }
    }

    private static void writeCommon(PrimitivesWriter primitivesWriter, FicheBlock ficheBlock) throws IOException {
        primitivesWriter.writeString(ficheBlock.getClassName());
        primitivesWriter.writeString(ficheBlock.getHtmlId());
    }

    private static void writeCommonZoneBlock(PrimitivesWriter primitivesWriter, ZoneBlock zoneBlock) throws IOException {
        writeCommon(primitivesWriter, zoneBlock);
        ParagraphBlock numero = zoneBlock.getNumero();
        writeCommon(primitivesWriter, numero);
        writeTextContentList(primitivesWriter, numero);
        ParagraphBlock legende = zoneBlock.getLegende();
        writeCommon(primitivesWriter, legende);
        writeTextContentList(primitivesWriter, legende);
    }

    private static void readCommonZoneBlock(PrimitivesReader primitivesReader, ZoneBlock zoneBlock) throws IOException {
        String className = primitivesReader.readString();
        String htmlId = primitivesReader.readString();
        zoneBlock.setClassName(className);
        zoneBlock.setHtmlId(htmlId);
        ParagraphBlock numero = zoneBlock.getNumero();
        readCommon(primitivesReader, numero);
        readTextContent(primitivesReader, numero);
        ParagraphBlock legende = zoneBlock.getLegende();
        readCommon(primitivesReader, legende);
        readTextContent(primitivesReader, legende);
    }

    private static void writeDecimal(PrimitivesWriter primitivesWriter, Decimal decimal) throws IOException {
        primitivesWriter.writeLong(decimal.getPartieEntiere());
        primitivesWriter.writeByte(decimal.getZeroLength());
        primitivesWriter.writeInt(decimal.getPartieDecimale());
    }

    private static void writeDegreDecimal(PrimitivesWriter primitivesWriter, DegreDecimal decimal) throws IOException {
        primitivesWriter.writeInt(decimal.getPartieEntiere());
        primitivesWriter.writeByte(decimal.getZeroLength());
        primitivesWriter.writeInt(decimal.getPartieDecimale());
    }

    private static void writeTextContentList(PrimitivesWriter primitivesWriter, TextContentList contentList) throws IOException {
        if (contentList == null) {
            primitivesWriter.writeInt(0);
            return;
        }
        int size = contentList.getTextContentCount();
        primitivesWriter.writeInt(size);
        for (int i = 0; i < size; i++) {
            Object obj = contentList.getTextContent(i);
            if (obj instanceof String) {
                primitivesWriter.writeByte((byte) 1);
                primitivesWriter.writeString((String) obj);
            } else if (obj instanceof S) {
                primitivesWriter.writeByte((byte) 2);
                writeS(primitivesWriter, (S) obj);
            }
        }
    }

    private static void readCommon(PrimitivesReader primitivesReader, ParagraphBlock paragraphBlock) throws IOException {
        String className = primitivesReader.readString();
        String htmlId = primitivesReader.readString();
        paragraphBlock.setClassName(className);
        paragraphBlock.setHtmlId(htmlId);
    }

    private static void readTextContent(PrimitivesReader primitivesReader, ParagraphBlock paragraphBlock) throws IOException {
        int size = primitivesReader.readInt();
        for (int i = 0; i < size; i++) {
            byte b = primitivesReader.readByte();
            switch ((int) b) {
                case 1:
                    paragraphBlock.addText(primitivesReader.readString());
                    break;
                case 2:
                    paragraphBlock.addS(readS(primitivesReader));
                    break;
                default:
                    throw new SwitchException("byte = " + b);
            }
        }
    }

    private static Para readPara(PrimitivesReader primitivesReader) throws IOException {
        Para.Builder builder = new Para.Builder();
        int size = primitivesReader.readInt();
        for (int i = 0; i < size; i++) {
            byte b = primitivesReader.readByte();
            switch ((int) b) {
                case 1:
                    builder.addText(primitivesReader.readString());
                    break;
                case 2:
                    builder.addS(readS(primitivesReader));
                    break;
                default:
                    throw new SwitchException("byte = " + b);
            }
        }
        return builder.toPara();
    }

    private static S readS(PrimitivesReader primitivesReader) throws IOException {
        short type = primitivesReader.readShort();
        String value = primitivesReader.readString();
        S s = new S(type, value);
        while (true) {
            char carac = primitivesReader.readChar();
            if (carac == 'x') {
                break;
            }
            String attr = primitivesReader.readString();
            switch (carac) {
                case 'r':
                    s.setRef(attr);
                    break;
                case 's':
                    s.setSrc(attr);
                    break;
                case 't':
                    s.setTitle(attr);
                    break;
                case 'l':
                    s.setLang(attr);
                    break;
                case 'c':
                    s.setClassName(attr);
                    break;
                case 'i':
                    s.setHtmlId(attr);
                    break;
                case 'u':
                    s.setSubtype(attr);
                    break;
                default:
                    throw new SwitchException("attribute = " + carac);
            }
        }
        return s;
    }

    private static void writeS(PrimitivesWriter primitivesWriter, S s) throws IOException {
        primitivesWriter.writeShort(s.getType());
        primitivesWriter.writeString(s.getValue());
        writeAttributes(primitivesWriter, s.getRef(), 'r');
        writeAttributes(primitivesWriter, s.getSrc(), 's');
        writeAttributes(primitivesWriter, s.getTitle(), 't');
        writeAttributes(primitivesWriter, s.getLang(), 'l');
        writeAttributes(primitivesWriter, s.getClassName(), 'c');
        writeAttributes(primitivesWriter, s.getHtmlId(), 'i');
        writeAttributes(primitivesWriter, s.getSubtype(), 'u');
        primitivesWriter.writeChar('x');
    }

    private static void writeAttributes(PrimitivesWriter primitivesWriter, String attr, char c) throws IOException {
        if ((attr == null) || (attr.length() == 0)) {
            return;
        }
        primitivesWriter.writeChar(c);
        primitivesWriter.writeString(attr);
    }

}
