/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.scrutariexport;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatDefParser;
import net.mapeadores.basedefiches.scrutariexport.CorpusScrutariDef;
import net.mapeadores.basedefiches.scrutariexport.CorpusScrutariDefBuilder;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDefBuilder;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusScrutariDef;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusScrutariDefBuilder;
import net.mapeadores.basedefiches.scrutariexport.WrongFormatSourceKeyException;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.xml.extraction.ExtractionDOMReader;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.text.LibelleUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportDOMReader {

    private ScrutariExportDefBuilder scrutariExportDefBuilder;
    private ScrutariExportXmlErrorHandler errorHandler;

    public ScrutariExportDOMReader(ScrutariExportDefBuilder scrutariExportDefBuilder, ScrutariExportXmlErrorHandler errorHandler) {
        this.scrutariExportDefBuilder = scrutariExportDefBuilder;
        this.errorHandler = errorHandler;
    }

    public void readScrutariExportDef(Element element_xml) {
        DOMUtils.readChildren(element_xml, new RootHandler());
    }


    private class RootHandler implements ElementHandler {

        private CorpusDefHandler corpusDefHandler = new CorpusDefHandler();
        private ThesaurusDefHandler thesaurusDefHandler = new ThesaurusDefHandler();

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("base-name")) {
                String baseName = DOMUtils.readSimpleElement(element);
                if (baseName.length() > 0) {
                    try {
                        scrutariExportDefBuilder.setBaseName(baseName);
                    } catch (ParseException pe) {
                        errorHandler.wrongElementValueError(tagName, baseName);
                    }
                }
            } else if (tagName.equals("base-icon")) {
                String baseIcon = DOMUtils.readSimpleElement(element);
                if (baseIcon.length() > 0) {
                    scrutariExportDefBuilder.setBaseIcon(baseIcon);
                }
            } else if (tagName.equals("langs-ui")) {
                LangsUiHandler langsUiHandler = new LangsUiHandler();
                DOMUtils.readChildren(element, langsUiHandler);
                if (!langsUiHandler.isEmpty()) {
                    scrutariExportDefBuilder.setLangUiArray(langsUiHandler.toArray());
                }
            } else if (tagName.equals("fiche-pattern")) {
                String patternString = DOMUtils.readSimpleElement(element);
                if (patternString.length() > 0) {
                    try {
                        AccoladePattern pattern = new AccoladePattern(patternString);
                        scrutariExportDefBuilder.setFicheHrefPattern(pattern);
                    } catch (ParseException pe) {
                        errorHandler.wrongElementValueError(tagName, patternString);
                    }
                }
            } else if (tagName.equals("motcle-pattern")) {
                String patternString = DOMUtils.readSimpleElement(element);
                if (patternString.length() > 0) {
                    try {
                        AccoladePattern pattern = new AccoladePattern(patternString);
                        scrutariExportDefBuilder.setMotcleHrefPattern(pattern);
                    } catch (ParseException pe) {
                        errorHandler.wrongElementValueError(tagName, patternString);
                    }
                }
            } else if (tagName.equals("corpus-def")) {
                String corpusString = element.getAttribute("corpus");
                if (corpusString.length() == 0) {
                    errorHandler.emptyAttributeError(tagName, "corpus");
                } else {
                    try {
                        SubsetKey corpusSubsetKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, corpusString);
                        corpusDefHandler.setCorpusSubsetKey(corpusSubsetKey);
                        DOMUtils.readChildren(element, corpusDefHandler);
                        scrutariExportDefBuilder.addCorpusScrutariDef(corpusDefHandler.toCorpusScrutariDef());
                    } catch (ParseException pe) {
                        errorHandler.wrongAttributeValueError(tagName, "corpus", corpusString);
                    }
                }
            } else if (tagName.equals("thesaurus-def")) {
                String thesaurusString = element.getAttribute("thesaurus");
                if (thesaurusString.length() == 0) {
                    errorHandler.emptyAttributeError(tagName, "thesaurus");
                } else {
                    try {
                        SubsetKey thesaurusSubsetKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, thesaurusString);
                        thesaurusDefHandler.setThesaurusSubsetKey(thesaurusSubsetKey);
                        DOMUtils.readChildren(element, thesaurusDefHandler);
                        scrutariExportDefBuilder.addThesaurusScrutariDef(thesaurusDefHandler.toThesaurusScrutariDef());
                    } catch (ParseException pe) {
                        errorHandler.wrongAttributeValueError(tagName, "thesaurus", thesaurusString);
                    }
                }
            } else if (tagName.equals("fiche-select")) {
                FicheSelectOptions ficheSelectOptions = ExtractionDOMReader.getFicheSelectOptions(element);
                if (ficheSelectOptions.containsOption()) {
                    scrutariExportDefBuilder.setFicheSelectOptions(ficheSelectOptions);
                }
            } else {
                errorHandler.unknownTagWarning(tagName);
            }
        }

    }


    private class LangsUiHandler implements ElementHandler {

        private Set<Integer> set = new LinkedHashSet<Integer>();

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("lang")) {
                String langString = DOMUtils.readSimpleElement(element);
                try {
                    int langInteger = LangInteger.parse(langString);
                    set.add(langInteger);
                } catch (LangIntegerException lie) {
                    errorHandler.wrongElementValueError(tagName, langString);
                }
            } else {
                errorHandler.unknownTagWarning(tagName);
            }
        }

        private boolean isEmpty() {
            return set.isEmpty();
        }

        private int[] toArray() {
            int size = set.size();
            int[] result = new int[size];
            int p = 0;
            for (int langInteger : set) {
                result[p] = langInteger;
                p++;
            }
            return result;
        }

    }


    private class CorpusDefHandler implements ElementHandler {

        private CorpusScrutariDefBuilder builder;
        private BasedefichesFormatDefHandler basedefichesFormatDefHandler;
        private ComplementHandler complementHandler;

        private CorpusDefHandler() {
            basedefichesFormatDefHandler = new BasedefichesFormatDefHandler();
            complementHandler = new ComplementHandler();
        }

        private void setCorpusSubsetKey(SubsetKey corpusSubsetKey) {
            builder = new CorpusScrutariDefBuilder(corpusSubsetKey);
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("date")) {
                String dateString = DOMUtils.readSimpleElement(element);
                try {
                    builder.setDateKey(dateString);
                } catch (ParseException pe) {
                    errorHandler.wrongElementValueError(tagName, dateString);
                } catch (WrongFormatSourceKeyException wfske) {
                    errorHandler.wrongElementValueError(tagName, dateString);
                }
            } else if (tagName.equals("titre-format")) {
                DOMUtils.readChildren(element, basedefichesFormatDefHandler);
                BasedefichesFormatDef basedefichesFormatDef = basedefichesFormatDefHandler.toBasedefichesFormatDef();
                if (basedefichesFormatDef != null) {
                    builder.setTitreBasedefichesFormatDef(basedefichesFormatDef);
                }
            } else if (tagName.equals("soustitre-format")) {
                DOMUtils.readChildren(element, basedefichesFormatDefHandler);
                BasedefichesFormatDef basedefichesFormatDef = basedefichesFormatDefHandler.toBasedefichesFormatDef();
                if (basedefichesFormatDef != null) {
                    builder.setSoustitreBasedefichesFormatDef(basedefichesFormatDef);
                }
            } else if (tagName.equals("complement")) {
                complementHandler.setNewComplementName(element.getAttribute("name"));
                DOMUtils.readChildren(element, complementHandler);
                complementHandler.addComplement(builder);
            } else if (tagName.equals("href-pattern")) {
                String hrefPatternString = DOMUtils.readSimpleElement(element);
                if (hrefPatternString.length() > 0) {
                    try {
                        AccoladePattern pattern = new AccoladePattern(hrefPatternString);
                        builder.setHrefPattern(pattern);
                    } catch (ParseException pe) {
                        errorHandler.wrongElementValueError(tagName, hrefPatternString);
                    }
                }
            } else if (tagName.equals("multilang-key")) {
                String multiLangKeyString = DOMUtils.readSimpleElement(element);
                if (multiLangKeyString.length() > 0) {
                    Object multiLangKey = null;
                    try {
                        multiLangKey = FieldKey.parse(multiLangKeyString);
                    } catch (ParseException pe) {
                    }
                    boolean done = false;
                    if (multiLangKey != null) {
                        try {
                            builder.setMultiLangKey(multiLangKey);
                            done = true;
                        } catch (IllegalArgumentException iae) {
                        }
                    }
                    if (!done) {
                        errorHandler.wrongElementValueError(tagName, multiLangKeyString);
                    }
                }
            } else {
                errorHandler.unknownTagWarning(tagName);
            }
        }

        public CorpusScrutariDef toCorpusScrutariDef() {
            CorpusScrutariDef corpusScrutariDef = builder.toCorpusScrutariDef();
            builder = null;
            return corpusScrutariDef;
        }

    }


    private class ThesaurusDefHandler implements ElementHandler {

        private ThesaurusScrutariDefBuilder builder;

        ThesaurusDefHandler() {
        }

        private void setThesaurusSubsetKey(SubsetKey thesaurusSubsetKey) {
            builder = new ThesaurusScrutariDefBuilder(thesaurusSubsetKey);
            builder.setSelected(true);
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("motcle-format")) {
                String motcleFormatPattern = DOMUtils.readSimpleElement(element);
                if (motcleFormatPattern.length() > 0) {
                    builder.setMotcleFormatPattern(motcleFormatPattern);
                }
            } else {
                errorHandler.unknownTagWarning(tagName);
            }
        }

        public ThesaurusScrutariDef toThesaurusScrutariDef() {
            ThesaurusScrutariDef thesaurusScrutariDef = builder.toThesaurusScrutariDef();
            builder = null;
            return thesaurusScrutariDef;
        }

    }


    private class BasedefichesFormatDefHandler implements ElementHandler {

        private List<String> valueList = new ArrayList<String>();
        private BasedefichesFormatDefErrorHandler basedefichesFormatDefErrorHandler;

        private BasedefichesFormatDefHandler() {
            basedefichesFormatDefErrorHandler = new BasedefichesFormatDefErrorHandler(errorHandler);
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("value")) {
                valueList.add(DOMUtils.readSimpleElement(element));
            } else {
                errorHandler.unknownTagWarning(tagName);
            }
        }

        private BasedefichesFormatDef toBasedefichesFormatDef() {
            int size = valueList.size();
            if (size == 0) {
                return null;
            }
            BasedefichesFormatDef basedefichesFormatDef = BasedefichesFormatDefParser.parse(valueList, basedefichesFormatDefErrorHandler);
            valueList.clear();
            return basedefichesFormatDef;
        }

    }


    private class ComplementHandler implements ElementHandler {

        private String complementName = null;
        private List<String> valueList = new ArrayList<String>();
        private List<Libelle> libelleList = new ArrayList<Libelle>();
        private BasedefichesFormatDefErrorHandler basedefichesFormatDefErrorHandler;

        private ComplementHandler() {
            basedefichesFormatDefErrorHandler = new BasedefichesFormatDefErrorHandler(errorHandler);
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("value")) {
                valueList.add(DOMUtils.readSimpleElement(element));
            } else if (tagName.equals("lib")) {
                try {
                    Libelle libelle = LibelleUtils.readLib(element);
                    if (libelle != null) {
                        libelleList.add(libelle);
                    }
                } catch (LangIntegerException ile) {
                    errorHandler.wrongAttributeValueError(tagName, "xml:lang", element.getAttribute("xml:lang"));
                }
            } else {
                errorHandler.unknownTagWarning(tagName);
            }
        }

        private void setNewComplementName(String name) {
            if (name.length() == 0) {
                name = null;
            }
            this.complementName = name;
            valueList.clear();
            libelleList.clear();
        }

        private void addComplement(CorpusScrutariDefBuilder builder) {
            BasedefichesFormatDef basedefichesFormatDef = null;
            LibelleHolder customLibelleHolder = null;
            if (valueList.size() > 0) {
                basedefichesFormatDef = BasedefichesFormatDefParser.parse(valueList, basedefichesFormatDefErrorHandler);
            }
            if (libelleList.size() > 0) {
                customLibelleHolder = LibelleUtils.toLibelleHolder(libelleList);
                if (customLibelleHolder.getLibelleCount() == 0) {
                    customLibelleHolder = null;
                }
            }
            builder.addComplement(complementName, customLibelleHolder, basedefichesFormatDef);
            valueList.clear();
            libelleList.clear();
        }

    }


    private static class BasedefichesFormatDefErrorHandler implements BasedefichesFormatDefParser.ErrorHandler {

        private ScrutariExportXmlErrorHandler errorHandler;

        private BasedefichesFormatDefErrorHandler(ScrutariExportXmlErrorHandler errorHandler) {
            this.errorHandler = errorHandler;
        }

        public void nullFormatSourceError() {
            handle("nullFormatSourceError");
        }

        public void wrongFormatSourceError() {
            handle("wrongFormatSourceError");
        }

        public void wrongFormatSourcePartError(String formatSourcePart) {
            handle("wrongFormatSourcePartError" + " : " + formatSourcePart);
        }

        public void invalidAsciiCharacterInstructionError(String formatInstructionPart) {
            handle("invalidAsciiCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void invalidEndCharacterInstructionError(String formatInstructionPart) {
            handle("invalidEndCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void invalidSeparatorCharacterInstructionError(String formatInstructionPart) {
            handle("invalidSeparatorCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void unknownArgumentKeyInstructionWarning(String argumentKey) {
            handle("unknownArgumentKeyInstructionWarning" + " : " + argumentKey);
        }

        public void missingArgumentValueInstructionError(String argumentKey) {
            handle("missingArgumentValueInstructionError" + " : " + argumentKey);
        }

        public void wrongArgumentValueInstructionError(String argumentKey, String argumentValue) {
            handle("wrongArgumentValueInstructionError" + " : " + argumentKey + " / " + argumentValue);
        }

        private void handle(String s) {
            errorHandler.wrongElementValueError("value", s);
        }

    }

}
