/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver;

import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.managers.transformation.*;
import fr.exemole.bdfserver.managers.tableexport.*;
import fr.exemole.bdfserver.storage.*;
import fr.exemole.bdfserver.storage.BdfResourceStorage;
import fr.exemole.bdfserver.storage.StorageProcessLog;
import net.mapeadores.basedefiches.*;
import java.io.*;
import fr.exemole.bdfserver.implementations.*;
import javax.xml.transform.*;
import net.mapeadores.basedefiches.xml.libs.*;
import fr.exemole.bdfserver.xsltfactory.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.xsltfactory.xslpattern.*;
import fr.exemole.bdfserver.libscol.*;
import fr.exemole.bdfserver.externalscript.*;
import fr.exemole.bdfserver.externalscript.impl.*;
import fr.exemole.bdfserver.util.BdfServerUtils;
import net.mapeadores.util.css.SacParserInit;
import net.mapeadores.util.buildinfo.BuildInfo;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.docstream.ClassResourceDocStream;
import net.mapeadores.util.io.docstream.StringDocStream;


/**
 *
 * @author Vincent Calame
 */
public class BdfServer implements XSLTFactoryConstants {

    static {
        SacParserInit.init();
    }

    Basedefiches basedefiches;
    BdfStorage bdfStorage;
    PermissionManager permissionManager;
    private PasswordManagerImpl passwordManager;
    ConfigurationManager configurationManager;
    UIManager uiManager;
    PolicyManager policyManager;
    BdfServerBasedefichesListener basedefichesListener;
    BalayageManager balayageManager;
    private TransformationURIResolver transformationURIResolver;
    private ScrutariExportManagerImpl scrutariExportManager;
    private SqlExportManagerImpl sqlExportManager;
    LibelleManager libelleManager;
    StorageProcessLog storageProcessLog;
    private TransformationManager transformationManager;
    private TableExportManager tableExportManager;
    private BuildInfo buildInfo;
    private File contextDirectory;
    private BdfResourceStorage bdfResourceStorage;
    private OutputStorage outputStorage;
    private BdfExtensionStorage bdfExtensionStorage;
    private DefaultExternalScriptManager externalScriptManager;

    public BdfServer(StorageCollection storageCollection) {
        this.storageProcessLog = storageCollection.getStorageProcessLog();
        BdfStorage.Check bdfStorageCheck = storageCollection.getBdfStorageCheck();
        this.bdfStorage = storageCollection.getBdfStorage();
        this.bdfResourceStorage = storageCollection.getInnerResourceStorage();
        this.outputStorage = storageCollection.getOutputStorage();
        this.bdfExtensionStorage = storageCollection.getBdfExtensionStorage();
        this.basedefiches = bdfStorage.getBasedefiches();
        this.permissionManager = new PermissionManager(this, bdfStorage.getRoleMap());
        this.passwordManager = new PasswordManagerImpl(this, bdfStorage.getPasswordStorage());
        passwordManager.check(bdfStorageCheck.getPasswordList());
        this.policyManager = new PolicyManager(this);
        this.configurationManager = new ConfigurationManager(this);
        this.scrutariExportManager = new ScrutariExportManagerImpl(this, bdfStorage.getScrutariExportStorage());
        scrutariExportManager.check(bdfStorageCheck.getScrutariExportDefList());
        this.sqlExportManager = new SqlExportManagerImpl(this, bdfStorage.getSqlExportStorage());
        sqlExportManager.check(bdfStorageCheck.getSqlExportDefList());
        this.uiManager = UIManager.newInstance(this, bdfStorage);
        this.externalScriptManager = new DefaultExternalScriptManager(this, storageCollection.getExternalScriptPath());
        bdfStorage.loadManager(permissionManager);
        bdfStorage.loadManager(passwordManager);
        bdfStorage.loadManager(configurationManager);
        bdfStorage.loadManager(policyManager);
        bdfStorage.loadManager(tableExportManager);
        bdfStorage.loadManager(scrutariExportManager);
        bdfStorage.loadManager(sqlExportManager);
        this.basedefichesListener = new BdfServerBasedefichesListener(this);
        this.balayageManager = new BalayageManager(this, storageCollection.getBalayagesStorage());
        this.tableExportManager = new TableExportManager(this, storageCollection.getTableExportStorage());
        this.transformationManager = new TransformationManager(this, storageCollection.getTransformationStorage());
        this.transformationURIResolver = new TransformationURIResolver(this);
        this.transformationManager.update();
        this.libelleManager = new LibelleManager(this);
        basedefiches.addBasedefichesListener(basedefichesListener);
        BdfServerUtils.checkBdfUserDef(this, bdfStorageCheck.getBdfUserDefList());
    }

    public StorageProcessLog getStorageProcessLog() {
        return storageProcessLog;
    }

    public int getDefaultWorkingLangInteger() {
        int[] array = configurationManager.getLangConfig().getWorkingLangIntegerArray();
        return array[0];
    }

    public BdfStorage getBdfStorage() {
        return bdfStorage;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public BasedefichesEditor getBasedefichesEditor() {
        return bdfStorage.getBasedefichesEditor();
    }

    public ConfigurationManager getConfigurationManager() {
        return configurationManager;
    }

    public PermissionManager getPermissionManager() {
        return permissionManager;
    }

    public PasswordManager getPasswordManager() {
        return passwordManager;
    }

    public TransformationManager getTransformationManager() {
        return transformationManager;
    }

    public UIManager getUIManager() {
        return uiManager;
    }

    public PolicyManager getPolicyManager() {
        return policyManager;
    }

    public TableExportManager getTableExportManager() {
        return tableExportManager;
    }

    public ScrutariExportManager getScrutariExportManager() {
        return scrutariExportManager;
    }

    public SqlExportManager getSqlExportManager() {
        return sqlExportManager;
    }

    public BalayageManager getBalayageManager() {
        return balayageManager;
    }

    public ExternalScriptManager getExternalScriptManager() {
        return externalScriptManager;
    }

    public LibelleManager getLibelleManager() {
        return libelleManager;
    }

    public URIResolver getURIResolver() {
        return transformationURIResolver;
    }

    public void bufferBasedefichesListener() {
        basedefichesListener.buffer();
    }

    public void flushBasedefichesListener() {
        basedefichesListener.flush();
    }

    public File getContextFile(String path) {
        return new File(contextDirectory, path);
    }

    public DocStream getInnerResourceDocStream(XsltResource xsltResource, boolean withTabs) {
        short resourceType = xsltResource.getResourceType();
        switch (resourceType) {
            case XsltResource.XSLCOMMON_RESOURCE:
                ClassResourceDocStream xslDocStream = ClassResourceDocStream.newInstance(BdfServer.class, "resources/xslt/_common_" + xsltResource.getSubname() + ".xsl");
                if (xslDocStream != null) {
                    xslDocStream.setMimeType("text/xml");
                    xslDocStream.setCharset("UTF-8");
                }
                return xslDocStream;
            case XsltResource.FORMATS_RESOURCE:
                return toDocStream(DecimalFormatXSLTWriter.getDecimalFormatXSLT(getConfigurationManager().getLangConfig().getAllLocaleSet(), withTabs));
            case XsltResource.LOCALESTRINGS_RESOURCE:
                return toDocStream(LocaleStringsXMLWriter.getLocaleStringsXML(xsltResource.getLocaleBundle(), withTabs));
            case XsltResource.DEFAULT_RESOURCE:
                return toDocStream(DefaultXSLTFactory.getDefaultStyleSheet(this, xsltResource.getTransformationKey(), XSL_FORMAT));
            case XsltResource.DEFAULT_PATTERN_RESOURCE:
                return toDocStream(XSLTPatternBuilder.getXSLTPattern(this, xsltResource.getTransformationKey(), false, XSL_FORMAT), "text/plain");
            case XsltResource.IMPORT_RESOURCE:
                return toDocStream(ImportXSLTWriter.getImportXSLT(xsltResource.getSubname(), this, withTabs));
            case XsltResource.LIBS_RESOURCE:
                return toDocStream(LibsXMLWriter.getLibsXML(xsltResource.getLocalisationInfo().getWorkingLangInteger(), basedefiches, xsltResource.getSubsetKey(), true));
            case XsltResource.LIBSCOL_RESOURCE:
                return toDocStream(LibsColXMLWriter.getLibsColXML(xsltResource.getLocalisationInfo().getWorkingLangInteger(), this, xsltResource.getLibsColKey(), true));
            case XsltResource.SPECIALLIBS_RESOURCE:
                return toDocStream(LibsXMLWriter.getLibsXML(xsltResource.getLocalisationInfo().getWorkingLangInteger(), basedefiches, xsltResource.getSubname(), true));
            default:
                throw new SwitchException("xsltResource.getResourceType() = " + xsltResource.getResourceType());
        }
    }

    private DocStream toDocStream(String s) {
        return toDocStream(s, "text/xml");
    }

    private DocStream toDocStream(String s, String mimeType) {
        if (s == null) {
            return null;
        }
        StringDocStream docStream = new StringDocStream(s);
        docStream.setMimeType(mimeType);
        return docStream;
    }

    public void saveUIManager() {
        transformationManager.clearDefaultTransformer();
        getBdfStorage().saveManager(uiManager);
    }

    public void savePermissionManager() {
        getBdfStorage().saveManager(permissionManager);
    }

    public void saveConfigurationManager() {
        transformationManager.update();
        getBdfStorage().saveManager(configurationManager);
    }

    public void savePolicyManager() {
        getBdfStorage().saveManager(policyManager);
    }

    /**
     *
     */
    public TransformerFactory newTransformationFactory(ErrorListener errorListener) {
        TransformerFactory factory = TransformerFactory.newInstance();
        factory.setURIResolver(transformationURIResolver);
        if (errorListener != null) {
            factory.setErrorListener(errorListener);
        }
        return factory;
    }

    /**
     * 
     * 
     * @throws NestedTransformerException si une exception TransformerException a été rencontrée.
     */
    public Transformer newTransformer(Source source, ErrorListener errorListener) {
        TransformerFactory transformerFactory = newTransformationFactory(errorListener);
        try {
            Transformer transformer = transformerFactory.newTransformer(source);
            transformer.setURIResolver(transformationURIResolver);
            if (errorListener != null) {
                transformer.setErrorListener(errorListener);
            }
            return transformer;
        } catch (TransformerException e) {
            throw new NestedTransformerException(e);
        }
    }

    /**
     * 
     * 
     * @throws NestedTransformerException si une exception TransformerException a été rencontrée.
     */
    public Transformer newTransformer(Templates templates) {
        try {
            Transformer transformer = templates.newTransformer();
            transformer.setURIResolver(transformationURIResolver);
            //transformer.setErrorListener(...);
            return transformer;
        } catch (TransformerException e) {
            throw new NestedTransformerException(e);
        }
    }

    public BuildInfo getBuildInfo() {
        return buildInfo;
    }

    public void setBuildInfo(BuildInfo buildInfo) {
        this.buildInfo = buildInfo;
    }

    public void setContextDirectory(File contextDirectory) {
        this.contextDirectory = contextDirectory;
    }

    public BdfResourceStorage getBdfResourceStorage() {
        return bdfResourceStorage;
    }

    public OutputStorage getOutputStorage() {
        return outputStorage;
    }

    public BdfExtensionStorage getBdfExtensionStorage() {
        return bdfExtensionStorage;
    }

}
