/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver;

import fr.exemole.bdfserver.ui.SubsetKeyTree;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.event.BasedefichesAdapter;
import net.mapeadores.basedefiches.l10n.BasedefichesL10n;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;


/**
 * Signale à l'instance de BdfServer à laquelle est rattachée l'instance de BasedefichesListener les événements les plus importants
 * sur la base de fiches.
 *
 * @author Vincent Calame
 */
public class BdfServerBasedefichesListener extends BasedefichesAdapter {

    public final static int SAVE_UIMANAGER = 1;
    public final static int SAVE_POLICYMANAGER = 2;
    public final static int SAVE_PERMISSIONMANAGER = 4;
    public final static int CLEAR_DEFAULTTEMPLATES = 16;
    private BdfServer bdfServer;
    private boolean onMultiChange = false;
    private int multiChangeMask = 0;

    public BdfServerBasedefichesListener(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public void buffer() {
        if (!onMultiChange) {
            onMultiChange = true;
            multiChangeMask = 0;
        }
    }

    private void saveUIManager() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | SAVE_UIMANAGER;
        } else {
            bdfServer.saveUIManager();
        }
    }

    private void savePolicyManager() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | SAVE_POLICYMANAGER;
        } else {
            bdfServer.savePolicyManager();
        }
    }

    private void savePermissionManager() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | SAVE_PERMISSIONMANAGER;
        } else {
            bdfServer.savePermissionManager();
        }
    }

    private void clearDefaultTemplates() {
        if (onMultiChange) {
            multiChangeMask = multiChangeMask | CLEAR_DEFAULTTEMPLATES;
        } else {
            bdfServer.getTransformationManager().clearDefaultTransformer();
        }
    }

    public void flush() {
        if ((multiChangeMask & CLEAR_DEFAULTTEMPLATES) != 0) {
            bdfServer.getTransformationManager().clearDefaultTransformer();
        }
        if ((multiChangeMask & SAVE_UIMANAGER) != 0) {
            bdfServer.saveUIManager();
        }
        if ((multiChangeMask & SAVE_POLICYMANAGER) != 0) {
            bdfServer.savePolicyManager();
        }
        if ((multiChangeMask & SAVE_PERMISSIONMANAGER) != 0) {
            bdfServer.savePermissionManager();
        }
        onMultiChange = false;
        multiChangeMask = 0;
    }

    @Override
    public void thesaurusCreated(Thesaurus thesaurus) {
        bdfServer.getPolicyManager().createThesaurusPolicy(thesaurus);
        bdfServer.getUIManager().thesaurusCreated(thesaurus);
        savePolicyManager();
        saveUIManager();
    }

    @Override
    public void thesaurusRemoved(SubsetKey thesaurusKey) {
        bdfServer.getUIManager().thesaurusRemoved(thesaurusKey);
        bdfServer.getPolicyManager().removeThesaurusPolicy(thesaurusKey);
        clearDefaultTemplates();
        saveUIManager();
        savePolicyManager();
    }

    @Override
    public void sphereCreated(Sphere sphere) {
        bdfServer.getUIManager().sphereCreated(sphere);
        saveUIManager();
    }

    @Override
    public void sphereRemoved(SubsetKey sphereKey) {
        bdfServer.getUIManager().sphereRemoved(sphereKey);
        saveUIManager();
    }

    @Override
    public void albumCreated(Album album) {
        bdfServer.getUIManager().albumCreated(album);
        saveUIManager();
    }

    @Override
    public void albumRemoved(SubsetKey albumKey) {
        bdfServer.getUIManager().albumRemoved(albumKey);
        saveUIManager();
    }

    @Override
    public void redacteurRemoved(RedacteurKey redacteurKey) {
        bdfServer.getBdfStorage().getBdfUserStorage().removeBdfUserDef(redacteurKey);
        bdfServer.getPasswordManager().setPassword(redacteurKey.toString(), null);
    }

    @Override
    public void corpusCreated(Corpus corpus) {
        initLibelle(bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey()));
        bdfServer.getUIManager().corpusCreated(corpus);
        saveUIManager();
    }

    @Override
    public void corpusRemoved(SubsetKey corpusKey, Subset masterSubset) {
        bdfServer.getUIManager().corpusRemoved(corpusKey, masterSubset);
        clearDefaultTemplates();
        saveUIManager();
    }

    @Override
    public void corpusFieldCreated(Corpus corpus, CorpusField corpusField) {
        FieldKey fieldKey = corpusField.getFieldKey();
        if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
            initLibelle(corpusEditor, corpusField);
        }
        bdfServer.getUIManager().corpusFieldCreated(corpus, corpusField);
        saveUIManager();
    }

    @Override
    public void corpusFieldRemoved(Corpus corpus, FieldKey fieldKey) {
        bdfServer.getUIManager().corpusFieldRemoved(corpus, fieldKey);
        saveUIManager();
    }

    private void initLibelle(CorpusEditor corpusEditor) {
        initLibelle(corpusEditor, FieldKey.IDCORPUS);
        initLibelle(corpusEditor, FieldKey.LANG);
        initLibelle(corpusEditor, FieldKey.TITRE);
        initLibelle(corpusEditor, FieldKey.REDACTEURS);
    }

    private void initLibelle(CorpusEditor corpusEditor, FieldKey fieldKey) {
        initLibelle(corpusEditor, corpusEditor.getCorpus().getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey));
    }

    private void initLibelle(CorpusEditor corpusEditor, CorpusField corpusField) {
        int[] workingLangIntegerArray = bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray();
        LibelleHolder libelleHolder = BasedefichesL10n.getSpecialFieldKeyLibelle(corpusField.getFieldKey());
        int libelleCount = libelleHolder.getLibelleCount();
        for (int j = 0; j < libelleCount; j++) {
            Libelle libelle = libelleHolder.getLibelleByIndex(j);
            int langInteger = libelle.getLangInteger();
            for (int i = 0; i < workingLangIntegerArray.length; i++) {
                if (langInteger == workingLangIntegerArray[i]) {
                    corpusEditor.getCorpusMetadataEditor().putLibelle(corpusField, libelle);
                    break;
                }
            }
        }
    }

    private void addSubsetKey(SubsetKeyTree tree, SubsetKey subsetKey) {
        tree.addSubsetKey(subsetKey, null);
    }

}
