/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.TransformationManager;
import fr.exemole.bdfserver.managers.transformation.TransformerParameters;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import java.util.HashMap;
import java.util.Map;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.io.TransformerErrorListener;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author Vincent Calame
 */
public class TransformManagerBuilder implements BalayageConstants {

    private ErrorHandler errorHandler;
    private BdfServer bdfServer;
    private Balayage balayage;

    public TransformManagerBuilder(BdfServer bdfServer, Balayage balayage, ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.bdfServer = bdfServer;
        this.balayage = balayage;
    }

    public TransformManager build(Output output) {
        String xsltPath = output.getXsltPath();
        if (xsltPath == null) {
            errorHandler.undefinedXslPath();
            return null;
        }
        if (xsltPath.startsWith("bdf:")) {
            return buildFromTransformationKey(output, xsltPath.substring(4));
        } else {
            return buildFromBalayage(output, xsltPath);
        }
    }

    private TransformManager buildFromBalayage(Output output, String xsltPath) {
        Source transformerSource = bdfServer.getBalayageManager().getTransformerSource(balayage, xsltPath);
        if (transformerSource == null) {
            errorHandler.wrongXslPath(xsltPath);
            return null;
        }
        TransformerErrorListener errorListener = new TransformerErrorListener();
        Transformer transformer = null;
        try {
            transformer = bdfServer.newTransformer(transformerSource, errorListener);
        } catch (NestedTransformerException ute) {
            errorListener.checkFatalTransformerException(ute.geTransformerException());
        }
        if (errorListener.hasError()) {
            errorHandler.transformerError(xsltPath, errorListener);
            return null;
        }
        return new BalayageTransformManager(output, transformer);
    }

    private TransformManager buildFromTransformationKey(Output output, String key) {
        if (output.getBalayageUnit().getType() != FICHE_TYPE) {
            errorHandler.wrongBalayageUnitType();
            return null;
        }
        if ((key.length() == 0) || (key.equals("_default"))) {
            key = null;
        }
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        Map<SubsetKey, Transformer> transformerMap = new HashMap();
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            SubsetKey corpusKey = basedefiches.getCorpus(i).getSubsetKey();
            TransformationKey transformationKey = new TransformationKey(corpusKey);
            XslTransformerKey xslTransformerKey = new XslTransformerKey(transformationKey, key);
            Transformer transformer = transformationManager.getXslTransformer(xslTransformerKey);
            transformerMap.put(corpusKey, transformer);
        }
        int defaultWorkingLang = bdfServer.getConfigurationManager().getLangConfig().getDefaultWorkingLangInteger();
        return new TransformationKeyTransformManager(output, transformerMap, LangInteger.toString(defaultWorkingLang));
    }


    public interface ErrorHandler {

        public void wrongBalayageUnitType();

        public void transformerError(String xsltPath, TransformerErrorListener errorListener);

        public void undefinedXslPath();

        public void wrongXslPath(String xsltPath);

    }


    private static abstract class AbstractTransformManager implements TransformManager {

        TransformerParameters transformerParameters;

        AbstractTransformManager(Output output) {
            transformerParameters = new TransformerParameters();
            int outputParamCount = output.getOutputParamCount();
            boolean reversePathDone = false;
            if (outputParamCount > 0) {
                for (int i = 0; i < outputParamCount; i++) {
                    OutputParam param = output.getOutputParam(i);
                    String name = param.getName();
                    if (name.equals(BDF_REVERSEPATH_PARAMETER)) {
                        reversePathDone = true;
                    }
                    transformerParameters.setPriorParam(name, param.getValue());
                }
            }
            if (!reversePathDone) {
                transformerParameters.setParam(BDF_REVERSEPATH_PARAMETER, output.getReversePath());
            }
        }

        public void setCurrentFileName(String fileName) {
            transformerParameters.setParam(BDF_FILENAME_PARAMETER, fileName);
        }

        public void setBalayageRoot(String balayageRoot) {
            transformerParameters.setParam(BDF_BALAYAGEROOT_PARAMETER, balayageRoot);
        }

        public void setCurrentPath(String path) {
            if (path == null) {
                path = "";
            }
            transformerParameters.setParam(BDF_PATH_PARAMETER, path);
        }

        public void setCurrentLang(String lang) {
            transformerParameters.setParam(BDF_LANG_PARAMETER, lang);
        }

        void initParameters(Transformer transformer) {
            transformerParameters.initParameters(transformer);
        }

    }


    private static class BalayageTransformManager extends AbstractTransformManager {

        private Transformer transformer;

        BalayageTransformManager(Output output, Transformer transformer) {
            super(output);
            this.transformer = transformer;
        }

        public Transformer getTransformer(Object extractionObject) {
            initParameters(transformer);
            return transformer;
        }

    }


    private static class TransformationKeyTransformManager extends AbstractTransformManager {

        private Map<SubsetKey, Transformer> transformerMap;
        private String defaultLang;

        TransformationKeyTransformManager(Output output, Map<SubsetKey, Transformer> transformerMap, String defaultLang) {
            super(output);
            this.transformerMap = transformerMap;
            this.defaultLang = defaultLang;
        }

        public Transformer getTransformer(Object extractionObject) {
            AbregeFiche abregeFiche = (AbregeFiche) extractionObject;
            SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
            Transformer transformer = transformerMap.get(corpusKey);
            initParameters(transformer);
            return transformer;
        }

        @Override
        public void setCurrentLang(String lang) {
            transformerParameters.setParam(BDF_LANG_PARAMETER, lang);
            if (lang == null) {
                lang = defaultLang;
            }
            transformerParameters.setParam(WORKINGLANG_PARAMETER, lang);
            transformerParameters.setParam(FORMATLOCALE_PARAMETER, lang);
        }

    }

}
