/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageConstants;
import fr.exemole.bdfserver.balayages.BalayageMode;
import fr.exemole.bdfserver.balayages.BalayageUnit;
import fr.exemole.bdfserver.balayages.engine.runners.RunnerFactory;
import fr.exemole.bdfserver.externalscript.ExternalScript;
import fr.exemole.bdfserver.managers.ScrutariExportManager;
import fr.exemole.bdfserver.runners.ScrutariExportRunner;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;


/**
 *
 * @author Vincent Calame
 */
public class BalayageEngine implements BalayageConstants {

    private BalayageParameters balayageParameters;
    private List<BalayageUnitRunner> balayageUnitRunnerList = new ArrayList<BalayageUnitRunner>();
    private SiteMapBuilder siteMapBuilder;

    public BalayageEngine(BalayageParameters balayageParameters) {
        this.balayageParameters = balayageParameters;
        this.siteMapBuilder = (SiteMapBuilder) balayageParameters.getFileGenerationListener();
        Balayage balayage = balayageParameters.getBalayage();
        int balayageUnitCount = balayage.getBalayageUnitCount();
        for (int i = 0; i < balayageUnitCount; i++) {
            BalayageUnit balayageUnit = balayage.getBalayageUnit(i);
            if (doBalayageUnit(balayageUnit)) {
                BalayageUnitRunner bu = RunnerFactory.get(balayageParameters, i, balayageUnit);
                balayageUnitRunnerList.add(bu);
            }
        }
    }

    private boolean doBalayageUnit(BalayageUnit balayageUnit) {
        BalayageMode balayageMode = balayageParameters.getBalayageMode();
        if (balayageMode == null) {
            return true;
        }
        return balayageUnit.hasMode(balayageMode.getName());
    }

    public void run() {
        BalayageLog balayageLog = balayageParameters.getBalayageLog();
        balayageLog.runStarted();
        for (BalayageUnitRunner bu : balayageUnitRunnerList) {
            bu.run();
            bu.dispose();
        }
        balayageUnitRunnerList.clear();
        if (siteMapBuilder != null) {
            try {
                String path = siteMapBuilder.writeSiteMapFile(balayageParameters.getBalayageRootDirectory());
                balayageLog.addPostscriptumLog(BalayageLog.SITEMAP_KEY, path);
            } catch (IOException ioe) {
                balayageLog.addPostscriptumError(BalayageLog.SITEMAP_KEY, BalayageLog.IO_EXCEPTION, getMessage(ioe));
            }
        }
        execPostscriptum();
        balayageLog.runEnded();
    }

    private void execPostscriptum() {
        BalayageLog balayageLog = balayageParameters.getBalayageLog();
        if (balayageLog.hasError()) {
            return;
        }
        BdfServer bdfServer = balayageParameters.getBdfServer();
        Balayage.Postscriptum postscriptum = balayageParameters.getBalayage().getPostscriptum();
        int scrutariExportCount = postscriptum.getScrutariExportCount();
        if (scrutariExportCount > 0) {
            ScrutariExportManager scrutariExportManager = bdfServer.getScrutariExportManager();
            for (int i = 0; i < scrutariExportCount; i++) {
                String name = postscriptum.getScrutariExportName(i);
                ScrutariExportDef scrutariExportDef = scrutariExportManager.getScrutariExportDef(name);
                if (scrutariExportDef != null) {
                    try {
                        ScrutariExportRunner.run(scrutariExportDef, bdfServer);
                        balayageLog.addPostscriptumLog(BalayageLog.SCRUTARIEXPORT_KEY, name);
                    } catch (IOException ioe) {
                        balayageLog.addPostscriptumError(BalayageLog.SCRUTARIEXPORT_KEY, BalayageLog.IO_EXCEPTION, getMessage(ioe));
                    }
                } else {
                    balayageLog.addPostscriptumError(BalayageLog.SCRUTARIEXPORT_KEY, BalayageLog.UNKNOWN_NAME, name);
                }
            }
        }
        if (balayageLog.hasError()) {
            return;
        }
        String command = postscriptum.getCommand();
        if (command == null) {
            return;
        }
        ExternalScript externalScript = bdfServer.getExternalScriptManager().getExternalScript(command);
        if (externalScript == null) {
            balayageLog.addPostscriptumError(BalayageLog.COMMAND_KEY, BalayageLog.COMMANDCHECK_EXCEPTION, "wrong script name : " + command);
            return;
        }
        try {
            externalScript.exec();
            balayageLog.addPostscriptumLog(BalayageLog.COMMAND_KEY, command);
        } catch (IOException ioe) {
            balayageLog.addPostscriptumError(BalayageLog.COMMAND_KEY, BalayageLog.COMMANDIO_EXCEPTION, getMessage(ioe));
        } /*catch(InterruptedException ie) {
        balayageLog.addCommandException(BalayageLog.COMMANDINTERRUPTED_EXCEPTION, ie.getMessage());
        return false;
        }*/
    }

    private String getMessage(IOException ioe) {
        String msg = ioe.getMessage();
        String message = ioe.getClass().getName();
        if (msg != null) {
            message = message + " : " + msg;
        }
        return message;
    }

}
