/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.balayages.BalayageUnit;
import fr.exemole.bdfserver.balayages.BalayageUtils;
import fr.exemole.bdfserver.balayages.Output;
import java.util.List;
import java.util.ArrayList;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class BalayageLog {

    public static final short UNRESOLVED_PATH = 1;
    public static final short WRONG_PATTERN = 2;
    public static final short NO_OUTPUT = 3;
    public static final short EXTRACTION_NOT_FOUND = 4;
    public static final short EXTRACTION_EXCEPTION = 5;
    public static final short OUTPUTPATH_EXCEPTION = 6;
    public static final short TRANSFORMER_NOT_FOUND = 7;
    public static final short TRANSFORMER_EXCEPTION = 8;
    public static final short NO_XSLTPATH = 9;
    public static final short WRONG_PATH = 10;
    public static final short TRANSFORMATION_EXCEPTION = 11;
    public static final short IO_EXCEPTION = 12;
    public static final short COMMANDCHECK_EXCEPTION = 13;
    public static final short COMMANDIO_EXCEPTION = 14;
    public static final short COMMANDINTERRUPTED_EXCEPTION = 15;
    public static final short UNKNOWN_NAME = 16;
    public static final String EXTRACTION_ATTRIBUTE = "extraction";
    public static final String SITEMAP_KEY = "sitemap";
    public static final String COMMAND_KEY = "command";
    public static final String SCRUTARIEXPORT_KEY = "scrutariexport";
    private List<LogItem> logList = new ArrayList<LogItem>();
    private long runStartTime;
    private PrintWriter fileWriter;
    private boolean hasError = false;

    public BalayageLog(String balayageName, File logFile) {
        try {
            fileWriter = new PrintWriter(new OutputStreamWriter(new FileOutputStream(logFile), "UTF-8"));
        } catch (IOException ioe) {
            fileWriter = null;
        }
    }

    public boolean hasError() {
        return hasError;
    }

    public void addBalayageUnitLog(int balayageUnitIndex, BalayageUnit balayageUnit) {
        LogItem logItem = new BalayageUnitLogItem(balayageUnitIndex, toString(balayageUnit));
        logList.add(logItem);
        printAll(logItem);
    }

    public void addBalayageUnitError(int balayageUnitIndex, BalayageUnit balayageUnit, short type, String value, String message) {
        LogItem logItem = new BalayageUnitErrorItem(balayageUnitIndex, toString(balayageUnit), type, value, message);
        logList.add(logItem);
        hasError = true;
        printAll(logItem);
    }

    public void addOutputError(int balayageUnitIndex, int outputIndex, Output output, short type, String value, String message) {
        LogItem logItem = new OutputErrorItem(balayageUnitIndex, toString(output.getBalayageUnit()), outputIndex, type, value, message);
        logList.add(logItem);
        hasError = true;
        printAll(logItem);
    }

    public void addPostscriptumLog(String key, String value) {
        LogItem logItem = new PostscriptumLogItem(key, value);
        logList.add(logItem);
        printAll(logItem);
    }

    public void addPostscriptumError(String key, short type, String message) {
        LogItem logItem = new PostscriptumErrorItem(key, type, message);
        logList.add(logItem);
        hasError = true;
        printAll(logItem);
    }

    public void addFileGeneration(int balayageUnitIndex, int outputIndex, File f, long time) {
        LogItem logItem = new FileGeneration(balayageUnitIndex, outputIndex, f, time);
        printAll(logItem);
    }

    public void runStarted() {
        runStartTime = System.currentTimeMillis();
    }

    public void runEnded() {
        LogItem logItem = new RunEnded();
        logList.add(logItem);
        printAll(logItem);
    }

    private void printAll(LogItem logItem) {
        if (fileWriter != null) {
            logItem.print(fileWriter, runStartTime);
        }
    }

    public void close() {
        if (fileWriter != null) {
            fileWriter.close();
        }
    }

    public void printLog(PrintWriter pw) {
        for (LogItem logItem : logList) {
            logItem.print(pw, runStartTime);
        }
    }


    private static abstract class LogItem {

        public abstract void print(PrintWriter pw, long runStartTime);

    }


    private static class RunEnded extends LogItem {

        private long time;

        private RunEnded() {
            time = System.currentTimeMillis();
        }

        public void print(PrintWriter pw, long runStartTime) {
            pw.println("");
            pw.print("===== Generation time  : ");
            printTime(pw, time - runStartTime);
            pw.println("");
        }

    }


    private static class BalayageUnitLogItem extends LogItem {

        private int balayageUnitIndex;
        private long time;
        private int freeMemory;
        private String balayageUnitString;

        private BalayageUnitLogItem(int balayageUnitIndex, String balayageUnitString) {
            this.balayageUnitIndex = balayageUnitIndex;
            this.time = System.currentTimeMillis();
            this.freeMemory = (int) Math.floor(((double) Runtime.getRuntime().freeMemory()) / (1024 * 1024));
            this.balayageUnitString = balayageUnitString;
        }

        public void print(PrintWriter pw, long runStartTime) {
            pw.println("");
            pw.print("+------- ");
            pw.print("balayageUnit ");
            pw.print(balayageUnitIndex + 1);
            pw.print(": ");
            pw.print(balayageUnitString);
            pw.print(" (+ ");
            printTime(pw, time - runStartTime);
            pw.println(") / mem = " + freeMemory);
        }

    }


    private static class BalayageUnitErrorItem extends LogItem {

        private int balayageUnitIndex;
        private String message;
        private short type;
        private String value;
        private String balayageUnitString;

        private BalayageUnitErrorItem(int balayageUnitIndex, String balayageUnitString, short type, String value, String message) {
            this.balayageUnitIndex = balayageUnitIndex;
            this.message = message;
            this.type = type;
            this.value = value;
            this.balayageUnitString = balayageUnitString;
        }

        public void print(PrintWriter pw, long runStartTime) {
            pw.println("");
            pw.print("[! ");
            pw.print(typeToString(type));
            pw.print(" !] ");
            pw.print("balayageUnit ");
            pw.print(balayageUnitIndex + 1);
            pw.print(": ");
            pw.print(balayageUnitString);
            if (value != null) {
                pw.print(", value: ");
                pw.print(value);
            }
            if (message != null) {
                pw.print(", message: ");
                pw.print(message);
            }
            pw.println("");

        }

    }


    private static class OutputErrorItem extends LogItem {

        private int balayageUnitIndex;
        private int outputIndex;
        private String message;
        private short type;
        private String value;
        private String balayageUnitString;

        private OutputErrorItem(int balayageUnitIndex, String balayageUnitString, int outputIndex, short type, String value, String message) {
            this.balayageUnitIndex = balayageUnitIndex;
            this.outputIndex = outputIndex;
            this.message = message;
            this.type = type;
            this.value = value;
            this.balayageUnitString = balayageUnitString;
        }

        public void print(PrintWriter pw, long runStartTime) {
            pw.println("");
            pw.print("[! ");
            pw.print(typeToString(type));
            pw.print(" !] ");
            pw.print("balayageUnit ");
            pw.print(balayageUnitIndex + 1);
            pw.print(": ");
            pw.print(balayageUnitString);
            pw.print(", output: ");
            pw.print(outputIndex + 1);
            if (value != null) {
                pw.print(", value: ");
                pw.print(value);
            }
            if (message != null) {
                pw.print(", message: ");
                pw.print(message);
            }
            pw.println("");

        }

    }


    private static class PostscriptumLogItem extends LogItem {

        private String key;
        private String message;

        private PostscriptumLogItem(String key, String message) {
            this.key = key;
            this.message = message;
        }

        public void print(PrintWriter pw, long runStartTime) {
            pw.println("");
            pw.print("+------- ");
            pw.print(key);
            pw.print(": ");
            pw.println(message);
        }

    }


    private static class PostscriptumErrorItem extends LogItem {

        private String key;
        private short type;
        private String message;

        private PostscriptumErrorItem(String key, short type, String message) {
            this.key = key;
            this.type = type;
            this.message = message;
        }

        public void print(PrintWriter pw, long runStartTime) {
            pw.println("");
            pw.print("[! ");
            pw.print(key);
            pw.print(": ");
            pw.print(typeToString(type));
            pw.print(" !] ");
            pw.print(message);
            pw.println("");

        }

    }


    private static class FileGeneration extends LogItem {

        private int balayageUnitIndex;
        private int outputIndex;
        private String path;
        private long time;
        private int freeMemory;

        private FileGeneration(int balayageUnitIndex, int outputIndex, File f, long time) {
            this.balayageUnitIndex = balayageUnitIndex;
            this.outputIndex = outputIndex;
            this.time = time;
            this.path = f.getAbsolutePath();
            this.freeMemory = (int) Math.floor(((double) Runtime.getRuntime().freeMemory()) / (1024 * 1024));
        }

        public void print(PrintWriter pw, long runStartTime) {
            pw.print("out: ");
            pw.print(balayageUnitIndex + 1);
            pw.print(".");
            pw.print(outputIndex + 1);
            pw.print(" path: ");
            pw.print(path);
            pw.print(" (");
            printTime(pw, time);
            pw.println(") / mem = " + freeMemory);
        }

    }

    private static String typeToString(short type) {
        switch (type) {
            case UNRESOLVED_PATH:
                return "unresolvedPath";
            case WRONG_PATTERN:
                return "wrongPattern";
            case NO_OUTPUT:
                return "noOutput";
            case EXTRACTION_NOT_FOUND:
                return "extractionNotFound";
            case EXTRACTION_EXCEPTION:
                return "extractionException";
            case OUTPUTPATH_EXCEPTION:
                return "outputPathException";
            case TRANSFORMER_NOT_FOUND:
                return "transformerNotFound";
            case TRANSFORMER_EXCEPTION:
                return "transformerException";
            case TRANSFORMATION_EXCEPTION:
                return "transformerExceptionDuringTransformation";
            case IO_EXCEPTION:
                return "ioException";
            case NO_XSLTPATH:
                return "noXsltPath";
            case WRONG_PATH:
                return "wrongPath";
            case COMMANDCHECK_EXCEPTION:
                return "commandCheck";
            case COMMANDIO_EXCEPTION:
                return "commandIOException";
            case COMMANDINTERRUPTED_EXCEPTION:
                return "commandInterruptedException";
            case UNKNOWN_NAME:
                return "unknownName";
            default:
                return "";
        }
    }

    private static void printTime(PrintWriter pw, long time) {
        float f = ((float) time) / 1000;
        pw.print(String.valueOf(f));
        pw.print("s");
    }

    private static String toString(BalayageUnit balayageUnit) {
        StringBuilder buf = new StringBuilder();
        buf.append(BalayageUtils.typeToString(balayageUnit.getType()));
        String extractionPath = balayageUnit.getExtractionPath();
        if (extractionPath != null) {
            buf.append("/");
            buf.append(extractionPath);
        }
        return buf.toString();
    }

}
