/* BdfServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageUnit;
import fr.exemole.bdfserver.balayages.FileGenerationListener;
import fr.exemole.bdfserver.balayages.Output;
import fr.exemole.bdfserver.balayages.TransformManager;
import fr.exemole.bdfserver.balayages.TransformManagerBuilder;
import fr.exemole.bdfserver.balayages.engine.BalayageLog;
import fr.exemole.bdfserver.balayages.engine.BalayageParameters;
import fr.exemole.bdfserver.balayages.engine.EngineUtils;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.Writer;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.util.io.TransformerErrorListener;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.text.ValueResolver;


/**
 *
 * @author Vincent Calame
 */
public class TransformOutputRunner extends AbstractOutputRunner {

    private short langOption = LANG_OPTION_UNKNOWN;
    private String[] langArray;
    private TransformManager transformManager;
    private boolean withTransformation;
    private FileGenerationListener fileGenerationListener;
    private BalayageUnit balayageUnit;

    public TransformOutputRunner(BalayageParameters balayageParameters, int outputIndex, Output output, int balayageUnitIndex) {
        super(balayageParameters, outputIndex, output, balayageUnitIndex);
        this.balayageUnit = output.getBalayageUnit();
        Balayage balayage = balayageParameters.getBalayage();
        BdfServer bdfServer = balayageParameters.getBdfServer();
        this.withTransformation = balayage.isWithTransformation();
        this.fileGenerationListener = balayageParameters.getFileGenerationListener();
        initLangArray();
        langOption = output.getLangOption();
        if (langOption == LANG_OPTION_UNKNOWN) {
            langOption = balayage.getDefaultLangOption();
        }
        if (balayage.isWithTransformation()) {
            InternalTransformerBuilderErrorHandler transformerBuilderErrorHandler = new InternalTransformerBuilderErrorHandler();
            TransformManagerBuilder transformerBuilder = new TransformManagerBuilder(bdfServer, balayage, transformerBuilderErrorHandler);
            transformManager = transformerBuilder.build(output);
            if (transformManager != null) {
                File balayageRootDirectory = balayageParameters.getBalayageRootDirectory();
                String absolutePath = balayageRootDirectory.getAbsolutePath();
                if (!absolutePath.endsWith(File.separator)) {
                    absolutePath = absolutePath + File.separator;
                }
                transformManager.setBalayageRoot(absolutePath);
            }
        }
    }


    private class InternalTransformerBuilderErrorHandler implements TransformManagerBuilder.ErrorHandler {

        private InternalTransformerBuilderErrorHandler() {
        }

        public void transformerError(String xsltPath, TransformerErrorListener errorListener) {
            isValid = false;
            balayageLog.addOutputError(balayageUnitIndex, outputIndex, output, BalayageLog.TRANSFORMER_EXCEPTION, xsltPath, errorListener.getErrorMessage());
        }

        public void wrongXslPath(String xsltPath) {
            balayageLog.addBalayageUnitError(balayageUnitIndex, balayageUnit, BalayageLog.TRANSFORMER_NOT_FOUND, xsltPath, null);
            isValid = false;
        }

        public void undefinedXslPath() {
            balayageLog.addOutputError(balayageUnitIndex, outputIndex, output, BalayageLog.NO_XSLTPATH, null, null);
            isValid = false;
        }

        public void wrongBalayageUnitType() {
        }

    }

    private void initLangArray() {
        LocalisationLists localisationsLists = output.getLocalisationLists();
        if (localisationsLists == null) {
            localisationsLists = balayageParameters.getExtractionContext().getLocalisationLists();
        }
        int count = localisationsLists.getWorkingLangCount();
        langArray = new String[count];
        for (int i = 0; i < count; i++) {
            langArray[i] = LangInteger.toString(localisationsLists.getWorkingLangInteger(i));
        }
    }

    public void write(Object extractionObject, ValueResolver valueResolver, String extractionString, long startTime) {
        String fileName = EngineUtils.getFileName(output, valueResolver, output.getBalayageUnit().getType(), withTransformation);
        String path = EngineUtils.getPath(output, valueResolver);
        if (langOption == LANG_OPTION_NONE) {
            write(extractionObject, fileName, path, langArray[0], extractionString, startTime);
        } else {
            long time = startTime;
            for (int i = 0; i < langArray.length; i++) {
                time = write(extractionObject, fileName, path, langArray[i], extractionString, time);
            }
        }
    }

    private long write(Object extractionObject, String fileName, String path, String lang, String extractionString, long startTime) {
        File directory = rootDirectory;
        if (langOption == LANG_OPTION_DIR) {
            directory = new File(directory, lang);
        }
        if (path != null) {
            directory = new File(directory, path);
        }
        prepareDirectory(directory);
        File file = new File(directory, fileName);
        boolean done = false;
        if (transformManager != null) {
            transformManager.setCurrentFileName(fileName);
            transformManager.setCurrentLang(lang);
            transformManager.setCurrentPath(path);
            done = writeTransformation(extractionObject, file, extractionString);
        } else {
            done = writeXml(file, extractionString);
        }
        if (done) {
            if (fileGenerationListener != null) {
                fileGenerationListener.generateFile(lang, path, fileName);
            }
            long currentTime = System.currentTimeMillis();
            balayageLog.addFileGeneration(balayageUnitIndex, outputIndex, file, currentTime - startTime);
            return currentTime;
        } else {
            return startTime;
        }
    }

    boolean writeXml(File file, String extractionString) {
        try {
            Writer writer = new OutputStreamWriter(new FileOutputStream(file), "UTF-8");
            writer.write(extractionString);
            writer.close();
            return true;
        } catch (IOException ioe) {
            ioe.printStackTrace();
            String msg = ioe.getMessage();
            String message = ioe.getClass().getName();
            if (msg != null) {
                message = message + " : " + msg;
            }
            balayageLog.addOutputError(balayageUnitIndex, outputIndex, output, BalayageLog.IO_EXCEPTION, file.getName(), message);
            return false;
        }
    }

    private boolean writeTransformation(Object extractionObject, File file, String extractionString) {
        try {
            StringReader stringReader = new StringReader(extractionString);
            StreamSource streamSource = new StreamSource(stringReader);
            OutputStream outputStream = new FileOutputStream(file);
            StreamResult streamResult = new StreamResult(outputStream);
            Transformer transformer = transformManager.getTransformer(extractionObject);
            transformer.transform(streamSource, streamResult);
            outputStream.close();
            stringReader = null;
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            String msg = e.getMessage();
            String message = e.getClass().getName();
            if (msg != null) {
                message = message + " : " + msg;
            }
            balayageLog.addOutputError(balayageUnitIndex, outputIndex, output, BalayageLog.TRANSFORMATION_EXCEPTION, file.getName(), message);
            return false;
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        transformManager = null;
        langArray = null;
    }

}
