/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUILibelleMap;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.format.FormatSourceLibelleProvider;
import net.mapeadores.basedefiches.include.DefaultIncludeLibelleMap;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusConstants;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class LibelleManager {

    private DefaultIncludeLibelleMap defaultInclude;
    private BdfServer bdfServer;
    private Map ilmMap = new HashMap();
    private InternalFormatSourceLibelleProvider formatSourceLibelleProvider = new InternalFormatSourceLibelleProvider();

    public LibelleManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
        defaultInclude = new DefaultIncludeLibelleMap(bdfServer.getBasedefiches());
    }

    public FormatSourceLibelleProvider getFormatSourceLibelleProvider() {
        return formatSourceLibelleProvider;
    }

    public IncludeUILibelleMap getIncludeUILibelleMap(SubsetKey corpusKey) {
        return get(corpusKey);
    }

    private BdfServerIncludeLibelleMap get(SubsetKey corpusKey) {
        BdfServerIncludeLibelleMap ilm = (BdfServerIncludeLibelleMap) ilmMap.get(corpusKey);
        if (ilm == null) {
            ilm = new BdfServerIncludeLibelleMap(corpusKey);
            ilmMap.put(corpusKey, ilm);
        }
        return ilm;
    }


    private class BdfServerIncludeLibelleMap implements IncludeUILibelleMap {

        SubsetKey corpusKey;

        BdfServerIncludeLibelleMap(SubsetKey corpusKey) {
            this.corpusKey = corpusKey;
        }

        public String getLibelleString(IncludeKey includeKey, int langInteger) {
            Corpus corpus = (Corpus) bdfServer.getBasedefiches().getSubset(corpusKey);
            CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
            if (corpusUI == null) {
                return defaultInclude.getLibelleString(includeKey, langInteger);
            }
            IncludeUI includeUI = (IncludeUI) corpusUI.getIncludeUI(includeKey);
            if (includeUI == null) {
                return defaultInclude.getLibelleString(includeKey, langInteger);
            } else {
                return getLibelleString(includeUI, langInteger);
            }
        }

        public String getLibelleString(IncludeUI includeUI, int langInteger) {
            LibelleHolder libelleHolder = includeUI.getCustomLibelle();
            if (libelleHolder == null) {
                return defaultInclude.getLibelleString(includeUI.getIncludeKey(), langInteger);
            } else {
                String lib = libelleHolder.getLibelleString(langInteger, false);
                if (lib == null) {
                    return defaultInclude.getLibelleString(includeUI.getIncludeKey(), langInteger);
                }
                return lib;
            }
        }

    }


    private class InternalFormatSourceLibelleProvider implements FormatSourceLibelleProvider {

        public LibelleHolder getLibelleHolder(Subset subset, FormatSourceKey formatSourceKey) {
            LibelleHolder libelleHolder = innerGetLibelleHolder(subset, formatSourceKey);
            if (libelleHolder == null) {
                return LibelleUtils.toEmptyLibelleHolder(formatSourceKey.toString());
            } else {
                return libelleHolder;
            }
        }

        private LibelleHolder innerGetLibelleHolder(Subset subset, FormatSourceKey formatSourceKey) {
            SubsetKey slaveCorpusKey = formatSourceKey.getSlaveCorpusKey();
            if (slaveCorpusKey != null) {
                Corpus corpus = (Corpus) bdfServer.getBasedefiches().getSubset(slaveCorpusKey);
                if (corpus == null) {
                    return null;
                } else {
                    return getCorpusLibelleHolder(corpus, formatSourceKey);
                }
            }
            if (subset instanceof Corpus) {
                return getCorpusLibelleHolder((Corpus) subset, formatSourceKey);
            } else {
                return null;
            }
        }

        private LibelleHolder getCorpusLibelleHolder(Corpus corpus, FormatSourceKey formatSourceKey) {
            if (formatSourceKey.isFieldKey()) {
                CorpusField corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey(formatSourceKey.getFieldKey());
                if (corpusField == null) {
                    return null;
                } else {
                    return corpusField;
                }
            } else if (formatSourceKey.isIncludeKey()) {
                IncludeKey includeKey = formatSourceKey.getIncludeKey();
                CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
                if (corpusUI == null) {
                    return null;
                }
                IncludeUI includeUI = (IncludeUI) corpusUI.getIncludeUI(includeKey);
                LibelleHolder libelleHolder = null;
                if (includeUI != null) {
                    libelleHolder = includeUI.getCustomLibelle();
                }
                if (libelleHolder == null) {
                    libelleHolder = getDefaultLibelleHolder(includeKey);
                }
                return libelleHolder;
            } else {
                return null;
            }
        }

        private LibelleHolder getDefaultLibelleHolder(IncludeKey includeKey) {
            SubsetKey subsetKey = includeKey.getSubsetKey();
            if (subsetKey != null) {
                if (subsetKey.isThesaurusSubset()) {
                    Thesaurus thesaurus = (Thesaurus) bdfServer.getBasedefiches().getSubset(subsetKey);
                    if (thesaurus != null) {
                        return thesaurus.getThesaurusMetadata().getIntituleByName(ThesaurusConstants.THESAURUS);
                    }
                } else if (subsetKey.isCorpusSubset()) {
                    Corpus corpus = (Corpus) bdfServer.getBasedefiches().getSubset(subsetKey);
                    if (corpus != null) {
                        return corpus.getCorpusMetadata().getIntituleByName(CorpusConstants.CORPUS);
                    }
                }
            }
            return null;
        }

    }

}
