/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.actions.BdfAction;
import fr.exemole.bdfserver.actions.CreateFiche;
import fr.exemole.bdfserver.actions.FicheAction;
import fr.exemole.bdfserver.actions.ReadFiche;
import fr.exemole.bdfserver.roles.CorpusPermission;
import fr.exemole.bdfserver.roles.FicheActionPermission;
import fr.exemole.bdfserver.roles.Role;
import fr.exemole.bdfserver.roles.RoleMap;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class PermissionManager {

    private Set<RedacteurKey> adminSet = new HashSet<RedacteurKey>();
    private BdfServer bdfServer;
    private RoleMap roleMap;

    public PermissionManager(BdfServer bdfServer, RoleMap roleMap) {
        this.bdfServer = bdfServer;
        this.roleMap = roleMap;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public Redacteur getRedacteur(RedacteurKey redacteurKey) {
        return SphereUtils.getRedacteur(bdfServer.getBasedefiches(), redacteurKey);
    }

    public boolean isRemoveable(Redacteur redacteur) {
        if (!bdfServer.getBasedefiches().isRemoveable(redacteur)) {
            return false;
        }
        if ((isAdmin(redacteur.getRedacteurKey()))) {
            if (getAdminCount() == 1) {
                return false;
            }
        }
        return true;
    }

    public boolean isAdmin(RedacteurKey redacteurKey) {
        return adminSet.contains(redacteurKey);
    }

    public boolean setAdmin(Redacteur redacteur, boolean isAdmin) {
        RedacteurKey redacteurKey = redacteur.getRedacteurKey();
        if (isAdmin) {
            if (adminSet.contains(redacteurKey)) {
                return false;
            } else {
                adminSet.add(redacteurKey);
                if (!redacteur.isActif()) {
                    bdfServer.getBasedefichesEditor().getSphereEditor(redacteurKey.getSphereKey()).setActif(redacteur, true);
                }
                return true;
            }
        } else {
            if (adminSet.contains(redacteurKey)) {
                if (getAdminCount() == 1) {
                    return false;
                }
                adminSet.remove(redacteurKey);
                return true;
            } else {
                return false;
            }
        }
    }

    public int getAdminCount() {
        return adminSet.size();
    }

    public List<RedacteurKey> getAdminList() {
        List<RedacteurKey> arrayList = new ArrayList<RedacteurKey>(adminSet);
        return arrayList;
    }

    public Role getDefaultRole() {
        return getRole(RoleMap.DEFAULT_ROLE);
    }

    public Role getRole(String name) {
        return roleMap.getRole(name);
    }

    public Role[] getRoleArray(boolean defaultRoleInclude) {
        return roleMap.getRoleArray(defaultRoleInclude);
    }

    public boolean isDefaultRole(Role role) {
        return (role.getName().equals(RoleMap.DEFAULT_ROLE));
    }

    public Role createRole(String roleName) {
        return roleMap.newRole(roleName);
    }

    public void saveRole(Role role) {
        roleMap.checkRoleChange(role.getName());
    }

    public void removeRole(Role role) {
        roleMap.removeRole(role.getName());
    }

    public boolean canDo(BdfUser bdfUser, BdfAction bdfAction) {
        return can(bdfUser.getRedacteurKey(), bdfUser.getRoleNameArray(), bdfAction, false);
    }

    public boolean cannotDo(BdfUser bdfUser, BdfAction bdfAction) {
        boolean test = can(bdfUser.getRedacteurKey(), bdfUser.getRoleNameArray(), bdfAction, true);
        return !test;
    }

    public boolean canDo(RedacteurKey redacteurKey, String[] roleNameArray, BdfAction bdfAction) {
        return can(redacteurKey, roleNameArray, bdfAction, false);
    }

    public boolean cannotDo(RedacteurKey redacteurKey, String[] roleNameArray, BdfAction bdfAction) {
        boolean test = can(redacteurKey, roleNameArray, bdfAction, true);
        return !test;
    }

    private boolean can(RedacteurKey redacteurKey, String[] roleNameArray, BdfAction bdfAction, boolean cannot) {
        if (isAdmin(redacteurKey)) {
            if (bdfAction instanceof CreateFiche) {
                CreateFiche createFicheAction = (CreateFiche) bdfAction;
                Corpus corpus = createFicheAction.getCorpus();
                int idcorpus = createFicheAction.getIdcorpus();
                if (idcorpus == -1) {
                    return true;
                }
                if (corpus.getAbregeFicheByIdcorpus(idcorpus) != null) {
                    return false;
                }
                Subset masterSubset = corpus.getMasterSubset();
                if (masterSubset == null) {
                    return true;
                }
                SubsetItem masterSubsetItem = masterSubset.getSubsetItemById(idcorpus);
                if (masterSubsetItem == null) {
                    return false;
                }
            }
            return true;
        }
        boolean testDefaultRole = false;
        if (roleNameArray != null) {
            int length = roleNameArray.length;
            for (int i = 0; i < length; i++) {
                String roleName = roleNameArray[i];
                if (roleName.equals(RoleMap.DEFAULT_ROLE)) {
                    testDefaultRole = true;
                } else {
                    Role role = roleMap.getRole(roleName);
                    if (role != null) {
                        boolean roleTest = testRole(role, redacteurKey, bdfAction, cannot);
                        if (roleTest) {
                            return true;
                        }
                    }
                }
            }
        } else {
            testDefaultRole = true;
        }
        if (testDefaultRole) {
            boolean roleTest = testRole(getDefaultRole(), redacteurKey, bdfAction, cannot);
            if (roleTest) {
                return true;
            }
        }
        return false;
    }

    private CorpusPermission getCorpusPermission(Role role, SubsetKey corpusKey) {
        CorpusPermission corpusPermission = role.getCorpusPermission(corpusKey);
        if (corpusPermission == null) {
            if (isDefaultRole(role)) {
                return CorpusPermission.newDefaultInstance();
            } else {
                return null;
            }
        }
        return corpusPermission;
    }

    private boolean testRole(Role role, RedacteurKey redacteurKey, BdfAction bdfAction, boolean cannot) {
        if (bdfAction instanceof CreateFiche) {
            CreateFiche createFicheAction = (CreateFiche) bdfAction;
            Corpus corpus = createFicheAction.getCorpus();
            CorpusPermission corpusPermission = getCorpusPermission(role, corpus.getSubsetKey());
            if (corpusPermission == null) {
                return false;
            }
            boolean canCreate = corpusPermission.canCreateFiche();
            if (!canCreate) {
                return false;
            }
            int idcorpus = createFicheAction.getIdcorpus();
            if (idcorpus == -1) {
                return true;
            }
            if (corpus.getAbregeFicheByIdcorpus(idcorpus) != null) {
                return false;
            }
            Subset masterSubset = corpus.getMasterSubset();
            if (masterSubset == null) {
                return true;
            }
            SubsetItem masterSubsetItem = masterSubset.getSubsetItemById(idcorpus);
            if (masterSubsetItem == null) {
                return false;
            }
            if (masterSubsetItem instanceof AbregeFiche) {
                FicheAction masterFicheAction = FicheAction.getWriteFiche((AbregeFiche) masterSubsetItem);
                return testRole(role, redacteurKey, masterFicheAction, cannot);
            } else {
                return true;
            }
        }
        if (bdfAction instanceof FicheAction) {
            FicheAction ficheAction = (FicheAction) bdfAction;
            CorpusPermission corpusPermission = getCorpusPermission(role, ficheAction.getCorpusKey());
            if (corpusPermission == null) {
                return false;
            }
            FicheActionPermission ficheActionPermission = corpusPermission.getFicheActionPermission(ficheAction);
            if (ficheAction.isAllFicheAction()) {
                short type = ficheActionPermission.getType();
                if (cannot) {
                    if (type == FicheActionPermission.NONE) {
                        return false;
                    } else {
                        return true;
                    }
                } else {
                    if (type == FicheActionPermission.ALL) {
                        return true;
                    } else {
                        return false;
                    }
                }
            } else {
                return resolveFicheActionPermission(ficheActionPermission, redacteurKey, ficheAction.getAbregeFiche());
            }
        }
        throw new SwitchException("bdfAction instanceof " + bdfAction.getClass().getName());
    }

    private boolean resolveFicheActionPermission(FicheActionPermission ficheActionPermission, RedacteurKey redacteurKey, AbregeFiche abregeFiche) {
        short type = ficheActionPermission.getType();
        switch (type) {
            case FicheActionPermission.ALL:
                return true;
            case FicheActionPermission.NONE:
                return false;
            case FicheActionPermission.OWN:
                return abregeFiche.hasRedacteur(redacteurKey);
            case FicheActionPermission.SPHERE:
                return abregeFiche.hasRedacteurInSphere(redacteurKey.getSphereKey());
            case FicheActionPermission.SELECTION:
            default:
                throw new SwitchException("ficheActionPermission.getType() = " + ficheActionPermission.getType());
        }
    }

    public FicheEligibility getFicheEligibility(BdfUser bdfUser) {
        return new BdfUserFicheEligibility(bdfUser);
    }

    public Role[] getRoleArray(BdfUser bdfUser) {
        String[] roleNameArray = bdfUser.getRoleNameArray();
        if (roleNameArray == null) {
            Role[] roleArray = new Role[1];
            roleArray[0] = getDefaultRole();
            return roleArray;
        } else {
            int length = roleNameArray.length;
            Role[] tempArray = new Role[length];
            int p = 0;
            boolean withDefault = false;
            for (int i = 0; i < length; i++) {
                Role role = getRole(roleNameArray[i]);
                if (role != null) {
                    if (isDefaultRole(role)) {
                        withDefault = true;
                    } else {
                        tempArray[p] = role;
                        p++;
                    }
                }
            }
            if (withDefault) {
                tempArray[p] = getDefaultRole();
                p++;
            }
            if (p < length) {
                Role[] roleArray = new Role[p];
                System.arraycopy(tempArray, 0, roleArray, 0, p);
                return roleArray;
            } else {
                return tempArray;
            }
        }
    }


    private class BdfUserFicheEligibility implements FicheEligibility {

        RedacteurKey redacteurKey;
        boolean isAdmin;
        Role[] roleArray;

        BdfUserFicheEligibility(BdfUser bdfUser) {
            this.redacteurKey = bdfUser.getRedacteurKey();
            isAdmin = isAdmin(redacteurKey);
            roleArray = getRoleArray(bdfUser);
        }

        public boolean accept(AbregeFiche abregeFiche) {
            if (isAdmin) {
                return true;
            }
            ReadFiche bdfAction = FicheAction.getReadFiche(abregeFiche);
            int length = roleArray.length;
            for (int i = 0; i < length; i++) {
                boolean roleTest = testRole(roleArray[i], redacteurKey, bdfAction, false);
                if (roleTest) {
                    return true;
                }
            }
            return false;
        }

    }

}
