/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.ui.AlterUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.SubsetGroupConstants;
import fr.exemole.bdfserver.ui.SubsetKeyGroupTemplates;
import fr.exemole.bdfserver.ui.SubsetKeyTree;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUIUtils;
import java.util.LinkedHashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class UIManager {

    private Map<SubsetKey, CorpusUI> corpusUIMap = new LinkedHashMap<SubsetKey, CorpusUI>();
    private BdfServer bdfServer;
    private SubsetKeyGroupTemplates subsetKeyGroupTemplates = new SubsetKeyGroupTemplates();
    private SubsetKeyTree corpusSubsetKeyTree = new SubsetKeyTree(subsetKeyGroupTemplates, SubsetGroupConstants.CORPUS_GROUP);
    private SubsetKeyTree thesaurusSubsetKeyTree = new SubsetKeyTree(subsetKeyGroupTemplates, SubsetGroupConstants.THESAURUS_GROUP);
    private SubsetKeyTree sphereSubsetKeyTree = new SubsetKeyTree(subsetKeyGroupTemplates, SubsetGroupConstants.SPHERE_GROUP);
    private SubsetKeyTree albumSubsetKeyTree = new SubsetKeyTree(subsetKeyGroupTemplates, SubsetGroupConstants.ALBUM_GROUP);

    private UIManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public SubsetKeyTree getCorpusSubsetKeyTree() {
        return corpusSubsetKeyTree;
    }

    public SubsetKeyTree getThesaurusSubsetKeyTree() {
        return thesaurusSubsetKeyTree;
    }

    public SubsetKeyTree getSphereSubsetKeyTree() {
        return sphereSubsetKeyTree;
    }

    public SubsetKeyTree getAlbumSubsetKeyTree() {
        return albumSubsetKeyTree;
    }

    public SubsetKeyTree getSubsetKeyTree(short type) {
        switch (type) {
            case SubsetGroupConstants.CORPUS_GROUP:
                return getCorpusSubsetKeyTree();
            case SubsetGroupConstants.THESAURUS_GROUP:
                return getThesaurusSubsetKeyTree();
            case SubsetGroupConstants.SPHERE_GROUP:
                return getSphereSubsetKeyTree();
            case SubsetGroupConstants.ALBUM_GROUP:
                return getAlbumSubsetKeyTree();
            default:
                throw new SwitchException("wrong type = " + type);
        }
    }

    public SubsetKeyGroupTemplates getSubsetKeyGroupTemplates() {
        return subsetKeyGroupTemplates;
    }

    public CorpusUI getCorpusUI(Corpus corpus) {
        return (CorpusUI) corpusUIMap.get(corpus.getSubsetKey());
    }

    public void corpusCreated(Corpus corpus) {
        CorpusUI corpusUI = new CorpusUI(corpus);
        corpusUIMap.put(corpus.getSubsetKey(), corpusUI);
        checkMasterSubsetUI(corpus.getMasterSubset());
        corpusSubsetKeyTree.addSubsetKey(corpus.getSubsetKey(), null);
    }

    public void corpusRemoved(SubsetKey corpusKey, Subset masterSubset) {
        corpusUIMap.remove(corpusKey);
        corpusSubsetKeyTree.removeSubsetKey(corpusKey);
        for (CorpusUI corpusUI : corpusUIMap.values()) {
            corpusUI.removeAllIncludeUI(corpusKey);
        }
        checkMasterSubsetUI(masterSubset);
    }

    public void sphereCreated(Sphere sphere) {
        sphereSubsetKeyTree.addSubsetKey(sphere.getSubsetKey(), null);
    }

    public void sphereRemoved(SubsetKey sphereKey) {
        sphereSubsetKeyTree.removeSubsetKey(sphereKey);
    }

    public void albumCreated(Album album) {
        albumSubsetKeyTree.addSubsetKey(album.getSubsetKey(), null);
    }

    public void albumRemoved(SubsetKey albumKey) {
        albumSubsetKeyTree.removeSubsetKey(albumKey);
    }

    public void thesaurusCreated(Thesaurus thesaurus) {
        thesaurusSubsetKeyTree.addSubsetKey(thesaurus.getSubsetKey(), null);
    }

    public void thesaurusRemoved(SubsetKey thesaurusKey) {
        thesaurusSubsetKeyTree.removeSubsetKey(thesaurusKey);
        for (CorpusUI corpusUI : corpusUIMap.values()) {
            corpusUI.removeAllIncludeUI(thesaurusKey);
        }
    }

    public void corpusFieldCreated(Corpus corpus, CorpusField corpusField) {
        CorpusUI corpusUI = internalGetCorpusUI(corpus);
        corpusUI.checkCorpus();
    }

    public void corpusFieldRemoved(Corpus corpus, FieldKey fieldKey) {
        CorpusUI corpusUI = internalGetCorpusUI(corpus);
        corpusUI.removeAllFieldUI(fieldKey);
    }

    public ComponentUIList getComponentUIList(Corpus corpus, String uiName) {
        CorpusUI corpusUI = getCorpusUI(corpus);
        AlterUI alterUI = null;
        if (uiName != null) {
            alterUI = bdfServer.getBdfStorage().getAlterUI(corpusUI, uiName);
        }
        if (alterUI == null) {
            return corpusUI;
        } else {
            switch (alterUI.getType()) {
                case AlterUI.COMPONENTUI_FILTER:
                    return CorpusUIUtils.filter(corpusUI, alterUI.getComponentUIFilter());
                case AlterUI.COMPONENTUI_LISTE:
                    return alterUI.getComponentUIList();
                default:
                    throw new SwitchException("Unknown type : " + alterUI.getType());
            }
        }

    }

    private CorpusUI internalGetCorpusUI(Corpus corpus) {
        return corpusUIMap.get(corpus.getSubsetKey());
    }

    private void checkMasterSubsetUI(Subset masterSubset) {
        if (masterSubset == null) {
            return;
        }
        if (masterSubset instanceof Corpus) {
            CorpusUI masterCorpusUI = getCorpusUI((Corpus) masterSubset);
            masterCorpusUI.checkRattachement();
        }
    }

    /**
     * À appeler impérativement à la fin d'une initialisation afin de garantir la cohérence par rapport à la base de fiches
     */
    private void endInit() {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        corpusSubsetKeyTree.checkBasedefiches(basedefiches);
        thesaurusSubsetKeyTree.checkBasedefiches(basedefiches);
        sphereSubsetKeyTree.checkBasedefiches(basedefiches);
        albumSubsetKeyTree.checkBasedefiches(basedefiches);
    }

    public static UIManager newInstance(BdfServer bdfServer, BdfStorage bdfStorage) {
        UIManager uiManager = new UIManager(bdfServer);
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            CorpusUI corpusUI = new CorpusUI(corpus);
            uiManager.corpusUIMap.put(corpus.getSubsetKey(), corpusUI);
        }
        bdfStorage.loadManager(uiManager);
        uiManager.endInit();
        return uiManager;
    }

}
