/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.tableexport;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.storage.TableExportStorage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.ColDefTestResult;
import net.mapeadores.basedefiches.tableexport.SubsetTableExport;
import net.mapeadores.basedefiches.tableexport.SubsetTableExportBuilder;
import net.mapeadores.basedefiches.tableexport.SubsetTableExportDef;
import net.mapeadores.basedefiches.tableexport.croisement.CroisementTableExport;
import net.mapeadores.basedefiches.tableexport.croisement.CroisementTableExportInfo;
import net.mapeadores.basedefiches.tableexport.logging.TableExportLog;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.logging.ProcessLog;


/**
 *
 * @author Vincent Calame
 */
public class TableExportManager {

    private BdfServer bdfServer;
    private TableExportStorage tableExportStorage;
    private TreeMap<String, InternalTableExportInfo> tableExportMap = new TreeMap<String, InternalTableExportInfo>();
    private Set<String> unmodifiableSet = Collections.unmodifiableSet(tableExportMap.keySet());
    private TreeMap<String, TableExportLog> wrongMap = new TreeMap<String, TableExportLog>();
    private Set<String> wrongUnmodifiableSet = Collections.unmodifiableSet(wrongMap.keySet());

    public TableExportManager(BdfServer bdfServer, TableExportStorage tableExportStorage) {
        this.bdfServer = bdfServer;
        this.tableExportStorage = tableExportStorage;
        update();
    }

    public void update() {
        tableExportMap.clear();
        wrongMap.clear();
        List<TableExportStorage.TableExportStorageInfo> currentList = tableExportStorage.checkStorage(bdfServer);
        for (TableExportStorage.TableExportStorageInfo info : currentList) {
            String name = info.getName();
            int count = info.getSubsetCount();
            if (count > 0) {
                InternalTableExportInfo internalTableExportInfo = new InternalTableExportInfo(info.getName(), info.getProcessLog());
                for (int i = 0; i < count; i++) {
                    SubsetKey subsetKey = info.getSubsetKey(i);
                    ColDefList colDefList = info.getColDefList(i);
                    internalTableExportInfo.initSubset(subsetKey, colDefList);
                }
                int croisementCount = info.getCroisementInfoCount();
                for (int i = 0; i < croisementCount; i++) {
                    CroisementTableExportInfo croisementInfo = info.getCroisementInfo(i);
                    internalTableExportInfo.initCroisementInfo(croisementInfo);
                }
                tableExportMap.put(name, internalTableExportInfo);
            } else {
                wrongMap.put(name, info.getProcessLog());
            }

        }
    }

    public void saveColDefList(String tableExportName, SubsetKey corpusKey, ColDefList colDefList) throws IOException {
        tableExportStorage.saveSubsetColDefList(tableExportName, corpusKey, colDefList);
    }

    public Set<String> getTableExportNameSet() {
        return unmodifiableSet;
    }

    public Set<String> getWrongNameSet() {
        return wrongUnmodifiableSet;
    }

    public ProcessLog getProcessLog(String tableExportName) {
        InternalTableExportInfo internalTableExportInfo = (InternalTableExportInfo) tableExportMap.get(tableExportName);
        if (internalTableExportInfo == null) {
            return (TableExportLog) wrongMap.get(tableExportName);
        }
        return internalTableExportInfo.getProcessLog();
    }

    public TableExport getTableExport(String tableExportName) {
        return (InternalTableExportInfo) tableExportMap.get(tableExportName);
    }

    public TableExport getDefaultTableExport(TableExportParametersBean tableExportParametersBean) {
        return new DefaultTableExport(tableExportParametersBean);
    }


    private class DefaultTableExport implements TableExport {

        TableExportParametersBean tableExportParametersBean;

        DefaultTableExport(TableExportParametersBean tableExportParametersBean) {
            this.tableExportParametersBean = tableExportParametersBean;
        }

        public SubsetTableExport[] getSubsetTableExportArray(LocalisationInfo localisationInfo) {
            Basedefiches basedefiches = bdfServer.getBasedefiches();
            int corpusCount = basedefiches.getCorpusCount();
            SubsetTableExport[] array = new SubsetTableExport[corpusCount];
            for (int j = 0; j < corpusCount; j++) {
                Corpus corpus = basedefiches.getCorpus(j);
                ColDefList coldDefList = TableExportUtils.getColDefListFromCorpusUI(bdfServer, corpus, tableExportParametersBean);
                SubsetTableExportBuilder corpusTableExportBuilder = new SubsetTableExportBuilder(corpus, localisationInfo);
                corpusTableExportBuilder.addColDefList(coldDefList);
                array[j] = corpusTableExportBuilder.toSubsetTableExport();
            }
            return array;
        }

        public CroisementTableExport[] getCroisementTableExportArray() {
            CroisementTableExport[] array = new CroisementTableExport[0];
            return array;
        }

        public SubsetKey[] getSubsetKeyArray() {
            Basedefiches basedefiches = bdfServer.getBasedefiches();
            int corpusCount = basedefiches.getCorpusCount();
            SubsetKey[] result = new SubsetKey[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                result[i] = basedefiches.getCorpus(i).getSubsetKey();
            }
            return result;
        }

    }


    private class InternalTableExportInfo implements TableExport {

        String name;
        TableExportLog tableExportLog;
        List<SubsetTableExportDef> subsetTableExportDefList = new ArrayList<SubsetTableExportDef>();
        List croisementBundleList = new ArrayList();

        InternalTableExportInfo(String name, TableExportLog tableExportLog) {
            this.name = name;
            this.tableExportLog = tableExportLog;
        }

        ProcessLog getProcessLog() {
            return tableExportLog;
        }

        String getName() {
            return name;
        }

        public CroisementTableExport[] getCroisementTableExportArray() {
            int size = croisementBundleList.size();
            CroisementTableExport[] array = new CroisementTableExport[size];
            for (int i = 0; i < size; i++) {
                CroisementBundle croisementBundle = (CroisementBundle) croisementBundleList.get(i);
                Croisement croisement = croisementBundle.getCroisement();
                CroisementTableExport tableExport = new CroisementTableExport(croisement, croisementBundle.inverse);
                tableExport.setTableName(croisementBundle.tableName);
                array[i] = tableExport;
            }
            return array;
        }

        void initCroisementInfo(CroisementTableExportInfo croisementInfo) {
            CroisementBundle croisementBundle = new CroisementBundle(croisementInfo);
            if (croisementBundle.testCroisement()) {
                croisementBundleList.add(croisementBundle);
            }
        }

        void initSubset(SubsetKey subsetKey, ColDefList colDefList) {
            Subset subset = bdfServer.getBasedefiches().getSubset(subsetKey);
            if (subset == null) {
                return;
            }
            TestColDefList testColDefList = new TestColDefList(colDefList);
            SubsetTableExportBuilder corpusTableExportBuilder = new SubsetTableExportBuilder(subset, null);
            int count = colDefList.getColDefCount();
            for (int i = 0; i < count; i++) {
                ColDef colDef = colDefList.getColDef(i);
                if (colDef.getType() == ColDef.FICHEFORMAT_TYPE) {
                    ColDefTestResult addResult = corpusTableExportBuilder.addColDef(colDef);
                    if (addResult.isError()) {
                        tableExportLog.addMetadataErrorLogItem(subset.getSubsetKey(), addResult.getErrorKey(), addResult.getErrorValue());
                    } else {
                        testColDefList.addColDef(colDef);
                    }
                } else {
                    testColDefList.addColDef(colDef);
                }
            }
            if (testColDefList.getColDefCount() == 0) {
                return;
            }
            SubsetTableExportDef subsetTableExportDef = new SubsetTableExportDef(subsetKey, testColDefList);
            subsetTableExportDefList.add(subsetTableExportDef);
        }

        public SubsetKey[] getSubsetKeyArray() {
            int size = subsetTableExportDefList.size();
            SubsetKey[] result = new SubsetKey[size];
            for (int i = 0; i < size; i++) {
                result[i] = subsetTableExportDefList.get(i).getSubsetKey();
            }
            return result;
        }

        public SubsetTableExport[] getSubsetTableExportArray(LocalisationInfo localisationInfo) {
            List<SubsetTableExport> result = new ArrayList<SubsetTableExport>();
            int size = subsetTableExportDefList.size();
            for (int i = 0; i < size; i++) {
                SubsetTableExportDef subsetTableExportDef = subsetTableExportDefList.get(i);
                Subset subset = bdfServer.getBasedefiches().getSubset(subsetTableExportDef.getSubsetKey());
                if (subset == null) {
                    continue;
                }
                SubsetTableExportBuilder subsetTableExportBuilder = new SubsetTableExportBuilder(subset, localisationInfo);
                subsetTableExportBuilder.addColDefList(subsetTableExportDef.getColDefList());
                result.add(subsetTableExportBuilder.toSubsetTableExport());
            }
            return result.toArray(new SubsetTableExport[result.size()]);
        }


        class CroisementBundle {

            String tableName;
            SubsetCouple subsetCouple;
            boolean inverse;

            CroisementBundle(CroisementTableExportInfo croisementInfo) {
                this.tableName = croisementInfo.getTableName();
                SubsetKey subsetKey1 = croisementInfo.getSubsetKey1();
                SubsetKey subsetKey2 = croisementInfo.getSubsetKey2();
                subsetCouple = new SubsetCouple(subsetKey1, subsetKey2);
                inverse = subsetCouple.getSubsetKey1().equals(subsetKey2);
            }

            Croisement getCroisement() {
                return bdfServer.getBasedefiches().getCroisement(subsetCouple.getSubsetKey1(), subsetCouple.getSubsetKey2());
            }

            boolean testCroisement() {
                return (getCroisement() != null);
            }

        }

    }


    private static class TestColDefList implements ColDefList {

        private ColDefList colDefList;
        private List<ColDef> list = new ArrayList<ColDef>();

        TestColDefList(ColDefList colDefList) {
            this.colDefList = colDefList;
        }

        void addColDef(ColDef colDef) {
            list.add(colDef);
        }

        public int getColDefCount() {
            return list.size();
        }

        public ColDef getColDef(int i) {
            return list.get(i);
        }

        public String getParameterValue(String parameterName) {
            return colDefList.getParameterValue(parameterName);
        }

    }

}
