/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.tableexport;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatDefBuilder;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.include.IncludeSummaryBuilder;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.ColDefUtils;
import net.mapeadores.basedefiches.tableexport.TableExportCommon;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.format.FormatConstants;


/**
 *
 * @author Vincent Calame
 */
public class TableExportUtils {

    private TableExportUtils() {
    }

    public static ColDefList getColDefListFromCorpusUI(BdfServer bdfServer, Corpus corpus, TableExportParametersBean tableExportParametersBean) {
        ColDefBuilder builder = new ColDefBuilder(bdfServer, corpus, tableExportParametersBean);
        builder.init();
        return ColDefUtils.fromArray(builder.toColDefArray());
    }


    private static class ColDefBuilder {

        private BdfServer bdfServer;
        private Corpus corpus;
        private CorpusMetadata corpusMetadata;
        private TableExportParametersBean tableExportParametersBean;
        private List<ColDef> colDefList = new ArrayList<ColDef>();
        private boolean noIndexation;
        private IncludeSummaryBuilder includeSummaryBuilder;

        private ColDefBuilder(BdfServer bdfServer, Corpus corpus, TableExportParametersBean tableExportParametersBean) {
            this.bdfServer = bdfServer;
            this.corpus = corpus;
            this.corpusMetadata = corpus.getCorpusMetadata();
            this.tableExportParametersBean = tableExportParametersBean;
            short indexationIncludeType = tableExportParametersBean.getIndexationIncludeType();
            noIndexation = (indexationIncludeType == TableExportCommon.NO_INDEXATION);
            if (indexationIncludeType == TableExportCommon.ALL_INDEXATION) {
                includeSummaryBuilder = new IncludeSummaryBuilder();
            }
        }

        private ColDef[] toColDefArray() {
            int size = colDefList.size();
            return colDefList.toArray(new ColDef[size]);
        }

        private void init() {
            colDefList.add(getFieldKeyColDef(FieldKey.IDCORPUS));
            if (tableExportParametersBean.isWithChrono()) {
                colDefList.add(getChronoColDef(IncludeKey.DATE_CREATION));
                colDefList.add(getChronoColDef(IncludeKey.DATE_MODIFICATION));
            }
            ComponentUIList componentUIList = bdfServer.getUIManager().getCorpusUI(corpus);
            int componentCount = componentUIList.getComponentUICount();
            for (int i = 0; i < componentCount; i++) {
                ComponentUI obj = componentUIList.getComponentUI(i);
                if (obj instanceof FieldUI) {
                    FieldUI fieldUI = (FieldUI) obj;
                    CorpusField corpusField = fieldUI.getCorpusField();
                    if ((!tableExportParametersBean.isWithTextes()) && (corpusField.getFieldKey().ownsToCorpsdefiche())) {
                        continue;
                    }
                    colDefList.add(getFieldKeyColDef(corpusField));
                } else if (obj instanceof IncludeUI) {
                    IncludeUI includeUI = (IncludeUI) obj;
                    IncludeKey includeKey = includeUI.getIncludeKey();
                    if (testInclude(includeKey)) {
                        colDefList.add(getIncludeKeyColDef(includeKey));
                    }
                }
            }
            if (includeSummaryBuilder != null) {
                IncludeSummary includeSummary = includeSummaryBuilder.getIncludeSummary();
                Basedefiches basedefiches = bdfServer.getBasedefiches();
                int thesaurusCount = basedefiches.getThesaurusCount();
                for (int i = 0; i < thesaurusCount; i++) {
                    SubsetKey thesaurusKey = basedefiches.getThesaurus(i).getSubsetKey();
                    IncludeSummary.SubsetInfo info = includeSummary.getSubsetInfo(thesaurusKey);
                    if ((info == null) || (!info.hasAnyPoidsInclude())) {
                        colDefList.add(getIncludeKeyColDef(IncludeKey.newInstance(thesaurusKey)));
                    }
                }
            }
        }

        private boolean testInclude(IncludeKey includeKey) {
            if (includeKey.isSpecialInclude()) {
                if (includeKey.equals(IncludeKey.LIAGE)) {
                    return tableExportParametersBean.isWithLiage();
                } else if (includeKey.equals(IncludeKey.ASSOCIATION)) {
                    return tableExportParametersBean.isWithAssociation();
                } else if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
                    return false;
                } else {
                    throw new SwitchException("unkonwn special key : " + includeKey.toString());
                }
            } else if (includeKey.isThesaurusInclude()) {
                if (noIndexation) {
                    return false;
                }
                if (includeSummaryBuilder != null) {
                    includeSummaryBuilder.append(includeKey);
                }
            } else if (includeKey.isCorpusInclude()) {
                if (!tableExportParametersBean.isWithLiage()) {
                    return false;
                }
            }
            return true;
        }

        private ColDef getFieldKeyColDef(FieldKey fieldKey) {
            return getFieldKeyColDef(corpusMetadata.getCorpusFieldByFieldKey(fieldKey));
        }

        private ColDef getFieldKeyColDef(CorpusField corpusField) {
            FieldKey fieldKey = corpusField.getFieldKey();
            BasedefichesFormatDefBuilder basedefichesFormatDefBuilder = new BasedefichesFormatDefBuilder();
            basedefichesFormatDefBuilder.addSource(FormatSourceKey.newFieldKeyInstance(fieldKey));
            if (fieldKey.equals(FieldKey.IDCORPUS)) {
                basedefichesFormatDefBuilder.setCastType(FormatConstants.INTEGER_CAST);
            }
            if (fieldKey.ownsToCorpsdefiche()) {
                basedefichesFormatDefBuilder.setSeparator("\n");
            }
            int ficheItemType = corpusField.getFicheItemType();
            switch (ficheItemType) {
                case CorpusField.MONTANT_FIELD:
                case CorpusField.NOMBRE_FIELD:
                    basedefichesFormatDefBuilder.addFormatPattern("{lib}");
                    break;
            }
            ColDef colDef = ColDefUtils.newBasedefichesFormatDefInstance(fieldKey.toString(), null, basedefichesFormatDefBuilder.toBasedefichesFormatDef());
            return colDef;
        }

        private ColDef getIncludeKeyColDef(IncludeKey includeKey) {
            BasedefichesFormatDefBuilder basedefichesFormatDefBuilder = new BasedefichesFormatDefBuilder();
            basedefichesFormatDefBuilder.addSource(FormatSourceKey.newIncludeKeyInstance(includeKey));
            ColDef colDef = ColDefUtils.newBasedefichesFormatDefInstance(includeKey.toString(), null, basedefichesFormatDefBuilder.toBasedefichesFormatDef());
            return colDef;
        }

        private ColDef getChronoColDef(IncludeKey includeKey) {
            BasedefichesFormatDefBuilder basedefichesFormatDefBuilder = new BasedefichesFormatDefBuilder();
            basedefichesFormatDefBuilder.addSource(FormatSourceKey.newIncludeKeyInstance(includeKey));
            basedefichesFormatDefBuilder.setCastType(FormatConstants.DATE_CAST);
            basedefichesFormatDefBuilder.addFormatPattern("{iso}");
            ColDef colDef = ColDefUtils.newBasedefichesFormatDefInstance(includeKey.toString(), null, basedefichesFormatDefBuilder.toBasedefichesFormatDef());
            return colDef;
        }

    }

}
