/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.transformation;

import java.io.Serializable;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public final class StreamTransformerKey implements Serializable {

    private TransformationKey transformationKey;
    private String transformerName;
    private String extension;
    private int hashCode;

    /**
     *
     *@throws IllegalArgumentException si transformationKey ou extension sont nul
     */
    public StreamTransformerKey(TransformationKey transformationKey, String extension, String transformerName) {
        if (transformationKey == null) {
            throw new IllegalArgumentException("transformationKey is null");
        }
        if (extension == null) {
            throw new IllegalArgumentException("extension is null");
        }
        if (extension.length() == 0) {
            throw new IllegalArgumentException("extension is empty");
        }
        if (extension.indexOf('/') != -1) {
            throw new IllegalArgumentException("extension contains a '/'");
        }
        this.transformationKey = transformationKey;
        this.transformerName = transformerName;
        this.extension = extension;
        initHashCode();
    }

    private void initHashCode() {
        hashCode = extension.hashCode() + transformationKey.hashCode();
        if (transformerName != null) {
            hashCode = hashCode + transformerName.hashCode();
        }
    }

    public String getTransformerName() {
        return transformerName;
    }

    public TransformationKey getTransformationKey() {
        return transformationKey;
    }

    public String getExtension() {
        return extension;
    }

    public int hashCode() {
        return hashCode;
    }

    public static StreamTransformerKey getDefaultStreamTransformerKey(TransformationKey transformationKey, String extension) {
        return new StreamTransformerKey(transformationKey, extension, null);
    }

    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj.hashCode() != this.hashCode) {
            return false;
        }
        if (!(obj instanceof StreamTransformerKey)) {
            return false;
        }
        StreamTransformerKey other = (StreamTransformerKey) obj;
        if (!other.extension.equals(this.extension)) {
            return false;
        }
        if (!other.transformationKey.equals(this.transformationKey)) {
            return false;
        }
        if (this.transformerName == null) {
            return (other.transformerName == null);
        }
        if (other.transformerName == null) {
            return false;
        }
        return (other.transformerName.equals(this.transformerName));
    }

    public String toString() {
        if (transformerName == null) {
            return transformationKey.toString() + "/" + extension + "/";
        }
        return transformationKey.toString() + "/" + extension + "/" + transformerName;
    }

    public boolean isDefaultTransformerKey() {
        return (transformerName == null);
    }

    public StreamTransformerKey toDefaultTransformerKey() {
        if (transformerName == null) {
            return this;
        }
        return new StreamTransformerKey(transformationKey, extension, null);
    }

    public static StreamTransformerKey parse(String s) throws ParseException {
        if (s == null) {
            throw new ParseException("s is null", 0);
        }
        int idx1 = s.indexOf('/');
        if (idx1 == -1) {
            throw new ParseException("no '/' character", s.length() - 1);
        }
        int idx2 = s.indexOf('/', idx1 + 1);
        if (idx2 == -1) {
            throw new ParseException("no second '/' character", s.length() - 1);
        }
        TransformationKey transformationKey = TransformationKey.parse(s.substring(0, idx1));
        String extension = s.substring(idx1 + 1, idx2);
        if (extension.length() == 0) {
            throw new ParseException("extension is empty", idx1);
        }
        String transformerName = s.substring(idx2 + 1);
        if (transformerName.length() == 0) {
            transformerName = null;
        }
        return new StreamTransformerKey(transformationKey, extension, transformerName);
    }

    public static String toStringKey(TransformationKey transformationKey, String extension) {
        return transformationKey.toString() + "/" + extension;
    }

    public static String toStringKey(StreamTransformerKey streamTransformerKey) {
        return toStringKey(streamTransformerKey.getTransformationKey(), streamTransformerKey.getExtension());
    }

}
