/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.transformation;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.storage.TransformationStorage;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUIUtils;
import fr.exemole.bdfserver.xsltfactory.DefaultXSLTFactory;
import fr.exemole.bdfserver.xsltfactory.XSLTFactoryConstants;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.MotcleFilter;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.builders.BuilderUtils;
import net.mapeadores.basedefiches.extraction.builders.ExtractionDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtraitCorpusDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.FicheFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.IntituleClauseBuilder;
import net.mapeadores.basedefiches.extraction.builders.MotcleFilterBuilder;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitList;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.io.TransformerErrorListener;


/**
 *
 * @author Vincent Calame
 */
public class TransformationManager {

    private static Map<Object, String[]> availableMap = new HashMap<Object, String[]>();

    static {
        String[] array = {"odt"};
        availableMap.put(TransformationKey.COMPILATION_TRANSFORMATIONKEY, array);
        availableMap.put("corpus", array);
    }

    private BdfServer bdfServer;
    private TransformationStorage transformationStorage;
    private Map<TransformationKey, TransformationInfo> transformationInfoMap = new HashMap<TransformationKey, TransformationInfo>();
    private Map<Object, Object> transformerMap = new HashMap<Object, Object>();
    private Map<TransformationKey, ExtractionDef> extractionDefMap = new HashMap<TransformationKey, ExtractionDef>();

    public TransformationManager(BdfServer bdfServer, TransformationStorage transformationStorage) {
        this.bdfServer = bdfServer;
        this.transformationStorage = transformationStorage;
    }

    public void clearDefaultTransformer() {
        for (Iterator it = transformerMap.keySet().iterator(); it.hasNext();) {
            Object obj = it.next();
            if (obj instanceof XslTransformerKey) {
                XslTransformerKey key = (XslTransformerKey) obj;
                if (key.isDefaultTransformerKey()) {
                    it.remove();
                }
            } else if (obj instanceof StreamTransformerKey) {
                StreamTransformerKey key = (StreamTransformerKey) obj;
                if (key.isDefaultTransformerKey()) {
                    it.remove();
                }
            }
        }
        extractionDefMap.clear();
    }

    public void update() {
        transformerMap.clear();
        transformationInfoMap.clear();
        extractionDefMap.clear();
        List<TransformationStorage.TransformationStorageInfo> list = transformationStorage.getTransformationStorageInfoList(bdfServer);
        for (TransformationStorage.TransformationStorageInfo info : list) {
            update(info);
        }
    }

    public void saveTransformer(Object transformerKey, InputStream inputStream, boolean pattern) throws IOException {
        transformationStorage.saveTransformer(transformerKey, inputStream, pattern);
        update();
    }

    public File getEtcCommonsFile(String relativePath) {
        File etcDir = transformationStorage.getEtcCommonsDirectory();
        return new File(etcDir, relativePath);
    }

    public File getVarCommonsFile(String relativePath) {
        File varDir = transformationStorage.getVarCommonsDirectory();
        return new File(varDir, relativePath);
    }

    public Source getEtcCommonsSource(String relativePath) throws TransformerException {
        File etcDir = transformationStorage.getEtcCommonsDirectory();
        File f = new File(etcDir, relativePath);
        if (f.exists() && (!f.isDirectory())) {
            return new StreamSource(f);
        } else {
            throw new TransformerException(relativePath + " (No such file in Etc Commons )");
        }
    }

    public Source getVarCommonsSource(String relativePath) throws TransformerException {
        File varDir = transformationStorage.getVarCommonsDirectory();
        File f = new File(varDir, relativePath);
        if (f.exists() && (!f.isDirectory())) {
            return new StreamSource(f);
        } else {
            throw new TransformerException(relativePath + " (No such file in Var Commons )");
        }
    }

    private void update(TransformationStorage.TransformationStorageInfo info) {
        TransformerFactory factory = bdfServer.newTransformationFactory(null);
        TransformationKey transformationKey = info.getTransformationKey();
        TransformationInfo.Builder builder = TransformationInfo.getBuilder(transformationKey);
        checkDefaultStreamTransformerAvailable(builder);
        for (XslTransformerInfo xslTransformerInfo : info.getXslTransformerInfoList()) {
            if (xslTransformerInfo.isValid()) {
                Source source = transformationStorage.getTransformerSource(xslTransformerInfo.getXslTransformerKey());
                if (source != null) {
                    TransformerErrorListener errorListener = new TransformerErrorListener();
                    factory.setErrorListener(errorListener);
                    try {
                        Templates templates = factory.newTemplates(source);
                        transformerMap.put(xslTransformerInfo.getXslTransformerKey(), templates);
                    } catch (TransformerConfigurationException tce) {
                        errorListener.checkFatalTransformerException(tce);
                    }
                    if (errorListener.hasError()) {
                        xslTransformerInfo.setError(XslTransformerInfo.XSL_ERROR, errorListener.getErrorMessage());
                    }
                }
            }
            builder.addXslTransformerInfo(xslTransformerInfo);
        }
        for (StreamTransformerInfo streamTransformerInfo : info.getStreamTransformerInfoList()) {
            if (streamTransformerInfo.isValid()) {
                StreamTransformer streamTransformer = transformationStorage.getStreamTransformer(bdfServer, streamTransformerInfo);
                if (streamTransformer != null) {
                    transformerMap.put(streamTransformerInfo.getStreamTransformerKey(), streamTransformer);
                }
            }
            builder.addStreamTransformerInfo(streamTransformerInfo);
        }
        transformationInfoMap.put(transformationKey, builder.toTransformationInfo());
    }

    public TransformationInfo getTransformationInfo(TransformationKey transformationKey) {
        TransformationInfo transformationInfo = transformationInfoMap.get(transformationKey);
        if (transformationInfo == null) {
            transformationInfo = TransformationInfo.getEmptyInstance(transformationKey, getDefaultStreamTransformerAvailableArray(transformationKey));
            transformationInfoMap.put(transformationKey, transformationInfo);
        }
        return transformationInfo;
    }

    public ExtractionDef getExtractionDef(StreamTransformerKey streamTransformerKey) {
        return getDefaultExtractionDef(streamTransformerKey.getTransformationKey());
    }

    public StreamTransformer getStreamTransformer(StreamTransformerKey streamTransformerKey) {
        StreamTransformer streamTransformer = (StreamTransformer) transformerMap.get(streamTransformerKey);
        if (streamTransformer != null) {
            return streamTransformer;
        }
        if (!streamTransformerKey.isDefaultTransformerKey()) {
            streamTransformerKey = streamTransformerKey.toDefaultTransformerKey();
            StreamTransformer defaultStreamTransformer = (StreamTransformer) transformerMap.get(streamTransformerKey);
            if (defaultStreamTransformer != null) {
                return defaultStreamTransformer;
            }
        }
        if (isDefaultStreamTransformerAvailable(streamTransformerKey)) {
            streamTransformer = createDefaultStreamTransformer(streamTransformerKey.getTransformationKey(), streamTransformerKey.getExtension());
            transformerMap.put(streamTransformerKey, streamTransformer);
            return streamTransformer;
        }
        TransformationInfo transformationInfo = getTransformationInfo(streamTransformerKey.getTransformationKey());
        int index = transformationInfo.indexOfExtension(streamTransformerKey.getExtension());
        if (index == -1) {
            return null;
        } else {
            streamTransformerKey = transformationInfo.getExtensionList(index).getStreamTransformerKey(0);
            return (StreamTransformer) transformerMap.get(streamTransformerKey);
        }
    }

    public ExtractionDef getExtractionDef(XslTransformerKey xslTransformerKey) {
        return getDefaultExtractionDef(xslTransformerKey.getTransformationKey());
    }

    public Transformer getXslTransformer(XslTransformerKey xslTransformerKey) {
        Templates xslTemplate = (Templates) transformerMap.get(xslTransformerKey);
        if (xslTemplate != null) {
            return bdfServer.newTransformer(xslTemplate);
        }
        if (!xslTransformerKey.isDefaultTransformerKey()) {
            return getXslTransformer(xslTransformerKey.toDefaultTransformerKey());
        }
        xslTemplate = createDefaultXslTransformer(xslTransformerKey.getTransformationKey());
        return bdfServer.newTransformer(xslTemplate);
    }

    private ExtractionDef getDefaultExtractionDef(TransformationKey transformationKey) {
        ExtractionDef defaultExtractionDef = extractionDefMap.get(transformationKey);
        if (defaultExtractionDef != null) {
            return defaultExtractionDef;
        }
        if (transformationKey.isCorpusTransformationKey()) {
            defaultExtractionDef = getFicheExtractionDef(transformationKey.toCorpusKey());
        } else if (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY)) {
            defaultExtractionDef = getCompilationExtractionDef();
        } else {
            ExtractionDefBuilder extractionDefBuilder = new ExtractionDefBuilder();
            if (transformationKey.equals(TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY)) {
                MotcleFilter thesaurusFilter = MotcleFilterBuilder.getThesaurusInstance();
                extractionDefBuilder.setDynamicExtraitThesaurusDef(BuilderUtils.getExtraitThesaurusDef(thesaurusFilter));
                defaultExtractionDef = extractionDefBuilder.toExtractionDef();
            } else if (transformationKey.equals(TransformationKey.MOTCLE_TRANSFORMATIONKEY)) {
                MotcleFilter aloneFilter = MotcleFilterBuilder.getAloneInstance();
                extractionDefBuilder.setDynamicExtraitThesaurusDef(BuilderUtils.getExtraitThesaurusDef(aloneFilter));
                defaultExtractionDef = extractionDefBuilder.toExtractionDef();
            }
        }
        extractionDefMap.put(transformationKey, defaultExtractionDef);
        return defaultExtractionDef;
    }

    private ExtractionDef getCompilationExtractionDef() {
        ExtractionDefBuilder extractionDefBuilder = new ExtractionDefBuilder();
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        UIManager uiManager = bdfServer.getUIManager();
        int corpusCount = basedefiches.getCorpusCount();
        List<ExtraitCorpusDef> list = new ArrayList<ExtraitCorpusDef>();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            CorpusUI corpusUI = uiManager.getCorpusUI(corpus);
            FilterUnitList filterUnitList = CorpusUIUtils.toFilterUnitList(corpus, corpusUI, true, true);
            FicheFilter ficheFilter = FicheFilterBuilder.fromFilterUnitList(filterUnitList);
            ExtraitCorpusDefBuilder builder = new ExtraitCorpusDefBuilder();
            builder.setFicheFilter(ficheFilter);
            builder.setIntituleClause(IntituleClauseBuilder.DEFAULT);
            FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
            SubsetKey[] corpusKeyArray = new SubsetKey[1];
            corpusKeyArray[0] = corpus.getSubsetKey();
            ficheSelectOptions.setCorpusKeyArray(corpusKeyArray);
            builder.setFicheSelectOptionsList(ficheSelectOptions);
            list.add(builder.toExtraitCorpusDef());
        }
        extractionDefBuilder.setStaticExtraitCorpusDefList(BuilderUtils.toExtraitCorpusDefList(list));
        return extractionDefBuilder.toExtractionDef();
    }

    private ExtractionDef getFicheExtractionDef(SubsetKey corpusKey) {
        Corpus corpus = (Corpus) bdfServer.getBasedefiches().getSubset(corpusKey);
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        ExtractionDefBuilder extractionDefBuilder = new ExtractionDefBuilder();
        FilterUnitList filterUnitList = CorpusUIUtils.toFilterUnitList(corpus, corpusUI, true, true);
        FicheFilter ficheFilter = FicheFilterBuilder.fromFilterUnitList(filterUnitList);
        extractionDefBuilder.setDynamicExtraitCorpusDef(BuilderUtils.getExtraitCorpusDef(ficheFilter, null));
        return extractionDefBuilder.toExtractionDef();
    }

    private Templates createDefaultXslTransformer(TransformationKey transformationKey) {
        String xslt = DefaultXSLTFactory.getDefaultStyleSheet(bdfServer, transformationKey, XSLTFactoryConstants.XSL_FORMAT);
        if (xslt == null) {
            throw new IllegalArgumentException("Corpus defined in sourceKey is not valid (corpus = " + transformationKey.toCorpusKey().toString() + ")");
        }
        Source source = new StreamSource(new StringReader(xslt));
        try {
            TransformerFactory factory = bdfServer.newTransformationFactory(null);
            Templates xslTemplate = factory.newTemplates(source);
            transformerMap.put(XslTransformerKey.getDefaultXslTransformerKey(transformationKey), xslTemplate);
            return xslTemplate;
        } catch (TransformerConfigurationException tce) {
            throw new NestedTransformerException(tce);
        }
    }

    private StreamTransformer createDefaultStreamTransformer(TransformationKey transformationKey, String extension) {
        if (extension.equals("odt")) {
            return DefaultXSLTFactory.getDefaultStreamTransformer(bdfServer, transformationKey, XSLTFactoryConstants.ODT_FORMAT);
        } else {
            throw new SwitchException("extension = " + extension);
        }
    }

    private static boolean isDefaultStreamTransformerAvailable(StreamTransformerKey streamTransformerKey) {
        String[] array = getDefaultStreamTransformerAvailableArray(streamTransformerKey.getTransformationKey());
        if (array == null) {
            return false;
        }
        String extension = streamTransformerKey.getExtension();
        for (int i = 0; i < array.length; i++) {
            if (array[i].equals(extension)) {
                return true;
            }
        }
        return false;
    }

    private static void checkDefaultStreamTransformerAvailable(TransformationInfo.Builder builder) {
        String[] array = getDefaultStreamTransformerAvailableArray(builder.getTransformationKey());
        if (array == null) {
            return;
        }
        for (int i = 0; i < array.length; i++) {
            builder.addDefaultTransformerAvailable(array[i]);
        }
    }

    private static String[] getDefaultStreamTransformerAvailableArray(TransformationKey transformationKey) {
        if (transformationKey.isCorpusTransformationKey()) {
            return availableMap.get("corpus");
        }
        return availableMap.get(transformationKey);
    }

}
