/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import fr.exemole.bdfserver.l10n.LocaleBundle;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.request.TypoOptions;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.util.Locale;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.corpus.fiche.Courriel;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.fiche.Image;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Link;
import net.mapeadores.basedefiches.corpus.fiche.Montant;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.Nombre;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.Pays;
import net.mapeadores.basedefiches.corpus.fiche.RedacteurItem;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.CurrencyIntegerException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.primitives.DegreSexagesimal;
import net.mapeadores.util.primitives.IntegerList;
import net.mapeadores.util.text.DateFormatBundle;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FicheParser {

    private boolean fieldAppend;
    private Basedefiches basedefiches;
    private Fiche fiche;
    private RedacteurItemParser redacteurItemParser;
    private FicheBlockParser ficheBlockParser;
    private DateFormatBundle dateFormatBundle;
    private Locale locale;
    private TextParser textParser;
    private ParagraphParser paragraphParser;
    private SubsetKey defaultSphereKey;

    public FicheParser(Basedefiches basedefiches, Fiche fiche, SubsetKey defaultSphereKey, LocaleBundle localeBundle, TypoOptions typoOptions) {
        this.fiche = fiche;
        this.basedefiches = basedefiches;
        this.dateFormatBundle = localeBundle.getDateFormatBundle();
        this.locale = localeBundle.getLocale();
        this.defaultSphereKey = defaultSphereKey;
        textParser = new TextParser();
        textParser.setQuoteOptions(typoOptions.getQuoteOptions());
        redacteurItemParser = new RedacteurItemParser(basedefiches, textParser);
        paragraphParser = new ParagraphParser(textParser);
        ficheBlockParser = new FicheBlockParser(typoOptions);
    }

    public void setFieldAppend(boolean fieldAppend) {
        this.fieldAppend = fieldAppend;
    }

    public void parseLang(String s) {
        if (s == null) {
            return;
        }
        Langue langue = Langue.parseLangue(s, locale);
        if (langue != null) {
            fiche.setLangInteger(langue.getLangInteger());
        } else {
            fiche.setLangInteger(LangInteger.UNDETERMINED);
        }
    }

    public void parseTitre(String s) {
        s = StringUtils.cleanString(s);
        fiche.setTitre(textParser.parseString(s));
    }

    public void parseSoustitre(String s) {
        s = StringUtils.cleanString(s);
        if (s.length() > 0) {
            fiche.setSoustitre(parsePara(s));
        }
    }

    public void parseRedacteurs(String[] tokens) {
        MutableFicheItemList ficheItemList = new MutableFicheItemList();
        redacteurItemParser.parseRedacteurItemList(tokens, ficheItemList, defaultSphereKey);
        if (fieldAppend) {
            fiche.appendRedacteurs(ficheItemList);
        } else {
            fiche.setRedacteurs(ficheItemList);
        }
    }

    public void parsePropriete(CorpusField proprieteField, String s) {
        s = StringUtils.cleanString(s);
        FicheItem ficheItem = null;
        if (s.length() > 0) {
            ficheItem = parseFicheItem(s, proprieteField);
        }
        fiche.setPropriete(proprieteField.getFieldKey(), ficheItem);
    }

    public void parsePersonnePropriete(CorpusField proprieteField, String nom, String prenom, String original, String nomAvant) {
        nom = StringUtils.cleanString(nom);
        prenom = StringUtils.cleanString(prenom);
        original = StringUtils.cleanString(original);
        boolean nomAvantBool = false;
        if (nomAvant.equals("1")) {
            nomAvantBool = true;
        }
        RedacteurItem redacteurItem = redacteurItemParser.parseRedacteurItem(nom, prenom, original, nomAvantBool);
        fiche.setPropriete(proprieteField.getFieldKey(), redacteurItem);
    }

    public void parseMontantPropriete(CorpusField proprieteField, String value, String cur) {
        FicheItem ficheItem = null;
        value = StringUtils.cleanString(value);
        if (value.length() > 0) {
            cur = StringUtils.cleanString(cur);
            boolean itemAllowed = true;
            if (cur.length() == 0) {
                IntegerList currencyArray = proprieteField.getFieldOptions().getCurrencyList();
                cur = CurrencyInteger.toString(currencyArray.getInteger(0));
            }
            ficheItem = parseMontant(value, cur);
            if (ficheItem == null) {
                if (itemAllowed) {
                    if (!value.startsWith("?")) {
                        value = "?" + value;
                    }
                }
                ficheItem = parseItem(value, itemAllowed);
            }
        }
        fiche.setPropriete(proprieteField.getFieldKey(), ficheItem);
    }

    public void parseImagePropriete(CorpusField proprieteField, String src, String alt, String title) {
        Image image = null;
        src = StringUtils.cleanString(src);
        if (src.length() > 0) {
            alt = StringUtils.cleanString(alt);
            title = StringUtils.cleanString(title);
            image = new Image(src, alt, title);
        }
        fiche.setPropriete(proprieteField.getFieldKey(), image);
    }

    public void parseGeopointPropriete(CorpusField proprieteField, String latitude, String longitude) {
        latitude = StringUtils.cleanString(latitude);
        longitude = StringUtils.cleanString(longitude);
        DegreDecimal latitudeDecimal = parseDegreDecimal(latitude);
        DegreDecimal longitudeDecimal = parseDegreDecimal(longitude);
        FicheItem ficheItem = null;
        boolean itemAllowed = true;
        if ((latitudeDecimal != null) && (longitudeDecimal != null)) {
            ficheItem = new Geopoint(latitudeDecimal, longitudeDecimal);
        } else {
            if ((latitude.length() > 0) || (longitude.length() > 0)) {
                if (itemAllowed) {
                    if (!latitude.startsWith("?? ")) {
                        latitude = "?? " + latitude;
                    }
                }
                ficheItem = parseItem(latitude + " ?? " + longitude, itemAllowed);
            }
        }
        fiche.setPropriete(proprieteField.getFieldKey(), ficheItem);
    }

    public void parseInformation(CorpusField informationField, String[] tokens) {
        MutableFicheItemList ficheItemList = new MutableFicheItemList();
        if (tokens.length > 0) {
            for (int i = 0; i < tokens.length; i++) {
                FicheItem ficheItem = parseFicheItem(tokens[i], informationField);
                if (ficheItem != null) {
                    ficheItemList.addFicheItem(ficheItem);
                }
            }
        }
        FieldKey fieldKey = informationField.getFieldKey();
        if (fieldAppend) {
            fiche.appendInformation(fieldKey, ficheItemList);
        } else {
            fiche.setInformation(fieldKey, ficheItemList);
        }
    }

    public void parseSection(CorpusField sectionField, String s) {
        MutableFicheBlockList ficheBlockList = new MutableFicheBlockList();
        ficheBlockParser.parseFicheBlockList(s, ficheBlockList);
        FieldKey fieldKey = sectionField.getFieldKey();
        if (fieldAppend) {
            fiche.appendSection(fieldKey, ficheBlockList);
        } else {
            fiche.setSection(fieldKey, ficheBlockList);
        }
    }

    private FicheItem parseFicheItem(String token, CorpusField corpusField) {
        int ficheItemType = corpusField.getFicheItemType();
        boolean itemAllowed = true;
        if (ficheItemType == CorpusField.LANGUE_FIELD) {
            Langue langue = Langue.parseLangue(token, locale);
            if (langue != null) {
                return langue;
            } else {
                return parseItem(token, itemAllowed);
            }
        }
        if (ficheItemType == CorpusField.PAYS_FIELD) {
            Pays pays = Pays.parsePays(token, locale);
            if (pays != null) {
                return pays;
            } else {
                return parseItem(token, itemAllowed);
            }
        }
        if (ficheItemType == CorpusField.DATATION_FIELD) {
            try {
                int dateInteger = DateInteger.parse(token, dateFormatBundle);
                return new Datation(dateInteger);
            } catch (ParseException pe) {
                return parseItem(token, itemAllowed);
            }
        }
        if (ficheItemType == CorpusField.LINK_FIELD) {
            Link link = parseLink(token);
            if (link != null) {
                return link;
            } else {
                return parseItem(token, itemAllowed);
            }
        }
        if (ficheItemType == CorpusField.NOMBRE_FIELD) {
            try {
                Decimal decimal = StringUtils.parseDecimal(token);
                return new Nombre(decimal);
            } catch (NumberFormatException nfe) {
                if (itemAllowed) {
                    if (!token.startsWith("?")) {
                        token = "?" + token;
                    }
                }
                return parseItem(token, itemAllowed);
            }
        }
        if (ficheItemType == CorpusField.MONTANT_FIELD) {
            Montant montant = parseMontant(token);
            if (montant != null) {
                return montant;
            }
            if (itemAllowed) {
                if (!token.startsWith("?")) {
                    token = "?" + token;
                }
            }
            return parseItem(token, itemAllowed);
        }
        if (ficheItemType == CorpusField.COURRIEL_FIELD) {
            try {
                CourrielBean courrielBean = CourrielBean.parse(token);
                return new Courriel(courrielBean);
            } catch (ParseException pe) {
                return parseItem(token, itemAllowed);
            }
        }
        if (ficheItemType == CorpusField.PERSONNE_FIELD) {
            return redacteurItemParser.parseRedacteurItem(token, getDefaultSphereKey(corpusField));
        }
        if (ficheItemType == CorpusField.ITEM_FIELD) {
            return new Item(textParser.parseString(token));
        }
        if (ficheItemType == CorpusField.GEOPOINT_FIELD) {
            Geopoint geopoint = parseGeopoint(token);
            if (geopoint != null) {
                return geopoint;
            } else {
                return parseItem(token, itemAllowed);
            }
        }
        if (ficheItemType == CorpusField.PARA_FIELD) {
            return parsePara(token);
        }
        if (ficheItemType == CorpusField.IMAGE_FIELD) {
            Image image = parseImage(token);
            if (image != null) {
                return image;
            } else {
                return parseItem(token, itemAllowed);
            }
        }
        return null;
    }

    private Para parsePara(String token) {
        Para.Builder paraBuilder = new Para.Builder();
        paragraphParser.parseParagraph(paraBuilder, token);
        return paraBuilder.toPara();
    }

    private Item parseItem(String token, boolean itemAllowed) {
        if (itemAllowed) {
            return new Item(textParser.parseString(token));
        } else {
            return null;
        }
    }

    public Link parseLink(String s) {
        int separateurinterne = '*';
        int posseparateur = s.indexOf(separateurinterne);
        if (posseparateur == -1) {
            separateurinterne = '|';
            posseparateur = s.indexOf(separateurinterne);
        }
        String href = null;
        if (posseparateur == - 1) {
            href = s;
        } else {
            href = s.substring(0, posseparateur).trim();
        }
        if (href.startsWith("www.")) {
            href = "http://" + href;
        }
        try {
            URL url = new URL(href);
            href = url.toString();
            String title = "";
            String comment = "";
            if (posseparateur > -1) {
                String bout = s.substring(posseparateur + 1).trim();
                int pos2 = bout.indexOf(separateurinterne);
                if (pos2 > -1) {
                    title = bout.substring(0, pos2).trim();
                    comment = bout.substring(pos2 + 1).trim();
                } else {
                    title = bout;
                }
            }
            return new Link(href, textParser.parseString(title), textParser.parseString(comment));
        } catch (MalformedURLException mue) {
            return null;
        }
    }

    public Image parseImage(String s) {
        int separateurinterne = '*';
        int posseparateur = s.indexOf(separateurinterne);
        String src = null;
        String alt = "";
        String title = "";
        if (posseparateur == - 1) {
            src = s;
        } else {
            src = s.substring(0, posseparateur).trim();
            int pos2 = s.indexOf(separateurinterne, posseparateur + 1);
            if (pos2 == -1) {
                alt = s.substring(posseparateur + 1).trim();
            } else {
                alt = s.substring(posseparateur + 1, pos2).trim();
                title = s.substring(pos2 + 1).trim();
            }
        }
        return new Image(src, textParser.parseString(alt), textParser.parseString(title));
    }

    private Geopoint parseGeopoint(String s) {
        DegreDecimal latitude = null;
        DegreDecimal longitude = null;
        int idxAster = s.indexOf('*');
        if (idxAster != -1) {
            latitude = parseDegreDecimal(s.substring(0, idxAster));
            longitude = parseDegreDecimal(s.substring(idxAster + 1));
        } else {
        }
        if ((latitude != null) && (longitude != null)) {
            return new Geopoint(latitude, longitude);
        } else {
            return null;
        }
    }

    private DegreDecimal parseDegreDecimal(String s) {
        try {
            Decimal decimal = StringUtils.parseDecimal(s);
            return new DegreDecimal(decimal);
        } catch (NumberFormatException nfe) {
            try {
                DegreSexagesimal ds = DegreSexagesimal.parse(s);
                return new DegreDecimal(ds);
            } catch (ParseException pe) {
                return null;
            }
        }
    }

    private Montant parseMontant(String s) {
        char firstChar = s.charAt(0);
        if (isDigitOrMinus(firstChar)) {
            return parseCurAtEnd(s);
        } else {
            return parceCurAtFirst(s);
        }
    }

    private Montant parseCurAtEnd(String s) {
        int index = -1;
        for (int i = (s.length() - 1); i >= 0; i--) {
            char c = s.charAt(i);
            if (isDigit(c)) {
                index = i;
                break;
            }
        }
        if ((index == -1) || (index == (s.length() - 1))) {
            return null;
        }
        String curString = s.substring(index + 1).trim();
        String valString = s.substring(0, index + 1).trim();
        return parseMontant(valString, curString);
    }

    private Montant parceCurAtFirst(String s) {
        int index = -1;
        for (int i = 1; i < s.length(); i++) {
            char c = s.charAt(i);
            if (isDigitOrMinus(c)) {
                index = i;
                break;
            }
        }
        if (index == -1) {
            return null;
        }
        String curString = s.substring(0, index).trim();
        String valString = s.substring(index);
        return parseMontant(valString, curString);
    }

    private Montant parseMontant(String valString, String curString) {
        try {
            int currencyInteger = CurrencyInteger.parse(curString);
            Decimal decimal = StringUtils.parseDecimal(valString);
            return new Montant(decimal, currencyInteger);
        } catch (CurrencyIntegerException cie) {
            return null;
        } catch (NumberFormatException nfe) {
            return null;
        }
    }

    private boolean isDigitOrMinus(char carac) {
        if (carac == '-') {
            return true;
        }
        return isDigit(carac);
    }

    private boolean isDigit(char carac) {
        if (carac > '9') {
            return false;
        }
        if (carac < '0') {
            return false;
        }
        return true;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    private SubsetKey getDefaultSphereKey(CorpusField corpusField) {
        SubsetKey customSphereKey = corpusField.getFieldOptions().getDefaultSphereKey();
        if (customSphereKey == null) {
            customSphereKey = defaultSphereKey;
        }
        return customSphereKey;
    }

}
