/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.users.BdfUser;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionUtils;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FicheSelectParser {

    public final static String IDCORPUS_PARAMNAME = "identifiants";
    public final static String CORPUS_ALL_PARAMNAME = "corpus_all";
    public final static String FIELDCONDITION_PARAMNAME = "fieldcondition";
    public final static String FIELDCONDITION_SCOPE_PARAMNAME = "fieldcondition_scope";
    public final static String FIELDCONDITION_FIELDSELECTION_PARAMNAME = "fieldcondition_fieldselection";
    public final static String REDACTEURS_ALL_PARAMNAME = "redacteurs_all";
    public final static String REDACTEURS_SELECTION_PARAMNAME = "redacteurs_selection";
    public final static String CORPUS_SELECTION_PARAMNAME = "corpus_selection";
    public final static String MOTCLE_ALL_PREFIX = "motcle_all_";
    public final static String MOTCLE_MODE_PREFIX = "motcle_mode_";
    public final static String MOTCLE_INPUT_PREFIX = "motcle_input_";
    public final static String MOTCLE_SELECTION_PREFIX = "motcle_selection_";
    public final static String MOTCLE_MODE_PARAMNAME = "motcle_mode";
    private RequestMap requestMap;
    private BdfServer bdfServer;
    private BdfUser bdfUser;
    private FicheSelectOptions options;
    private TextParser textParser;

    private FicheSelectParser(RequestMap requestMap, BdfServer bdfServer, BdfUser bdfUser) {
        this.bdfServer = bdfServer;
        this.bdfUser = bdfUser;
        this.requestMap = requestMap;
        options = new FicheSelectOptions();
        textParser = new TextParser();
        textParser.setQuoteOptions(bdfUser.getTypoOptions().getQuoteOptions());
        parseIdcorpus();
        parseCorpus();
        parseFieldCondition();
        parseRedacteurs();
        parseMotscles();
    }

    public static FicheSelectOptions parse(RequestMap requestMap, BdfServer bdfServer, BdfUser bdfUser) {
        FicheSelectParser parser = new FicheSelectParser(requestMap, bdfServer, bdfUser);
        return parser.options;
    }

    private void parseIdcorpus() {
        String rangeString = requestMap.getParameter(IDCORPUS_PARAMNAME);
        if (rangeString != null) {
            RangeList rangeList = RangeUtils.positiveRangeParse(rangeString);
            options.setRangeList(rangeList);
        }
    }

    private void parseCorpus() {
        String touscorpus = requestMap.getParameter(CORPUS_ALL_PARAMNAME);
        if ((touscorpus == null) || (touscorpus.equals("0"))) {
            String[] selectioncorpus = requestMap.getParameterValues(CORPUS_SELECTION_PARAMNAME);
            if ((selectioncorpus != null) && (selectioncorpus.length > 0)) {
                SubsetKey[] corpusKeyArray = new SubsetKey[selectioncorpus.length];
                for (int i = 0; i < selectioncorpus.length; i++) {
                    try {
                        corpusKeyArray[i] = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, selectioncorpus[i]);
                    } catch (java.text.ParseException pe) {
                    }
                }
                options.setCorpusKeyArray(corpusKeyArray);
            }
        }
    }

    private void parseFieldCondition() {
        Condition fieldCondition = ConditionUtils.parseSimpleCondition(requestMap.getParameter(FIELDCONDITION_PARAMNAME));
        if ((fieldCondition != null) && (!ConditionUtils.isNeutralState(fieldCondition.getState()))) {
            short fieldConditionScope = FicheSelectOptions.TITRE_SCOPE;
            String scopeString = requestMap.getParameter(FIELDCONDITION_SCOPE_PARAMNAME);
            if (scopeString != null) {
                try {
                    fieldConditionScope = FicheSelectOptions.fieldConditionToShort(scopeString);
                } catch (IllegalArgumentException iae) {
                }
            }
            FieldKey[] fieldKeyArray  = null;
            if (fieldConditionScope == FicheSelectOptions.SELECTION_SCOPE) {
                fieldKeyArray = parseFieldKeyArray(requestMap.getParameter(FIELDCONDITION_FIELDSELECTION_PARAMNAME));
                if (fieldKeyArray == null) {
                    fieldConditionScope = FicheSelectOptions.TITRE_SCOPE;
                }
            }
            options.setFieldCondition(fieldCondition, fieldConditionScope, fieldKeyArray);

        }
    }

    private void parseRedacteurs() {
        String tousredacteurs = requestMap.getParameter(REDACTEURS_ALL_PARAMNAME);
        RedacteurKey[] redacteurKeyArray = null;
        SubsetKey[] subsetKeyArray = null;
        if ((tousredacteurs == null) || (tousredacteurs.equals("0"))) {
            String selectionredacteurs = requestMap.getParameter(REDACTEURS_SELECTION_PARAMNAME);
            if (selectionredacteurs != null) {
                SubsetKey defaultSphereKey = bdfUser.getRedacteurKey().getSphereKey();
                String[] tokens = StringUtils.getCleanedTokens(selectionredacteurs, ';');
                Set<SubsetKey> subsetKeySet = new LinkedHashSet<SubsetKey>();
                Set<RedacteurKey> redacteurKeySet = new LinkedHashSet<RedacteurKey>();
                int length = tokens.length;
                for (int i = 0; i < length; i++) {
                    String token = tokens[i];
                    if (token.charAt(0) == '[') {
                        if (token.charAt(token.length() - 1) != ']') {
                            continue;
                        }
                        try {
                            SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, token.substring(1, token.length() - 1));
                            subsetKeySet.add(sphereKey);
                        } catch (java.text.ParseException pe) {
                            continue;
                        }
                    } else {
                        RedacteurKey redacteurKey = RedacteurKey.parseFromUserString(token, defaultSphereKey);
                        if (redacteurKey != null) {
                            redacteurKeySet.add(redacteurKey);
                        }
                    }
                }
                int subsetSize = subsetKeySet.size();
                if (subsetSize > 0) {
                    subsetKeyArray = new SubsetKey[subsetSize];
                    int p = 0;
                    for (SubsetKey subsetKey : subsetKeySet) {
                        subsetKeyArray[p] = subsetKey;
                        p++;
                    }
                }
                int redacteurSize = redacteurKeySet.size();
                if (redacteurSize > 0) {
                    redacteurKeyArray = new RedacteurKey[redacteurSize];
                    int p = 0;
                    for (RedacteurKey redacteurKey : redacteurKeySet) {
                        redacteurKeyArray[p] = redacteurKey;
                        p++;
                    }
                }
            }
        }
        options.setRedacteurKeyArray(redacteurKeyArray);
        options.setSphereKeyArray(subsetKeyArray);
    }

    private void parseMotscles() {
        String motcleMode = requestMap.getParameter(MOTCLE_MODE_PARAMNAME);
        options.setMotcleSelectMode(ConditionUtils.modeToShort(motcleMode, ConditionUtils.MODE_AND));
        SortedSet<Integer> suffixSet = new TreeSet<Integer>();
        for (String parameterName : requestMap.getParameterNameSet()) {
            if (parameterName.startsWith(MOTCLE_INPUT_PREFIX)) {
                int idx = parameterName.lastIndexOf('_');
                if (idx != -1) {
                    try {
                        int number = Integer.parseInt(parameterName.substring(idx + 1));
                        suffixSet.add(new Integer(number));
                    } catch (NumberFormatException nfe) {
                    }
                }
            }
        }
        List<MotcleSelectOptions> optionsList = new ArrayList<MotcleSelectOptions>();
        for (Integer itg : suffixSet) {
            MotcleSelectOptions motcleSelectOptions = parseOptions(itg.toString());
            if (motcleSelectOptions != null) {
                optionsList.add(motcleSelectOptions);
            }
        }
        int size = optionsList.size();
        if (size > 0) {
            MotcleSelectOptions[] array = optionsList.toArray(new MotcleSelectOptions[size]);
            options.setMotcleSelectOptionsArray(array);
        }
    }

    private MotcleSelectOptions parseOptions(String suffix) {
        String chmtcl = requestMap.getParameter(MOTCLE_INPUT_PREFIX + suffix);
        if ((chmtcl == null) || (chmtcl.length() == 0)) {
            return null;
        }
        String modeSelection = requestMap.getParameter(MOTCLE_MODE_PREFIX + suffix);
        short typeMode = ConditionUtils.modeToShort(modeSelection, ConditionUtils.MODE_AND);
        Condition motcleCondition = ConditionUtils.parseCondition(chmtcl, typeMode);
        if (ConditionUtils.isNeutralState(motcleCondition.getState())) {
            return null;
        }
        MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
        motcleSelectOptions.setFieldCondition(motcleCondition);
        motcleSelectOptions.setIdalphaOnly(true);
        String tousthesaurus = requestMap.getParameter(MOTCLE_ALL_PREFIX + suffix);
        if ((tousthesaurus != null) && (tousthesaurus.equals("0"))) {
            String[] selectionthesaurus = requestMap.getParameterValues(MOTCLE_SELECTION_PREFIX + suffix);
            if ((selectionthesaurus != null) && (selectionthesaurus.length > 0)) {
                String thesaurusIdList = StringUtils.implode(selectionthesaurus, ';');
                SubsetKey[] thesaurusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_THESAURUS, thesaurusIdList);
                if (thesaurusKeyArray != null) {
                    motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
                }
            }
        }
        return motcleSelectOptions;
    }

    public static FieldKey[] parseFieldKeyArray(String s) {
        if (s == null) {
            return null;
        }
        String[] tokens = StringUtils.getTechnicalTokens(s);
        int length = tokens.length;
        if (length == 0) {
            return null;
        }
        Set<FieldKey> fieldKeySet = new LinkedHashSet<FieldKey>();
        for (int i = 0; i < length; i++) {
            try {
                FieldKey fieldKey = FieldKey.parse(tokens[i]);
                fieldKeySet.add(fieldKey);
            } catch (ParseException pe) {
            }
        }
        int size = fieldKeySet.size();
        if (size == 0) {
            return null;
        }
        return fieldKeySet.toArray(new FieldKey[size]);
    }

}
