/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.policies.PolicyUtils;
import fr.exemole.bdfserver.managers.policies.ThesaurusPolicy;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.DefaultMotcleCroisementList;
import net.mapeadores.basedefiches.croisement.MotcleCroisementList;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusEditor;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class IndexationParser {

    private Basedefiches basedefiches;
    private int workingLangInteger;
    private CroisementEditor croisementEditor;
    private BdfServer bdfServer;
    private Map<SubsetKey, ThesaurusParser> thesaurusParserMap = new HashMap<SubsetKey, ThesaurusParser>();
    private Map<SubsetKey, MotcleListBundle> currentMotcleListBundleMap = new HashMap<SubsetKey, MotcleListBundle>();

    public IndexationParser(BdfServer bdfServer, int workingLangInteger) {
        this.basedefiches = bdfServer.getBasedefiches();
        this.bdfServer = bdfServer;
        this.croisementEditor = bdfServer.getBasedefichesEditor().getCroisementEditor();
        this.workingLangInteger = workingLangInteger;
    }

    public void parse(String paramName, String[] tokens) {
        IncludeKey includeKey = null;
        try {
            includeKey = IncludeKey.parse(paramName);
        } catch (java.text.ParseException pe) {
            throw new IllegalArgumentException("not a thesaurus Parameter : " + paramName);
        }
        SubsetKey thesaurusKey = includeKey.getSubsetKey();
        if (!thesaurusKey.isThesaurusSubset()) {
            throw new IllegalArgumentException("not a thesaurus Parameter : " + paramName);
        }
        ThesaurusParser thesaurusParser = getThesaurusParser(thesaurusKey);
        thesaurusParser.parse(tokens, includeKey.getPoidsFilter());
    }

    private ThesaurusParser getThesaurusParser(SubsetKey thesaurusKey) {
        ThesaurusParser thesaurusParser = thesaurusParserMap.get(thesaurusKey);
        if (thesaurusParser == null) {
            thesaurusParser = new ThesaurusParser(thesaurusKey);
            thesaurusParserMap.put(thesaurusKey, thesaurusParser);
        }
        return thesaurusParser;
    }

    public void clear() {
        currentMotcleListBundleMap.clear();
        for (ThesaurusParser thesaurusParser : thesaurusParserMap.values()) {
            thesaurusParser.clear();
        }
    }

    public void save(AbregeFiche abregeFiche) {
        for (MotcleListBundle bundle : currentMotcleListBundleMap.values()) {
            croisementEditor.updateCroisement(abregeFiche, bundle.getMotcleList(), bundle.getUpdateType());
        }
    }


    private class ThesaurusParser {

        private SubsetKey thesaurusKey;
        private int thesaurusLangInteger;
        private ThesaurusPolicy thesaurusPolicy;
        private Thesaurus thesaurus;
        private boolean withIdalpha;
        private Thesaurus transferThesaurus;
        private Thesaurus[] verifThesaurusArray;
        private short typeajout;
        private MotcleListBundle currentMotcleListBundle;

        ThesaurusParser(SubsetKey thesaurusKey) {
            this.thesaurusKey = thesaurusKey;
            thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusKey);
            if (thesaurus == null) {
                throw new IllegalArgumentException("unknown thesaurus : " + thesaurusKey.toString());
            }
            thesaurusPolicy = bdfServer.getPolicyManager().getThesaurusPolicy(thesaurusKey);
            withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
            thesaurusLangInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, workingLangInteger);
            typeajout = thesaurusPolicy.getAjoutPolicy();
            if (typeajout == ThesaurusPolicy.AJOUT_TRANSFERT) {
                transferThesaurus = PolicyUtils.getTransferThesaurus(basedefiches, thesaurusPolicy);
            } else if (typeajout == ThesaurusPolicy.AJOUT_VERIFICATION) {
                verifThesaurusArray = PolicyUtils.getVerifThesaurusArray(basedefiches, thesaurusPolicy);
            }
        }

        public void clear() {
            currentMotcleListBundle = null;
        }

        public void parse(String[] tokens, int poidsFilter) {
            currentMotcleListBundle = getMotcleListBundle(thesaurusKey, true);
            for (int i = 0; i < tokens.length; i++) {
                PoidsMotcleToken motcleToken = PoidsMotcleToken.parse(tokens[i], withIdalpha);
                if (motcleToken != null) {
                    if (poidsFilter > 0) {
                        motcleToken.setPoids(poidsFilter);
                    }
                    addMotcle(motcleToken);
                }
            }
        }

        private void addMotcle(PoidsMotcleToken motcleToken) {
            Motcle motcle = PoidsMotcleToken.getMotcle(motcleToken, thesaurus, thesaurusLangInteger);
            if (motcle != null) {
                currentMotcleListBundle.addMotcle(motcle, motcleToken.getPoids());
            } else if ((!withIdalpha) && (!motcleToken.isIdthsBundle())) {
                ajout(motcleToken.getText(), motcleToken.getPoids());
            }
        }

        private void ajout(CleanedString lib, int poids) {
            if (typeajout == ThesaurusPolicy.AJOUT_AUCUN) {
                return;
            }
            if (typeajout == ThesaurusPolicy.AJOUT_AUTORISE) {
                Motcle motcle = createMotcle(thesaurusKey, lib);
                currentMotcleListBundle.addMotcle(motcle, poids);
                return;
            } else if (typeajout == ThesaurusPolicy.AJOUT_TRANSFERT) {
                SubsetKey key2 = transferThesaurus.getSubsetKey();
                MotcleListBundle bundle2 = getMotcleListBundle(key2, false);
                Motcle motcle = getMotcle(transferThesaurus, lib);
                if (motcle == null) {
                    motcle = createMotcle(key2, lib);
                }
                bundle2.addMotcle(motcle, poids);
                return;
            } else if (typeajout == ThesaurusPolicy.AJOUT_VERIFICATION) {
                for (int i = 0; i < verifThesaurusArray.length; i++) {
                    Thesaurus verifThesaurus = verifThesaurusArray[i];
                    Motcle motcle = getMotcle(verifThesaurus, lib);
                    if (motcle != null) {
                        MotcleListBundle bundle2 = getMotcleListBundle(verifThesaurus.getSubsetKey(), false);
                        bundle2.addMotcle(motcle, poids);
                        return;
                    }
                }
                Motcle motcle = createMotcle(thesaurusKey, lib);
                currentMotcleListBundle.addMotcle(motcle, poids);
            }
        }

        private Motcle createMotcle(SubsetKey thskey, CleanedString lib) {
            ThesaurusEditor thsedit = bdfServer.getBasedefichesEditor().getThesaurusEditor(thskey);
            try {
                Motcle motcle = thsedit.createMotcle(-1, null);
                if (lib != null) {
                    thsedit.putLibelle(motcle, LibelleUtils.toLibelle(thesaurusLangInteger, lib));
                }
                return motcle;
            } catch (net.mapeadores.basedefiches.ExistingIdException iiee) {
                return null;
            } catch (java.text.ParseException pe) {
                return null;
            }
        }

        private Motcle getMotcle(Thesaurus ths2, CleanedString lib) {
            if (lib == null) {
                return null;
            }
            int li = ThesaurusUtils.checkLangIntegerDisponibility(ths2, thesaurusLangInteger);
            return ths2.getMotcleByLibelle(lib.toString(), li);
        }

        private MotcleListBundle getMotcleListBundle(SubsetKey thesaurusKey, boolean clearNeeded) {
            MotcleListBundle motcleListBundle = (MotcleListBundle) currentMotcleListBundleMap.get(thesaurusKey);
            if (motcleListBundle == null) {
                motcleListBundle = new MotcleListBundle(thesaurusKey);
                currentMotcleListBundleMap.put(thesaurusKey, motcleListBundle);
            }
            if (clearNeeded) {
                motcleListBundle.setClearNeeded();
            }
            return motcleListBundle;
        }

    }


    private class MotcleListBundle {

        private DefaultMotcleCroisementList motcleList;
        private short updateType = CroisementEditor.APPEND_AND_POIDS_MAX;

        MotcleListBundle(SubsetKey thesaurusKey) {
            motcleList = new DefaultMotcleCroisementList(thesaurusKey, 16);
        }

        public MotcleCroisementList getMotcleList() {
            return motcleList;
        }

        public short getUpdateType() {
            return updateType;
        }

        private void setClearNeeded() {
            updateType = CroisementEditor.CLEAR_EXISTING;
        }

        private void addMotcle(Motcle motcle, int poids) {
            motcleList.add(motcle, poids);
        }

    }

}
