/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.corpus;

import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusUtils;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class CorpusUI implements ComponentUIList {

    private Map<String, ComponentUI> componentUIMap = new HashMap<String, ComponentUI>();
    private Set<SubsetKey> rattachementSet = Collections.emptySet();
    private Corpus corpus;
    private InternalComponentUIEditor internalComponentUIEditor = new InternalComponentUIEditor();
    private List<ComponentUI> componentUIList = new ArrayList<ComponentUI>();

    public CorpusUI(Corpus corpus) {
        this.corpus = corpus;
        checkCorpus();
    }

    public int getComponentUICount() {
        return componentUIList.size();
    }

    public ComponentUI getComponentUI(int i) {
        return componentUIList.get(i);
    }

    /**
     * Vérifie qu'il existe bien une information d'interface pour toutes les méta-données du corpus et
     * les crèe au besoin.
     */
    public void checkCorpus() {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        checkField(corpusMetadata, FieldKey.TITRE);
        checkField(corpusMetadata, FieldKey.SOUSTITRE);
        checkField(corpusMetadata, FieldKey.REDACTEURS);
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            checkCorpusField(corpusMetadata.getProprieteField(i));
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            checkCorpusField(corpusMetadata.getInformationField(i));
        }
        int sectionCount = corpusMetadata.getSectionFieldCount();
        for (int i = 0; i < sectionCount; i++) {
            checkCorpusField(corpusMetadata.getSectionField(i));
        }
        checkField(corpusMetadata, FieldKey.LANG);
        IncludeUI liageUI = (IncludeUI) componentUIMap.get(IncludeKey.LIAGE.toString());
        if (liageUI == null) {
            liageUI = new DefaultIncludeUI(IncludeKey.LIAGE);
            addIncludeUI(liageUI);
        }
        checkRattachement();
    }

    public void checkRattachement() {
        String rattachementKey = IncludeKey.RATTACHEMENT.toString();
        rattachementSet = CorpusUtils.getRattachementSet(corpus);
        IncludeUI rattachementUI = (IncludeUI) componentUIMap.get(rattachementKey);
        if (rattachementSet.size() == 0) {
            if (rattachementUI != null) {
                removeComponentUI(rattachementKey);
            }
            return;
        }
        if (rattachementUI == null) {
            rattachementUI = new DefaultIncludeUI(IncludeKey.RATTACHEMENT);
            addIncludeUI(rattachementUI);
        }
        List<String> keyToRemoveList = new ArrayList<String>();
        for (String key : componentUIMap.keySet()) {
            try {
                IncludeKey includeKey = IncludeKey.parse(key);
                SubsetKey subsetKey = includeKey.getSubsetKey();
                if (rattachementSet.contains(subsetKey)) {
                    keyToRemoveList.add(key);
                }

            } catch (java.text.ParseException pe) {
            }
        }
        for (String key : keyToRemoveList) {
            removeComponentUI(key);
        }
    }

    private void addIncludeUI(IncludeUI includeUI) {
        componentUIMap.put(includeUI.getIncludeKey().toString(), includeUI);
        componentUIList.add(includeUI);
    }

    private void addFieldUI(FieldUI fieldUI) {
        componentUIMap.put(fieldUI.getCorpusField().getFieldKey().toString(), fieldUI);
        componentUIList.add(fieldUI);
    }

    private void checkField(CorpusMetadata corpusMetadata, FieldKey fieldKey) {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        if (corpusField != null) {
            checkCorpusField(corpusField);
        }
    }

    private void checkCorpusField(CorpusField corpusField) {
        FieldUI fieldUI = (FieldUI) componentUIMap.get(corpusField.getFieldKey().toString());
        if (fieldUI == null) {
            fieldUI = new DefaultFieldUI(corpusField);
            addFieldUI(fieldUI);
        }
    }

    public boolean ownsToRattachement(SubsetKey subsetKey) {
        return (rattachementSet.contains(subsetKey));
    }

    public boolean isAssociationActive() {
        return componentUIMap.containsKey(IncludeKey.ASSOCIATION.toString());
    }

    public void setAssociationActive(boolean active) {
        String key = IncludeKey.ASSOCIATION.toString();
        if (active) {
            if (!isAssociationActive()) {
                requireIncludeUI(IncludeKey.ASSOCIATION);
            }
        } else {
            removeComponentUI(key);
        }
    }

    private CommentUI requireComment(String name) {
        String key = "comment_" + name;
        CommentUI comment = (CommentUI) componentUIMap.get(key);
        if (comment == null) {
            comment = new DefaultCommentUI(name);
            componentUIMap.put(key, comment);
            componentUIList.add(comment);
        }
        return comment;
    }

    public CommentUI getCommentUI(String name) {
        String key = "comment_" + name;
        return (CommentUI) componentUIMap.get(key);
    }

    public boolean removeComment(String name) {
        return removeComponentUI("comment_" + name);
    }

    public IncludeUI getIncludeUI(IncludeKey includeKey) {
        return (IncludeUI) componentUIMap.get(includeKey.toString());
    }

    public boolean removeInclude(IncludeKey includeKey) {
        return removeComponentUI(includeKey.toString());
    }

    private IncludeUI requireIncludeUI(IncludeKey includeKey) {
        IncludeUI includeUI = getIncludeUI(includeKey);
        if (includeUI != null) {
            return includeUI;
        }
        SubsetKey subsetKey = includeKey.getSubsetKey();
        if ((subsetKey != null) && (ownsToRattachement(subsetKey))) {
            return null;
        }
        if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
            return null;
        }
        if (includeKey.isChronoInclude()) {
            throw new IllegalArgumentException("no chrono include key allowed =" + includeKey.toString());
        }
        includeUI = new DefaultIncludeUI(includeKey);
        addIncludeUI(includeUI);
        return includeUI;
    }

    public FieldUI getFieldUI(FieldKey fieldKey) {
        return (FieldUI) componentUIMap.get(fieldKey.toString());
    }

    /**
     *
     *@throws IllegalArgumentExceptino si fieldKey correspond à un champ obligatoire.
     */
    public boolean removeAllFieldUI(FieldKey fieldKey) {
        if (fieldKey.isMandatoryFieldKey()) {
            throw new IllegalArgumentException("fieldKey.isMandatoryFieldKey()");
        }
        return removeComponentUI(fieldKey.toString());
    }

    /**
     * Supprime s'il existe
     */
    public boolean removeAllIncludeUI(SubsetKey subsetKey) {
        String racine = subsetKey.toString();
        List<String> keyList = new LinkedList<String>(componentUIMap.keySet());
        boolean done = false;
        for (String key : keyList) {
            if (key.startsWith(racine)) {
                if (removeComponentUI(key)) {
                    done = true;
                }
            }
        }
        return done;
    }

    private boolean removeComponentUI(String key) {
        ComponentUI componentUI = (ComponentUI) componentUIMap.get(key);
        if (componentUI == null) {
            return false;
        }
        componentUIMap.remove(key);
        componentUIList.remove(componentUI);
        if (componentUI instanceof CommentUI) {
            while (true) {
                boolean done = componentUIList.remove(componentUI);
                if (!done) {
                    break;
                }
            }
        }
        return true;
    }

    public SubsetKey getCorpusSubsetKey() {
        return corpus.getSubsetKey();
    }

    public String[] getPositionArray() {
        int size = componentUIList.size();
        String[] result = new String[size];
        for (int i = 0; i < size; i++) {
            result[i] = componentUIList.get(i).getKeyString();
        }
        return result;
    }

    private void parsePositionArray(String[] positionArray) {
        List<ComponentUI> newList = new ArrayList<ComponentUI>();
        Set<String> duplicateSet = new HashSet<String>();
        for (int i = 0; i < positionArray.length; i++) {
            String token = positionArray[i];
            ComponentUI componentUI = componentUIMap.get(token);
            if (componentUI != null) {
                if (!duplicateSet.contains(token)) {
                    newList.add(componentUI);
                    duplicateSet.add(token);
                } else {
                    if (componentUI instanceof CommentUI) {
                        newList.add(componentUI);
                    }
                }
            }
        }
        for (ComponentUI componentUI : componentUIList) {
            String token = componentUI.getKeyString();
            if (!duplicateSet.contains(token)) {
                newList.add(componentUI);
            }
        }
        componentUIList = newList;
    }

    public ComponentUIEditor getComponentUIEditor() {
        return internalComponentUIEditor;
    }


    private class InternalComponentUIEditor implements ComponentUIEditor {

        public FieldUI getFieldUI(FieldKey fieldKey) {
            return CorpusUI.this.getFieldUI(fieldKey);
        }

        public IncludeUI getIncludeUI(IncludeKey includeKey, boolean creation) {
            if (creation) {
                return CorpusUI.this.requireIncludeUI(includeKey);
            } else {
                return CorpusUI.this.getIncludeUI(includeKey);
            }
        }

        public boolean setInputOption(ComponentUI componentUI, InputOption inputOption) {
            if (componentUI instanceof DefaultFieldUI) {
                ((DefaultFieldUI) componentUI).innerUpdateInputOption(inputOption);
                return true;
            } else if (componentUI instanceof DefaultIncludeUI) {
                ((DefaultIncludeUI) componentUI).innerUpdateInputOption(inputOption);
                return true;
            }
            return false;
        }

        public boolean setLimitOption(ComponentUI componentUI, LimitOption limitOption) {
            if (componentUI instanceof DefaultIncludeUI) {
                ((DefaultIncludeUI) componentUI).innerUpdateLimitOption(limitOption);
                return true;
            }
            return false;
        }

        public boolean setMotcleSelectOptionsString(ComponentUI componentUI, String motcleSelectOptionsString) {
            if (componentUI instanceof DefaultIncludeUI) {
                ((DefaultIncludeUI) componentUI).innerSetMotcleSelectOptionsString(motcleSelectOptionsString);
                return true;
            }
            return false;
        }

        public boolean setValueOption(ComponentUI componentUI, ValueOption valueOption) {
            if (componentUI instanceof DefaultFieldUI) {
                ((DefaultFieldUI) componentUI).innerUpdateValueOption(valueOption);
                return true;
            } else if (componentUI instanceof DefaultIncludeUI) {
                ((DefaultIncludeUI) componentUI).innerUpdateValueOption(valueOption);
                return true;
            }
            return false;
        }

        public boolean putLibelle(IncludeUI includeUI, Libelle libelle) {
            if (includeUI instanceof DefaultIncludeUI) {
                return ((DefaultIncludeUI) includeUI).innerPutLibelle(libelle);
            }
            return false;
        }

        public boolean removeLibelle(IncludeUI includeUI, int langInteger) {
            if (includeUI instanceof DefaultIncludeUI) {
                return ((DefaultIncludeUI) includeUI).innerRemoveLibelle(langInteger);
            }
            return false;
        }

        public boolean setDefaultLibelleUsage(ComponentUI componentUI, boolean b) {
            if (componentUI instanceof DefaultIncludeUI) {
                return ((DefaultIncludeUI) componentUI).innerSetDefaultLibelleUsage(b);
            }
            return false;
        }

        public CommentUI getCommentUI(String name, boolean creation) {
            if (creation) {
                return CorpusUI.this.requireComment(name);
            } else {
                return CorpusUI.this.getCommentUI(name);
            }
        }

        public boolean setLocation(ComponentUI componentUI, int newLocation) {
            if (componentUI instanceof DefaultCommentUI) {
                return ((DefaultCommentUI) componentUI).innerSetLocation(newLocation);
            }
            return false;
        }

        public boolean setPositionArray(String[] positionArray) {
            parsePositionArray(positionArray);
            return true;
        }

        public boolean putHtmlString(CommentUI commentUI, int langInteger, String htmlString) {
            if (commentUI instanceof DefaultCommentUI) {
                return ((DefaultCommentUI) commentUI).innerPutHtmlString(langInteger, htmlString);
            }
            return false;
        }

    }

}
