/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.xml;

import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.DefaultCommentUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.MutableInputOption;
import fr.exemole.bdfserver.ui.options.MutableLimitOption;
import fr.exemole.bdfserver.ui.options.MutableValueOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class AbstractUiDOMReader {

    protected UiErrorHandler errorHandler;
    protected ComponentUIEditor editor;

    public AbstractUiDOMReader(ComponentUIEditor editor, UiErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.editor = editor;
    }

    protected FieldUI getFieldUI(Element element_xml) {
        String fieldKeyString = element_xml.getAttribute("field-key");
        if (fieldKeyString.length() == 0) {
            errorHandler.emptyAttributeError("field-ui", "field-key");
            return null;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(fieldKeyString);
            FieldUI fieldUI = editor.getFieldUI(fieldKey);
            if (fieldUI == null) {
                errorHandler.unknownKeyError("field-ui", fieldKey);
                return null;
            }
            return fieldUI;

        } catch (java.text.ParseException pe) {
            errorHandler.attributeSyntaxError("field-ui", "field-key", fieldKeyString);
            return null;
        }
    }

    protected void readFieldUI(Element element_xml, FieldUI fieldUI) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("input")) {
                    InputOption inputOption = readInputOption(element);
                    if (inputOption != null) {
                        editor.setInputOption(fieldUI, inputOption);
                    }
                } else if (tagName.equals("value")) {
                    ValueOption valueOption = readValueOption(element);
                    if (valueOption != null) {
                        editor.setValueOption(fieldUI, valueOption);
                    }
                }
            }
        }
    }

    protected IncludeUI getSubsetIncludeUI(Element element_xml, boolean creation) {
        SubsetKey subsetKey = null;
        int poids = -1;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("subset")) {
                    String subsetKeyString = XMLUtils.getData(element);
                    if (subsetKeyString.length() == 0) {
                        errorHandler.emptyTagError("subset");
                        return null;
                    }
                    try {
                        subsetKey = SubsetKey.parse(subsetKeyString);
                    } catch (java.text.ParseException pe) {
                        errorHandler.textSyntaxError("subset", subsetKeyString);
                        return null;
                    }
                } else if (tagName.equals("poids")) {
                    String poidsString = XMLUtils.getData(element);
                    if (poidsString.length() == 0) {
                        errorHandler.emptyTagError("poids");
                        return null;
                    }
                    try {
                        poids = Integer.parseInt(poidsString);
                    } catch (NumberFormatException nfe) {
                        errorHandler.textSyntaxError("poids", poidsString);
                        return null;
                    }
                }
            }
        }
        if (subsetKey == null) {
            errorHandler.emptyTagError("subset");
            return null;
        }
        try {
            IncludeKey includeKey = IncludeKey.newInstance(subsetKey, poids);
            return editor.getIncludeUI(includeKey, creation);
        } catch (IllegalArgumentException iae) {
            errorHandler.textSyntaxError("subset", subsetKey.toString());
            return null;
        }
    }

    protected void readIncludeUI(Element element_xml, IncludeUI includeUI) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("custom")) {
                    readCustom(includeUI, element);
                }
                if (tagName.equals("input")) {
                    InputOption inputOption = readInputOption(element);
                    if (inputOption != null) {
                        editor.setInputOption(includeUI, inputOption);
                    }
                } else if (tagName.equals("value")) {
                    ValueOption valueOption = readValueOption(element);
                    if (valueOption != null) {
                        editor.setValueOption(includeUI, valueOption);
                    }
                } else if (tagName.equals("limit")) {
                    LimitOption limitOption = readLimitOption(element);
                    if (limitOption != null) {
                        editor.setLimitOption(includeUI, limitOption);
                    }
                } else if (tagName.equals("pioche-select")) {
                    String motcleSelectOptionString = XMLUtils.getData(element_xml);
                    if (motcleSelectOptionString.length() > 0) {
                        editor.setMotcleSelectOptionsString(includeUI, motcleSelectOptionString);
                    }
                }
            }
        }
    }

    protected IncludeUI getSpecialIncludeUI(Element element_xml, boolean creation) {
        String includeKeyString = element_xml.getAttribute("include-key");
        if (includeKeyString.length() == 0) {
            errorHandler.emptyAttributeError("special-ui", "include-key");
            return null;
        }
        try {
            IncludeKey includeKey = IncludeKey.parse(includeKeyString);
            IncludeUI includeUI = editor.getIncludeUI(includeKey, creation);
            if ((!includeKey.isSpecialInclude()) || (includeUI == null)) {
                errorHandler.unknownKeyError("special-ui", includeKey);
                return null;
            }
            return includeUI;
        } catch (java.text.ParseException pe) {
            errorHandler.attributeSyntaxError("special-ui", "include-key", includeKeyString);
            return null;
        }
    }

    protected CommentUI getCommentUI(Element element_xml, boolean creation) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            errorHandler.emptyAttributeError(element_xml.getTagName(), "name");
            return null;
        }
        return editor.getCommentUI(name, true);
    }

    protected void readCommentUI(Element element_xml, CommentUI commentUI) {
        String locationString = element_xml.getAttribute("location");
        if (locationString.length() != 0) {
            int location = DefaultCommentUI.maskStringToInt(locationString);
            editor.setLocation(commentUI, location);
        }
        DOMUtils.readChildren(element_xml, new CommentUIHandler(commentUI));
    }

    private LimitOption readLimitOption(Element element_xml) {
        MutableLimitOption mutableLimitOption = new MutableLimitOption();
        String nonewString = element_xml.getAttribute("nonew");
        if (nonewString.equals("1")) {
            mutableLimitOption.setNoNewAllowed(true);
        } else if ((nonewString.length() == 0) || (nonewString.equals("0"))) {
            mutableLimitOption.setNoNewAllowed(false);
        } else {
            errorHandler.attributeSyntaxError("limit", "nonew", nonewString);
            return null;
        }
        return mutableLimitOption;
    }

    private ValueOption readValueOption(Element element_xml) {
        MutableValueOption mutableValueOption = new MutableValueOption();
        String mandatoryString = element_xml.getAttribute("mandatory");
        if (mandatoryString.equals("1")) {
            mutableValueOption.setMandatory(true);
        } else if ((mandatoryString.length() == 0) || (mandatoryString.equals("0"))) {
            mutableValueOption.setMandatory(false);
        } else {
            errorHandler.attributeSyntaxError("value", "mandatory", mandatoryString);
            return null;
        }
        String defaultValue = element_xml.getAttribute("default").trim();
        if (defaultValue.length() != 0) {
            mutableValueOption.setDefaultValue(defaultValue);
        }
        return mutableValueOption;
    }

    private InputOption readInputOption(Element element_xml) {
        MutableInputOption mutableInputOption = new MutableInputOption();
        String typeString = element_xml.getAttribute("type");
        if (typeString.length() == 0) {
            errorHandler.emptyAttributeError("input", "type");
            return null;
        }
        try {
            mutableInputOption.setInputType(DefaultInputOption.inputTypeToShort(typeString));
        } catch (IllegalArgumentException iae) {
            errorHandler.attributeSyntaxError("input", "type", typeString);
            return null;
        }
        String widthString = element_xml.getAttribute("width");
        if (widthString.length() == 0) {
            widthString = element_xml.getAttribute("size"); //Ancien nom de l'attribut
        }
        if (widthString.length() == 0) {
            /*if (mutableInputOption.getInputType() == InputOption.INPUT_TEXT) {
            errorHandler.emptyAttributeError("input", "width");
            return null;
            }*/
        } else {
            try {
                mutableInputOption.setWidthType(DefaultInputOption.widthTypeToShort(widthString));
            } catch (IllegalArgumentException iae) {
                errorHandler.attributeSyntaxError("input", "width", widthString);
            }
        }
        String rowsString = element_xml.getAttribute("rows");
        if (rowsString.length() == 0) {
            /*if (mutableInputOption.getInputType() == InputOption.INPUT_TEXT) {
            errorHandler.emptyAttributeError("input", "rows");
            return null;
            }*/
        } else {
            try {
                int row = Integer.parseInt(rowsString);
                mutableInputOption.setRows(row);
            } catch (NumberFormatException nfe) {
                errorHandler.attributeSyntaxError("input", "rows", rowsString);
            } catch (IllegalArgumentException iae) {
                errorHandler.attributeSyntaxError("input", "rows", rowsString);
            }
        }
        return mutableInputOption;
    }

    private void readCustom(IncludeUI includeUI, Element element_xml) {
        editor.setDefaultLibelleUsage(includeUI, false);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("lib")) {
                    try {
                        Libelle libelle = LibelleUtils.readLib(element);
                        if (libelle != null) {
                            editor.putLibelle(includeUI, libelle);
                        }
                    } catch (LangIntegerException ile) {
                        errorHandler.attributeSyntaxError("lib", "xml:lang", element.getAttribute("xml:lang"));
                    }
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }


    public class CommentUIHandler implements ElementHandler {

        private CommentUI commentUI;

        private CommentUIHandler(CommentUI commentUI) {
            this.commentUI = commentUI;
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if ((!tagName.equals("lib")) && (!tagName.equals("html"))) {
                errorHandler.unknownTagWarning(tagName);
                return;
            }
            int langInteger = LangInteger.UNDETERMINED;
            String langIntegerString = element.getAttribute("xml:lang");
            if (langIntegerString.length() > 0) {
                boolean langError = false;
                try {
                    langInteger = LangInteger.parse(langIntegerString);
                    if (langInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
                        langError = true;
                    }
                } catch (LangIntegerException lie) {
                    langError = true;
                }
                if (langError) {
                    errorHandler.attributeSyntaxError("html", "xml:lang", langIntegerString);
                }
            }
            String htmlString = XMLUtils.getRawData(element);
            editor.putHtmlString(commentUI, langInteger, htmlString);
        }

    }

}
