/* BdfServer - Copyright (c)2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory;

import fr.exemole.bdfserver.l10n.LocaleBundle;
import fr.exemole.bdfserver.libscol.LibsColKey;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import java.util.Locale;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class XsltResource {

    public final static short IMPORT_RESOURCE = 1;
    public final static short FORMATS_RESOURCE = 2;
    public final static short XSLCOMMON_RESOURCE = 3;
    public final static short DEFAULT_RESOURCE = 4;
    public final static short LIBS_RESOURCE = 5;
    public final static short LOCALESTRINGS_RESOURCE = 6;
    public final static short SPECIALLIBS_RESOURCE = 7;
    public final static short DEFAULT_PATTERN_RESOURCE = 8;
    public final static short LIBSCOL_RESOURCE = 9;
    private LocalisationInfo localisationInfo;
    private LocaleBundle localeBundle;
    private short resourceType;
    private String subname;
    private String path;
    private TransformationKey transformationKey;
    private SubsetKey subsetKey;
    private LibsColKey libsColKey;

    private XsltResource(short resourceType) {
        this.resourceType = resourceType;
    }

    private XsltResource(short resourceType, LocalisationInfo localisationInfo, String subname) {
        this.resourceType = resourceType;
        this.subname = subname;
        this.localisationInfo = localisationInfo;
    }

    private XsltResource(LibsColKey libsColKey, LocalisationInfo localisationInfo) {
        this.resourceType = LIBSCOL_RESOURCE;
        this.libsColKey = libsColKey;
        this.localisationInfo = localisationInfo;
    }

    private XsltResource(short resourceType, TransformationKey transformationKey) {
        this.resourceType = resourceType;
        this.transformationKey = transformationKey;
    }

    private XsltResource(short resourceType, String subname) {
        this.resourceType = resourceType;
        this.subname = subname;
    }

    private XsltResource(short resourceType, LocalisationInfo localisationInfo) {
        this.localisationInfo = localisationInfo;
        this.resourceType = resourceType;
    }

    private XsltResource(short resourceType, LocaleBundle localeBundle) {
        this.localeBundle = localeBundle;
        this.resourceType = resourceType;
    }

    private XsltResource(short resourceType, SubsetKey subsetKey, LocalisationInfo localisationInfo) {
        this.localisationInfo = localisationInfo;
        this.resourceType = resourceType;
        this.subsetKey = subsetKey;
    }

    private XsltResource(short resourceType, String subname, String path) {
        this.resourceType = resourceType;
        this.subname = subname;
        this.path = path;
    }

    /**
     * Non nul pour DEFAULT_RESOURCE, nul sinon.
     */
    public TransformationKey getTransformationKey() {
        return transformationKey;
    }

    public String getSubname() {
        return subname;
    }

    public LocalisationInfo getLocalisationInfo() {
        return localisationInfo;
    }

    public short getResourceType() {
        return resourceType;
    }

    public SubsetKey getSubsetKey() {
        return subsetKey;
    }

    public LibsColKey getLibsColKey() {
        return libsColKey;
    }

    public LocaleBundle getLocaleBundle() {
        return localeBundle;
    }

    public String getPath() {
        return path;
    }

    /**
     * Une ressource a toujours une extension d'au moins trois lettres.
     */
    public static XsltResource parse(String path) {
        int length = path.length();
        if (length < 5) {
            return null;
        }
        int idxPoint = path.lastIndexOf('.');
        if ((idxPoint == -1) || (idxPoint == (length - 1))) {
            return null;
        }
        if (path.lastIndexOf('/') != -1) {
            return null;
        }
        String extension = path.substring(idxPoint + 1);
        String name = path.substring(0, idxPoint);
        if (extension.equals("xsl")) {
            if (name.startsWith("_common_")) {
                return getXslCommonResource(name.substring(8));
            } else if (name.startsWith("_default_")) {
                return getDefaultResource(name.substring(9), "xsl");
            } else if (name.startsWith("_formats_")) {
                return new XsltResource(FORMATS_RESOURCE);
            } else if (name.startsWith("_import_")) {
                return new XsltResource(IMPORT_RESOURCE, name.substring(8));
            } else {
                //return new InnerResource(OTHER_RESOURCE, name, path);
                return null;
            }
        } else if (extension.equals("xml")) {
            if (name.startsWith("_localestrings_")) {
                return getLocaleStringsResource(name.substring(15));
            } else if (name.startsWith("_libs_")) {
                return getLibsResource(name.substring(6));
            } else if (name.startsWith("_libscol_")) {
                return getLibsColResource(name.substring(9));
            } else {
                //return new InnerResource(OTHER_RESOURCE, name, path);
                return null;
            }
        } else if (extension.equals("txt")) {
            if (name.startsWith("_default_")) {
                return getDefaultResource(name.substring(9), "txt");
            } else {
                //return new InnerResource(OTHER_RESOURCE, name, path);
                return null;
            }
        } else {
            //return new InnerResource(OTHER_RESOURCE, name, path);
            return null;
        }
    }

    private static XsltResource getLocaleStringsResource(String localisationSuffix) {
        try {
            int langInteger = LangInteger.parse(localisationSuffix);
            Locale locale = LangInteger.getLocale(langInteger);
            LocaleBundle localeBundle = LocaleBundle.getLocaleBundle(locale);
            return new XsltResource(LOCALESTRINGS_RESOURCE, localeBundle);
        } catch (LangIntegerException lie) {
            return null;
        }

    }

    private static XsltResource getLibsResource(String suffix) {
        int idx = suffix.indexOf('_');
        if (idx == -1) {
            return null;
        }
        int idx2 = suffix.indexOf('_', idx + 1);
        if (idx2 == -1) {
            return null;
        }
        LocalisationInfo localisationInfo = LocalisationInfo.parse(suffix.substring(idx2 + 1));
        if (localisationInfo == null) {
            return null;
        }
        String subname = suffix.substring(0, idx2);
        if (subname.startsWith("special_")) {
            return new XsltResource(SPECIALLIBS_RESOURCE, localisationInfo, subname.substring(8));
        } else {
            try {
                SubsetKey subsetKey = SubsetKey.parse(subname);
                return new XsltResource(LIBS_RESOURCE, subsetKey, localisationInfo);
            } catch (java.text.ParseException pe) {
                return null;
            }
        }
    }

    private static XsltResource getLibsColResource(String suffix) {
        int idxlang = suffix.lastIndexOf('_');
        LocalisationInfo localisationInfo = LocalisationInfo.parse(suffix.substring(idxlang + 1));
        if (localisationInfo == null) {
            return null;
        }
        try {
            LibsColKey key = LibsColKey.parse(suffix.substring(0, idxlang));
            return new XsltResource(key, localisationInfo);
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    private static XsltResource getXslCommonResource(String resourceSubname) {
        return new XsltResource(XSLCOMMON_RESOURCE, resourceSubname);
    }

    private static XsltResource getDefaultResource(String transformationSuffix, String extension) {
        TransformationKey transformationKey = null;
        try {
            transformationKey = TransformationKey.parse(transformationSuffix);

        } catch (java.text.ParseException pe) {
            return null;
        }
        XsltResource ir = null;
        if (extension.equals("txt")) {
            ir = new XsltResource(DEFAULT_PATTERN_RESOURCE, transformationKey);
        } else {
            ir = new XsltResource(DEFAULT_RESOURCE, transformationKey);
        }
        return ir;
    }

}
