/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern.bih;

import fr.exemole.bdfserver.xsltfactory.xslpattern.InsertionUnitTranslator;
import java.io.IOException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.include.IncludeUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.insertpattern.api.InsertionBuffer;
import net.mapeadores.util.text.insertpattern.api.InsertionUnit;
import net.mapeadores.util.xml.PatternXSLTResolver;
import net.mapeadores.util.xml.XSLTWriter;


/**
 *
 * @author Vincent Calame
 */
public class ClauseTranslator implements InsertionUnitTranslator {

    private PatternXSLTResolver xsltResolver;

    public ClauseTranslator(PatternXSLTResolver xsltResolver) {
        this.xsltResolver = xsltResolver;
    }

    public void translate(InsertionUnit insertionUnit) throws IOException {
        switch (insertionUnit.getType()) {
            case InsertionBuffer.START_IF:
                resolveStartIf(insertionUnit.getClause(), insertionUnit.isUnique());
                break;
            case InsertionBuffer.START_ELSEIF:
                resolveStartElseIf(insertionUnit.getClause());
                break;
            case InsertionBuffer.START_ELSE:
                resolveStartElse();
                break;
            case InsertionBuffer.END_IF:
                resolveEndIf(insertionUnit.isUnique(), insertionUnit.isOnElse());
                break;
            default:
                new SwitchException("type = " + insertionUnit.getType());
        }
    }

    private void resolveStartIf(String clause, boolean unique) throws IOException {
        XSLTWriter currentXsltWriter = xsltResolver.getCurrentXSLTWriter();
        String error = null;
        try {
            clause = resolveClause(clause);
        } catch (IllegalArgumentException iae) {
            clause = "true()";
            error = "# IF CLAUSE ERROR : " + iae.getMessage() + "# ";
        }
        if (unique) {
            currentXsltWriter.openXslIf(clause);
            if (error != null) {
                currentXsltWriter.escape(error);
            }
        } else {
            currentXsltWriter.openXslChoose();
            currentXsltWriter.openXslWhen(clause);
            if (error != null) {
                currentXsltWriter.escape(error);
            }
        }
    }

    private void resolveStartElseIf(String clause) throws IOException {
        XSLTWriter currentXsltWriter = xsltResolver.getCurrentXSLTWriter();
        currentXsltWriter.closeXslWhen();
        String error = null;
        try {
            clause = resolveClause(clause);
        } catch (IllegalArgumentException iae) {
            clause = "true()";
            error = "# ELIF CLAUSE ERROR : " + iae.getMessage() + "# ";
        }
        currentXsltWriter.openXslWhen(clause);
        if (error != null) {
            currentXsltWriter.escape(error);
        }
    }

    private void resolveStartElse() throws IOException {
        XSLTWriter currentXsltWriter = xsltResolver.getCurrentXSLTWriter();
        currentXsltWriter.closeXslWhen();
        currentXsltWriter.openXslOtherwise();

    }

    private void resolveEndIf(boolean unique, boolean onElse) throws IOException {
        XSLTWriter currentXsltWriter = xsltResolver.getCurrentXSLTWriter();
        if (unique) {
            currentXsltWriter.closeXslIf();
        } else {
            if (onElse) {
                currentXsltWriter.closeXslOtherwise();
            } else {
                currentXsltWriter.closeXslWhen();
            }
            currentXsltWriter.closeXslChoose();
        }
    }

    public boolean isTemplateInstruction(InsertionUnit insertionUnit) {
        return false;
    }

    private String resolveClause(String clause) {
        clause = clause.trim();
        ClauseBuffer clauseBuffer = new ClauseBuffer();
        clauseBuffer.parse(clause);
        return clauseBuffer.getXsltClause();
    }


    private class ClauseBuffer {

        private StringBuilder buffer = new StringBuilder();
        private StringBuilder keyBuffer = new StringBuilder();

        private ClauseBuffer() {
        }

        private String getXsltClause() {
            return buffer.toString();
        }

        private void parse(String raw) {
            int length = raw.length();
            for (int i = 0; i < length; i++) {
                char carac = raw.charAt(i);
                switch (carac) {
                    case '(':
                    case ')':
                        resolveKey();
                        buffer.append(carac);
                        break;
                    case '&':
                        resolveKey();
                        buffer.append(" and ");
                        break;
                    case '|':
                        resolveKey();
                        buffer.append(" or ");
                        break;
                    default:
                        keyBuffer.append(carac);
                }
            }
            resolveKey();
        }

        private void resolveKey() {
            String s = keyBuffer.toString().trim();
            keyBuffer = new StringBuilder();
            int length = s.length();
            if (length == 0) {
                return;
            }
            boolean negative = false;
            if (s.charAt(0) == '!') {
                if (length == 1) {
                    throw new IllegalArgumentException("« ! » alone");
                }
                s = s.substring(1);
                length = length - 1;
                negative = true;
            }
            Object obj = IncludeUtils.parseKey(s);
            if (obj == null) {
                if (negative) {
                    s = "!" + s;
                }
                throw new IllegalArgumentException("« " + s + " » is wrong");
            }
            if (negative) {
                buffer.append("boolean(");
            }
            if (obj != null) {
                if (obj instanceof FieldKey) {
                    BihUtils.appendPath(buffer, (FieldKey) obj, false, false);
                } else {
                    IncludeKey includeKey = (IncludeKey) obj;
                    if (includeKey.isSubsetInclude()) {
                        BihUtils.appendSubsetIncludeKey(buffer, includeKey, false);
                    } else {
                        if (includeKey.isChronoInclude()) {
                            BihUtils.appendChronoIncludeKey(buffer);
                        } else if (includeKey.isSpecialInclude()) {
                            if (includeKey.equals(IncludeKey.ASSOCIATION)) {
                                BihUtils.appendAssocation(buffer);
                            } else if (includeKey.equals(IncludeKey.LIAGE)) {
                                BihUtils.appendLiagePath(buffer, false);
                            } else if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
                                BihUtils.appendRattachementPath(buffer, false);
                            } else {
                                throw new SwitchException("unknown key = " + obj.toString());
                            }
                        } else {
                            throw new SwitchException("unknown key = " + obj.toString());
                        }
                    }
                }
            }
            if (negative) {
                buffer.append(")= false()");
            }

        }

    }

}
