/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.commands2.affichage.AffichageCommandFactory;
import fr.exemole.bdfserver.commands2.album.AlbumCommandFactory;
import fr.exemole.bdfserver.commands2.configuration.ConfigurationCommandFactory;
import fr.exemole.bdfserver.commands2.corpus.CorpusCommandFactory;
import fr.exemole.bdfserver.commands2.envoi.EnvoiCommandFactory;
import fr.exemole.bdfserver.commands2.saisie.SaisieCommandFactory;
import fr.exemole.bdfserver.commands2.selection.SelectionCommandFactory;
import fr.exemole.bdfserver.commands2.sphere.SphereCommandFactory;
import fr.exemole.bdfserver.extension.BdfExtension;
import fr.exemole.bdfserver.extension.BdfExtensionFactoryException;
import fr.exemole.bdfserver.extension.BdfExtensionManager;
import fr.exemole.bdfserver.htmlproducers.HtmlProducerFactory;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.instruction.InstructionUtils;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.managers.configuration.PageSubstitution;
import fr.exemole.bdfserver.response.BdfConstants;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceLocalisationProvider;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.ResponseDataType;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;
import net.mapeadores.util.servlets.handlers.HtmlResponseHandler;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;
import net.mapeadores.util.servlets.handlers.RedirectResponseHandler;
import net.mapeadores.util.servlets.handlers.SimpleResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class CommandInstruction implements BdfInstruction, BdfInstructionConstants, ResponseDataType {

    private static final Map<String, BdfCommand2Factory> commandFactoryMap = new HashMap<String, BdfCommand2Factory>();
    private BdfCommand2 bdfCommand2;
    private String actionName;
    private boolean noCommand = false;
    private CommandMessage initErrorMessage = null;
    private final BdfContext bdfContext;
    private final BdfServer bdfServer;
    private final BdfRequestMap requestMap;
    private BdfUser bdfUser;
    private LocalisationInfo localisationInfo;
    private ResourceLocalisationProvider resourceLocalisationProvider;

    static {
        commandFactoryMap.put("Album", new AlbumCommandFactory());
        commandFactoryMap.put("Saisie", new SaisieCommandFactory());
        commandFactoryMap.put("Sphere", new SphereCommandFactory());
        commandFactoryMap.put("Configuration", new ConfigurationCommandFactory());
        commandFactoryMap.put("Selection", new SelectionCommandFactory());
        commandFactoryMap.put("Affichage", new AffichageCommandFactory());
        commandFactoryMap.put("Envoi", new EnvoiCommandFactory());
        commandFactoryMap.put("Corpus", new CorpusCommandFactory());
    }

    public CommandInstruction(BdfContext bdfContext, BdfRequestMap requestMap, String actionName) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.requestMap = requestMap;
        this.actionName = actionName;
        this.resourceLocalisationProvider = (ResourceLocalisationProvider) bdfContext.getContextObject(BdfContextConstants.RESOURCELOCALISATIONPROVIDER_NAME);
        String cmd = requestMap.getCommandParameter();
        if (cmd != null) {
            BdfCommand2Factory factory = commandFactoryMap.get(actionName);
            if (factory != null) {
                bdfCommand2 = factory.getBdfCommand(bdfContext, requestMap, cmd);
                if (bdfCommand2 == null) {
                    initErrorMessage = CommandMessage.error("global_err_bdfcommand", cmd);
                }
            } else {
                initErrorMessage = CommandMessage.error("global_err_bdfcommandfactory", actionName);
            }
        } else {
            noCommand = true;
        }
    }

    public short getBdfUserNeed() {
        if (noCommand) {
            return getNoCommandUserNeed(actionName, requestMap);
        } else {
            if (bdfCommand2 == null) {
                return NONE_BDFUSER;
            } else {
                return bdfCommand2.getBdfUserNeed();
            }
        }
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        this.bdfUser = bdfUser;
        if (bdfUser != null) {
            localisationInfo = bdfUser.getLocalisationInfo();
        } else {
            localisationInfo = InstructionUtils.getLocalisationInfo(requestMap, bdfServer);
        }
        ResourceLocalisation resourceLocalisation = resourceLocalisationProvider.getResourceLocalisation(localisationInfo.getFormatLocale());
        if (noCommand) {
            return getResponseHandler(null, resourceLocalisation);
        }
        if (initErrorMessage != null) {
            return CommandMessageResponseHandler.getInstance(initErrorMessage, resourceLocalisation, requestMap);
        }
        if (requestMap.isTest()) {
            try {
                CommandMessage commandMessage = bdfCommand2.testCommand(bdfUser);
                return CommandMessageResponseHandler.getInstance(commandMessage, resourceLocalisation, requestMap);
            } catch (BdfCommand2Exception bce) {
                CommandMessage errorMessage = BdfCommand2Utils.toCommandMessage(bce);
                return CommandMessageResponseHandler.getInstance(errorMessage, resourceLocalisation, requestMap);
            }
        }
        try {
            BdfCommand2Result bdfCommandResult = null;
            if (bdfCommand2.needSynchronisation()) {
                synchronized (bdfServer) {
                    bdfCommandResult = bdfCommand2.doCommand(bdfUser);
                }
            } else {
                bdfCommandResult = bdfCommand2.doCommand(bdfUser);
            }
            return getResponseHandler(bdfCommandResult, resourceLocalisation);
        } catch (BdfCommand2Exception bce) {
            CommandMessage errorMessage = BdfCommand2Utils.toCommandMessage(bce);
            return CommandMessageResponseHandler.getInstance(errorMessage, resourceLocalisation, requestMap);
        }
    }

    private ResponseHandler getResponseHandler(BdfCommand2Result bdfCommandResult, ResourceLocalisation resourceLocalisation) {
        short responseDataType = requestMap.getResponseDataType();
        switch (responseDataType) {
            case JSON_DATATYPE:
            case JSON_IN_HTML_DATATYPE:
            case JSON_P_DATATYPE:
                CommandMessage commandMessage = bdfCommandResult.getCommandMessage();
                if (commandMessage.isErrorMessage()) {
                    return CommandMessageResponseHandler.getJsonInstance(commandMessage, resourceLocalisation, requestMap);
                } else {
                    JsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, resourceLocalisation);
                    return new JsonResponseHandler(jsonProducer, responseDataType, requestMap.getCallbackParameter());
                }
        }
        if (requestMap.isRedirectDefined()) {
            return new RedirectResponseHandler(requestMap.getRedirectParameter());
        }
        if (!requestMap.isPageDefined()) {
            return CommandMessageResponseHandler.getHtmlInstance(CommandMessage.error("global_err_nopage", null), resourceLocalisation);
        }
        String page = requestMap.getPageParameter();
        CommandMessage commandeMessage = null;
        if (bdfCommandResult != null) {
            commandeMessage = bdfCommandResult.getCommandMessage();
            if ((commandeMessage != null) && (commandeMessage.isErrorMessage())) {
                String errorPage = requestMap.getParameter("page-err");
                if (errorPage != null) {
                    page = errorPage;
                }
            }
        }
        BdfCommandPageParameters bdfCommandPageParameters = new BdfCommandPageParameters();
        bdfCommandPageParameters.setBdfUser(bdfUser);
        bdfCommandPageParameters.setRequestMap(requestMap);
        bdfCommandPageParameters.setActionName(actionName);
        bdfCommandPageParameters.setBdfCommandResult(bdfCommandResult);
        if (!page.startsWith("ext:")) {
            PageSubstitution pageSubstitution = bdfServer.getConfigurationManager().getPageSubstitution(actionName + "/" + page);
            if (pageSubstitution != null) {
                page = pageSubstitution.getSubstitutionName();
            }
        }
        if (page.startsWith("ext:")) {
            int idx = page.indexOf('/');
            if (idx == -1) {
                return CommandMessageResponseHandler.getHtmlInstance(CommandMessage.error("global_err_extpage_slash", page), resourceLocalisation);
            }
            String extensionName = page.substring(4, idx);
            String objectKey = page.substring(idx + 1);
            BdfExtensionManager bdfExtensionManager = (BdfExtensionManager) bdfContext.getContextObject(BdfContextConstants.BDFEXTENSIONMANAGER_NAME);
            BdfExtension bdfExtension = null;
            try {
                bdfExtension = bdfExtensionManager.getBdfExtension(bdfContext, extensionName);
            } catch (BdfExtensionFactoryException befe) {
                String[] errorArray = new String[2];
                errorArray[0] = extensionName;
                Throwable cause = befe.getCause();
                if (cause != null) {
                    errorArray[1] = cause.getClass().getName() + ": " + cause.getMessage();
                } else {
                    errorArray[1] = "BdfExtensionFactoryException: " + befe.getMessage();
                }
                return CommandMessageResponseHandler.getHtmlInstance(CommandMessage.error("global_err_extpage_exception", errorArray), resourceLocalisation);
            }
            if (bdfExtension == null) {
                return CommandMessageResponseHandler.getHtmlInstance(CommandMessage.error("global_err_extpage_noext", extensionName), resourceLocalisation);
            }
            ResponseHandler responseHandler = (ResponseHandler) bdfExtension.newInstance("page:" + objectKey, bdfCommandPageParameters);
            if (responseHandler == null) {
                return CommandMessageResponseHandler.getHtmlInstance(CommandMessage.error("global_err_extpage_nopage", objectKey), resourceLocalisation);
            }
            return responseHandler;
        } else if (page.equals(BdfConstants.MESSAGE_PAGE)) {
            if (commandeMessage != null) {
                return CommandMessageResponseHandler.getHtmlInstance(commandeMessage, resourceLocalisation);
            } else {
                SimpleResponseHandler srh = new SimpleResponseHandler("");
                srh.setHtml(true);
                return srh;
            }
        } else {
            HtmlProducer htmlProducer = null;
            try {
                htmlProducer = HtmlProducerFactory.getHtmlProducer(page, bdfContext, bdfCommandPageParameters);
            } catch (BdfCommand2Exception bce) {
                CommandMessage errorMessage = BdfCommand2Utils.toCommandMessage(bce);
                return CommandMessageResponseHandler.getHtmlInstance(errorMessage, resourceLocalisation);
            }
            if (htmlProducer == null) {
                return CommandMessageResponseHandler.getHtmlInstance(CommandMessage.error("global_err_unknownpage", page), resourceLocalisation);
            }
            return new HtmlResponseHandler(htmlProducer);
        }

    }

    public static boolean isValidActionName(String actionName) {
        return (commandFactoryMap.get(actionName) != null);
    }

    public static short getNoCommandUserNeed(String actionName, RequestMap requestMap) {
        if (actionName.equals("Album")) {
            return ADMIN_ONLY_BDFUSER;
        }
        if (actionName.equals("Configuration")) {
            return ADMIN_ONLY_BDFUSER;
        }
        return MANDATORY_BDFUSER;
    }

}
