/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.tools.duplication.DuplicationConstants;
import net.mapeadores.basedefiches.tools.duplication.DuplicationDef;
import net.mapeadores.basedefiches.tools.duplication.DuplicationDefBuilder;
import net.mapeadores.basedefiches.tools.duplication.DuplicationEngine;
import net.mapeadores.basedefiches.tools.duplication.DuplicationLog;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class DuplicationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Duplication";
    public final static String COMMANDKEY = "CRP-23";
    public final static String IDCORPUSTOKENS_PARAMNAME = "idcorpustokens";
    public final static String POIDSWITHORIGIN_PARAMNAME = "poidswithorigin";
    private DuplicationDef duplicationDef;
    private Corpus corpus;

    public DuplicationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        DuplicationLog duplicationLog = DuplicationEngine.run(duplicationDef, bdfServer.getBasedefichesEditor());
        if (duplicationLog.getErrorCount() > 0) {
            throw new ShouldNotOccurException("test done before");
        }
        int count = duplicationLog.getResultCount();
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                buf.append(", ");
            }
            DuplicationLog.DuplicationResult result = duplicationLog.getResult(i);
            buf.append(result.getOriginIdcorpus());
            buf.append(" → ");
            buf.append(result.getDestinationIdcorpus());
        }
        setDone("corpus_done_duplication", buf.toString());
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        DuplicationDefBuilder duplicationDefBuilder = new DuplicationDefBuilder(corpus.getSubsetKey());
        String idcorpusTokens = BdfCommand2Utils.getMandatoryParameter(requestMap, IDCORPUSTOKENS_PARAMNAME);
        StringBuilder tokenErrorBuffer = new StringBuilder();
        String[] tokens = StringUtils.getTechnicalTokens(idcorpusTokens, true);
        int tokenLength = tokens.length;
        for (int i = 0; i < tokenLength; i++) {
            String token = tokens[i];
            boolean done = false;
            try {
                int idcorpus = Integer.parseInt(token);
                if (idcorpus > 0) {
                    done = true;
                    duplicationDefBuilder.addOriginIdcorpus(idcorpus);
                }
            } catch (NumberFormatException nfe) {
            }
            if (!done) {
                if (tokenErrorBuffer.length() > 0) {
                    tokenErrorBuffer.append("; ");
                }
                tokenErrorBuffer.append(token);
            }
        }
        if (tokenErrorBuffer.length() > 0) {
            setError("corpus_err_wrongidcorpustokens", tokenErrorBuffer.toString());
            return false;
        }
        String poidsWithOriginString = requestMap.getParameter(POIDSWITHORIGIN_PARAMNAME);
        if (poidsWithOriginString != null) {
            poidsWithOriginString = poidsWithOriginString.trim();
            if (poidsWithOriginString.length() > 0) {
                boolean done = false;
                try {
                    int poidsWithOrigin = Integer.parseInt(poidsWithOriginString);
                    if ((poidsWithOrigin == -1) || (poidsWithOrigin > 0)) {
                        duplicationDefBuilder.setPoidsWithOrigin(poidsWithOrigin);
                        done = true;
                    }
                } catch (NumberFormatException nfe) {
                }
                if (!done) {
                    setError("corpus_err_wrongpoidswithorigin", poidsWithOriginString);
                    return false;
                }
            }

        }
        duplicationDef = duplicationDefBuilder.toDuplicationDef();
        if (duplicationDef.getOriginIdcorpusCount() == 0) {
            setError("corpus_err_noidcorpustokens", null);
            return false;
        }
        DuplicationLog duplicationLog = DuplicationEngine.test(duplicationDef, basedefiches);
        int errorCount = duplicationLog.getErrorCount();
        if (errorCount > 0) {
            setMultiErrorTitle("corpus_err_duplicationerrorlist", null);
            for (int i = 0; i < errorCount; i++) {
                DuplicationLog.DuplicationError error = duplicationLog.getError(i);
                Object value = getErrorValue(error);
                String errorKey = error.getErrorKey();
                if (errorKey.equals(DuplicationConstants.MISSING_ORIGINFICHE_IDCORPUSERROR)) {
                    addMultiError("corpus_err_missingoriginfiche", value);
                } else if (errorKey.equals(DuplicationConstants.EXISTING_DESTINATIONFICHE_IDCORPUSERROR)) {
                    addMultiError("corpus_err_existingdestinationfiche", value);
                } else {
                    addMultiError("corpus_err_duplicationerror", errorKey);
                }
            }
            return false;
        } else {
            return true;
        }

    }

    private Object getErrorValue(DuplicationLog.DuplicationError error) {
        int idcorpusCount = error.getIdcorpusCount();
        if (idcorpusCount == 0) {
            return null;
        }
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < idcorpusCount; i++) {
            if (i > 0) {
                buf.append("; ");
            }
            buf.append(error.getIdcorpus(i));
        }
        return buf.toString();
    }

}
