/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionException;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsConstants;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsUtils;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatDefParser;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.CurrencyIntegerException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FieldOptionsCommand extends AbstractBdfCommand2 implements FieldOptionsConstants {

    public final static String COMMANDNAME = "FieldOptions";
    public final static String COMMANDKEY = "CRP-07";
    public final static String DEFAULTSPHERE_SUFFIX = "|defaultsphere";
    public final static String BLOCKDISPLAY_SUFFIX = "|blockdisplay";
    public final static String SUBFIELDDISPLAY_SUFFIX = "|subfielddisplay";
    public final static String CURRENCYARRAY_SUFFIX = "|currencyarray";
    public final static String BASEURL_SUFFIX = "|baseurl";
    public final static String TITREGENERATIONPATTERN_PARAMNAME = "titregenerationpattern";
    public final static String TITREGENERATION_SOURCE_PARAMNAME = "titregeneration_source";
    public final static String TITREGENERATION_PATTERN_PARAMNAME = "titregeneration_pattern";
    public final static String TITREGENERATION_INSTRUCTION_PARAMNAME = "titregeneration_instruction";
    public final static String ADDRESSFIELDARRAY_SUFFIX = "|addressfieldarray";
    public final static String GEOLOCALISATIONFIELD_PARAMNAME = "geolocalisationfield";
    private Corpus corpus;
    private BasedefichesFormatDef ficheFormatDef = null;
    private boolean initTitreGeneration = false;
    private boolean initGeolocalisationField = false;
    private CorpusField geolocalisationField = null;

    public FieldOptionsCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        CorpusMetadataEditor corpusMetadataEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey()).getCorpusMetadataEditor();
        boolean done = false;
        if (initTitreGeneration) {
            CorpusField titreField = corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE);
            try {
                done = corpusMetadataEditor.setFieldOption(titreField, TITREGENERATIONFORMAT_OPTION, ficheFormatDef);
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }
        if (initGeolocalisationField) {
            boolean geoDone = false;
            try {
                geoDone = corpusMetadataEditor.setGeolocalisationField(geolocalisationField);
                if (geoDone) {
                    done = true;
                }
            } catch (IllegalArgumentException iae) {
                throw new ImplementationException(iae);
            }
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            if (setOptions(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            if (setOptions(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (done) {
            setDone("corpus_done_options", null);
            bdfServer.getTransformationManager().clearDefaultTransformer();
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        String formatSource = requestMap.getParameter(TITREGENERATION_SOURCE_PARAMNAME);
        if (formatSource == null) {
            initTitreGeneration = false;
        } else {
            initTitreGeneration = true;
            formatSource = formatSource.trim();
            if (formatSource.length() > 0) {
                String formatPattern = BdfCommand2Utils.getMandatoryParameter(requestMap, TITREGENERATION_PATTERN_PARAMNAME);
                String formatInstruction = BdfCommand2Utils.getMandatoryParameter(requestMap, TITREGENERATION_INSTRUCTION_PARAMNAME);
                FicheFormatErrorHandler ffeh = new FicheFormatErrorHandler(formatSource);
                ficheFormatDef = BasedefichesFormatDefParser.parse(formatSource, formatPattern, formatInstruction, ffeh);
                if (hasMultiError()) {
                    setMultiErrorTitle("corpus_err_options_list", null);
                    return false;
                }
            }
        }
        String geolocalisationFieldString = requestMap.getParameter(GEOLOCALISATIONFIELD_PARAMNAME);
        if (geolocalisationFieldString == null) {
            initGeolocalisationField = false;
        } else {
            initGeolocalisationField = true;
            if (geolocalisationFieldString.length() == 0) {
                geolocalisationField = null;
            } else {
                boolean done = false;
                try {
                    FieldKey fieldKey = FieldKey.parse(geolocalisationFieldString);
                    if (fieldKey.isProprieteFieldKey()) {
                        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
                        if ((corpusField != null) && (corpusField.getFicheItemType() == CorpusField.GEOPOINT_FIELD)) {
                            done = true;
                            geolocalisationField = corpusField;
                        }
                    }
                } catch (java.text.ParseException pe) {
                }
                if (!done) {
                    throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGFIELDKEY, GEOLOCALISATIONFIELD_PARAMNAME, geolocalisationFieldString);
                }
            }
        }
        return true;
    }

    private boolean setOptions(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        boolean done = false;
        if (FieldOptionsUtils.testFieldOption(DEFAULTSPHEREKEY_OPTION, corpusField)) {
            if (setDefaultSphere(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(INFORMATIONDISPLAY_OPTION, corpusField)) {
            if (setDisplay(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(SUBFIELDDISPLAY_OPTION, corpusField)) {
            if (setSubfieldDisplay(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(CURRENCYARRAY_OPTION, corpusField)) {
            if (setCurrencyArray(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(ADDRESSFIELDARRAY_OPTION, corpusField)) {
            if (setAddressFieldArray(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(BASEURL_OPTION, corpusField)) {
            if (setStringOption(corpusMetadataEditor, corpusField, BASEURL_OPTION, BASEURL_SUFFIX)) {
                done = true;
            }
        }
        return done;
    }

    private boolean setDisplay(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + BLOCKDISPLAY_SUFFIX);
        short value = FieldOptionsUtils.INLINE_DISPLAY;
        if ((param != null) && (param.equals("1"))) {
            value = FieldOptionsUtils.BLOCK_DISPLAY;
        }
        boolean done = false;
        try {
            done = corpusMetadataEditor.setFieldOption(corpusField, INFORMATIONDISPLAY_OPTION, new Short(value));
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setDefaultSphere(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + DEFAULTSPHERE_SUFFIX);
        if (param == null) {
            return false;
        }
        boolean done = false;
        SubsetKey defaultSphereKey = null;
        if (param.length() > 0) {
            try {
                defaultSphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, param);
            } catch (java.text.ParseException pe) {
            }
        }
        try {
            done = corpusMetadataEditor.setFieldOption(corpusField, DEFAULTSPHEREKEY_OPTION, defaultSphereKey);
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setSubfieldDisplay(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        if (!FieldOptionsUtils.haveSubfield(corpusField.getFicheItemType())) {
            return false;
        }
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + SUBFIELDDISPLAY_SUFFIX);
        boolean done = false;
        try {
            if ((param == null) || (!param.equals("1"))) {
                done = corpusMetadataEditor.setFieldOption(corpusField, SUBFIELDDISPLAY_OPTION, null);
            } else {
                done = corpusMetadataEditor.setFieldOption(corpusField, SUBFIELDDISPLAY_OPTION, Boolean.TRUE);
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setStringOption(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField, String optionName, String suffix) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + suffix);
        System.out.println("String option = " + param);
        try {
            boolean done = corpusMetadataEditor.setFieldOption(corpusField, optionName, param);
            return done;
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
    }

    private boolean setAddressFieldArray(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        if (corpusField.getFicheItemType() != CorpusField.GEOPOINT_FIELD) {
            return false;
        }
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + ADDRESSFIELDARRAY_SUFFIX);
        boolean done = false;
        try {
            if (param == null) {
                done = corpusMetadataEditor.setFieldOption(corpusField, ADDRESSFIELDARRAY_OPTION, null);
            } else {
                List<FieldKey> fieldKeyList = new ArrayList<FieldKey>();
                String[] tokenArray = StringUtils.getTechnicalTokens(param, false);
                int length = tokenArray.length;
                for (int i = 0; i < length; i++) {
                    try {
                        FieldKey fieldKey = FieldKey.parse(tokenArray[i]);
                        fieldKeyList.add(fieldKey);
                    } catch (java.text.ParseException pe) {
                    }
                }
                done = corpusMetadataEditor.setFieldOption(corpusField, ADDRESSFIELDARRAY_OPTION, fieldKeyList.toArray(new FieldKey[fieldKeyList.size()]));
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setCurrencyArray(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        if (corpusField.getFicheItemType() != CorpusField.MONTANT_FIELD) {
            return false;
        }
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + CURRENCYARRAY_SUFFIX);
        boolean done = false;
        try {
            int[] currencyIntegerArray = new int[0];
            if (param != null) {
                String[] tokenArray = StringUtils.getTechnicalTokens(param, false);
                int length = tokenArray.length;
                currencyIntegerArray = new int[length];
                int p = 0;
                for (int i = 0; i < length; i++) {
                    try {
                        int currencyInteger = CurrencyInteger.parse(tokenArray[i]);
                        currencyIntegerArray[p] = currencyInteger;
                        p++;
                    } catch (CurrencyIntegerException cie) {
                    }
                }
                if (p < length) {
                    int[] tempArray = new int[p];
                    System.arraycopy(currencyIntegerArray, 0, tempArray, 0, p);
                    currencyIntegerArray = tempArray;
                }
            }
            if (currencyIntegerArray.length == 0) {
                done = corpusMetadataEditor.setFieldOption(corpusField, CURRENCYARRAY_OPTION, null);
            } else {
                done = corpusMetadataEditor.setFieldOption(corpusField, CURRENCYARRAY_OPTION, currencyIntegerArray);
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }


    private class FicheFormatErrorHandler implements BasedefichesFormatDefParser.ErrorHandler {

        private String formatSource;

        private FicheFormatErrorHandler(String formatSource) {
            this.formatSource = formatSource;
        }

        private void addError(String key, Object value) {
            addMultiError("corpus_err_titregeneration_" + key, value);
        }

        public void nullFormatSourceError() {
            throw new ImplementationException("nullFormatSourceError");
        }

        public void wrongFormatSourceError() {
            addError("wrongformatsource", formatSource);
        }

        public void wrongFormatSourcePartError(String formatSourcePart) {
            addError("wrongformatsource", formatSourcePart);
        }

        public void invalidAsciiCharacterInstructionError(String formatInstructionPart) {
            addError("invalidcharacter", formatInstructionPart);
        }

        public void invalidEndCharacterInstructionError(String formatInstructionPart) {
            addError("invalidcharacter", formatInstructionPart);
        }

        public void invalidSeparatorCharacterInstructionError(String formatInstructionPart) {
            addError("invalidcharacter", formatInstructionPart);
        }

        public void unknownArgumentKeyInstructionWarning(String argumentKey) {
            addError("unknownargumentkey", argumentKey);
        }

        public void missingArgumentValueInstructionError(String argumentKey) {
            addError("missingargumentvalue", argumentKey);
        }

        public void wrongArgumentValueInstructionError(String argumentKey, String argumentValue) {
            String[] vals = {argumentKey, argumentValue};
            addError("wrongargumentvalue", vals);
        }

    }

}
