/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.PasswordManager;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.managers.PolicyManager;
import fr.exemole.bdfserver.managers.ScrutariExportManager;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.managers.xml.ConfigurationManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.ConfigurationManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.ManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.ManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.PasswordManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.PermissionManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.PermissionManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.PolicyManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.PolicyManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.SqlExportManagerDOMReader;
import fr.exemole.bdfserver.roles.RoleMap;
import fr.exemole.bdfserver.managers.SqlExportManager;
import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogUiErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.ManagerErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.TreesErrorHandler;
import fr.exemole.bdfserver.storage.directory.old.CastorUtils;
import fr.exemole.bdfserver.ui.AlterUI;
import fr.exemole.bdfserver.ui.DefaultAlterUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.xml.ComponentUiFilterDOMReader;
import fr.exemole.bdfserver.ui.xml.CorpusUiDOMReader;
import fr.exemole.bdfserver.ui.xml.DefaultUIFilterBuilder;
import fr.exemole.bdfserver.ui.xml.TreesDOMReader;
import fr.exemole.bdfserver.ui.xml.UiErrorHandler;
import fr.exemole.bdfserver.users.BdfUserDef;
import java.io.File;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;
import net.mapeadores.util.logging.MutableProcessLog;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryBdfStorage implements BdfStorage {

    private BdfDirectories bdfDirectories;
    private DirectoryBasedefiches directoryBasedefiches;
    private DirectoryRoleMap roleMap;
    private MutableProcessLog storageProcessLog;
    private BdfStorageException currentBdfStorageException;
    private InternalExceptionHandler exceptionHandler = new InternalExceptionHandler();
    private DirectoryPasswordStorage passwordStorage;
    private DirectoryScrutariExportStorage scrutariExportStorage;
    private DirectorySqlExportStorage sqlExportStorage;
    private DirectoryBdfUserStorage bdfUserStorage;
    private boolean conversion_0_20_Done = false;

    public DirectoryBdfStorage(File bdfStorageDir, File cacheDir, File backupDir, MutableProcessLog storageProcessLog) {
        this.storageProcessLog = storageProcessLog;
        bdfDirectories = new BdfDirectories(bdfStorageDir, cacheDir, backupDir);
        directoryBasedefiches = new DirectoryBasedefiches(bdfDirectories);
        this.conversion_0_20_Done = directoryBasedefiches.init(storageProcessLog, exceptionHandler);
        roleMap = new DirectoryRoleMap(bdfDirectories);
        roleMap.init(storageProcessLog);
        passwordStorage = new DirectoryPasswordStorage(bdfDirectories);
        scrutariExportStorage = new DirectoryScrutariExportStorage(bdfDirectories);
        sqlExportStorage = new DirectorySqlExportStorage(bdfDirectories);
        bdfUserStorage = new DirectoryBdfUserStorage(bdfDirectories);
    }

    public boolean isConversion_0_20_Done() {
        return conversion_0_20_Done;
    }

    public BdfStorage.Check check(MutableProcessLog storageProcessLog) {
        DirectoryCheck check = new DirectoryCheck();
        check.setPasswordList(passwordStorage.check());
        check.setScrutariExportDefList(scrutariExportStorage.check(storageProcessLog));
        check.setSqlExportDefList(sqlExportStorage.check(storageProcessLog));
        check.setBdfUserDefList(bdfUserStorage.check());
        return check;
    }

    private void checkException() {
        if (currentBdfStorageException != null) {
            throw currentBdfStorageException;
        }
    }

    public PasswordStorage getPasswordStorage() {
        checkException();
        return passwordStorage;
    }

    public ScrutariExportStorage getScrutariExportStorage() {
        checkException();
        return scrutariExportStorage;
    }

    public SqlExportStorage getSqlExportStorage() {
        checkException();
        return sqlExportStorage;
    }

    public BdfUserStorage getBdfUserStorage() {
        checkException();
        return bdfUserStorage;
    }

    public RoleMap getRoleMap() {
        return roleMap;
    }

    public void loadManager(Object manager) {
        if (manager instanceof PermissionManager) {
            loadManager(manager, "permissionmanager", "permission-manager-bean", new PermissionManagerDOMReader(), false);
        } else if (manager instanceof PasswordManager) {
            loadManager(manager, "passwordmanager", "password-manager-bean", new PasswordManagerDOMReader(), true);
        } else if (manager instanceof SqlExportManager) {
            loadManager(manager, "sqlexportmanager", "sql-export-manager-bean", new SqlExportManagerDOMReader(), true);
        } else if (manager instanceof PolicyManager) {
            loadManager(manager, "policymanager", "policy-manager-bean", new PolicyManagerDOMReader(), false);
        } else if (manager instanceof ConfigurationManager) {
            loadManager(manager, "configurationmanager", "configuration-manager-bean", new ConfigurationManagerDOMReader(), false);
        } else if (manager instanceof UIManager) {
            loadUiManager((UIManager) manager);
        } else if (manager instanceof ScrutariExportManager) {
            StorageFile storageFile = BdfStorageUtils.getConfStorageFile(bdfDirectories, "rchfexportmanager");
            File file = storageFile.getFile();
            if (file.exists()) {
                CastorUtils.fillManager(file, manager);
                storageFile.delete();
            }
        }
    }

    private void loadManager(Object manager, String managerName, String beanName, ManagerDOMReader managerDOMReader, boolean delete) {
        StorageFile storageFile = BdfStorageUtils.getConfStorageFile(bdfDirectories, managerName);
        Document doc = storageFile.readDocument();
        if (doc == null) {
            return;
        }
        storageProcessLog.setCurrentURI(storageFile.toProcessURI());
        String tagName = doc.getDocumentElement().getTagName();
        if (tagName.equals(beanName)) {
            File file = storageFile.getFile();
            if (file.exists()) {
                CastorUtils.fillManager(file, manager);
                saveManager(manager);
            }
        } else {
            ManagerErrorHandler errorHandler = new ManagerErrorHandler(storageProcessLog);
            managerDOMReader.setErrorHandler(errorHandler);
            managerDOMReader.readManager(manager, doc.getDocumentElement());
        }
        if (delete) {
            storageFile.delete();
        }
    }

    private void loadUiManager(UIManager uiManager) {
        boolean done = false;
        StorageFile treesStorageFile = BdfStorageUtils.getUiStorageFile(bdfDirectories, "trees");
        Document doc = treesStorageFile.readDocument();
        if (doc != null) {
            done = true;
            storageProcessLog.setCurrentURI(treesStorageFile.toProcessURI());
            TreesDOMReader treesDOMReader = new TreesDOMReader(new TreesErrorHandler(storageProcessLog));
            treesDOMReader.readTrees(uiManager, doc.getDocumentElement());
        }
        LogUiErrorHandler uiErrorHandler = new LogUiErrorHandler(storageProcessLog);
        Basedefiches basedefiches = uiManager.getBdfServer().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            StorageFile corpusUiStorageFile = BdfStorageUtils.getUiStorageFile(bdfDirectories, corpus.getSubsetKey().toString());
            Document doccorpus = corpusUiStorageFile.readDocument();
            if (doccorpus != null) {
                done = true;
                CorpusUI corpusUI = uiManager.getCorpusUI(corpus);
                storageProcessLog.setCurrentURI(corpusUiStorageFile.toProcessURI());
                CorpusUiDOMReader corpusUiDOMReader = new CorpusUiDOMReader(corpusUI.getComponentUIEditor(), uiErrorHandler, corpusUI.getCorpusSubsetKey());
                corpusUiDOMReader.readCorpusUI(doccorpus.getDocumentElement());
            }
        }
        StorageFile storageFile = BdfStorageUtils.getConfStorageFile(bdfDirectories, "uimanager");
        File file = storageFile.getFile();
        if (file.exists()) {
            if (!done) {
                CastorUtils.fillManager(file, uiManager);
                saveManager(uiManager);
            }
            storageFile.delete();
        }
    }

    public AlterUI getAlterUI(CorpusUI corpusUI, String alterName) {
        StorageFile alterUiStorageFile = BdfStorageUtils.getAlterUiStorageFile(bdfDirectories, corpusUI.getCorpusSubsetKey(), alterName);
        if (!alterUiStorageFile.exists()) {
            return null;
        }
        Document doc = alterUiStorageFile.readDocument();
        if (doc == null) {
            return null;
        }
        Element documentElement = doc.getDocumentElement();
        String tagName = documentElement.getTagName();
        if (tagName.equals("ui-filter")) {
            DefaultUIFilterBuilder builder = new DefaultUIFilterBuilder(corpusUI);
            ComponentUiFilterDOMReader reader = new ComponentUiFilterDOMReader(builder, new NullUiErrorHandler());
            reader.readComponentUiFilter(documentElement);
            DefaultAlterUI defaultAlterUI = new DefaultAlterUI(builder.toComponentUIFilter());
            return defaultAlterUI;
        } else {
            return null;
        }
    }

    public void saveManager(Object manager) {
        checkException();
        if (manager instanceof PermissionManager) {
            saveManager(manager, "permissionmanager", new PermissionManagerXMLWriter());
        } else if (manager instanceof PolicyManager) {
            saveManager(manager, "policymanager", new PolicyManagerXMLWriter());
        } else if (manager instanceof ConfigurationManager) {
            saveManager(manager, "configurationmanager", new ConfigurationManagerXMLWriter());
        } else if (manager instanceof UIManager) {
            BdfStorageUtils.saveUIManager(bdfDirectories, (UIManager) manager);
        }
    }

    private void saveManager(Object manager, String managerName, ManagerXMLWriter managerXMLWriter) {
        BdfStorageUtils.saveManager(bdfDirectories, manager, managerName, managerXMLWriter);
    }

    public Basedefiches getBasedefiches() {
        return directoryBasedefiches;
    }

    public BasedefichesEditor getBasedefichesEditor() {
        checkException();
        return directoryBasedefiches.getInternalBasedefichesEditor();
    }


    private class InternalExceptionHandler implements ExceptionHandler {

        public void handleBdfStorageException(BdfStorageException bdfStorageException) {
            currentBdfStorageException = bdfStorageException;
        }

    }


    private static class NullUiErrorHandler implements UiErrorHandler {

        public void unknownTagWarning(String tagName) {
        }

        public void emptyAttributeError(String tagName, String attributeName) {
        }

        public void attributeSyntaxError(String tagName, String type, String attributeValue) {
        }

        public void emptyTagError(String tagName) {
        }

        public void textSyntaxError(String tagName, String text) {
        }

        public void unknownKeyError(String tagName, Object key) {
        }

    }


    private static class DirectoryCheck implements BdfStorage.Check {

        private List<ScrutariExportDef> scrutariExportDefList;
        private List<SqlExportDef> sqlExportDefList;
        private List<BdfUserDef> bdfUserDefList;
        private List<String> passwordList;

        public List<ScrutariExportDef> getScrutariExportDefList() {
            return scrutariExportDefList;
        }

        private void setScrutariExportDefList(List<ScrutariExportDef> scrutariExportDefList) {
            this.scrutariExportDefList = scrutariExportDefList;
        }

        public List<SqlExportDef> getSqlExportDefList() {
            return sqlExportDefList;
        }

        private void setSqlExportDefList(List<SqlExportDef> sqlExportDefList) {
            this.sqlExportDefList = sqlExportDefList;
        }

        public List<BdfUserDef> getBdfUserDefList() {
            return bdfUserDefList;
        }

        private void setBdfUserDefList(List<BdfUserDef> bdfUserDefList) {
            this.bdfUserDefList = bdfUserDefList;
        }

        public List<String> getPasswordList() {
            return passwordList;
        }

        private void setPasswordList(List<String> passwordList) {
            this.passwordList = passwordList;
        }

    }

}
