/* BdfServer_DirectoryStorage - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.roles.Role;
import fr.exemole.bdfserver.roles.RoleMap;
import fr.exemole.bdfserver.roles.xml.RoleDOMReader;
import fr.exemole.bdfserver.roles.xml.RoleXMLWriter;
import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.storage.directory.bdf.utils.RoleErrorHandler;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.TreeMap;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;
import net.mapeadores.util.text.StringUtils;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryRoleMap implements RoleMap, BdfDirectoryConstants {

    private TreeMap<String, Role> roleMap = new TreeMap<String, Role>();
    private BdfDirectories bdfDirectories;

    public DirectoryRoleMap(BdfDirectories bdfDirectories) {
        this.bdfDirectories = bdfDirectories;
    }

    public Role getRole(String roleName) {
        return roleMap.get(roleName);
    }

    public Role newRole(String roleName) {
        Role newRole = getRole(roleName);
        if (newRole == null) {
            if (!StringUtils.isTechnicalName(roleName, true)) {
                throw new IllegalArgumentException("wrong role name = " + roleName);
            }
            newRole = new Role(roleName);
            roleMap.put(roleName, newRole);
            saveRole(newRole);
            return newRole;
        } else {
            return null;
        }
    }

    public boolean removeRole(String roleName) {
        if (roleName.equals(DEFAULT_ROLE)) {
            return false;
        }
        Role role = getRole(roleName);
        if (role != null) {
            removeRole(role);
            return true;
        } else {
            return false;
        }
    }

    public void checkRoleChange(String roleName) {
        Role role = getRole(roleName);
        if (role != null) {
            saveRole(role);
        }
    }

    public Role[] getRoleArray(boolean defaultRoleIncluded) {
        int size = roleMap.size();
        int p = 0;
        if (!defaultRoleIncluded) {
            size = size - 1;
        } else {
            p = 1;
        }
        Role[] result = new Role[size];
        for (Role role : roleMap.values()) {
            if (role.getName().equals(DEFAULT_ROLE)) {
                if (defaultRoleIncluded) {
                    result[0] = role;
                }
            } else {
                result[p] = role;
                p++;
            }
        }
        return result;
    }

    void init(MutableProcessLog storageProcessLog) {
        String rootPath = ROLES;
        File roleDirectory = getRoleDirectory();
        File[] fileList = roleDirectory.listFiles();
        for (int i = 0; i < fileList.length; i++) {
            File f = fileList[i];
            String fileName = f.getName();
            if (fileName.endsWith(".xml")) {
                String roleName = fileName.substring(0, fileName.length() - 4);
                storageProcessLog.setCurrentURI(ProcessURI.fromFile(f));
                if (!roleName.equals(DEFAULT_ROLE)) {
                    if (!StringUtils.isTechnicalName(roleName, true)) {
                        ProcessLogItem rsl = storageProcessLog.newLogItemInstance();
                        rsl.setCategory("WARNING");
                        rsl.setKey("wrongRoleFileName");
                        rsl.setObject(fileName);
                        continue;
                    }
                }
                StorageFile storageFile = new StorageFile(bdfDirectories, rootPath + File.separator + fileName);
                Role role = new Role(roleName);
                Document doc = storageFile.readDocument();
                RoleDOMReader reader = new RoleDOMReader(new RoleErrorHandler(storageProcessLog));
                reader.readRole(role, doc.getDocumentElement());
                roleMap.put(roleName, role);
            }
        }
        if (!roleMap.containsKey(DEFAULT_ROLE)) {
            initDefaultRole();
        }
    }

    private void initDefaultRole() {
        Role role = new Role(DEFAULT_ROLE);
        roleMap.put(DEFAULT_ROLE, role);
    }

    private void saveRole(Role role) {
        StorageFile storageFile = getRoleFile(role);
        try {
            BufferedWriter buf = storageFile.getXMLWriter();
            RoleXMLWriter xmlWriter = new RoleXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(0);
            xmlWriter.appendRole(role);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }

    }

    private void removeRole(Role role) {
        StorageFile storageFile = getRoleFile(role);
        storageFile.delete();
        roleMap.remove(role.getName());
    }

    private File getRoleDirectory() {
        String rootPath = ROLES;
        File roleDirectory = new File(bdfDirectories.getBaseDirectory(), rootPath);
        if (roleDirectory.exists()) {
            if (!roleDirectory.isDirectory()) {
                throw new BdfStorageException(roleDirectory.getPath() + " is not a directory");
            }
        } else {
            roleDirectory.mkdirs();
        }
        return roleDirectory;
    }

    private StorageFile getRoleFile(Role role) {
        String rootPath = ROLES;
        getRoleDirectory();
        String fileName = role.getName() + ".xml";
        return new StorageFile(bdfDirectories, rootPath + File.separator + fileName);
    }

}
