/* BdfServer_DirectoryStorage - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.util.io.FileUtils;
import net.mapeadores.util.io.IOUtils;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationDirectory {

    private File dir;

    private IllustrationDirectory(File dir) {
        this.dir = dir;
    }

    public InputStream getImage() {
        File f = getImageFile();
        if (!f.exists()) {
            return null;
        }
        try {
            return new FileInputStream(f);
        } catch (IOException ioe) {
            throw new BdfStorageException("file : " + f.getAbsolutePath(), ioe);
        }
    }

    public void updateImage(InputStream inputStream, short imageType) {
        if (!dir.isDirectory()) {
            dir.delete();
        }
        dir.mkdirs();
        String extension = AlbumUtils.formatTypeToString(imageType);
        File[] fileArray = dir.listFiles();
        int length = fileArray.length;
        for (int i = 0; i < length; i++) {
            File f = fileArray[i];
            String name = f.getName();
            if (name.startsWith("img.")) {
                f.delete();
            }
        }
        String fileName = "img." + extension;
        File f = new File(dir, fileName);
        try {
            FileOutputStream outputStream = new FileOutputStream(f);
            IOUtils.copy(inputStream, outputStream);
            outputStream.close();
        } catch (IOException ioe) {
            throw new BdfStorageException("file : " + f.getAbsolutePath(), ioe);
        }
    }

    public boolean delete() {
        try {
            if (dir.exists()) {
                FileUtils.forceDelete(dir);
                return true;
            } else {
                return false;
            }
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    private File getImageFile() {
        File f = new File(dir, "img.png");
        if (!f.exists()) {
            f = new File(dir, "img.jpg");
        }
        return f;
    }

    public static IllustrationDirectory newInstance(BdfDirectories bdfDirectories, Illustration illustration) {
        return newInstance(bdfDirectories, illustration.getAlbum().getSubsetKey(), illustration.getIdalbum());
    }

    public static IllustrationDirectory newInstance(BdfDirectories bdfDirectories, SubsetKey albumSubsetKey, int idalbum) {
        File imgDirectory = new File(bdfDirectories.getBaseDirectory(), getPath(albumSubsetKey, idalbum));
        return new IllustrationDirectory(imgDirectory);
    }

    public static String getPath(SubsetKey albumSubsetKey, int idalbum) {
        StringBuilder buf = new StringBuilder();
        buf.append("basedefiches");
        buf.append(File.separatorChar);
        buf.append(SubsetKey.categoryToString(SubsetKey.CATEGORY_ALBUM));
        buf.append(File.separatorChar);
        buf.append(albumSubsetKey.getSubsetName());
        buf.append(File.separatorChar);
        buf.append("i.");
        if (idalbum < 1000) {
            buf.append("000");
        } else {
            if (idalbum < 10000) {
                buf.append("00");
            } else if (idalbum < 100000) {
                buf.append("0");
            }
            buf.append(idalbum / 1000);
        }
        buf.append(File.separatorChar);
        buf.append(idalbum);
        return buf.toString();
    }

    public static List<IllustrationInfo> getIllustrationInfoList(BdfDirectories bdfDirectories, SubsetKey albumSubsetKey) {
        List<IllustrationInfo> list = new ArrayList<IllustrationInfo>();
        File imageDirectory = new File(bdfDirectories.getBaseDirectory(), "basedefiches" + File.separator + SubsetKey.categoryToString(SubsetKey.CATEGORY_ALBUM) + File.separator + albumSubsetKey.getSubsetName());
        if ((!imageDirectory.exists()) || (!imageDirectory.isDirectory())) {
            return list;
        }
        File[] directories = imageDirectory.listFiles();
        for (int i = 0; i < directories.length; i++) {
            File imgDir = directories[i];
            if (!imgDir.isDirectory()) {
                continue;
            }
            String name = imgDir.getName();
            if ((name.length() < 2) || (!name.startsWith("i."))) {
                continue;
            }
            try {
                int directoryNumber = Integer.parseInt(name.substring(2));
                addIllustrationInfo(list, imgDir, directoryNumber);
            } catch (NumberFormatException nfe) {
            }
        }
        return list;
    }

    private static void addIllustrationInfo(List<IllustrationInfo> list, File intervalDirectory, int directoryNumber) {
        File[] directories = intervalDirectory.listFiles();
        for (int i = 0; i < directories.length; i++) {
            File directory = directories[i];
            if (!directory.isDirectory()) {
                continue;
            }
            String name = directory.getName();
            String errorMessage = null;
            try {
                int idalbum = Integer.parseInt(name);
                if (idalbum < 1) {
                    errorMessage = "idalbum < 1 (directoryNumber = " + directoryNumber;
                } else if (((idalbum < 1000) && (directoryNumber != 0)) || (idalbum / 1000 != directoryNumber)) {
                    errorMessage = "Wrong range : idalbum = " + idalbum + " / directoryNumber = " + directoryNumber;
                } else {
                    IllustrationInfo illustrationInfo = getIllustrationInfo(directory, idalbum);
                    if (illustrationInfo != null) {
                        list.add(illustrationInfo);
                    }
                }
            } catch (NumberFormatException nfe) {
            }
            if (errorMessage != null) {
                System.out.println(errorMessage);
            }
        }
    }

    private static IllustrationInfo getIllustrationInfo(File imageDir, int idalbum) {
        File imgFile = new File(imageDir, "img.png");
        short type = 0;
        if (imgFile.exists()) {
            type = AlbumConstants.PNG_FORMATTYPE;
        } else {
            imgFile = new File(imageDir, "img.jpg");
            if (imgFile.exists()) {
                type = AlbumConstants.JPG_FORMATTYPE;
            } else {
                return null;
            }
        }
        IllustrationInfo illustrationInfo = new IllustrationInfo(type, idalbum);
        return illustrationInfo;
    }


    public static class IllustrationInfo {

        private short type;
        private int idalbum;

        private IllustrationInfo(short type, int idalbum) {
            this.type = type;
            this.idalbum = idalbum;
        }

        public short getType() {
            return type;
        }

        public String getName() {
            return null;
        }

        public int getIdalbum() {
            return idalbum;
        }

    }

}
