/* BdfServer_DirectoryStorage - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.jdbm;

import fr.exemole.bdfserver.storage.BdfStorageException;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import java.io.IOException;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import jdbm.RecordManager;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesJdbm {

    private static final long JDBM_VERSION = 5L;
    private static final String FIELDKEY_CODER_NAME = "fieldkey_coder";
    private static final String SUBSETITEM_RECORDID_MAP_NAME = "subsetitem_recordid_map";
    private RecordManager recordManager;
    private int commitCounter = 0;
    private FieldKeyCoder fieldKeyCoder;
    private SubsetItemRecordIdMap subsetItemRecordIdMap;
    private EnteteFicheSerializer enteteFicheSerializer;
    private CorpsFicheSerializer corpsFicheSerializer;

    private BasedefichesJdbm(RecordManager recordManager) throws IOException {
        this.recordManager = recordManager;
        fieldKeyCoder = FieldKeyCoder.init(recordManager, FIELDKEY_CODER_NAME);
        subsetItemRecordIdMap = SubsetItemRecordIdMap.init(recordManager, SUBSETITEM_RECORDID_MAP_NAME);
        enteteFicheSerializer = new EnteteFicheSerializer(fieldKeyCoder);
        corpsFicheSerializer = new CorpsFicheSerializer(fieldKeyCoder);
    }

    public void setRecordManager(RecordManager recordManager) {
        this.recordManager = recordManager;
    }

    public static BasedefichesJdbm checkJdbmVersion(RecordManager recordManager) {
        try {
            long recordId = recordManager.getNamedObject("basedefiches_jdbm_version");
            if (recordId == 0) {
                return null;
            }
            Object obj = recordManager.fetch(recordId);
            if (!(obj instanceof Long)) {
                return null;
            }
            long l = ((Long) obj).longValue();
            if (l != JDBM_VERSION) {
                return null;
            }
            return new BasedefichesJdbm(recordManager);
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public static BasedefichesJdbm initEmptyInstance(RecordManager recordManager) {
        try {
            long recordId = recordManager.insert(new Long(JDBM_VERSION));
            recordManager.setNamedObject("basedefiches_jdbm_version", recordId);
            recordManager.commit();
            return new BasedefichesJdbm(recordManager);
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public Fiche getFiche(AbregeFiche abregeFiche, boolean ficheComplete) {
        try {
            if (ficheComplete) {
                return getFicheComplete(abregeFiche);
            } else {
                EnteteFiche enteteFiche = getEnteteFiche(abregeFiche);
                return enteteFiche.getFiche();
            }
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public void removeFiche(AbregeFiche abregeFiche) {
        try {
            SubsetItemRecordId recordId = subsetItemRecordIdMap.removeSubsetItem(abregeFiche);
            if (recordId == null) {
                return;
            }
            recordManager.delete(recordId.getRecid1());
            if (recordId.getType() == SubsetItemRecordId.DOUBLE_TYPE) {
                recordManager.delete(recordId.getRecid2());
            }
            saveSubsetItemRecordIdMap();
            testCommit();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public void updateFiche(AbregeFiche abregeFiche, Fiche fiche, boolean saveImmediatly) {
        int idcorpus = abregeFiche.getIdcorpus();
        EnteteFiche enteteFiche = new EnteteFiche(idcorpus, fiche);
        CorpsFiche corpsFiche = new CorpsFiche(fiche);
        try {
            SubsetItemRecordId recordIdObject = subsetItemRecordIdMap.getRecordId(abregeFiche);
            if (recordIdObject == null) {
                long recid1 = recordManager.insert(enteteFiche, enteteFicheSerializer);
                long recid2 = recordManager.insert(corpsFiche, corpsFicheSerializer);
                subsetItemRecordIdMap.add(abregeFiche.getCorpus(), idcorpus, recid1, recid2);
                if (saveImmediatly) {
                    saveSubsetItemRecordIdMap();
                }
            } else {
                long recid1 = recordIdObject.getRecid1();
                recordManager.update(recid1, enteteFiche, enteteFicheSerializer);
                long recid2 = recordIdObject.getRecid2();
                recordManager.update(recid2, corpsFiche, corpsFicheSerializer);
            }
            if (saveImmediatly) {
                saveFieldKeydCoder();
            }
            testCommit();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public void commit() {
        try {
            recordManager.commit();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public void endInit() {
        try {
            saveSubsetItemRecordIdMap();
            saveFieldKeydCoder();
            recordManager.commit();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    private EnteteFiche getEnteteFiche(AbregeFiche abregeFiche) throws IOException {
        SubsetItemRecordId recordId = subsetItemRecordIdMap.getRecordId(abregeFiche);
        if (recordId == null) {
            return null;
        }
        EnteteFiche enteteFiche = (EnteteFiche) recordManager.fetch(recordId.getRecid1(), enteteFicheSerializer);
        return enteteFiche;
    }

    private CorpsFiche getCorpsFiche(AbregeFiche abregeFiche) throws IOException {
        SubsetItemRecordId recordId = subsetItemRecordIdMap.getRecordId(abregeFiche);
        if (recordId == null) {
            return null;
        }
        CorpsFiche corpsFiche = (CorpsFiche) recordManager.fetch(recordId.getRecid2(), corpsFicheSerializer);
        return corpsFiche;
    }

    private Fiche getFicheComplete(AbregeFiche abregeFiche) throws IOException {
        SubsetItemRecordId recordId = subsetItemRecordIdMap.getRecordId(abregeFiche);
        if (recordId == null) {
            return null;
        }
        EnteteFiche enteteFiche = (EnteteFiche) recordManager.fetch(recordId.getRecid1(), enteteFicheSerializer);
        CorpsFiche corpsFiche = (CorpsFiche) recordManager.fetch(recordId.getRecid2(), corpsFicheSerializer);
        Fiche fiche = enteteFiche.getFiche();
        corpsFiche.setCorpsTo(fiche);
        return fiche;
    }

    private void testCommit() {
        commitCounter++;
        if (commitCounter > 500) {
            commit();
            commitCounter = 0;
        }
    }

    private void saveSubsetItemRecordIdMap() throws IOException {
        subsetItemRecordIdMap.save(recordManager, SUBSETITEM_RECORDID_MAP_NAME);
    }

    private void saveFieldKeydCoder() throws IOException {
        if (fieldKeyCoder.isWithChange()) {
            fieldKeyCoder.save(recordManager, FIELDKEY_CODER_NAME);
        }
    }

    public EnteteFiche[] getEnteteFicheArray(Corpus corpus) {
        Map<Integer, SubsetItemRecordId> map = subsetItemRecordIdMap.getSubsetItemMap(corpus);
        if ((map == null) || (map.size() == 0)) {
            return new EnteteFiche[0];
        }
        SortedMap<Integer, EnteteFiche> sortedMap = new TreeMap<Integer, EnteteFiche>();
        try {
            for (Map.Entry<Integer, SubsetItemRecordId> entry : map.entrySet()) {
                int idcorpus = entry.getKey();
                SubsetItemRecordId recordId = entry.getValue();
                EnteteFiche enteteFiche = (EnteteFiche) recordManager.fetch(recordId.getRecid1(), enteteFicheSerializer);
                sortedMap.put(idcorpus, enteteFiche);
            }
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
        EnteteFiche[] array = new EnteteFiche[sortedMap.size()];
        return sortedMap.values().toArray(array);
    }

}
