/* BdfServer_DirectoryStorage - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.tableexportstorage;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.storage.TableExportStorage;
import fr.exemole.bdfserver.storage.TableExportStorage.TableExportStorageInfo;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.tableexport.ColDefErrorHandler;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.ColDefListParser;
import net.mapeadores.basedefiches.tableexport.ColDefTestHandler;
import net.mapeadores.basedefiches.tableexport.ColDefUtils;
import net.mapeadores.basedefiches.tableexport.croisement.CroisementListParser;
import net.mapeadores.basedefiches.tableexport.croisement.CroisementTableExportInfo;
import net.mapeadores.basedefiches.tableexport.logging.SubsetProcessURI;
import net.mapeadores.basedefiches.tableexport.logging.TableExportLog;
import net.mapeadores.util.exceptions.NestedIOException;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryTableExportStorage implements TableExportStorage {

    private File etcTableExportDirectory;
    private File varTableExportDirectory;

    public DirectoryTableExportStorage(File etcTableExportDirectory, File varTableExportDirectory) {
        if ((etcTableExportDirectory.exists()) && (!etcTableExportDirectory.isDirectory())) {
            throw new IllegalArgumentException("tableExportDirectory is not a directory : " + etcTableExportDirectory.getPath());
        }
        this.etcTableExportDirectory = etcTableExportDirectory;
        this.varTableExportDirectory = varTableExportDirectory;
    }

    public List<TableExportStorageInfo> checkStorage(BdfServer bdfServer) {
        Map<String, TableExportStorageInfo> map = new LinkedHashMap<String, TableExportStorageInfo>();
        scanDir(varTableExportDirectory, map, bdfServer);
        scanDir(etcTableExportDirectory, map, bdfServer);
        List<TableExportStorageInfo> result = new ArrayList<TableExportStorageInfo>();
        result.addAll(map.values());
        return result;
    }

    public void saveSubsetColDefList(String tableExportName, SubsetKey subsetKey, ColDefList colDefList) throws IOException {
        File directory = new File(varTableExportDirectory, tableExportName);
        if (!directory.exists()) {
            directory.mkdirs();
        }
        File destination = new File(directory, subsetKey.toString() + ".txt");
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(destination), "UTF-8"));
        ColDefUtils.writeColDefList(writer, colDefList);
        writer.close();
    }

    private void scanDir(File dir, Map<String, TableExportStorageInfo> map, BdfServer bdfServer) {
        if (!dir.exists()) {
            return;
        }
        if (!dir.isDirectory()) {
            return;
        }
        File[] list = dir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File file = list[i];
            if (file.isDirectory()) {
                String name = file.getName();
                if (!name.startsWith(".")) {
                    if (map.containsKey(name)) {
                        continue;
                    }
                    try {
                        DirectoryTableExportStorageInfo directoryTableExportStorageInfo = scanDirectory(name, file, bdfServer);
                        if (directoryTableExportStorageInfo != null) {
                            map.put(name, directoryTableExportStorageInfo);
                        }
                    } catch (IOException ioe) {
                        throw new NestedIOException(ioe);
                    }
                }
            }
        }
    }

    private DirectoryTableExportStorageInfo scanDirectory(String name, File directory, BdfServer bdfServer) throws IOException {
        DirectoryTableExportStorageInfo directoryTableExportStorageInfo = new DirectoryTableExportStorageInfo(name);
        File[] files = directory.listFiles();
        int count = 0;
        for (int i = 0; i < files.length; i++) {
            File f = files[i];
            if (!f.isDirectory()) {
                String fileName = f.getName();
                try {
                    if (fileName.equals("croisement.txt")) {
                        directoryTableExportStorageInfo.processCroisement(f);
                        count++;
                    } else if (fileName.endsWith(".txt")) {
                        SubsetKey corpusKey = SubsetKey.parse(fileName.substring(0, fileName.length() - 4));
                        if ((corpusKey.isCorpusSubset()) || (corpusKey.isThesaurusSubset())) {
                            directoryTableExportStorageInfo.processSubset(corpusKey, f, bdfServer);
                            count++;
                        }
                    }
                } catch (java.text.ParseException pe) {
                }
            }
        }
        if (count == 0) {
            return null;
        }
        return directoryTableExportStorageInfo;
    }


    private class DirectoryTableExportStorageInfo implements TableExportStorageInfo {

        private String name;
        private TableExportLog tableExportLog;
        private List<SubsetInfo> infoList = new ArrayList<SubsetInfo>();
        private List<CroisementTableExportInfo> croisementInfoList = new ArrayList<CroisementTableExportInfo>();

        DirectoryTableExportStorageInfo(String name) {
            this.name = name;
            tableExportLog = new TableExportLog();
        }

        public String getName() {
            return name;
        }

        public int getSubsetCount() {
            return infoList.size();
        }

        public SubsetKey getSubsetKey(int i) {
            return infoList.get(i).getSubsetKey();
        }

        public ColDefList getColDefList(int i) {
            return infoList.get(i).getColDefList();
        }

        public TableExportLog getProcessLog() {
            return tableExportLog;
        }

        public int getCroisementInfoCount() {
            return croisementInfoList.size();
        }

        public CroisementTableExportInfo getCroisementInfo(int i) {
            return croisementInfoList.get(i);
        }

        private void processCroisement(File f) throws IOException {
            Reader reader = new InputStreamReader(new FileInputStream(f), "UTF-8");
            CroisementListParser.Result result = CroisementListParser.parse(reader);
            reader.close();
            croisementInfoList = result.getCroisementTableExportInfoList();
        }

        private void processSubset(SubsetKey subsetKey, File f, BdfServer bdfServer) throws IOException {
            Subset subset = bdfServer.getBasedefiches().getSubset(subsetKey);
            if (subset == null) {
                tableExportLog.addURILogItem(TableExportLog.UNKNOWNSUBSET_URI, SubsetProcessURI.fromFile(f, subsetKey));
                return;
            }
            SubsetInfo ci = new SubsetInfo();
            ci.process(subset, f);
            if (!ci.hasColDef()) {
                tableExportLog.addURILogItem(TableExportLog.EMPTY_URI, SubsetProcessURI.fromFile(f, subsetKey));
            } else {
                infoList.add(ci);
            }
        }


        private class SubsetInfo {

            private SubsetKey subsetKey;
            private ColDefList colDefList;

            private SubsetInfo() {
            }

            private void process(Subset subset, File f) throws IOException {
                subsetKey = subset.getSubsetKey();
                Reader reader = new InputStreamReader(new FileInputStream(f), "UTF-8");
                ColDefErrorHandler errorHandler = tableExportLog.getColDefListParserErrorHandler(SubsetProcessURI.fromFile(f, subsetKey));
                ColDefTestHandler testHandler = ColDefUtils.getSubsetTestHandler(subset, errorHandler);
                colDefList = ColDefListParser.parse(reader, testHandler, errorHandler);
                reader.close();
            }

            private SubsetKey getSubsetKey() {
                return subsetKey;
            }

            private boolean hasColDef() {
                return (colDefList.getColDefCount() != 0);
            }

            private ColDefList getColDefList() {
                return colDefList;
            }

        }

    }

}
