/* BdfServer_DirectoryStorage - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.transformationstorage;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerInfo;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.XslTransformerInfo;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.storage.TransformationStorage;
import fr.exemole.bdfserver.storage.TransformationStorage.TransformationStorageInfo;
import fr.exemole.bdfserver.storage.directory.transformationstorage.stream.MessageStreamTransformerErrorHandler;
import fr.exemole.bdfserver.xsltfactory.odcss.FileBdfCssSource;
import fr.exemole.bdfserver.xsltfactory.xslpattern.PatternErrorHandler;
import fr.exemole.bdfserver.xsltfactory.xslpattern.PatternUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.opendocument.css.elements.ElementMaps;
import net.mapeadores.opendocument.css.output.StylesOutput;
import net.mapeadores.opendocument.css.parse.CssParser;
import net.mapeadores.opendocument.css.parse.CssSource;
import net.mapeadores.opendocument.css.parse.LogCssErrorHandler;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.transform.ODStreamTransformer;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.sxio.SXIO;
import net.mapeadores.util.sxio.SXStreamTransformer;
import net.mapeadores.util.xml.XMLUtils;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryTransformationStorage implements TransformationStorage {

    private File etcTransformationDirectory;
    private File varTransformationDirectory;

    public DirectoryTransformationStorage(File etcTransformationDirectory, File varTransformationDirectory) {
        if ((etcTransformationDirectory.exists()) && (!etcTransformationDirectory.isDirectory())) {
            throw new IllegalArgumentException("transformationDirectory is not a directory : " + etcTransformationDirectory.getPath());
        }
        this.etcTransformationDirectory = etcTransformationDirectory;
        this.varTransformationDirectory = varTransformationDirectory;
    }

    public File getEtcCommonsDirectory() {
        return new File(etcTransformationDirectory, "_commons");
    }

    public File getVarCommonsDirectory() {
        return new File(varTransformationDirectory, "_commons");
    }

    public Source getTransformerSource(XslTransformerKey xslTransformerKey) {
        if (xslTransformerKey.isDefaultTransformerKey()) {
            throw new IllegalArgumentException("xslTransformerKey is default transformer key");
        }
        String path = xslTransformerKey.getTransformationKey().toString() + File.separator + xslTransformerKey.getTransformerName() + ".xsl";
        File compilFile = new File(varTransformationDirectory, "_compil" + File.separator + path);
        if (compilFile.exists()) {
            return new StreamSource(compilFile);
        }
        File varFile = new File(varTransformationDirectory, path);
        if (varFile.exists()) {
            return new StreamSource(varFile);
        }
        File etcFile = new File(etcTransformationDirectory, path);
        if (etcFile.exists()) {
            return new StreamSource(etcFile);
        }
        return null;
    }

    public List<TransformationStorageInfo> getTransformationStorageInfoList(BdfServer bdfServer) {
        Map<TransformationKey, InternalTransformationStorageInfo> map = new LinkedHashMap<TransformationKey, InternalTransformationStorageInfo>();
        scanDir(varTransformationDirectory, bdfServer, map);
        scanDir(etcTransformationDirectory, bdfServer, map);
        List<TransformationStorageInfo> result = new ArrayList<TransformationStorageInfo>();
        result.addAll(map.values());
        return result;
    }

    public StreamTransformer getStreamTransformer(BdfServer bdfServer, StreamTransformerInfo streamTransformerInfo) {
        StreamTransformerKey streamTransformerKey = streamTransformerInfo.getStreamTransformerKey();
        String extension = streamTransformerKey.getExtension();
        String path = streamTransformerKey.getTransformationKey().toString() + File.separator + streamTransformerKey.getTransformerName() + "." + extension;
        File dir = new File(varTransformationDirectory, path);
        String baseURI = "bdfvar:" + path + "/";
        if (!dir.exists()) {
            dir = new File(etcTransformationDirectory, path);
            baseURI = "bdfetc:" + path + "/";
        }
        if (!dir.exists()) {
            return null;
        }
        MessageStreamTransformerErrorHandler eh = new MessageStreamTransformerErrorHandler();
        StreamTransformer streamTransformer = null;
        try {
            if (OdUtils.isOdExtension(extension)) {
                File cssStyles = new File(dir, "styles.css");
                File xmlStyles = null;
                File xmlAutomaticStyles = null;
                if (cssStyles.exists()) {
                    File compilDir = getCompilDir(streamTransformerKey);
                    xmlStyles = new File(compilDir, "styles.xml");
                    xmlAutomaticStyles = new File(compilDir, "_content_automaticstyles.xml");
                    CssSource cssSource = new FileBdfCssSource(baseURI, cssStyles, bdfServer);
                    LogCssErrorHandler logCssErrorHandler = new LogCssErrorHandler();
                    ElementMaps elementMaps = CssParser.parse(cssSource, logCssErrorHandler);
                    StylesOutput.write(xmlStyles, elementMaps);
                    StylesOutput.writeAutomaticStyles(xmlAutomaticStyles, elementMaps);
                    int warningCount = logCssErrorHandler.getLogItemCount();
                    for (int i = 0; i < warningCount; i++) {
                        streamTransformerInfo.addWarning(logCssErrorHandler.getLogItem(i).toString());
                    }
                    if (logCssErrorHandler.hasFatalError()) {
                        streamTransformerInfo.setError(StreamTransformerInfo.PROCESS_ERROR, logCssErrorHandler.getFatalErrorLogItem().toString());
                        return null;
                    }
                } else {
                    xmlStyles = new File(dir, "styles.xml");
                    xmlAutomaticStyles = new File(dir, "_content_automaticstyles.xml");
                }
                File txtFile = new File(dir, "content.txt");
                File contentXslFile = null;
                if (txtFile.exists()) {
                    contentXslFile = compile(txtFile, streamTransformerInfo, bdfServer);
                    if (contentXslFile == null) {
                        return null;
                    }
                } else {
                    contentXslFile = new File(dir, "content.xsl");
                }
                ODStreamTransformer.Builder odStreamTransformerBuilder = new ODStreamTransformer.Builder(extension);
                byte[] stylesByteArray = testFile(xmlStyles, eh, false);
                if (stylesByteArray != null) {
                    odStreamTransformerBuilder.setStyles(stylesByteArray);
                }
                byte[] contentAutomaticStylesArray = testFile(xmlAutomaticStyles, eh, false);
                if (contentAutomaticStylesArray != null) {
                    odStreamTransformerBuilder.setContentAutomaticStyles(contentAutomaticStylesArray);
                }
                byte[] contentXsltByteArray = testFile(contentXslFile, eh, true);
                if (contentXsltByteArray != null) {
                    odStreamTransformerBuilder.setURIResolver(bdfServer.getURIResolver());
                    odStreamTransformerBuilder.setContentURI(contentXslFile.toURI().toString());
                    odStreamTransformerBuilder.setStreamTransformerErrorHandler(eh);
                    odStreamTransformerBuilder.setContentXsl(contentXsltByteArray);
                }
                streamTransformer = odStreamTransformerBuilder.toODStreamTransformer();
            } else {
                String mimeType = SXIO.getMimeType(extension);
                if (mimeType.length() > 0) {
                    streamTransformer = SXStreamTransformer.buildFromDirectory(extension, dir, eh);
                } else {
                    streamTransformerInfo.setError(StreamTransformerInfo.UNKNOWN_EXTENSION_ERROR, extension);
                    return null;
                }
            }
        } catch (IOException ioe) {
            streamTransformerInfo.setError(StreamTransformerInfo.PROCESS_ERROR, ioe.getMessage());
            return null;
        }
        if (eh.hasError()) {
            streamTransformerInfo.setError(StreamTransformerInfo.STREAM_TRANSFORMER_ERROR, eh.getErrorMessage());
            return null;
        }
        return streamTransformer;
    }

    private void scanDir(File dir, BdfServer bdfServer, Map<TransformationKey, InternalTransformationStorageInfo> map) {
        if (!dir.exists()) {
            return;
        }
        if (!dir.isDirectory()) {
            return;
        }
        File[] list = dir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File file = list[i];
            if (file.isDirectory()) {
                String name = file.getName();
                if ((name.equals("_commons")) || (name.equals("_compil"))) {
                    continue;
                }
                try {
                    TransformationKey transformationKey = TransformationKey.parse(name);
                    InternalTransformationStorageInfo transformationStorageInfo = map.get(transformationKey);
                    if (transformationStorageInfo == null) {
                        transformationStorageInfo = new InternalTransformationStorageInfo(transformationKey);
                        map.put(transformationKey, transformationStorageInfo);
                    }
                    transformationStorageInfo.scan(file, bdfServer);
                    map.put(transformationKey, transformationStorageInfo);
                } catch (java.text.ParseException pe) {
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }
        }
    }

    public void saveTransformer(Object transformerKey, InputStream inputStream, boolean pattern) throws IOException {
        if (transformerKey instanceof XslTransformerKey) {
            saveXslTransformer((XslTransformerKey) transformerKey, inputStream, pattern);
        } else if (transformerKey instanceof StreamTransformerKey) {
            saveStreamTransformer((StreamTransformerKey) transformerKey, inputStream, pattern);
        }
    }

    private void saveXslTransformer(XslTransformerKey xslTransformerKey, InputStream inputStream, boolean pattern) throws IOException {
        File dir = checkTransformationDirectory(xslTransformerKey.getTransformationKey());
        String transformerName = xslTransformerKey.getTransformerName();
        if (transformerName == null) {
            transformerName = "_default";
        }
        if (pattern) {
            transformerName = transformerName + ".txt";
        } else {
            transformerName = transformerName + ".xsl";
        }
        File destination = new File(dir, transformerName);
        if (destination.exists()) {
            destination.delete();
        }
        OutputStream outputStream = new FileOutputStream(destination);
        IOUtils.copy(inputStream, outputStream);
        outputStream.close();
    }

    private void saveStreamTransformer(StreamTransformerKey streamTransformerKey, InputStream inputStream, boolean pattern) throws IOException {
        File dir = checkTransformationDirectory(streamTransformerKey.getTransformationKey());
        String extension = streamTransformerKey.getExtension();
        String transformerName = streamTransformerKey.getTransformerName();
        if (transformerName == null) {
            transformerName = "_default";
        }
        File streamDir = new File(dir, transformerName + "." + extension);
        checkDirectory(streamDir);
        String fileName = null;
        if (extension.equals("odt")) {
            fileName = "content";
            if (pattern) {
                fileName = fileName + ".txt";
            } else {
                fileName = fileName + ".xsl";
            }
        }
        File destination = new File(streamDir, fileName);
        if (destination.exists()) {
            destination.delete();
        }
        OutputStream outputStream = new FileOutputStream(destination);
        IOUtils.copy(inputStream, outputStream);
        outputStream.close();
        if (extension.equals("odt")) {
            File cssFile = new File(streamDir, "styles.css");
            File stylesFile = new File(streamDir, "styles.xml");
            if ((!cssFile.exists()) && (!stylesFile.exists())) {
                OutputStream outputStream2 = new FileOutputStream(cssFile);
                IOUtils.copy(getDefaultOdCss(streamTransformerKey.getTransformationKey()), outputStream2);
                outputStream2.close();
            }
        }
    }

    private static InputStream getDefaultOdCss(TransformationKey transformationKey) {
        return DirectoryTransformationStorage.class.getResourceAsStream("_default_od.css");
    }

    private File checkTransformationDirectory(TransformationKey transformationKey) {
        String name = transformationKey.toString();
        File dir = new File(varTransformationDirectory, name);
        checkDirectory(dir);
        return dir;
    }

    private void checkDirectory(File dir) {
        if (!dir.exists()) {
            dir.mkdirs();
        } else if (!dir.isDirectory()) {
            dir.delete();
            dir.mkdirs();
        }
    }

    private void testXslt(File xslFile, XslTransformerInfo xslTransformerInfo) {
        xslTransformerInfo.setXslURI(xslFile.getPath());
        Object test = XMLUtils.testXml(xslFile);
        if (!(test.equals(Boolean.TRUE))) {
            xslTransformerInfo.setError(XslTransformerInfo.XMLOFXSL_ERROR, test.toString());
        }
    }

    private File compile(File txtFile, XslTransformerInfo xslTransformerInfo, BdfServer bdfServer) throws IOException {
        XslTransformerKey xslTransformerKey = xslTransformerInfo.getXslTransformerKey();
        File dir = new File(varTransformationDirectory, "_compil" + File.separator + xslTransformerKey.getTransformationKey().toString());
        dir.mkdirs();
        File compilFile = new File(dir, xslTransformerKey.getTransformerName() + ".xsl");
        PatternErrorHandler patternErrorHandler = PatternUtils.processPattern(txtFile, compilFile, bdfServer);
        xslTransformerInfo.setPatternWarning(patternErrorHandler.getPatternWarning());
        xslTransformerInfo.setPatternURI(txtFile.getPath());
        if (patternErrorHandler.hasError()) {
            xslTransformerInfo.setError(XslTransformerInfo.PATTERN_ERROR, patternErrorHandler.getPatternError());
            return null;
        } else {
            return compilFile;
        }
    }

    private File compile(File txtFile, StreamTransformerInfo streamTransformerInfo, BdfServer bdfServer) throws IOException {
        StreamTransformerKey streamTransformerKey = streamTransformerInfo.getStreamTransformerKey();
        File dir = getCompilDir(streamTransformerKey);
        String name = txtFile.getName();
        File compilFile = new File(dir, name.substring(0, name.length() - 4) + ".xsl");
        PatternErrorHandler patternErrorHandler = PatternUtils.processPattern(txtFile, compilFile, bdfServer);
        if (patternErrorHandler.hasWarning()) {
            streamTransformerInfo.addWarning(patternErrorHandler.getPatternWarning());
        }
        if (patternErrorHandler.hasError()) {
            streamTransformerInfo.setError(StreamTransformerInfo.PROCESS_ERROR, patternErrorHandler.getPatternError());
            return null;
        } else {
            return compilFile;
        }
    }

    private File getCompilDir(StreamTransformerKey streamTransformerKey) {
        File dir = new File(varTransformationDirectory, "_compil" + File.separator + streamTransformerKey.getTransformationKey().toString() + File.separator + streamTransformerKey.getTransformerName() + "." + streamTransformerKey.getExtension());
        dir.mkdirs();
        return dir;
    }


    private class InternalTransformationStorageInfo implements TransformationStorageInfo {

        private TransformationKey transformationKey;
        private SortedMap<String, XslTransformerInfo> xslMap = new TreeMap<String, XslTransformerInfo>();
        private SortedMap<String, StreamTransformerInfo> streamMap = new TreeMap<String, StreamTransformerInfo>();

        InternalTransformationStorageInfo(TransformationKey transformationKey) {
            this.transformationKey = transformationKey;
        }

        public TransformationKey getTransformationKey() {
            return transformationKey;
        }

        public List<XslTransformerInfo> getXslTransformerInfoList() {
            List<XslTransformerInfo> list = new ArrayList<XslTransformerInfo>();
            list.addAll(xslMap.values());
            return list;
        }

        public List<StreamTransformerInfo> getStreamTransformerInfoList() {
            List<StreamTransformerInfo> list = new ArrayList<StreamTransformerInfo>();
            list.addAll(streamMap.values());
            return list;
        }

        private void scan(File directory, BdfServer bdfServer) throws IOException {
            File[] list = directory.listFiles();
            Map<String, File> xslTempMap = new HashMap<String, File>();
            for (int i = 0; i < list.length; i++) {
                File file = list[i];
                if (file.isDirectory()) {
                    scanStreamTransformerDirectory(file, bdfServer);
                } else {
                    String fileName = file.getName();
                    int idx = fileName.lastIndexOf('.');
                    if (idx < 1) {
                        continue;
                    }
                    String extension = fileName.substring(idx + 1);
                    String templateName = fileName.substring(0, idx);
                    if (extension.equals("txt")) {
                        if (!xslMap.containsKey(templateName)) {
                            XslTransformerKey xslTransformerKey = new XslTransformerKey(transformationKey, templateName);
                            XslTransformerInfo xslTransformerInfo = new XslTransformerInfo(xslTransformerKey);
                            xslTransformerInfo.setFromPattern(true);
                            File xslFile = compile(file, xslTransformerInfo, bdfServer);
                            if (xslFile != null) {
                                testXslt(xslFile, xslTransformerInfo);
                            }
                            xslMap.put(templateName, xslTransformerInfo);
                        }
                    } else if (extension.equals("xsl")) {
                        xslTempMap.put(templateName, file);
                    }
                }
            }
            for (Map.Entry<String, File> entry : xslTempMap.entrySet()) {
                String templateName = entry.getKey();
                if (!xslMap.containsKey(templateName)) {
                    XslTransformerKey xslTransformerKey = new XslTransformerKey(transformationKey, templateName);
                    XslTransformerInfo xslTransformerInfo = new XslTransformerInfo(xslTransformerKey);
                    xslTransformerInfo.setFromPattern(false);
                    testXslt(entry.getValue(), xslTransformerInfo);
                    xslMap.put(templateName, xslTransformerInfo);
                }
            }
        }

        private void scanStreamTransformerDirectory(File directory, BdfServer bdfServer) throws IOException {
            String name = directory.getName();
            int idx = name.lastIndexOf('.');
            if (idx < 1) {
                return;
            }
            String extension = name.substring(idx + 1);
            String transformerName = name.substring(0, idx);
            StreamTransformerKey streamTransformerKey = new StreamTransformerKey(transformationKey, extension, transformerName);
            String stks = streamTransformerKey.toString();
            if (!streamMap.containsKey(stks)) {
                StreamTransformerInfo streamTransformerInfo = new StreamTransformerInfo(streamTransformerKey);
                if (!OdUtils.isOdExtension(extension)) {
                    String mimeType = SXIO.getMimeType(extension);
                    if (mimeType.length() == 0) {
                        streamTransformerInfo.setError(StreamTransformerInfo.UNKNOWN_EXTENSION_ERROR, extension);
                    }
                }
                streamMap.put(stks, streamTransformerInfo);
            }
        }

    }

    private byte[] testFile(File result, MessageStreamTransformerErrorHandler eh, boolean isMissingError) {
        if (!result.exists()) {
            if (isMissingError) {
                eh.missingResourceError(result.toURI().toString());
            }
            return null;
        }
        if (result.isDirectory()) {
            eh.wrongResourceError(result.toURI().toString(), "directory URI");
            return null;
        }
        Object xmlTest = XMLUtils.testXml(result);
        if (!xmlTest.equals(Boolean.TRUE)) {
            eh.wrongResourceError(result.toURI().toString(), "XML check error :: " + xmlTest.toString());
            return null;
        }
        try {
            InputStream inputStream = new FileInputStream(result);
            byte[] array = IOUtils.toByteArray(inputStream);
            inputStream.close();
            return array;
        } catch (IOException ioe) {
            eh.wrongResourceError(result.toURI().toString(), ioe.getMessage());
            return null;
        }
    }

}
