/* BdfServer_Get - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageConstants;
import fr.exemole.bdfserver.balayages.BalayageUnit;
import fr.exemole.bdfserver.balayages.BalayageUtils;
import fr.exemole.bdfserver.balayages.Output;
import fr.exemole.bdfserver.balayages.TransformManager;
import fr.exemole.bdfserver.balayages.TransformManagerBuilder;
import fr.exemole.bdfserver.balayages.engine.BalayageParameters;
import fr.exemole.bdfserver.balayages.engine.EngineUtils;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.ExtractionStream;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.managers.BalayageManager;
import fr.exemole.bdfserver.users.BdfUser;
import javax.xml.transform.Transformer;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.util.io.TransformerErrorListener;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.servlets.HttpAccessException;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.ValueResolver;


/**
 *
 * @author Vincent Calame
 */
public class BalayagesInstruction implements BdfInstruction, BdfInstructionConstants {

    private BdfServer bdfServer;
    private BdfContext bdfContext;
    private String filePath;

    public BalayagesInstruction(BdfContext bdfContext, String filePath) {
        this.bdfContext = bdfContext;
        this.filePath = filePath;
        this.bdfServer = bdfContext.getBdfServer();
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        BalayageUnitInitializer balayageUnitInitializer = new BalayageUnitInitializer(bdfServer);
        boolean done = balayageUnitInitializer.initFilePath(filePath);
        if (!done) {
            return null;
        }
        balayageUnitInitializer.initExtraction();
        ExtractionContext extractionContext = balayageUnitInitializer.getExtractionContext();
        ExtractionDef extractionDef = balayageUnitInitializer.getExtractionDef();
        Object dynamicObject = balayageUnitInitializer.getDynamicObject();
        if (balayageUnitInitializer.isXmlVersion()) {
            return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, extractionContext, extractionDef, dynamicObject, null);
        } else {
            if (balayageUnitInitializer.hasError()) {
                throw new HttpAccessException(HttpAccessException.INTERNAL_ERROR, balayageUnitInitializer.getErrorKey());
            }
            return ExtractionStream.toHtmlTransformationResponseHandler(bdfServer, bdfUser, extractionContext, extractionDef, dynamicObject, balayageUnitInitializer.getTransformer());
        }
    }


    private static class BalayageUnitInitializer {

        private BdfServer bdfServer;
        private Balayage balayage;
        private BalayageUnit balayageUnit;
        private SubsetKey subsetKey;
        private Subset subset;
        private SubsetItem subsetItem;
        private int langInteger = 0;
        private boolean xmlVersion;
        private ExtractionContext extractionContext;
        private ExtractionDef extractionDef;
        private Object dynamicObject;
        private String errorKey = null;
        private TransformManager transformManager;
        private Output output;

        private BalayageUnitInitializer(BdfServer bdfServer) {
            this.bdfServer = bdfServer;
        }

        public boolean isXmlVersion() {
            return xmlVersion;
        }

        public ExtractionContext getExtractionContext() {
            return extractionContext;
        }

        public ExtractionDef getExtractionDef() {
            return extractionDef;
        }

        public Object getDynamicObject() {
            return dynamicObject;
        }

        public Transformer getTransformer() {
            return transformManager.getTransformer(dynamicObject);
        }

        public boolean hasError() {
            return (errorKey != null);
        }

        public String getErrorKey() {
            return errorKey;
        }

        public boolean initFilePath(String filePath) {
            int ln = filePath.length();
            int idx1 = filePath.indexOf('/');
            if (idx1 == -1) {
                return false;
            }
            int idx2 = filePath.indexOf('/', idx1 + 1);
            if ((idx2 == -1) || (idx2 > (ln - 5))) {
                return false;
            }
            String balayageName = filePath.substring(0, idx1);
            boolean test = initBalayage(balayageName);
            if (!test) {
                return false;
            }
            String balayageUnitName = filePath.substring(idx1 + 1, idx2);
            test = initBalayageUnit(balayageUnitName);
            if (!test) {
                return false;
            }
            String baseName = filePath.substring(idx2 + 1);
            test = initSubset(baseName);
            if (!test) {
                return false;
            }
            return true;
        }

        public void initExtraction() {
            BalayageManager balayageManager = bdfServer.getBalayageManager();
            BalayageParameters balayageParameters = new BalayageParameters(balayage, bdfServer);
            extractionContext = balayageParameters.getExtractionContext();
            if (langInteger == 0) {
                langInteger = extractionContext.getLocalisationLists().getWorkingLangInteger(0);
            }
            String lang = LangInteger.toString(langInteger);
            short balayageType = balayageUnit.getType();
            String extractionPath = balayageUnit.getExtractionPath();
            if (extractionPath != null) {
                extractionDef = balayageManager.getExtractionDef(balayage, extractionPath);
            } else {
                extractionDef = balayageManager.getDefaultExtractionDef(balayageType);
            }
            if (dynamicObject == null) {
                dynamicObject = subset;
            }
            if (!xmlVersion) {
                if (!balayage.isWithTransformation()) {
                    xmlVersion = true;
                } else {
                    Output checkedOutput = checkOutput();
                    if (checkedOutput == null) {
                        xmlVersion = true;
                    } else {
                        initTransformer(checkedOutput, lang);
                    }
                }
            }
        }

        private void initTransformer(Output output, String lang) {
            InternalTransformerBuilderErrorHandler transformerBuilderErrorHandler = new InternalTransformerBuilderErrorHandler();
            TransformManagerBuilder transformerBuilder = new TransformManagerBuilder(bdfServer, balayage, transformerBuilderErrorHandler);
            transformManager = transformerBuilder.build(output);
            if (transformManager != null) {
                ValueResolver patternValueSet = EngineUtils.getValueResolver(dynamicObject);
                String path = EngineUtils.getPath(output, patternValueSet);
                String fileName = EngineUtils.getFileName(output, patternValueSet, balayageUnit.getType(), true);
                transformManager.setCurrentFileName(fileName);
                transformManager.setCurrentPath(path);
                transformManager.setCurrentLang(lang);
            }
        }

        private boolean initBalayage(String balayageName) {
            int idxtiret = balayageName.indexOf('-');
            if (idxtiret != -1) {
                try {
                    langInteger = LangInteger.parse(balayageName.substring(idxtiret + 1));
                } catch (LangIntegerException lie) {
                    return false;
                }
                balayageName = balayageName.substring(0, idxtiret);
            }
            balayage = bdfServer.getBalayageManager().getBalayage(balayageName);
            if (balayage == null) {
                return false;
            }
            return true;
        }

        private boolean initBalayageUnit(String balayageUnitName) {
            int idxtiret = balayageUnitName.indexOf('-');
            String outputName = null;
            if (idxtiret != -1) {
                if (idxtiret < 2) {
                    return false;
                }
                outputName = balayageUnitName.substring(idxtiret + 1);
                balayageUnitName = balayageUnitName.substring(0, idxtiret);
            }
            balayageUnit = BalayageUtils.getBalayageUnit(balayage, balayageUnitName);
            if (balayageUnit == null) {
                return false;
            }
            if (outputName != null) {
                output = BalayageUtils.getOutput(balayageUnit, outputName);
                if (output == null) {
                    return false;
                }
            }
            return true;
        }

        private boolean initSubset(String baseName) {
            if (baseName.endsWith(".xml")) {
                xmlVersion = true;
                baseName = baseName.substring(0, baseName.length() - 4);
            }
            int idxtiret = baseName.indexOf('-');
            boolean needId = false;
            try {
                switch (balayageUnit.getType()) {
                    case BalayageConstants.UNIQUE_TYPE:
                        if (baseName.equals("unique")) {
                            return true;
                        } else {
                            return false;
                        }
                    case BalayageConstants.CORPUS_TYPE:
                        if (idxtiret != -1) {
                            return false;
                        }
                        subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, baseName);
                        break;
                    case BalayageConstants.THESAURUS_TYPE:
                        if (idxtiret != -1) {
                            return false;
                        }
                        subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, baseName);
                        break;
                    case BalayageConstants.FICHE_TYPE:
                        if (idxtiret < 2) {
                            return false;
                        }
                        subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, baseName.substring(0, idxtiret));
                        needId = true;
                        break;
                    case BalayageConstants.MOTCLE_TYPE:
                        if (idxtiret < 2) {
                            return false;
                        }
                        subsetKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, baseName.substring(0, idxtiret));
                        needId = true;
                        break;
                    default:
                        return false;
                }
            } catch (java.text.ParseException pe) {
                return false;
            }
            subset = bdfServer.getBasedefiches().getSubset(subsetKey);
            if (subset == null) {
                return false;
            }
            if (needId) {
                try {
                    int subsetItemId = Integer.parseInt(baseName.substring(idxtiret + 1));
                    if (subsetItemId < 1) {
                        return false;
                    }
                    subsetItem = subset.getSubsetItemById(subsetItemId);
                    if (subsetItem == null) {
                        return false;
                    }
                } catch (NumberFormatException nfe) {
                    return false;
                }
            }
            return true;
        }

        private Output checkOutput() {
            if (output != null) {
                return output;
            }
            int count = balayageUnit.getOutputCount();
            if (count == 0) {
                return null;
            }
            return balayageUnit.getOutput(0);
        }


        private class InternalTransformerBuilderErrorHandler implements TransformManagerBuilder.ErrorHandler {

            public void transformerError(String xsltPath, TransformerErrorListener errorListener) {
                errorKey = "XSLT error : " + errorListener.getErrorMessage();
            }

            public void wrongXslPath(String xsltPath) {
                errorKey = "wrong xslt path : " + xsltPath;
            }

            public void undefinedXslPath() {
                errorKey = "xslt attribute of output element is not defined";
            }

            public void wrongBalayageUnitType() {
            }

        }

    }

}
