/* BdfServer_Get - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.actions.FicheAction;
import fr.exemole.bdfserver.actions.ReadFiche;
import fr.exemole.bdfserver.get.htmlproducers.bdfcontent.GeolocalisationHtmlProducer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.ExtractionStream;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusUtils;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.ExtractionContextUtils;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceLocalisationProvider;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.servlets.HttpAccessException;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;
import net.mapeadores.util.servlets.handlers.HtmlResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class FichesInstruction implements BdfInstruction, BdfInstructionConstants {

    private BdfServer bdfServer;
    private BdfContext bdfContext;
    private String filePath;
    private Corpus corpus;
    private AbregeFiche abregeFiche;
    private ResourceLocalisationProvider resourceLocalisationProvider;

    public FichesInstruction(BdfContext bdfContext, String filePath) {
        this.bdfContext = bdfContext;
        this.filePath = filePath;
        this.bdfServer = bdfContext.getBdfServer();
        this.resourceLocalisationProvider = (ResourceLocalisationProvider) bdfContext.getContextObject(BdfContextConstants.RESOURCELOCALISATIONPROVIDER_NAME);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        FilePathInfo filePathInfo = FilePathInfo.getSimpleFile(filePath);
        if (filePathInfo == null) {
            return null;
        }
        String baseName = filePathInfo.getBaseName();
        String extension = filePathInfo.getExtension();
        if (baseName.equals("geolocalisation")) {
            if (!extension.equals("html")) {
                return null;
            }
            HtmlProducer geolocalisationHtmlProducer = new GeolocalisationHtmlProducer(bdfContext, bdfUser);
            return new HtmlResponseHandler(geolocalisationHtmlProducer);
        }
        if (baseName.equals("compilation")) {
            return getCompilationResponseHandler(bdfUser, extension);
        }
        int idxtiret = baseName.indexOf('-');
        if (idxtiret < 2) {
            return null;
        }
        try {
            SubsetKey corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, baseName.substring(0, idxtiret));
            corpus = (Corpus) bdfServer.getBasedefiches().getSubset(corpusKey);
        } catch (java.text.ParseException pe) {
            return null;
        }
        if (corpus == null) {
            return null;
        }
        try {
            int idcorpus = Integer.parseInt(baseName.substring(idxtiret + 1));
            if (idcorpus < 1) {
                return null;
            }
            abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
            if (abregeFiche == null) {
                return null;
            }
        } catch (NumberFormatException nfe) {
            return null;
        }
        return getFicheResponseHandler(bdfUser, abregeFiche, extension);
    }

    private ResponseHandler getFicheResponseHandler(BdfUser bdfUser, AbregeFiche abregeFiche, String extension) {
        ReadFiche readAction = FicheAction.getReadFiche(abregeFiche);
        boolean canRead = bdfServer.getPermissionManager().canDo(bdfUser, readAction);
        if (!canRead) {
            if (extension.equals("html")) {
                ResourceLocalisation resourceLocalisation = resourceLocalisationProvider.getResourceLocalisation(bdfUser.getFormatLocale());
                return CommandMessageResponseHandler.getHtmlInstance(CommandMessage.error("global_err_readfiche", CorpusUtils.toKeyString(abregeFiche)), resourceLocalisation);
            } else {
                throw new HttpAccessException(HttpAccessException.FORBIDDEN_ACCESS);
            }
        }
        TransformationKey transformationKey = new TransformationKey(abregeFiche.getCorpus().getSubsetKey());
        if (extension.equals("xsl")) {
            return ExtractionStream.toRedirectResponseHandler(bdfServer, bdfUser, transformationKey);
        }
        LocalisationLists localisationLists = LocalisationLists.newInstance(bdfUser.getLocalisationInfo());
        ExtractionContext extractionContext = ExtractionContextUtils.newExtractionContext(bdfServer.getBasedefiches(), localisationLists);
        if (extension.equals("xml")) {
            return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, transformationKey, extractionContext, abregeFiche, null);
        } else if (extension.equals("xml+xsl")) {
            XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(transformationKey);
            String stylesheetPath = ExtractionStream.getTemplatePath(xslTransformerKey);
            return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, transformationKey, extractionContext, abregeFiche, stylesheetPath);
        } else if (extension.equals("html")) {
            return ExtractionStream.toHtmlTransformationResponseHandler(bdfServer, bdfUser, extractionContext, abregeFiche, transformationKey);
        } else {
            return ExtractionStream.toStreamTransformationResponseHandler(bdfServer, bdfUser, transformationKey, extension, extractionContext, abregeFiche);
        }
    }

    private ResponseHandler getCompilationResponseHandler(BdfUser bdfUser, String extension) {
        TransformationKey transformationKey = TransformationKey.COMPILATION_TRANSFORMATIONKEY;
        if (extension.equals("xsl")) {
            return ExtractionStream.toRedirectResponseHandler(bdfServer, bdfUser, transformationKey);
        }
        AbregeFicheSortedList abregeFicheSortedList = bdfUser.getAbregeFicheSortedList();
        LocalisationLists localisationLists = LocalisationLists.newInstance(bdfUser.getLocalisationInfo());
        ExtractionContext extractionContext = ExtractionContextUtils.newExtractionContext(bdfServer.getBasedefiches(), localisationLists, null, abregeFicheSortedList);
        if (extension.equals("xml")) {
            return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, transformationKey, extractionContext, null, null);
        } else if (extension.equals("xml+xsl")) {
            XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(transformationKey);
            String stylesheetPath = ExtractionStream.getTemplatePath(xslTransformerKey);
            return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, transformationKey, extractionContext, null, stylesheetPath);
        } else if (extension.equals("html")) {
            return ExtractionStream.toHtmlTransformationResponseHandler(bdfServer, bdfUser, extractionContext, null, transformationKey);
        } else {
            return ExtractionStream.toStreamTransformationResponseHandler(bdfServer, bdfUser, transformationKey, extension, extractionContext, null);
        }

    }

}


