/* BdfServer_Get - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.ExtractionStream;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.LexieMapOdsProducer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.LexieMapXmlProducer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.LibsXmlProducer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.StatsThesaurusOdsProducer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.ThesaurusOdsProducer;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.ExtractionContextUtils;
import net.mapeadores.basedefiches.misc.ThesaurusInverseFactory;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.localisation.ResourceLocalisationProvider;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.StreamResponseHandler;
import net.mapeadores.util.servlets.handlers.XmlResponseHandler;
import net.mapeadores.util.text.lexie.LexieMap;
import net.mapeadores.util.text.lexie.xml.LexieMapXMLWriter;
import net.mapeadores.util.xml.XmlProducer;


/**
 *
 * @author Vincent Calame
 */
public class MotsclesInstruction implements BdfInstruction, BdfInstructionConstants {

    public final static short STATS_OPTION = 1;
    public final static short STATSSEL_OPTION = 2;
    public final static short INVERSE_OPTION = 3;
    public final static short LIBS_OPTION = 4;
    public final static short TABLE_OPTION = 5;
    public final static short MOTCLE_OPTION = 6;
    private BdfServer bdfServer;
    private BdfContext bdfContext;
    private String filePath;
    private ResourceLocalisationProvider resourceLocalisationProvider;
    private Thesaurus thesaurus;
    private Motcle motcle;
    private short outputOption = 0;
    private int langInteger;
    private String baseName;
    private String extension;

    public MotsclesInstruction(BdfContext bdfContext, String filePath) {
        this.bdfContext = bdfContext;
        this.filePath = filePath;
        this.bdfServer = bdfContext.getBdfServer();
        this.resourceLocalisationProvider = (ResourceLocalisationProvider) bdfContext.getContextObject(BdfContextConstants.RESOURCELOCALISATIONPROVIDER_NAME);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        FilePathInfo filePathInfo = FilePathInfo.getSimpleFile(filePath);
        if (filePathInfo == null) {
            return null;
        }
        baseName = filePathInfo.getBaseName();
        extension = filePathInfo.getExtension();
        boolean optionInit = initOption();
        if (!optionInit) {
            return null;
        }
        TransformationKey transformationKey = getTransformationKey(outputOption);
        if (extension.equals("xsl")) {
            if (transformationKey != null) {
                return ExtractionStream.toRedirectResponseHandler(bdfServer, bdfUser, transformationKey);
            } else {
                return null;
            }
        }
        if (outputOption == INVERSE_OPTION) {
            return getInverseResponseHandler(bdfUser);
        }
        if (outputOption == LIBS_OPTION) {
            if (extension.equals("xml")) {
                XmlProducer libsXmlProducer = new LibsXmlProducer(thesaurus, langInteger);
                return new XmlResponseHandler(libsXmlProducer);
            } else {
                return null;
            }
        }
        FicheEligibility ficheEligibility = getFicheEligibility(bdfUser);
        LocalisationLists localisationLists = LocalisationLists.newInstance(bdfUser.getLocalisationInfo());
        ExtractionContext extractionContext = ExtractionContextUtils.newExtractionContext(bdfServer.getBasedefiches(), localisationLists, ficheEligibility);
        if (extension.equals("xml")) {
            switch (outputOption) {
                case STATS_OPTION:
                case STATSSEL_OPTION:
                    return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, transformationKey, extractionContext, thesaurus, null);
                case MOTCLE_OPTION:
                    return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, transformationKey, extractionContext, motcle, null);
                default:
                    return null;
            }
        } else if (extension.equals("html")) {
            switch (outputOption) {
                case STATS_OPTION:
                case STATSSEL_OPTION:
                    return ExtractionStream.toHtmlTransformationResponseHandler(bdfServer, bdfUser, extractionContext, thesaurus, transformationKey);
                case MOTCLE_OPTION:
                    return ExtractionStream.toHtmlTransformationResponseHandler(bdfServer, bdfUser, extractionContext, motcle, transformationKey);
                default:
                    return null;
            }
        } else if (extension.equals("ods")) {
            StreamProducer streamProducer = null;
            switch (outputOption) {
                case STATS_OPTION:
                case STATSSEL_OPTION:
                    streamProducer = new StatsThesaurusOdsProducer(thesaurus, ficheEligibility, bdfUser.getWorkingLangInteger());
                    break;
                case TABLE_OPTION:
                    streamProducer = new ThesaurusOdsProducer(thesaurus);
                    break;
                default:
                    return null;
            }
            return new StreamResponseHandler(streamProducer);
        } else {
            return null;
        }
    }

    private ResponseHandler getInverseResponseHandler(BdfUser bdfUser) {
        TransformationKey transformationKey = TransformationKey.INVERSETHESAURUS_TRANSFORMATIONKEY;
        int thesaurusLangInteger = bdfUser.getWorkingLangInteger();
        thesaurusLangInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, thesaurusLangInteger);
        LexieMap lexieMap = ThesaurusInverseFactory.getLibelleLexieMap(thesaurus, thesaurusLangInteger);
        if (extension.equals("xml")) {
            XmlProducer lexieMapXmlProducer = new LexieMapXmlProducer(lexieMap);
            return new XmlResponseHandler(lexieMapXmlProducer);
        } else if (extension.equals("html")) {
            return ExtractionStream.toHtmlTransformationResponseHandler(bdfServer, bdfUser, LexieMapXMLWriter.getLexieMapXML(lexieMap, false), transformationKey);
        } else if (extension.equals("ods")) {
            LexieMapOdsProducer lexieMapOdsProducer = new LexieMapOdsProducer(lexieMap, thesaurus);
            return new StreamResponseHandler(lexieMapOdsProducer);
        } else {
            return null;
        }
    }

    private boolean initOption() {
        int idxLang = baseName.indexOf('_');
        if (idxLang != -1) {
            try {
                langInteger = LangInteger.parse(baseName.substring(idxLang + 1));
            } catch (LangIntegerException lie) {
                return false;
            }
            baseName = baseName.substring(0, idxLang);
        }
        int idx = baseName.indexOf('-');
        String outputString = null;
        if (idx != -1) {
            outputString = baseName.substring(idx + 1);
            baseName = baseName.substring(0, idx);

        }
        thesaurus = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), baseName);
        if ((thesaurus == null) || (outputString == null)) {
            return false;
        }
        outputOption = toShort(outputString);
        if (outputOption == LIBS_OPTION) {
            if (langInteger == 0) {
                return false;
            } else {
                return true;
            }
        } else if (outputOption != 0) {
            return true;
        }
        try {
            int idths = Integer.parseInt(outputString);
            motcle = thesaurus.getMotcleByIdths(idths);
            if (motcle != null) {
                outputOption = MOTCLE_OPTION;
                return true;
            } else {
                return false;
            }
        } catch (NumberFormatException nfe) {
            return false;
        }
    }

    private FicheEligibility getFicheEligibility(BdfUser bdfUser) {
        switch (outputOption) {
            case STATSSEL_OPTION:
                return bdfUser.getAbregeFicheSortedList();
            default:
                return null;
        }
    }

    private static short toShort(String optionString) {
        if (optionString.equals("stats")) {
            return STATS_OPTION;
        }
        if (optionString.equals("statssel")) {
            return STATSSEL_OPTION;
        }
        if (optionString.equals("inverse")) {
            return INVERSE_OPTION;
        }
        if (optionString.equals("table")) {
            return TABLE_OPTION;
        }
        if (optionString.equals("libs")) {
            return LIBS_OPTION;
        }
        return 0;
    }

    private static TransformationKey getTransformationKey(short outputOption) {
        switch (outputOption) {
            case STATS_OPTION:
                return TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY;
            case STATSSEL_OPTION:
                return TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY;
            case INVERSE_OPTION:
                return TransformationKey.INVERSETHESAURUS_TRANSFORMATIONKEY;
            case MOTCLE_OPTION:
                return TransformationKey.MOTCLE_TRANSFORMATIONKEY;
            default:
                return null;
        }
    }

}
