/* @@@@@APPLICATION@@@@@ - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.stream;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfResourceUtils;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.xsltfactory.XsltResource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.association.VersionName;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.MimeTypeResolver;
import net.mapeadores.util.io.docstream.FileDocStream;
import net.mapeadores.util.io.docstream.WrappedDocStream;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.DocStreamResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class StreamInstruction implements BdfInstruction, StreamConstants, BdfInstructionConstants {

    private String filePath;
    private BdfContext bdfContext;
    private BdfServer bdfServer;
    private BdfUser bdfUser;
    private short dirType;

    public StreamInstruction(BdfContext bdfContext, short dirType, String filePath) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.dirType = dirType;
        this.filePath = filePath;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        this.bdfUser = bdfUser;
        switch (dirType) {
            case OUTPUT:
                return getOutputResponseHandler(filePath);
        }
        DocStream docStream = getDocStream();
        if (docStream != null) {
            return new DocStreamResponseHandler(docStream);
        } else {
            return null;
        }
    }

    public short getBdfUserNeed() {
        switch (dirType) {
            case PRV:
                return MANDATORY_BDFUSER;
            default:
                return NONE_BDFUSER;
        }
    }

    private DocStream getDocStream() {
        switch (dirType) {
            case JS:
                return getBdfResourceDocStream(BdfResourceConstants.JS_RESOURCE, filePath);
            case CSS:
                return getBdfResourceDocStream(BdfResourceConstants.CSS_RESOURCE, filePath);
            case RSC:
                return getBdfResourceDocStream(BdfResourceConstants.OTHER_RESOURCE, filePath);
            case THEME:
                return getBdfResourceDocStream(BdfResourceConstants.THEME_RESOURCE, filePath);
            case JQUERY:
                return getBdfResourceDocStream(BdfResourceConstants.JQUERY_RESOURCE, filePath);
            case XSLT:
                return getXsltDocStream(filePath);
            case DOCS:
                return getDocumentDocStream(filePath);
            case PRV:
                return getOutputDocStream(ConfigurationManager.PRIVATE_OUTPUTDIRECTORY, filePath);
            case PUB:
                return getOutputDocStream(ConfigurationManager.PUBLIC_OUTPUTDIRECTORY, filePath);
            default:
                throw new SwitchException("wrong dirType : " + dirType);
        }
    }

    private DocStream getBdfResourceDocStream(short resourceType, String resourcePath) {
        return BdfResourceUtils.getBdfResourceDocStream(bdfServer.getBdfResourceStorage(), resourceType, resourcePath);
    }

    /**
     * Retourne une feuille de style XSLT via le chemin /xslt.
     * Si le fichier existe dans le répertoire _xslt du contexte, ce fichier est renvoyé.
     *Traite également les requêtes /xslt/_common_{}.xsl et _localisation_{}
     */
    private DocStream getXsltDocStream(String filePath) {
        File xslFile = bdfServer.getContextFile("_xslt" + File.separator + filePath);
        String mimeType = null;
        if (filePath.endsWith(".txt")) {
            mimeType = "text/plain";
        } else {
            mimeType = "text/xml";
        }
        if (xslFile.exists()) {
            FileDocStream fileDocStream = new FileDocStream(xslFile);
            fileDocStream.setMimeType(mimeType);
            fileDocStream.setCharset("UTF-8");
            return fileDocStream;
        }
        String resourceName = filePath;
        int lastIndex = filePath.lastIndexOf('/');
        if (lastIndex != -1) {
            resourceName = filePath.substring(lastIndex + 1);
        }
        XsltResource innerResource = XsltResource.parse(resourceName);
        if (innerResource == null) {
            return null;
        }
        DocStream original = bdfServer.getInnerResourceDocStream(innerResource, true);
        if (original == null) {
            return null;
        }
        String oldMimeType = original.getMimeType();
        if (!oldMimeType.equals("application/octet-stream")) {
            return original;
        }
        WrappedDocStream newDocStream = new WrappedDocStream(original);
        newDocStream.setMimeType(mimeType);
        newDocStream.setCharset("UTF-8");
        return newDocStream;
    }

    private DocStream getDocumentDocStream(String filePath) {
        Addenda addenda = (Addenda) bdfServer.getBasedefiches().getSubset(SubsetKey.ADDENDA);
        VersionName.ParseResult parseResult = VersionName.parse(filePath, true);
        Document document = addenda.getDocumentByBasename(parseResult.getBasename());
        if (document == null) {
            return null;
        }
        Version version = document.getVersionByExtension(parseResult.getExtension());
        if (version == null) {
            return null;
        }
        VersionDocStream docStream = new VersionDocStream(version);
        String mimeType = getMimeTypeFromFileName(version.getVersionName().toString());
        docStream.setMimeType(mimeType);
        return docStream;
    }

    private DocStream getOutputDocStream(String outputName, String filePath) {
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        File outputDirectory = configurationManager.getOutputDirectory(outputName);
        File f = new File(outputDirectory, filePath);
        if (!f.exists()) {
            return null;
        }
        String mimeType = getMimeTypeFromFileName(f.getName());
        FileDocStream fileDocStream = new FileDocStream(f);
        fileDocStream.setMimeType(mimeType);
        return fileDocStream;
    }

    private ResponseHandler getOutputResponseHandler(String filePath) {
        File f = bdfServer.getOutputStorage().getOutputFile(filePath);
        if (!f.exists()) {
            return null;
        }
        if (f.isDirectory()) {
            if (!filePath.endsWith("/")) {
                return null;
            }
            return null;
        }
        String mimeType = getMimeTypeFromFileName(f.getName());
        FileDocStream fileDocStream = new FileDocStream(f);
        fileDocStream.setMimeType(mimeType);
        return new DocStreamResponseHandler(fileDocStream);
    }

    private String getMimeTypeFromFileName(String fileName) {
        MimeTypeResolver mimeTypeResolver = bdfServer.getBdfResourceStorage().getMimeTypeResolver();
        return mimeTypeResolver.getMimeTypeFromFileName(fileName);
    }


    private static class VersionDocStream implements DocStream {

        private Version version;
        private String mimeType;
        private String charset;

        VersionDocStream(Version version) {
            this.version = version;
        }

        public InputStream getInputStream() throws IOException {
            return version.getInputStream();
        }

        public int getLength() {
            return (int) version.getFileLength().getValue();
        }

        public String getMimeType() {
            return mimeType;
        }

        public void setMimeType(String mimeType) {
            this.mimeType = mimeType;
        }

        public String getCharset() {
            return charset;
        }

        public void setCharset(String charset) {
            this.charset = charset;
        }

        public long getLastModified() {
            return -1;
        }

    }

    public static short testDir(String dirPath) {
        if (dirPath.equals("js")) {
            return JS;
        } else if (dirPath.equals("xslt")) {
            return XSLT;
        } else if (dirPath.equals("docs")) {
            return DOCS;
        } else if (dirPath.equals("prv")) {
            return PRV;
        } else if (dirPath.equals("pub")) {
            return PUB;
        } else if (dirPath.equals("output")) {
            return OUTPUT;
        } else if (dirPath.equals("rsc")) {
            return RSC;
        } else if (dirPath.equals("theme")) {
            return THEME;
        } else if (dirPath.equals("jquery")) {
            return JQUERY;
        } else if (dirPath.equals("css")) {
            return CSS;
        } else {
            return 0;
        }
    }

}
