/* OdLib_Io - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.transform;

import java.io.ByteArrayInputStream;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.io.StreamTransformerErrorHandler;
import net.mapeadores.util.io.TransformerErrorListener;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.TransformZipEntrySource;
import net.mapeadores.opendocument.io.FileZipEntrySource;
import net.mapeadores.opendocument.io.ByteArrayZipEntrySource;
import java.util.Map;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.URIResolver;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.stream.StreamSource;




/**
 *
 * @author Vincent Calame
 */
public class ODStreamTransformer implements StreamTransformer {
    private short odType;
    private String extension;
    private String mimeType;
    private String name;
    private Templates contentTemplate;
    private byte[] stylesByteArray;
    private byte[] contentAutomaticStylesByteArray;
    private File stylesFile;
    private InternalURIResolver internalURIResolver = new InternalURIResolver();
    private TransformerErrorListener internalErrorListener = new TransformerErrorListener();
    
    private ODStreamTransformer(String extension) {
        this.extension = extension;
        odType = OdUtils.getOdType(extension);
        mimeType = OdUtils.getMimeType(odType);
    }
    
    public String getMimeType() {
        return mimeType;
    }
    
    public String getExtension() {
        return extension;
    }
    
    public String getName() {
        return name;
    }
    
    public void transform(Source source, OutputStream outputStream, Map<String, String> transformerParameters) throws TransformerException {
        OdZip odZip = new OdZip();
        odZip.setOdtype(odType);
        if (stylesByteArray != null) {
            odZip.setStylesZipEntrySource(new ByteArrayZipEntrySource(stylesByteArray));
        } else if (stylesFile != null) {
            odZip.setStylesZipEntrySource(new FileZipEntrySource(stylesFile));
        }
        Transformer transformer = contentTemplate.newTransformer();
        transformer.setURIResolver(internalURIResolver);
        transformer.setErrorListener(internalErrorListener);
        for (Map.Entry<String, String> entry : transformerParameters.entrySet()) {
            transformer.setParameter(entry.getKey(), entry.getValue());
        }
        TransformZipEntrySource transformZipEntrySource = new TransformZipEntrySource(transformer, source);
        odZip.setContentZipEntrySource(transformZipEntrySource);
        try {
            odZip.zip(outputStream);
        } catch(IOException ioe) {
            throw new TransformerException(ioe);
        } catch(NestedTransformerException nte) {
            throw nte.getTransformerException();
        }
    }
    
    public static class Builder {
        private ODStreamTransformer odStreamTransformer;
        private StreamTransformerErrorHandler streamTransformerErrorHandler;
        private String contentURI;
        private TransformerErrorListener factoryErrorListener = new TransformerErrorListener();
        
        public Builder(String extension) {
            odStreamTransformer = new ODStreamTransformer(extension);
        }
        
        public ODStreamTransformer toODStreamTransformer() {
            return odStreamTransformer;
        }
        
        public void setStyles(InputStream inputStream) throws IOException {
            setStyles(IOUtils.toByteArray(inputStream));
        }
        
        public void setStyles(byte[] byteArray) {
            odStreamTransformer.stylesByteArray = byteArray;
            odStreamTransformer.stylesFile = null;
        }
        
        public void setStyles(File stylesFile) {
            odStreamTransformer.stylesByteArray = null;
            odStreamTransformer.stylesFile = stylesFile;
        }
        
        public void setContentAutomaticStyles(InputStream inputStream) throws IOException {
            setContentAutomaticStyles(IOUtils.toByteArray(inputStream));
        }
        
        public void setContentAutomaticStyles(byte[] byteArray) throws IOException {
            odStreamTransformer.contentAutomaticStylesByteArray = byteArray;
        }
        
        public void setContentXsl(byte[] byteArray) {
            StreamSource streamSource = new StreamSource(new ByteArrayInputStream(byteArray));
            initTemplates(streamSource);
        }
        
        public void setContentXsl(InputStream inputStream) {
            StreamSource streamSource = new StreamSource(inputStream);
            initTemplates(streamSource);
        }
        
        public void setURIResolver(URIResolver uriResolver) {
            odStreamTransformer.internalURIResolver.setURIResolver(uriResolver);
        }
        
        public void setErrorListener(ErrorListener errorListener) {
            factoryErrorListener.setCustomErrorListener(errorListener);
            odStreamTransformer.internalErrorListener.setCustomErrorListener(errorListener);
        }
        
        private void initTemplates(Source source) {
            TransformerFactory tFactory = TransformerFactory.newInstance();
            tFactory.setURIResolver(odStreamTransformer.internalURIResolver);
            tFactory.setErrorListener(factoryErrorListener);
            try {
                odStreamTransformer.contentTemplate = tFactory.newTemplates(source);
            }
            catch(TransformerException te) {
                factoryErrorListener.checkFatalTransformerException(te);
            }
            if (factoryErrorListener.hasError()) {
                 streamTransformerErrorHandler.wrongResourceError(contentURI,  "XSL error :: " + factoryErrorListener.getErrorMessage());
            }
        }

        public void setStreamTransformerErrorHandler(StreamTransformerErrorHandler streamTransformerErrorHandler) {
            this.streamTransformerErrorHandler = streamTransformerErrorHandler;
        }

        public void setContentURI(String contentURI) {
            this.contentURI = contentURI;
        }

    }
    
    public byte[] getStylesByteArray() {
        return stylesByteArray;
    }
    
    public byte[] getContentAutomaticStylesByteArray() {
        return contentAutomaticStylesByteArray;
    }
    
    public File getStylesFile() {
        return stylesFile;
    }
    
    static private StreamSource getStreamSource(byte[] byteArray) {
        return new StreamSource(new ByteArrayInputStream(byteArray));
    }
    
    private class InternalURIResolver implements URIResolver {
        private URIResolver customURIResolver;
        
        private void setURIResolver(URIResolver customURIResolver) {
            this.customURIResolver = customURIResolver;
        }
        
        public Source resolve(String href,String base) throws TransformerException {
            if (href.equals("_content_automaticstyles.xml")) {
                byte[] byteArray = getContentAutomaticStylesByteArray();
                if (byteArray != null) return getStreamSource(byteArray);
            } else if (href.equals("styles.xml")) {
                byte[] byteArray = getStylesByteArray();
                if (byteArray != null) return getStreamSource(byteArray);
                File stylesFile =  getStylesFile();
                if (stylesFile != null) return new StreamSource(stylesFile);
            }
            if (customURIResolver != null) {
                return customURIResolver.resolve(href,base);
            } else {
                return null;
            }
        }
    }

}
