/* ScrutariDataExport - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.dataexport.xml;

import net.scrutari.dataexport.api.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class XmlScrutariDataExport implements ScrutariDataExport {

    private int currentState = START;
    private XmlWriter xmlWriter;
    private XmlBuilder xmlBuilder;
    private XmlFicheExport ficheExport = new XmlFicheExport();
    private XmlMotcleExport motcleExport = new XmlMotcleExport();
    private Map<String, Map<String, Integer>> indexationMap = new LinkedHashMap<String, Map<String, Integer>>();

    public XmlScrutariDataExport(XmlWriter xmlWriter) {
        this.xmlWriter = xmlWriter;
    }

    public BaseMetadataExport startExport() {
        if (currentState != START) {
            throw new ExportStateException(currentState, "startExport");
        }
        xmlWriter.openTag("base");
        currentState = BASEMETADATA;
        XmlBaseMetadataExport baseMetadataExport = new XmlBaseMetadataExport();
        this.xmlBuilder = baseMetadataExport;
        return baseMetadataExport;
    }

    public CorpusMetadataExport newCorpus(String corpusName) {
        commonTest("newCorpus");
        flushXmlBuilder();
        flushSubset();
        xmlWriter.openTagWithAttribute("corpus", "corpus-name", corpusName);
        currentState = CORPUSMETADATA;
        XmlCorpusMetadataExport corpusMetadataExport = new XmlCorpusMetadataExport();
        this.xmlBuilder = corpusMetadataExport;
        return corpusMetadataExport;
    }

    public FicheExport newFiche(String ficheId) {
        switch (currentState) {
            case START:
            case END:
            case BASEMETADATA:
            case THESAURUSMETADATA:
            case MOTCLE:
                throw new ExportStateException(currentState, "newFiche");
        }
        flushXmlBuilder();
        currentState = FICHE;
        ficheExport.reinit(ficheId);
        this.xmlBuilder = ficheExport;
        return ficheExport;
    }

    public ThesaurusMetadataExport newThesaurus(String thesaurusName) {
        commonTest("newThesaurus");
        flushXmlBuilder();
        flushSubset();
        xmlWriter.openTagWithAttribute("thesaurus", "thesaurus-name", thesaurusName);
        currentState = THESAURUSMETADATA;
        XmlThesaurusMetadataExport thesaurusMetadataExport = new XmlThesaurusMetadataExport();
        this.xmlBuilder = thesaurusMetadataExport;
        return thesaurusMetadataExport;
    }

    public MotcleExport newMotcle(String motcleId) {
        switch (currentState) {
            case START:
            case END:
            case BASEMETADATA:
            case CORPUSMETADATA:
            case FICHE:
                throw new ExportStateException(currentState, "newFiche");
        }
        flushXmlBuilder();
        currentState = MOTCLE;
        motcleExport.reinit(motcleId);
        this.xmlBuilder = motcleExport;
        return motcleExport;
    }

    public void addIndexation(String corpusName, String ficheId, String thesaurusName, String motcleId, int poids) {
        commonTest("addIndexation");
        if (poids < 1) {
            poids = 1;
        }
        Map<String, Integer> map = getIndexationGroupMap(corpusName, thesaurusName);
        String key = ficheId + '\t' + motcleId;
        map.put(key, new Integer(poids));
    }

    public void endExport() {
        commonTest("endExport");
        flushXmlBuilder();
        flushSubset();
        writeIndexation();
        xmlWriter.closeTag("base");
        currentState = END;
    }

    public int getState() {
        return currentState;
    }

    private void writeIndexation() {
        String[] indexationGroupNameArray = {"corpus-path", "thesaurus-path"};
        String[] indexationNameArray = {"fiche-id", "motcle-id", "poids"};
        for (Map.Entry<String, Map<String, Integer>> mapEntry : indexationMap.entrySet()) {
            String key = mapEntry.getKey();
            int idx = key.indexOf('\t');
            String[] indexationGroupValueArray = new String[2];
            indexationGroupValueArray[0] = key.substring(0, idx);
            indexationGroupValueArray[1] = key.substring(idx + 1);
            xmlWriter.openTagWithAttributes("indexation-group", indexationGroupNameArray, indexationGroupValueArray);
            Map<String, Integer> map = mapEntry.getValue();
            for (Map.Entry<String, Integer> entry2 : map.entrySet()) {
                String key2 = entry2.getKey();
                int idx2 = key2.indexOf('\t');
                String[] indexationValueArray = new String[3];
                indexationValueArray[0] = key2.substring(0, idx2);
                indexationValueArray[1] = key2.substring(idx2 + 1);
                int poids = entry2.getValue().intValue();
                String poidsString = null;
                if (poids > 1) {
                    poidsString = String.valueOf(poids);
                }
                indexationValueArray[2] = poidsString;
                xmlWriter.addEmptyElement("indexation", indexationNameArray, indexationValueArray);
            }
            xmlWriter.closeTag("indexation-group");
        }
    }

    private Map<String, Integer> getIndexationGroupMap(String corpusName, String thesaurusName) {
        String key = corpusName + '\t' + thesaurusName;
        Map<String, Integer> map = indexationMap.get(key);
        if (map == null) {
            map = new LinkedHashMap<String, Integer>();
            indexationMap.put(key, map);
        }
        return map;
    }

    private void commonTest(String methodName) {
        switch (currentState) {
            case START:
            case END:
                throw new ExportStateException(currentState, methodName);
        }
    }

    private void flushXmlBuilder() {
        if (xmlBuilder != null) {
            xmlBuilder.writeXML(xmlWriter);
            xmlBuilder = null;
        }
    }

    private void flushSubset() {
        switch (currentState) {
            case CORPUSMETADATA:
            case FICHE:
                xmlWriter.closeTag("corpus");
                break;
            case THESAURUSMETADATA:
            case MOTCLE:
                xmlWriter.closeTag("thesaurus");
                break;
        }
    }

}
