/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.impl.InstructionParser;


/**
 *
 * @author Vincent Calame
 */
public class AttributeParser {

    private static final InstructionParser.ErrorHandler NULL_BRIDGE = new NullErrorHandlerBridge();

    public static AttributeHolder parseAttributeHolder(String s, NameSpaceString nameSpace, ErrorHandler errorHandler) {
        InstructionParser.ErrorHandler instructionParserErrorHandler = NULL_BRIDGE;
        if (errorHandler != null) {
            instructionParserErrorHandler = new ErrorHandlerBridge(errorHandler);
        }
        Instruction instruction = InstructionParser.parse(s, instructionParserErrorHandler);
        if (instruction == null) {
            return null;
        }
        int count = instruction.getLength();
        if (count == 1) {
            Argument argument = instruction.getArgument(0);
            CleanedString cleanedString = CleanedString.newInstance(argument.getValue());
            if (cleanedString == null) {
                return null;
            }
            try {
                AttributeKey attributeKey = AttributeKey.parse(argument.getKey(), nameSpace);
                Attribute attribute = AttributeBuilder.toAttribute(attributeKey, cleanedString);
                return AttributeUtils.toSingletonAttributeHolder(attribute);
            } catch (ParseException pe) {
                return null;
            }

        }
        Map<AttributeKey, Attribute> attributeMap = new HashMap<AttributeKey, Attribute>();
        List<Attribute> attributeList = new ArrayList<Attribute>();
        for (int i = 0; i < count; i++) {
            Argument argument = instruction.getArgument(i);
            CleanedString cleanedString = CleanedString.newInstance(argument.getValue());
            if (cleanedString == null) {
                continue;
            }
            try {
                AttributeKey attributeKey = AttributeKey.parse(argument.getKey(), nameSpace);
                Attribute attribute = AttributeBuilder.toAttribute(attributeKey, cleanedString);
                attributeMap.put(attributeKey, attribute);
                attributeList.add(attribute);
            } catch (ParseException pe) {
                continue;
            }
        }
        int size = attributeList.size();
        if (size == 0) {
            return null;
        }
        Attribute[] attributeArray = attributeList.toArray(new Attribute[size]);
        return new InternalAttributeHolder(attributeMap, attributeArray);
    }


    public static interface ErrorHandler {

        public void invalidAsciiCharacterError(String part, int row, int col);

        public void invalidEndCharacterError(String part, int row, int col);

        public void invalidSeparatorCharacterError(String part, int row, int col);

    }


    private static class InternalAttributeHolder implements AttributeHolder {

        private Map<AttributeKey, Attribute> attributeMap;
        private Attribute[] attributeArray;

        private InternalAttributeHolder(Map<AttributeKey, Attribute> attributeMap, Attribute[] attributeArray) {
            this.attributeMap = attributeMap;
            this.attributeArray = attributeArray;
        }

        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeMap.get(key);
        }

        public int getAttributeCount() {
            return attributeArray.length;
        }

        public Attribute getAttributeByIndex(int i) {
            return attributeArray[i];
        }

    }


    private static class NullErrorHandlerBridge implements InstructionParser.ErrorHandler {

        public void invalidAsciiCharacterError(String part, int row, int col) {
        }

        public void invalidEndCharacterError(String part, int row, int col) {
        }

        public void invalidSeparatorCharacterError(String part, int row, int col) {
        }

    }


    private static class ErrorHandlerBridge implements InstructionParser.ErrorHandler {

        private ErrorHandler errorHandler;

        private ErrorHandlerBridge(ErrorHandler errorHandler) {
            this.errorHandler = errorHandler;
        }

        public void invalidAsciiCharacterError(String part, int row, int col) {
            errorHandler.invalidAsciiCharacterError(part, row, col);
        }

        public void invalidEndCharacterError(String part, int row, int col) {
            errorHandler.invalidEndCharacterError(part, row, col);
        }

        public void invalidSeparatorCharacterError(String part, int row, int col) {
            errorHandler.invalidSeparatorCharacterError(part, row, col);
        }

    }

}
