/* UtilLib - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.exceptions.SwitchException;
import java.text.Collator;


/**
 *
 * @author Vincent Calame
 */
public class ConditionUtils implements ConditionConstants {

    private ConditionUtils() {
    }

    public static short modeToShort(String s, short defaultMode) {
        if (s == null) {
            return defaultMode;
        }
        if (s.equals("and")) {
            return ConditionUtils.MODE_AND;
        } else if (s.equals("or")) {
            return ConditionUtils.MODE_OR;
        }
        return defaultMode;
    }

    public static String modeToString(short mode) {
        switch (mode) {
            case MODE_AND:
                return "and";
            case MODE_OR:
                return "or";
            default:
                throw new IllegalArgumentException();
        }
    }

    public static short getModeInverse(short mode) {
        if (mode == MODE_AND) {
            return MODE_OR;
        }
        return MODE_AND;
    }

    public static boolean accept(TextTest textTest, String contentString) {
        if (contentString == null) {
            contentString = "";
        }
        short testType = textTest.getTestType();
        if (testType == EST_VIDE) {
            return (contentString.length() == 0);
        }
        if (testType == EST_NON_VIDE) {
            return (contentString.length() > 0);
        }
        String testString = textTest.getText().toLowerCase();
        contentString = contentString.toLowerCase();
        if (isExcludingPartTestType(testType)) {
            switch (testType) {
                case NE_CORRESPOND_PAS_A:
                    return !contentString.equals(testString);
                case NE_COMMENCE_PAS_PAR:
                    return !contentString.startsWith(testString);
                case NE_FINIT_PAS_PAR:
                    return !contentString.endsWith(testString);
                default:
                    return !contentString.contains(testString);
            }
        } else {
            switch (testType) {
                case CORRESPOND_A:
                    return contentString.equals(testString);
                case COMMENCE_PAR:
                    return contentString.startsWith(testString);
                case FINIT_PAR:
                    return contentString.endsWith(testString);
                default:
                    return contentString.contains(testString);
            }
        }
    }

    /**
     * Si la chaine ne contient aucun des caractères suivants : !*;
     * alors les espaces de la chaine sont traités comme des séparateurs
     */
    public static Condition parseSimpleCondition(String conditionString) {
        if (conditionString == null) {
            return null;
        }
        conditionString = conditionString.trim();
        int length = conditionString.length();
        if (length == 0) {
            return null;
        }
        short mode = MODE_AND;
        if (conditionString.charAt(0) == '|') {
            if (length == 1) {
                return null;
            }
            mode = MODE_OR;
            conditionString = conditionString.substring(1).trim();
            length = conditionString.length();
        }
        if ((length == 1) && (conditionString.charAt(0) == '!')) {
            return null;
        }
        for (int i = 0; i < length; i++) {
            char carac = conditionString.charAt(i);
            switch (carac) {
                case '*':
                case ';':
                    return parseCondition(conditionString, mode);
            }
        }
        DefaultCondition condition = new DefaultCondition(mode);
        String[] tokens = StringUtils.getCleanedTokens(conditionString, ' ');
        int tokenCount = tokens.length;
        for (int i = 0; i < tokenCount; i++) {
            String token = tokens[i];
            short testType = CONTIENT;
            if (token.charAt(0) == '!') {
                if (token.length() == 1) {
                    continue;
                }
                testType = NE_CONTIENT_PAS;
                token = token.substring(1);
            }
            condition.innerAddTextTest(new DefaultTextTest(testType, token));
        }
        return condition;
    }

    /**
     * Reconstruit une instance de Condition à partir d'une chaîne.
     * Le point-virgule est le séparateur des différentes conditions.
     */
    public static Condition parseCondition(String conditionString, short mode) {
        DefaultCondition condition = new DefaultCondition(mode);
        int length = conditionString.length();
        boolean previousEscaped = false;
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < length; i++) {
            char carac = conditionString.charAt(i);
            if (previousEscaped) {
                if (carac != ';') {
                    buf.append('\\');
                }
                buf.append(carac);
                previousEscaped = false;
            } else {
                if (carac == ';') {
                    if (buf.length() > 0) {
                        String testString = buf.toString();
                        TextTest textTest = parseTextTest(testString);
                        if (textTest != null) {
                            condition.innerAddTextTest(parseTextTest(testString));
                        }
                        buf = new StringBuilder();
                    }

                } else if (carac == '\\') {
                    previousEscaped = true;
                } else {
                    buf.append(carac);
                }
            }
        }
        if (previousEscaped) {
            buf.append('\\');
        }
        String testString = buf.toString();
        TextTest textTest = parseTextTest(testString);
        if (textTest != null) {
            condition.innerAddTextTest(textTest);
        }
        return condition;
    }

    /**
     * Reconstruit une instance de ContentConditionBean à partir d'une chaîne.
     */
    public static TextTest parseTextTest(String testString) {
        if (testString == null) {
            return null;
        }
        testString = testString.trim();
        if (testString.length() == 0) {
            return null;
        }
        boolean non = (testString.charAt(0) == '!');
        if ((non) && (testString.length() == 1)) {
            return new DefaultTextTest(EST_VIDE, "");
        }
        if (non) {
            testString = testString.substring(1);
        }
        String text = "";
        short testType = 0;
        boolean commence = false;
        if ((testString.charAt(0) == '*') || (testString.charAt(0) == '%')) {
            commence = true;
            text = testString.substring(1);
        } else if (testString.startsWith("..")) {
            commence = true;
            text = testString.substring(2);
        } else {
            text = testString;
        }
        text = StringUtils.cleanString(text);
        int taille = text.length();
        if (text.length() == 0) {
            if (non) {
                testType = EST_VIDE;
            } else {
                testType = EST_NON_VIDE;
            }
        } else {
            boolean finit = false;
            if ((text.charAt(taille - 1) == '*') || (text.charAt(taille - 1) == '%')) {
                text = text.substring(0, taille - 1);
                finit = true;
            } else if (text.endsWith("..")) {
                text = text.substring(0, taille - 2);
                finit = true;
            }
            if (commence) {
                if (finit) {
                    if (non) {
                        testType = NE_CONTIENT_PAS;
                    } else {
                        testType = CONTIENT;
                    }
                } else {
                    if (non) {
                        testType = NE_FINIT_PAS_PAR;
                    } else {
                        testType = FINIT_PAR;
                    }
                }
            } else {
                if (finit) {
                    if (non) {
                        testType = NE_COMMENCE_PAS_PAR;
                    } else {
                        testType = COMMENCE_PAR;
                    }
                } else {
                    if (non) {
                        testType = NE_CORRESPOND_PAS_A;
                    } else {
                        testType = CORRESPOND_A;
                    }
                }
            }
        }
        return new DefaultTextTest(testType, text);
    }

    public static TextTest toOpposite(TextTest textTest) {
        short oppositeType = getOppositeTestType(textTest.getTestType());
        return new DefaultTextTest(oppositeType, textTest.getText());
    }

    public static short getOppositeTestType(short testType) {
        switch (testType) {
            case COMMENCE_PAR:
                return NE_COMMENCE_PAS_PAR;
            case FINIT_PAR:
                return NE_FINIT_PAS_PAR;
            case CONTIENT:
                return NE_CONTIENT_PAS;
            case CORRESPOND_A:
                return NE_CORRESPOND_PAS_A;
            case EST_VIDE:
                return EST_NON_VIDE;
            case NE_COMMENCE_PAS_PAR:
                return COMMENCE_PAR;
            case NE_FINIT_PAS_PAR:
                return FINIT_PAR;
            case NE_CONTIENT_PAS:
                return CONTIENT;
            case NE_CORRESPOND_PAS_A:
                return CORRESPOND_A;
            case EST_NON_VIDE:
                return EST_VIDE;
            default:
                return -1;
        }
    }

    public static String toSQL(TextTest textTest) {
        StringBuilder buf = new StringBuilder();
        switch (textTest.getTestType()) {
            case EST_VIDE:
                buf.append(" IS NULL");
                break;
            case EST_NON_VIDE:
                buf.append(" IS NOT NULL");
                break;
            case COMMENCE_PAR:
                buf.append(" LIKE '");
                preparation(buf, textTest);
                buf.append("%'");
                break;
            case FINIT_PAR:
                buf.append(" LIKE '%");
                preparation(buf, textTest);
                buf.append("'");
                break;
            case CONTIENT:
                buf.append(" LIKE '%");
                preparation(buf, textTest);
                buf.append("%'");
                break;
            case CORRESPOND_A:
                buf.append(" LIKE '");
                preparation(buf, textTest);
                buf.append("'");
                break;
            case NE_COMMENCE_PAS_PAR:
                buf.append(" NOT LIKE '%");
                preparation(buf, textTest);
                buf.append("%'");
                break;
            case NE_FINIT_PAS_PAR:
                buf.append(" NOT LIKE '%");
                preparation(buf, textTest);
                buf.append("'");
                break;
            case NE_CONTIENT_PAS:
                buf.append(" NOT LIKE '%");
                preparation(buf, textTest);
                buf.append("%'");
                break;
            case NE_CORRESPOND_PAS_A:
                buf.append(" NOT LIKE '");
                preparation(buf, textTest);
                buf.append("'");
                break;
        }
        return buf.toString();
    }

    private static void preparation(StringBuilder buf, TextTest textTest) {
        String text = textTest.getText();
        for (int i = 0; i < text.length(); i++) {
            char carac = text.charAt(i);
            if (carac == '\'') {
                buf.append("''");
            } else {
                buf.append(carac);
            }
        }
    }

    public static boolean isExcludingPartTestType(short conditionType) {
        return (conditionType == NE_COMMENCE_PAS_PAR) || (conditionType == NE_FINIT_PAS_PAR) || (conditionType == NE_CONTIENT_PAS) || (conditionType == NE_CORRESPOND_PAS_A);
    }

    public static boolean isIncludingPartTestType(short conditionType) {
        return (conditionType == COMMENCE_PAR) || (conditionType == FINIT_PAR) || (conditionType == CONTIENT) || (conditionType == CORRESPOND_A);
    }

    public static Condition toPartCondition(Condition condition) {
        short state = condition.getState();
        if (state == P) {
            return condition;
        }
        if (state == PA) {
            DefaultCondition defaultCondition = new DefaultCondition(condition.getMode());
            defaultCondition.innerSetState(P);
            int includingSize = condition.getIncludingTextTestCount();
            int excludingSize = condition.getExcludingTextTestCount();
            for (int i = 0; i < includingSize; i++) {
                defaultCondition.innerAddTextTest(condition.getIncludingTextTest(i));
            }
            for (int i = 0; i < excludingSize; i++) {
                defaultCondition.innerAddTextTest(condition.getExcludingTextTest(i));
            }
            return defaultCondition;
        }
        return null;
    }

    public static String conditionToSimpleString(Condition condition) {
        String result = internalConditionToSimpleString(condition);
        if (result == null) {
            return conditionToString(condition);
        } else {
            return result;
        }
    }

    private static String internalConditionToSimpleString(Condition condition) {
        short state = condition.getState();
        int excludingSize = condition.getExcludingTextTestCount();
        if (state == PA) {
            if (excludingSize == 0) {
                return null;
            }
        } else if (state != P) {
            return null;
        }
        int includingSize = condition.getIncludingTextTestCount();
        StringBuilder buf = new StringBuilder();
        if (condition.getMode() == MODE_OR) {
            buf.append('|');
        }
        for (int i = 0; i < includingSize; i++) {
            TextTest textTest = condition.getIncludingTextTest(i);
            if (textTest.getTestType() != CONTIENT) {
                return null;
            }
            String text = textTest.getText();
            if (text.indexOf(' ') != -1) {
                return null;
            }
            if (text.equals("!")) {
                return null;
            }
            if (text.equals("|")) {
                return null;
            }
            buf.append(text);
            buf.append(' ');
        }
        for (int i = 0; i < excludingSize; i++) {
            TextTest textTest = condition.getExcludingTextTest(i);
            if (textTest.getTestType() != NE_CONTIENT_PAS) {
                return null;
            }
            String text = textTest.getText();
            if (text.indexOf(' ') != -1) {
                return null;
            }
            buf.append('!');
            buf.append(text);
            buf.append(' ');
        }
        return buf.toString();
    }

    public static String conditionToString(Condition condition) {
        ConditionGroupBean conditionGroupBean = toConditionGroupBean(condition);
        StringBuilder buf = new StringBuilder();
        ContentConditionBean[] array = conditionGroupBean.getContentConditionBeanArray();
        if ((array == null) || (array.length == 0)) {
            return "";
        }
        for (int i = 0; i < array.length; i++) {
            if (i > 0) {
                buf.append(';');
            }
            buf.append(textTestToString(array[i]));
        }
        return buf.toString();
    }

    /**
     * Exprime la condition sous forme d'une chaîne.
     */
    public static String textTestToString(TextTest textTest) {
        String text = textTest.getText();
        switch (textTest.getTestType()) {
            case COMMENCE_PAR:
                return text + "*";
            case FINIT_PAR:
                return "*" + text;
            case CONTIENT:
                return "*" + text + "*";
            case CORRESPOND_A:
                return text;
            case EST_VIDE:
                return "!*";
            case NE_COMMENCE_PAS_PAR:
                return "!" + text + "*";
            case NE_FINIT_PAS_PAR:
                return "!*" + text;
            case NE_CONTIENT_PAS:
                return "!*" + text + "*";
            case NE_CORRESPOND_PAS_A:
                return "!" + text;
            case EST_NON_VIDE:
                return "*";
            default:
                return "";
        }
    }

    public static String testTypeToString(short testType) {
        switch (testType) {
            case COMMENCE_PAR:
                return ".*";
            case FINIT_PAR:
                return "*.";
            case CONTIENT:
                return "*.*";
            case CORRESPOND_A:
                return ".";
            case EST_VIDE:
                return "!*";
            case NE_COMMENCE_PAS_PAR:
                return "!.*";
            case NE_FINIT_PAS_PAR:
                return "!*.";
            case NE_CONTIENT_PAS:
                return "!*.*";
            case NE_CORRESPOND_PAS_A:
                return "!.";
            case EST_NON_VIDE:
                return "*";
            default:
                throw new IllegalArgumentException("wrong testType value = " + testType);
        }
    }

    public static short testTypeToShort(String type) {
        if (type.equals(".*")) {
            return COMMENCE_PAR;
        } else if (type.equals("*.")) {
            return FINIT_PAR;
        } else if (type.equals("*.*")) {
            return CONTIENT;
        } else if (type.equals(".")) {
            return CORRESPOND_A;
        } else if (type.equals("!*")) {
            return EST_VIDE;
        } else if (type.equals("!.*")) {
            return NE_COMMENCE_PAS_PAR;
        } else if (type.equals("!*.")) {
            return NE_FINIT_PAS_PAR;
        } else if (type.equals("!*.*")) {
            return NE_CONTIENT_PAS;
        } else if (type.equals("!.")) {
            return NE_CORRESPOND_PAS_A;
        } else if (type.equals("*")) {
            return EST_NON_VIDE;
        } else {
            throw new IllegalArgumentException("wrong type value = " + type);
        }
    }

    /**
     *
     *@throws IllegalArgumentException si testType est inconnu.
     */
    public static short getState(short testType) {
        switch (testType) {
            case COMMENCE_PAR:
            case FINIT_PAR:
            case CONTIENT:
            case CORRESPOND_A:
                return P;
            case EST_VIDE:
                return A;
            case NE_COMMENCE_PAS_PAR:
            case NE_FINIT_PAS_PAR:
            case NE_CONTIENT_PAS:
            case NE_CORRESPOND_PAS_A:
                return PA;
            case EST_NON_VIDE:
                return T;
            default:
                throw new IllegalArgumentException("unknown conditionType");
        }
    }

    public static short merge(short mode, short currentState, short addingState) {
        if (currentState == U) {
            return addingState;
        }
        if (currentState == addingState) {
            return currentState;
        }
        if (mode == MODE_AND) {
            return mergeAndMode(currentState, addingState);
        } else {
            return mergeOrMode(currentState, addingState);
        }
    }

    public static short mergeAndMode(short currentState, short addingState) {
        if (currentState == X) {
            return X;
        }
        if (currentState == N) {
            return addingState;
        }
        if (currentState == A) {
            switch (addingState) {
                case T:
                case P:
                    return X;
                case PA:
                    return A;
                default:
                    return currentState;
            }
        }
        if (currentState == T) {
            switch (addingState) {
                case A:
                    return X;
                case P:
                case PA:
                    return P;
                default:
                    return currentState;
            }
        }
        if (currentState == P) {
            switch (addingState) {
                case A:
                    return X;
                case T:
                case PA:
                    return P;
                default:
                    return currentState;
            }
        }
        if (currentState == PA) {
            switch (addingState) {
                case A:
                    return A;
                case T:
                case P:
                    return P;
                default:
                    return currentState;
            }
        }
        throw new IllegalArgumentException();
    }

    public static short mergeOrMode(short currentState, short addingState) {
        if (currentState == X) {
            return addingState;
        }
        if (currentState == N) {
            return N;
        }
        if (currentState == A) {
            switch (addingState) {
                case T:
                    return N;
                case P:
                case PA:
                    return PA;
                default:
                    return currentState;
            }
        }
        if (currentState == T) {
            switch (addingState) {
                case A:
                    return N;
                case P:
                    return T;
                case PA:
                    return N;
                default:
                    return currentState;
            }
        }
        if (currentState == P) {
            switch (addingState) {
                case A:
                    return PA;
                case T:
                    return T;
                case PA:
                    return PA;
                default:
                    return currentState;
            }
        }
        if (currentState == PA) {
            switch (addingState) {
                case A:
                    return PA;
                case T:
                    return N;
                case P:
                    return PA;
                default:
                    return currentState;
            }
        }
        throw new IllegalArgumentException();
    }

    public static boolean isImpossibleState(short state) {
        return (state == X);
    }

    public static boolean isNeutralState(short state) {
        return ((state == U) || (state == N));
    }

    public static boolean isNotEmptyState(short state) {
        return (state == T);
    }

    public static boolean isEmptyState(short state) {
        return ((state == A) || (state == PA));
    }

    public static boolean isPartState(short state) {
        return ((state == P) || (state == PA));
    }

    public static int toTextConstants(short testType) {
        switch (testType) {
            case COMMENCE_PAR:
                return TextConstants.STARTSWITH;
            case FINIT_PAR:
                return TextConstants.ENDSWITH;
            case CONTIENT:
                return TextConstants.CONTAINS;
            case CORRESPOND_A:
                return TextConstants.MATCHES;
            default:
                throw new IllegalArgumentException("wrong condition value");
        }
    }

    public static Condition toCondition(ConditionGroupBean bean) {
        DefaultCondition condition = new DefaultCondition(bean.getMode());
        ContentConditionBean[] contentConditionBeanArray = bean.getContentConditionBeanArray();
        if (contentConditionBeanArray != null) {
            for (int i = 0; i < contentConditionBeanArray.length; i++) {
                if (contentConditionBeanArray[i] != null) {
                    condition.innerAddTextTest(contentConditionBeanArray[i]);
                }
            }
        }
        return condition;
    }

    public static ConditionGroupBean toConditionGroupBean(Condition condition) {
        ContentConditionBean[] array = getContentConditionArray(condition);
        return new ConditionGroupBean(condition.getMode(), array);
    }

    public static ContentConditionBean[] getContentConditionArray(Condition condition) {
        switch (condition.getState()) {
            case A:
                return getA();
            case T:
                return getT();
            case P:
                return getP(condition);
            case PA:
                return getPA(condition);
            case X:
                return getX();
            case U:
            case N:
                return new ContentConditionBean[0];
            default:
                throw new SwitchException("state =" + condition.getState());
        }
    }

    private static ContentConditionBean[] getA() {
        ContentConditionBean[] array = new ContentConditionBean[1];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        return array;
    }

    private static ContentConditionBean[] getT() {
        ContentConditionBean[] array = new ContentConditionBean[1];
        array[0] = new ContentConditionBean(EST_NON_VIDE, "");
        return array;
    }

    private static ContentConditionBean[] getX() {
        ContentConditionBean[] array = new ContentConditionBean[2];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        array[1] = new ContentConditionBean(EST_NON_VIDE, "");
        return array;
    }

    private static ContentConditionBean[] getP(Condition condition) {
        int includingSize = condition.getIncludingTextTestCount();
        int excludingSize = condition.getExcludingTextTestCount();
        ContentConditionBean[] array = new ContentConditionBean[includingSize + excludingSize];
        for (int i = 0; i < includingSize; i++) {
            array[i] = new ContentConditionBean(condition.getIncludingTextTest(i));
        }
        for (int i = 0; i < excludingSize; i++) {
            array[i + includingSize] = new ContentConditionBean(condition.getExcludingTextTest(i));
        }
        return array;
    }

    private static ContentConditionBean[] getPA(Condition condition) {
        int excludingSize = condition.getExcludingTextTestCount();
        if (excludingSize > 0) {
            return getP(condition);
        }
        int includingSize = condition.getIncludingTextTestCount();
        ContentConditionBean[] array = new ContentConditionBean[1 + includingSize];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        for (int i = 0; i < includingSize; i++) {
            array[i + 1] = new ContentConditionBean(condition.getIncludingTextTest(i));
        }
        return array;
    }

}
