/* UtilLib - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.io.AppendableWriter;
import java.io.PrintWriter;
import java.io.Writer;
import net.mapeadores.util.localisation.ResourceLocalisationPack;


/**
 *
 * @author Vincent Calame
 */
public class HtmlPrinter {

    public final static String CHECKBOX_TYPE = "checkbox";
    public final static String RADIO_TYPE = "radio";
    protected PrintWriter pw;
    private ResourceLocalisationPack resourceLocalisationPack = new ResourceLocalisationPack();
    private ResourceLocalisation resourceLocalisation = resourceLocalisationPack.getResourceLocalisationInterface();
    protected boolean breakLine = true;

    public HtmlPrinter() {
    }

    public void addResourceLocalisation(ResourceLocalisation resourceLocalisation) {
        resourceLocalisationPack.addResourceLocalisation(resourceLocalisation);
    }

    public ResourceLocalisation getResourceLocalisation() {
        return resourceLocalisation;
    }

    public void initPrinter(PrintWriter pw) {
        this.pw = pw;
    }

    public void initPrinter(Appendable appendable) {
        if (appendable instanceof PrintWriter) {
            this.pw = (PrintWriter) appendable;
        } else if (appendable instanceof Writer) {
            this.pw = new PrintWriter((Writer) appendable);
        } else {
            this.pw = new PrintWriter(new AppendableWriter(appendable));
        }
    }

    public void flush() {
        pw.flush();
    }

    public PrintWriter getPrintWriter() {
        return pw;
    }

    public void appendStrictDocType() {
        pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">");
    }

    public void appendTransitionalDocType() {
        pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
    }

    public void append(String s) {
        pw.print(s);
    }

    public void space() {
        pw.print(" ");
    }

    public void doublespace() {
        pw.print("&nbsp; ");
    }

    public void escape(String s) {
        if (s == null) {
            pw.print("null");
            return;
        } else if (s.length() == 0) {
            return;
        }
        int taille = s.length();
        for (int i = 0; i < taille; i++) {
            char carac = s.charAt(i);
            if (carac < 32) {
                pw.print(' ');
            } else {
                switch (carac) {
                    case '&':
                        pw.print("&amp;");
                        break;
                    case '"':
                        pw.print("&quot;");
                        break;
                    case '<':
                        pw.print("&lt;");
                        break;
                    case '>':
                        pw.print("&gt;");
                        break;
                    default:
                        pw.print(carac);
                }
            }
        }
    }

    public void escapeScriptString(String s) {
        if (s == null) {
            return;
        } else if (s.length() == 0) {
            return;
        }
        int taille = s.length();
        for (int i = 0; i < taille; i++) {
            char carac = s.charAt(i);
            if (carac < 32) {
                pw.print(' ');
            } else {
                switch (carac) {
                    case '"':
                        pw.print("\\\"");
                        break;
                    case '\'':
                        pw.print("\\'");
                        break;
                    case '\\':
                        pw.print("\\\\");
                        break;
                    default:
                        pw.print(carac);
                }
            }
        }
    }

    public void escape(String s, boolean preformatted) {
        if (preformatted) {
            HTMLUtils.printPreformattedHTML(pw, s);
        } else {
            escape(s);
        }
    }

    public void localize(String key) {
        escape(getLocalization(key, null));
    }

    public String getLocalization(String key) {
        return getLocalization(key, null);
    }

    public String getLocalization(ResourceItem resourceItem) {
        return getLocalization(resourceItem.getResourceKey(), resourceItem.getValue());
    }

    public String getLocalization(String resourceKey, Object value) {
        return resourceLocalisation.toString(resourceKey, value);
    }

    public String getEscapedLocalization(String resourceKey, Object value) {
        String localization = getLocalization(resourceKey, value);
        if (localization == null) {
            return null;
        }
        StringBuffer buf = new StringBuffer();
        HTMLUtils.appendHTML(buf, localization);
        return buf.toString();
    }

    public void localize(String key, Object value) {
        escape(getLocalization(key, value));
    }

    public void localize(ResourceItem resourceItem) {
        escape(getLocalization(resourceItem));
    }

    public void startA(String href) {
        openTag("a");
        printAttribute("href", href);
        closeTag();
    }

    public void startA(String href, String target) {
        openTag("a");
        printAttribute("href", href);
        printAttribute("target", target);
        closeTag();
    }

    public void startA(HTMLAttributes htmlAttributes) {
        openTag("a");
        if (htmlAttributes != null) {
            printAttribute("href", htmlAttributes.getHref());
            printAttribute("target", htmlAttributes.getTarget());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endA() {
        endTag("a");
    }

    public void appendA(String href, String key) {
        startA(href);
        localize(key);
        endA();
    }

    public void appendA(String href, String key, String target) {
        startA(href, target);
        localize(key);
        endA();
    }

    public void startBody() {
        startBody(null);
    }

    public void startBody(HTMLAttributes htmlAttributes) {
        openTag("body");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endBody() {
        endTag("body");
        breakLine();
    }

    public void appendBr() {
        pw.print("<br>");
    }

    public void appendBr(int i) {
        for (int j = 0; j < i; j++) {
            appendBr();
        }
    }

    public void startCell(int colspan, int rowspan) {
        openTag("td");
        if (colspan > 1) {
            pw.print(" colspan=\"");
            pw.print(String.valueOf(colspan));
            pw.print("\"");
        }
        if (rowspan > 1) {
            pw.print(" rowspan=\"");
            pw.print(String.valueOf(rowspan));
            pw.print("\"");
        }
        closeTag();
    }

    public void startCell() {
        pw.print("<td>");
    }

    public void startCellWithTopAlign() {
        openTag("td");
        printAttribute("valign", "top");
        closeTag();
    }

    public void startCell(String width) {
        openTag("td");
        printAttribute("width", width);
        closeTag();
    }

    public void startCell(HTMLAttributes htmlAttributes) {
        openTag("td");
        if (htmlAttributes != null) {
            int colSpan = htmlAttributes.getColSpan();
            if (colSpan > 1) {
                printAttribute("colspan", String.valueOf(colSpan));
            }
            int rowSpan = htmlAttributes.getRowSpan();
            if (rowSpan > 1) {
                printAttribute("rowspan", String.valueOf(rowSpan));
            }
            printAttribute("width", htmlAttributes.getWidth());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endCell() {
        endTag("td");
        breakLine();
    }

    public void appendEmptyCell() {
        startCell();
        endCell();
    }

    public void appendCol(HTMLAttributes htmlAttributes) {
        openTag("col");
        if (htmlAttributes != null) {
            printAttribute("width", htmlAttributes.getWidth());
            /*printAttribute("height",htmlAttributes.getS.getHeight());*/
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void startCode() {
        pw.print("<code>");
    }

    public void endCode() {
        endTag("code");
    }

    public void startEm() {
        pw.print("<em>");
    }

    public void endEm() {
        endTag("em");
    }

    public void startDiv(HTMLAttributes htmlAttributes) {
        openTag("div");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endDiv() {
        endTag("div");
        breakLine();
    }

    public void startFieldset(HTMLAttributes htmlAttributes) {
        openTag("fieldset");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endFieldset() {
        endTag("fieldset");
    }

    public void startForm(HTMLAttributes htmlAttributes) {
        openTag("form");
        if (htmlAttributes != null) {
            printAttribute("action", htmlAttributes.getAction());
            printAttribute("method", htmlAttributes.getMethod());
            printAttribute("target", htmlAttributes.getTarget());
            printAttribute("onsubmit", htmlAttributes.getOnSubmit());
            printAttribute("enctype", htmlAttributes.getEnctype());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endForm() {
        endTag("form");
    }

    public void startH1() {
        pw.print("<h1>");
    }

    public void endH1() {
        endTag("h1");
        breakLine();
    }

    public void appendH1(String key) {
        startH1();
        localize(key);
        endH1();
    }

    public void printH1(String key, Object value) {
        startH1();
        localize(key, value);
        endH1();
    }

    public void startH2() {
        pw.print("<h2>");
    }

    public void endH2() {
        endTag("h2");
        breakLine();
    }

    public void appendH2(String key) {
        startH2();
        localize(key);
        endH2();
    }

    public void startH3() {
        pw.print("<h3>");
    }

    public void startH3(HTMLAttributes htmlAttributes) {
        openTag("h3");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endH3() {
        endTag("h3");
        breakLine();
    }

    public void appendH3(String key) {
        startH3();
        localize(key);
        endH3();
    }

    public void startHead() {
        pw.print("<head>");
        breakLine();
    }

    public void endHead() {
        endTag("head");
        breakLine();
    }

    public void appendHr() {
        pw.print("<hr>");
    }

    public void appendHr(HTMLAttributes htmlAttributes) {
        openTag("hr");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void startHtml(int langInteger) {
        openTag("html");
        if (langInteger != 0) {
            printAttribute("lang", LangInteger.toString(langInteger));
            if (LangInteger.isRTLScript(langInteger)) {
                printAttribute("dir", "rtl");
            }

        }
        closeTag();
        breakLine();
    }

    public void endHtml() {
        endTag("html");
    }

    public void appendImg(HTMLAttributes htmlAttributes) {
        openTag("img");
        if (htmlAttributes != null) {
            printAttribute("src", htmlAttributes.getSrc());
            printEscapedAttribute("alt", htmlAttributes.getAlt());
            printAttribute("width", htmlAttributes.getWidth());
            printAttribute("height", htmlAttributes.getHeight());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void appendImg(String src, String key, Object value, int width, int height, String className) {
        String title = getLocalization(key, value);
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setClasses(className);
        htmlAttributes.setSrc(src);
        htmlAttributes.setAlt(title);
        htmlAttributes.setTitle(title);
        htmlAttributes.setWidth((String.valueOf(width)));
        htmlAttributes.setHeight(String.valueOf(height));
        appendImg(htmlAttributes);
    }

    public void startIframe(HTMLAttributes htmlAttributes) {
        openTag("iframe");
        if (htmlAttributes != null) {
            printAttribute("src", htmlAttributes.getSrc());
            printAttribute("width", htmlAttributes.getWidth());
            printAttribute("height", htmlAttributes.getHeight());
            printAttribute("scrolling", htmlAttributes.getScrolling());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endIframe() {
        endTag("iframe");
        breakLine();
    }

    public void startLegend(HTMLAttributes htmlAttributes) {
        openTag("legend");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endLegend() {
        endTag("legend");
        breakLine();
    }

    public void appendLegend(String key) {
        startLegend(null);
        localize(key);
        endLegend();
    }

    public void startLi() {
        startLi(null);
    }

    public void startLi(HTMLAttributes htmlAttributes) {
        openTag("li");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endLi() {
        endTag("li");
        breakLine();
    }

    public void appendLink(HTMLAttributes htmlAttributes) {
        openTag("link");
        if (htmlAttributes != null) {
            printAttribute("href", htmlAttributes.getHref());
            printAttribute("rel", htmlAttributes.getRel());
            printAttribute("type", htmlAttributes.getType());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void appendMeta(HTMLAttributes htmlAttributes) {
        openTag("meta");
        if (htmlAttributes != null) {
            printAttribute("http-equiv", htmlAttributes.getHttpEquiv());
            printAttribute("content", htmlAttributes.getContent());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void startP() {
        pw.print("<p>");
    }

    public void startP(HTMLAttributes htmlAttributes) {
        openTag("p");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void startP(String className) {
        openTag("p");
        printAttribute("class", className);
        closeTag();
    }

    public void endP() {
        endTag("p");
        breakLine();
    }

    public void appendP(String className, String key) {
        startP(className);
        localize(key);
        endP();
    }

    public void startPre() {
        pw.print("<pre>");
    }

    public void startPre(HTMLAttributes htmlAttributes) {
        openTag("pre");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endPre() {
        endTag("pre");
    }

    public void startOption(String value, boolean selected) {
        openTag("option");
        printValueAttribute(value);
        if (selected) {
            pw.print(" selected");
        }
        closeTag();
    }

    public void startOption(HTMLAttributes htmlAttributes) {
        openTag("option");
        if (htmlAttributes != null) {
            printValueAttribute(htmlAttributes.getValue());
            printBooleanAttribute("selected", htmlAttributes.isSelected());
            printBooleanAttribute("disabled", htmlAttributes.isDisabled());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endOption() {
        endTag("option");
        breakLine();
    }

    public void appendOption(String value, boolean selected) {
        startOption(value, selected);
        append(value);
        endOption();
    }

    public void appendOption(String value, boolean selected, String key) {
        startOption(value, selected);
        localize(key);
        endOption();
    }

    public void startOptGroup(HTMLAttributes htmlAttributes) {
        openTag("optgroup");
        if (htmlAttributes != null) {
            printEscapedAttribute("label", htmlAttributes.getLabel());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endOptGroup() {
        endTag("optgroup");
        breakLine();
    }

    public void startRow() {
        pw.print("<tr>");
        breakLine();
    }

    public void startRow(String className) {
        openTag("tr");
        printAttribute("class", className);
        closeTag();
        breakLine();
    }

    public void endRow() {
        endTag("tr");
        breakLine();
    }

    public void startScript(HTMLAttributes htmlAttributes) {
        openTag("script");
        if (htmlAttributes != null) {
            printTypeAttribute(htmlAttributes.getType());
            printAttribute("src", htmlAttributes.getSrc());
            printBooleanAttribute("defer", htmlAttributes.isDefer());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endScript() {
        endTag("script");
        breakLine();
    }

    public void startNoscript() {
        pw.print("<noscript>");
    }

    public void endNoscript() {
        endTag("noscript");
    }

    public void startSelect(String name) {
        openTag("select");
        printNameAttribute(name);
        closeTag();
        breakLine();
    }

    public void startSelect(HTMLAttributes htmlAttributes) {
        openTag("select");
        printHTMLAttributes(htmlAttributes);
        printBooleanAttribute("multiple", htmlAttributes.isMultiple());
        printAttribute("size", htmlAttributes.getSize());
        closeTag();
        breakLine();
    }

    public void endSelect() {
        pw.print("</select>");
    }

    public void startSmall() {
        pw.print("<small>");
    }

    public void endSmall() {
        pw.print("</small>");
    }

    public void startSpan(String className) {
        openTag("span");
        printAttribute("class", className);
        closeTag();
    }

    public void startSpan(HTMLAttributes htmlAttributes) {
        openTag("span");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endSpan() {
        endTag("span");
    }

    public void startStrong() {
        pw.print("<strong>");
    }

    public void endStrong() {
        endTag("strong");
    }

    public void startTable() {
        pw.print("<table>");
        breakLine();
    }

    public void startTable(String width) {
        openTag("table");
        printAttribute("width", width);
        closeTag();
        breakLine();
    }

    public void startTable(HTMLAttributes htmlAttributes) {
        openTag("table");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endTable() {
        endTag("table");
        breakLine();
    }

    public void startTHead(HTMLAttributes htmlAttributes) {
        openTag("thead");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endTHead() {
        endTag("thead");
        breakLine();
    }

    public void startTFoot(HTMLAttributes htmlAttributes) {
        openTag("tfoot");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endTFoot() {
        endTag("tfoot");
        breakLine();
    }

    public void startTBody(HTMLAttributes htmlAttributes) {
        openTag("tbody");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endTBody() {
        pw.print("</tbody>");
        breakLine();
    }

    public void startTitle() {
        pw.print("<title>");
    }

    public void endTitle() {
        endTag("title");
        breakLine();
    }

    public void startTextArea(String name, int cols, int rows) {
        startTextArea(name, cols, rows, null);
    }

    public void startTextArea(String name, int cols, int rows, String id) {
        pw.print("<textarea name=\"");
        pw.print(name);
        pw.print("\"");
        if (cols > 0) {
            pw.print(" cols=\"");
            pw.print(String.valueOf(cols));
            pw.print("\"");
        }
        if (rows > 0) {
            pw.print(" rows=\"");
            pw.print(String.valueOf(rows));
            pw.print("\"");
        }
        printAttribute("id", id);
        closeTag();
    }

    public void startTextArea(HTMLAttributes htmlAttributes) {
        openTag("textarea");
        int cols = 0;
        int rows = 0;
        if (htmlAttributes != null) {
            cols = htmlAttributes.getCols();
            rows = htmlAttributes.getRows();
        }
        if (cols < 1) {
            cols = 50;
        }
        if (rows < 1) {
            rows = 3;
        }
        printAttribute("cols", cols);
        printAttribute("rows", rows);
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endTextArea() {
        endTag("textarea");
    }

    public void startUl() {
        pw.print("<ul>");
        breakLine();
    }

    public void startUl(HTMLAttributes htmlAttributes) {
        openTag("ul");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endUl() {
        endTag("ul");
        breakLine();
    }

    public void appendHiddenInput(String name, String value) {
        appendHiddenInput(name, value, false);
    }

    public void appendHiddenInput(String name, String value, boolean preformatted) {
        openInputTag();
        printTypeAttribute("hidden");
        printNameAttribute(name);
        pw.print(" value=\"");
        escape(value, preformatted);
        pw.print("\">");
    }

    public void appendInput(HTMLAttributes htmlAttributes) {
        openInputTag();
        if (htmlAttributes != null) {
            printTypeAttribute(htmlAttributes.getType());
            printValueAttribute(htmlAttributes.getValue());
            printAttribute("size", htmlAttributes.getSize());
            printBooleanAttribute("checked", htmlAttributes.isChecked());
            printBooleanAttribute("disabled", htmlAttributes.isDisabled());
            printBooleanAttribute("readonly", htmlAttributes.isReadonly());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void appendSmallestInput(String name, String value) {
        appendTextInput(name, value, 3);
    }

    public void appendSmallInput(String name, String value) {
        appendTextInput(name, value, 15);
    }

    public void appendMediumInput(String name, String value) {
        appendTextInput(name, value, 30);
    }

    public void appendLargeInput(String name, String value) {
        appendTextInput(name, value, 60);
    }

    public void appendTextInput(String name, String value, int size) {
        openInputTag();
        printTypeAttribute("text");
        printNameAttribute(name);
        printValueAttribute(value);
        if (size > 0) {
            printAttribute("size", String.valueOf(size));
        }
        closeTag();
    }

    public void appendTextInput(HTMLAttributes htmlAttributes) {
        openInputTag();
        printTypeAttribute("text");
        String value = htmlAttributes.getValue();
        if (value == null) {
            value = "";
        }
        printValueAttribute(value);
        printAttribute("size", htmlAttributes.getSize());
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void appendPasswordInput(String name, int size) {
        openInputTag();
        printTypeAttribute("password");
        printNameAttribute(name);
        if (size > 0) {
            printAttribute("size", String.valueOf(size));
        }
        closeTag();
    }

    public void appendRadioInput(String name, String value, boolean checked, String key) {
        openInputTag();
        printTypeAttribute("radio");
        printNameAttribute(name);
        printValueAttribute(value);
        if (checked) {
            pw.print(" checked");
        }
        closeTag();
        if (key != null) {
            localize(key);
        }
    }

    public void appendCheckInput(String name, String value, boolean checked, String key) {
        openInputTag();
        printTypeAttribute("checkbox");
        printNameAttribute(name);
        printValueAttribute(value);
        printBooleanAttribute("checked", checked);
        closeTag();
        if (key != null) {
            localize(key);
        }
    }

    /**
     * retourne <label><input/></label>
     */
    public void addCheck(HTMLAttributes checkAttributes, String labelString, HTMLAttributes labelAttributes) {
        startLabel(labelAttributes);
        checkAttributes = checkAttributes.copy();
        checkAttributes.setType(CHECKBOX_TYPE);
        appendInput(checkAttributes);
        if (labelString != null) {
            String label = getLocalization(labelString);
            if (label == null) {
                label = labelString;
            }
            escape(label);
        }
        endLabel();
    }

    public void addRadio(HTMLAttributes radioAttributes, String labelString, HTMLAttributes labelAttributes) {
        startLabel(labelAttributes);
        radioAttributes = radioAttributes.copy();
        radioAttributes.setType("radio");
        appendInput(radioAttributes);
        if (labelString != null) {
            String label = getLocalization(labelString);
            if (label == null) {
                label = labelString;
            }
            escape(label);
        }
        endLabel();
    }

    public void startLabel(HTMLAttributes htmlAttributes) {
        openTag("label");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endLabel() {
        endTag("label");
    }

    public void breakLine() {
        if (breakLine) {
            pw.println("");
        }
    }

    private boolean printHTMLAttributes(HTMLAttributes htmlAttributes) {
        if (htmlAttributes == null) {
            return false;
        }
        printAttribute("class", htmlAttributes.getClasses());
        printAttribute("id", htmlAttributes.getId());
        printAttribute("style", htmlAttributes.getStyle());
        printAttribute("onchange", htmlAttributes.getOnChange());
        printAttribute("onclick", htmlAttributes.getOnClick());
        printAttribute("onblur", htmlAttributes.getOnBlur());
        printAttribute("onfocus", htmlAttributes.getOnFocus());
        printAttribute("onmouseover", htmlAttributes.getOnMouseOver());
        printAttribute("onmousedown", htmlAttributes.getOnMouseDown());
        printAttribute("onmouseup", htmlAttributes.getOnMouseUp());
        printAttribute("onmouseout", htmlAttributes.getOnMouseOut());
        printAttribute("onkeyup", htmlAttributes.getOnKeyUp());
        printAttribute("name", htmlAttributes.getName());
        printEscapedAttribute("title", htmlAttributes.getTitle());
        return true;
    }

    public void printAttribute(String name, String value) {
        if (value == null) {
            return;
        }
        pw.print(" ");
        pw.print(name);
        pw.print("=\"");
        pw.print(value);
        pw.print("\"");
    }

    public void printAttribute(String name, int value) {
        pw.print(" ");
        pw.print(name);
        pw.print("=\"");
        pw.print(String.valueOf(value));
        pw.print("\"");
    }

    private void printBooleanAttribute(String name, boolean value) {
        if (value) {
            pw.print(" ");
            pw.print(name);
        }
    }

    private void printNameAttribute(String name) {
        printAttribute("name", name);
    }

    private void printValueAttribute(String value) {
        printEscapedAttribute("value", value);
    }

    private void printTypeAttribute(String type) {
        printAttribute("type", type);
    }

    private void openInputTag() {
        pw.print("<input");
    }

    public void openTag(String tag) {
        pw.print("<");
        pw.print(tag);
    }

    public void closeTag() {
        pw.print(">");
    }

    public void endTag(String tag) {
        pw.print("</");
        pw.print(tag);
        pw.print(">");
    }

    public void appendScriptLine(String s) {
        appendScriptLine(s, 0);
    }

    public void appendScriptLine(String s, int tab) {
        pw.print(" ");
        for (int i = 0; i < tab; i++) {
            pw.print("   ");
        }
        pw.print(s);
        pw.print("\n");
    }

    private void printEscapedAttribute(String name, String value) {
        if (value == null) {
            return;
        }
        pw.print(" ");
        pw.print(name);
        pw.print("=\"");
        escape(value);
        pw.print("\"");
    }

    public void startRowsFrameset(String rows) {
        openTag("frameset");
        printAttribute("rows", rows);
        closeTag();
        breakLine();
    }

    public void startColsFrameset(String cols) {
        openTag("frameset");
        printAttribute("cols", cols);
        closeTag();
        breakLine();
    }

    public void appendFrame(String name, String src) {
        openTag("frame");
        printAttribute("name", name);
        printAttribute("src", src);
        closeTag();
        breakLine();
    }

    public void endFrameset() {
        endTag("frameset");
        breakLine();
    }

}
