/* UtilLib - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceItem;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class UtilHtmlPrinter extends HtmlPrinter {

    public static final String COMMANDKEY_DIVCLASS = "d_GLB_CommandKey";
    public static final String SUBMIT_DIVCLASS = "d_GLB_Submit";
    private final static HTMLAttributes SUBMITDIV_ATTR = new HTMLAttributes();

    static {
        SUBMITDIV_ATTR.setClasses(SUBMIT_DIVCLASS);
    }

    private String htmlpage_title;
    private HTMLAttributes bodyHTMLAttributes;
    private ResourceLocalisation commandKeyLocalisation;
    private CommandBoxParameters commandBoxParameters = new CommandBoxParameters();

    {
        commandBoxParameters.getCommandKeyDivAttributes().setClasses(COMMANDKEY_DIVCLASS);
        commandBoxParameters.getSubmitParameters().getDivAttributes().setClasses(SUBMIT_DIVCLASS);
    }

    private boolean onCommandBox = false;
    private String currentCommandKey = null;
    private List<String> cssUrlList = new ArrayList<String>();
    private List<String> jsUrlList = new ArrayList<String>();
    private String iconPng = null;

    public UtilHtmlPrinter() {
    }

    public void setCommandKeyLocalisation(ResourceLocalisation commandKeyLocalisation) {
        this.commandKeyLocalisation = commandKeyLocalisation;
    }

    public ResourceLocalisation getCommandKeyLocalisation() {
        return commandKeyLocalisation;
    }

    public CommandBoxParameters getCommandBoxParameters() {
        return commandBoxParameters;
    }

    public void setTitle(String title) {
        this.htmlpage_title = title;
    }

    public void localizeTitle(String key, Object value) {
        this.htmlpage_title = getLocalization(key, value);
        if (htmlpage_title == null) {
            htmlpage_title = key;
        }
    }

    public String getTitle() {
        return htmlpage_title;
    }

    public void setBodyHTMLAttributes(HTMLAttributes bodyHTMLAttributes) {
        this.bodyHTMLAttributes = bodyHTMLAttributes;
    }

    public void addCssUrl(String cssUrl) {
        cssUrlList.add(cssUrl);
    }

    public void addJsUrl(String jsUrl) {
        jsUrlList.add(jsUrl);
    }

    public void start(int langInteger) {
        appendTransitionalDocType();
        startHtml(langInteger);
        startHead();
        startTitle();
        escape(htmlpage_title);
        endTitle();
        append("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
        breakLine();
        if (!jsUrlList.isEmpty()) {
            append("<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
            breakLine();
        }
        for (String cssUrl : cssUrlList) {
            append("<link href=\"");
            append(cssUrl);
            append("\" rel=\"stylesheet\" type=\"text/css\">");
            breakLine();
        }
        if (iconPng != null) {
            append("<link href=\"");
            append(iconPng);
            append("\" rel=\"icon\" type=\"image/png\">");
            breakLine();
        }
        if (!jsUrlList.isEmpty()) {
            HTMLAttributes scriptAttributes = new HTMLAttributes();
            scriptAttributes.setType("text/javascript");
            int length = jsUrlList.size();
            for (int i = 0; i < length; i++) {
                scriptAttributes.setSrc(jsUrlList.get(i));
                startScript(scriptAttributes);
                endScript();
                breakLine();
            }
        }
        endHead();
        startBody(bodyHTMLAttributes);
    }

    public void startForm(HTMLAttributes formAttributes, String commandString) {
        startForm(formAttributes);
        appendHiddenInput("cmd", commandString);
    }

    public void startCommandBox(CommandBox commandBox) {
        if (onCommandBox) {
            throw new IllegalStateException("last CommandBox is not ended");
        }
        onCommandBox = true;
        HTMLAttributes formAttributes = commandBoxParameters.getFormAttributes();
        formAttributes.setAction(commandBox.getFormAction());
        formAttributes.setMethod("POST");
        if (commandBox.isMultipart()) {
            formAttributes.setEnctype(HTMLAttributes.MULTIPART_ENCTYPE);
        }
        startForm(formAttributes);
        appendHiddenInput("cmd", commandBox.getCommandString());
        String page = commandBox.getPage();
        if (page != null) {
            appendHiddenInput("page", page);
        }
        String errorPage = commandBox.getErrorPage();
        if (errorPage != null) {
            appendHiddenInput("page-err", errorPage);
        }
        Map<String, String> hiddenAttributesMap = commandBox.getHiddenAttributesMap();
        if (hiddenAttributesMap != null) {
            for (Map.Entry<String, String> entry : hiddenAttributesMap.entrySet()) {
                appendHiddenInput(entry.getKey(), entry.getValue());
            }
        }
        HTMLAttributes fieldsetAttributes = commandBoxParameters.getFieldsetAttributes();
        String commandFamily = commandBox.getCommandFamily();
        if (commandFamily != null) {
            fieldsetAttributes.setClasses("fs_" + commandFamily);
        } else {
            fieldsetAttributes.setClasses(null);
        }
        startFieldset(fieldsetAttributes);
        String commandKey = commandBox.getCommandKey();
        if (commandKey != null) {
            startLegend(commandBoxParameters.getLegendAttributes());
            escape(commandKeyLocalisation.toString(commandKey, null));
            endLegend();
            currentCommandKey = commandKey;
        }
    }

    public void endCommandBox(String submitKey) {
        if (!onCommandBox) {
            throw new IllegalStateException("no CommandBox started yet");
        }
        if (submitKey != null) {
            SubmitParameters submitParameters = commandBoxParameters.getSubmitParameters();
            addSubmit(new ResourceItem(submitKey), submitParameters);
        }
        if (currentCommandKey != null) {
            startDiv(commandBoxParameters.getCommandKeyDivAttributes());
            append(currentCommandKey);
            endDiv();
        }
        endFieldset();
        endForm();
        currentCommandKey = null;
        onCommandBox = false;
    }

    public void end() {
        if (onCommandBox) {
            throw new IllegalStateException("last CommandBox is not ended");
        }
        endBody();
        endHtml();
    }

    public void addSubmit(String submitKey, boolean encloseInDiv) {
        SubmitParameters submitParameters = new SubmitParameters();
        submitParameters.setEncloseInDiv(encloseInDiv);
        addSubmit(new ResourceItem(submitKey), submitParameters);
    }

    public void addSubmit(ResourceItem submitResourceItem, SubmitParameters submitParameters) {
        boolean encloseInDiv = submitParameters.isEncloseInDiv();
        if (encloseInDiv) {
            HTMLAttributes divAttr = submitParameters.getDivAttributes();
            if (divAttr == null) {
                divAttr = SUBMITDIV_ATTR;
            }
            startDiv(divAttr);
        }
        HTMLAttributes inputAttr = submitParameters.getInputAttributes();
        if (inputAttr == null) {
            inputAttr = new HTMLAttributes();
        } else {
            inputAttr = inputAttr.copy();
        }
        if (submitResourceItem != null) {
            inputAttr.setValue(getLocalization(submitResourceItem));
        }
        inputAttr.setType("submit");
        appendInput(inputAttr);
        if (encloseInDiv) {
            endDiv();
        }
    }

    @Deprecated
    public void printColon() {
        HtmlPrinterUtils.printColon(this);
    }

    @Deprecated
    public void printLibColon(String libKey) {
        HtmlPrinterUtils.printLibColon(this, libKey);
    }

    @Deprecated
    public void printLibCells(String libKey, boolean topAlign) {
        HtmlPrinterUtils.printLibCells(this, libKey, topAlign);
    }

    @Deprecated
    public void printLitteralLibCells(String value, boolean topAlign) {
        if (topAlign) {
            startCellWithTopAlign();
        } else {
            startCell();
        }
        escape(value);
        endCell();
        if (topAlign) {
            startCellWithTopAlign();
        } else {
            startCell();
        }
        append("&nbsp;:&nbsp;");
        endCell();
    }

    public void setIconPng(String iconPng) {
        this.iconPng = iconPng;
    }

}
