/* UtilLib - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.langinteger;

import net.mapeadores.util.exceptions.NestedIOException;
import java.util.Locale;
import java.io.IOException;
import java.io.Serializable;


/**
 * Représente une langue sous la forme d'un entier.
 *  L'utilisation d'une valeur entière à l'avantage d'un gain de place et de performance.
 * Cette valeur entière ne provient pas d'une table de correspondance mais est déduite directement de la chaîne de langue.
 * <p>Cela part du principe que les caractères d'un code de langue font partie de la tranche a-z et peuvent donc être codés sur cinq bits.
 * Un entier étant codé sur 32 bits, sa valeur peut représenter 6 caractères de a à z, 2 bits n'étant pas utilisé.
 * Un entier peut donc représenter toutes les langues sous la forme d'un code ISO de deux lettres (fr), de trois lettres (abk), de deux lettres + le code du pays (fr-BE), ou trois lettres + le code du pays
 *<p>La classe LangInteger essaie de suivre autant que faire ce peut la norme RFC 3066. Elle est insensible à la casse (mais utilise la représentation classique langue en minuscules, pays en majuscules)
 * Elle ne gére pas les cas particuliers comme « en-cockney » ou les langues enregistrées auprès l'IANA (commençant par i-)
 *<p>Les entiers déduits d'une valeur de langue sont toujours négatifs, cela permet à un système utilisant LangInteger d'utiliser des entiers positifs pour des valeurs
 * qui lui sont propres (grosso modo, on peut imaginer une procédure qui teste le signe, utilise LangInteger si c'est négatif et un autre système si positif).
 *<p>Conséquence de ce signe négatif, c'est que l'ordre des entiers est celui de l'ordre alphabétique : "en" < "fr" et LangInteger.ENGLISH < LangInteger.FRENCH 
 *
 *<p>Les entiers sont codés de la manière suivante :
 *<ul>
 * <li>premier caractère :         0011 1110 0000 0000 0000 0000 0000 0000 (0x3E000000)
 * <li>deuxième caractère :        0000 0001 1111 0000 0000 0000 0000 0000 (0x1F0000)
 * <li>trosième caractère :        0000 0000 0000 1111 1000 0000 0000 0000 (0xF8000)
 * <li>quatrième caractère :       0000 0000 0000 0000 0111 1100 0000 0000 (0x7C00)
 * <li>cinquième caractère :       0000 0000 0000 0000 0000 0011 1110 0000 (0x3E0)
 * <li>sixième caractère :         0000 0000 0000 0000 0000 0000 0001 1111 (0x1F)
 *</ul>
 *<p>Ce qui donne pour le français : 1000 1101 0001 0000 0000 0000 0000 0000
 *<p>Le premier, le deuxième et éventuellement le troisième caractères sont utilisés pour le code de langue,
 * le quatrième et le cinquième pour le code du pays, le sixième caractère n'est pas utilisé pour le moment.
 * *
 * @author  Vincent Calame
 */
public final class LangInteger implements Serializable {

    private static final long serialVersionUID = 1L;
    /**
     * Correspond au code ISO 639-2 : "zxx" aucun contenu linguistique.
     */
    public final static int NO_LINGUISTIC_CONTENT = -1249116160;
    /**
     * Correspond au code ISO 639-2 : "und" : langue indéterminée. À utiliser
     * lorsque l'attribut langue est absent.
     */
    public final static int UNDETERMINED = -1428029440;
    public final static int FRENCH = -1927282688;
    public final static int ENGLISH = -1965031424;
    public final static int SPANISH = -1959788544;
    public final static int PORTUGUESE = -1589641216;
    public final static int GERMAN = -2008023040;
    public final static int ARABIC = -2095054848;
    public final static int HEBREW = -1873805312;
    public final static int LANG_PART = 0xFFFF8000;
    //public final static int COUNTRY_PART = 0xFFFF8000;
    private int intValue;

    public LangInteger(int intValue) {
        this.intValue = intValue;
    }

    public int intValue() {
        return intValue;
    }

    @Override
    public int hashCode() {
        return intValue;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof LangInteger) {
            return (intValue == ((LangInteger) obj).intValue());
        }
        return false;
    }

    @Override
    public String toString() {
        return toString(intValue);
    }

    /**
     * Les codes de pays sont sous la forme zxx-{code du pays}.
     * UK est assimilé à GB.
     */
    public static int parse(String langIntegerString) throws LangIntegerException {
        return parse(langIntegerString, false);
    }

    public static int parse(String langIntegerString, boolean noLinguisticContentAllowed) throws LangIntegerException {
        if (langIntegerString == null) {
            throw new LangIntegerException();
        }
        return parse(langIntegerString, 0, langIntegerString.length(), noLinguisticContentAllowed);
    }

    public static int parse(String langIntegerString, int index, int length, boolean noLinguisticContentAllowed) throws LangIntegerException {
        if (langIntegerString == null) {
            throw new LangIntegerException();
        }
        if (length == -1) {
            length = langIntegerString.length() - index;
        }
        if (length < 2) {
            throw new LangIntegerException(langIntegerString.substring(index, index + length));
        }
        if (length > 6) {
            throw new LangIntegerException(langIntegerString.substring(index, index + length));
        }
        int langInteger = 0x80000000;
        char carac1 = langIntegerString.charAt(index);
        char carac2 = langIntegerString.charAt(index + 1);
        langInteger = putChar(langInteger, carac1, 1);
        langInteger = putChar(langInteger, carac2, 2);
        if (length == 2) {
            return LangUtils.checkCode(langInteger);
        }
        char carac3 = langIntegerString.charAt(index + 2);
        char caracPays1 = 0;
        char caracPays2 = 0;
        if ((carac3 != '-') && (carac3 != '_')) { //doit être sous la forme fre, ou fre-BE
            int iso3 = putChar(langInteger, carac3, 3);
            langInteger = LangUtils.checkCode(iso3);
            if (!noLinguisticContentAllowed) {
                if (langInteger == NO_LINGUISTIC_CONTENT) {
                    throw new LangIntegerException(langIntegerString.substring(index, index + length));
                }
            }
            if (length == 3) {
                return langInteger;
            } else {
                if (length != 6) {
                    throw new LangIntegerException(langIntegerString.substring(index, index + length));
                }
                char sep = langIntegerString.charAt(index + 3);
                if ((sep != '-') && (sep != '_')) {
                    throw new LangIntegerException(langIntegerString.substring(index, index + length));
                }
                caracPays1 = langIntegerString.charAt(index + 4);
                caracPays2 = langIntegerString.charAt(index + 5);
            }
        } else {// doit être sous la forme fr-BE
            if (length != 5) {
                throw new LangIntegerException(langIntegerString.substring(index, index + length));
            }
            caracPays1 = langIntegerString.charAt(index + 3);
            caracPays2 = langIntegerString.charAt(index + 4);
        }
        if (length < 5) {
            throw new LangIntegerException(langIntegerString.substring(index, index + length));
        }
        langInteger = putChar(langInteger, caracPays1, 4);
        langInteger = putChar(langInteger, caracPays2, 5);
        return langInteger;
    }

    private static int putChar(int langInteger, char c, int position) throws LangIntegerException {
        return LangUtils.putCinqbits(langInteger, LangUtils.toCinqbits(c), position);
    }

    /**
     * Retourne Undertined si la valeur de locale.getLanguage() est incorrecte
     */
    public static int fromLocale(Locale locale) {
        try {
            String language = locale.getLanguage();
            if (language.length() == 0) {
                return UNDETERMINED;
            }
            String country = locale.getCountry();
            if (country.length() > 0) {
                return parse(language + "-" + country);
            } else {
                return parse(language);
            }
        } catch (LangIntegerException mcle) {
        }
        return UNDETERMINED;
    }

    public static String toString(int langInteger) {
        return toString(langInteger, '-');
    }

    public static String toString(int langInteger, char separateur) {
        StringBuilder buf = new StringBuilder(16);
        appendString(buf, langInteger, separateur);
        return buf.toString();
    }

    public static void appendString(Appendable buf, int langInteger) {
        try {
            writeString(buf, langInteger, '-');
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public static void appendString(Appendable buf, int langInteger, char separateur) {
        try {
            writeString(buf, langInteger, separateur);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public static void writeString(Appendable buf, int langInteger) throws IOException {
        writeString(buf, langInteger, '-');
    }

    public static void writeString(Appendable buf, int langInteger, char separateur) throws IOException {
        buf.append(getLanguage(langInteger));
        String country = getCountry(langInteger);
        if (country.length() > 0) {
            buf.append(separateur);
            buf.append(country);
        }
    }

    /**
     * Retourne l'instance de Locale correspondant à la valeur langInteger.
     * Dans la mesure où LangInteger couvre un champ plus vaste que les langues codées
     * sur deux lettres, c'est l'instance de Locale par défaut qui est retournée, s'il n'est
     * pas possible de résoudre la langue. Il n'est donc pas garanti d'avoir :
     *langInteger = LangInteger.fromLocale(LangInteger.getLocale(langInteger))
     */
    public static Locale getLocale(int langInteger) {
        if (langInteger == UNDETERMINED) {
            return Locale.getDefault();
        }
        if (langInteger == NO_LINGUISTIC_CONTENT) {
            return Locale.getDefault();
        }
        return new Locale(getLanguage(langInteger), getCountry(langInteger));
    }

    /**
     * Retourne la valeur de la langue diminuée des éventuelles informations de pays.
     */
    public static int getLangPart(int langInteger) {
        if (langInteger > 0) {
            return langInteger;
        }
        return langInteger & LANG_PART;
    }

    public static boolean isRTLScript(int langInteger) {
        int langPart = getLangPart(langInteger);
        switch (langPart) {
            case ARABIC:
            case HEBREW:
                return true;
            default:
                return false;
        }
    }

    /**
     * Retourne une chaîne vide si la valeur ne correspond pas à un codage.
     */
    public static String getLanguage(int langInteger) {
        if (langInteger >= 0) {
            return "";
        }
        char carac1 = getChar(langInteger, 0, false);
        if (carac1 == '\00') {
            return "";
        }
        char carac2 = getChar(langInteger, 1, false);
        if (carac2 == '\00') {
            return "";
        }
        char carac3 = getChar(langInteger, 2, false);
        if (carac3 != '\00') {
            char[] caracs = new char[3];
            caracs[0] = carac1;
            caracs[1] = carac2;
            caracs[2] = carac3;
            return new String(caracs);
        } else {
            char[] caracs = new char[2];
            caracs[0] = carac1;
            caracs[1] = carac2;
            return new String(caracs);
        }
    }

    public static String getCountry(int langInteger) {
        if (((0x1F) & langInteger) != 0) {
            return "";
        }
        char caracPays1 = getChar(langInteger, 3, true);
        char caracPays2 = getChar(langInteger, 4, true);
        if (caracPays1 == '\00') {
            return "";
        }
        if (caracPays2 == '\00') {
            return "";
        }
        char[] caracs = new char[2];
        caracs[0] = caracPays1;
        caracs[1] = caracPays2;
        return new String(caracs);
    }

    private static char toChar(int cinqbits, boolean majuscule) {
        if (cinqbits == 0) {
            return '\00';
        }
        if (majuscule) {
            return (char) (cinqbits + 64);
        } else {
            return (char) (cinqbits + 96);
        }
    }

    private static char getChar(int langInteger, int position, boolean majuscule) {
        switch (position) {
            case 0:
                return toChar((langInteger & 0x3E000000) >>> 25, majuscule);
            case 1:
                return toChar((langInteger & 0x1FC0000) >>> 20, majuscule);
            case 2:
                return toChar((langInteger & 0xF8000) >>> 15, majuscule);
            case 3:
                return toChar((langInteger & 0x7C00) >>> 10, majuscule);
            case 4:
                return toChar((langInteger & 0x3E0) >>> 5, majuscule);
            case 5:
                return toChar((langInteger & 0x1F), majuscule);
            default:
                return '\00';
        }
    }

    /**
     * Retourne l'entier correspondant à "zxx-{code du pays}". Si le code
     * du pays n'est pas défini, retourne "zxx" (NO_LINGUISTIC_CONTENT)
     */
    public static int toCountryInteger(int langInteger) {
        if (langInteger >= 0) {
            return NO_LINGUISTIC_CONTENT;
        }
        int countryPart = langInteger & 0x7FE0;
        return NO_LINGUISTIC_CONTENT | countryPart;
    }

    public static boolean isCountryInteger(int codeInteger) {
        if (getLangPart(codeInteger) != NO_LINGUISTIC_CONTENT) {
            return false;
        }
        if (codeInteger == NO_LINGUISTIC_CONTENT) {
            return false;
        }
        return true;
    }

}
