/* UtilLib - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.lexie.LexieFilter;
import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Locale;
import java.util.Properties;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;


/**
 *
 * @author Vincent Calame
 */
public class LocalisationLexieFilters {

    static final DefaultWordFilter defaultWordFilter = new DefaultWordFilter();
    static final IntHashMap currentFilter = new IntHashMap();

    private LocalisationLexieFilters() {
    }

    public static LexieFilter getLexieFilter(int langInteger) {
        LexieFilter wordFilter = (LexieFilter) currentFilter.get(langInteger);
        if (wordFilter == null) {
            wordFilter = init(langInteger);
            currentFilter.put(langInteger, wordFilter);
        }
        return wordFilter;
    }

    private static LexieFilter init(int langInteger) {
        Class currentClass = LocalisationLexieFilters.class;
        String langString = LangInteger.toString(langInteger);
        InputStream propertiesStream = currentClass.getResourceAsStream("resources/wordfilter_" + langString + ".properties");
        InputStream forbiddenStream = currentClass.getResourceAsStream("resources/forbiddenwords_" + langString + ".txt");
        try {
            LexieFilter wordFilter = init(langInteger, propertiesStream, forbiddenStream);
            if (wordFilter == null) {
                wordFilter = defaultWordFilter;
            }
            return wordFilter;
        } catch (IOException ioe) {
            ioe.printStackTrace();
            return defaultWordFilter;
        }
    }

    private static LexieFilter init(int langInteger, InputStream propertiesStream, InputStream forbiddenStream) throws IOException {
        if (propertiesStream == null) {
            return null;
        }
        if (forbiddenStream == null) {
            return null;
        }
        Properties properties = new Properties();
        properties.load(propertiesStream);
        propertiesStream.close();
        List<String> forbiddenList = new ArrayList<String>();
        BufferedReader reader = new BufferedReader(new InputStreamReader(forbiddenStream, "UTF-8"));
        String ligne = null;
        while ((ligne = reader.readLine()) != null) {
            ligne = ligne.trim();
            if ((ligne.length() > 0) && (ligne.charAt(0) != '#')) {
                forbiddenList.add(ligne);
            }
        }
        reader.close();
        LocalisationWordFilter wordFilter = new LocalisationWordFilter(langInteger, properties, forbiddenList);
        return wordFilter;
    }


    private static class DefaultWordFilter implements LexieFilter {

        public boolean acceptLexie(String word) {
            int length = word.length();
            if (length == 0) {
                return false;
            } else if (length == 1) {
                return StringUtils.isIdeogram(word.charAt(0));
            } else if (length == 2) {
                return false;
            } else {
                return true;
            }
        }

    }


    private static class LocalisationWordFilter implements LexieFilter {

        int minlength = 3;
        int minpartlength = 2;
        int maxForbiddenWordLength = 0;
        Set<String> forbiddenWordSet = null;
        Locale locale;

        LocalisationWordFilter(int langInteger, Properties properties, List<String> forbiddenList) {
            String minlengthString = properties.getProperty("minlength");
            if (minlengthString != null) {
                try {
                    minlength = Integer.parseInt(minlengthString);
                } catch (NumberFormatException nfe) {
                }
            }
            String minpartlengthString = properties.getProperty("minpartlength");
            if (minpartlengthString != null) {
                try {
                    minpartlength = Integer.parseInt(minpartlengthString);
                } catch (NumberFormatException nfe) {
                }
            }
            locale = LangInteger.getLocale(langInteger);
            if (forbiddenList.size() > 0) {
                forbiddenWordSet = new HashSet<String>();
                for (String s : forbiddenList) {
                    s = s.toLowerCase(locale);
                    forbiddenWordSet.add(s);
                    maxForbiddenWordLength = Math.max(maxForbiddenWordLength, s.length());
                }
            }
        }

        public boolean acceptLexie(String word) {
            int length = word.length();
            if (length == 1) {
                if (StringUtils.isIdeogram(word.charAt(0))) {
                    return true;
                }
            }
            if (length < minlength) {
                if (length > 1) {
                    try {
                        Integer.parseInt(word);
                        return true;
                    } catch (NumberFormatException nfe) {
                    }
                }
                return false;
            }
            if (length > maxForbiddenWordLength) {
                return true;
            }
            if (forbiddenWordSet == null) {
                return true;
            }
            return !forbiddenWordSet.contains(word.toLowerCase(locale));
        }

    }

}
