/* UtilLib - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.text.DecimalFormatSymbols;
import java.io.Serializable;


/**
 * Les codes sont négatifs afin de les distinguer des codes numériques des monnaies.
 * Voir : http://www.iso.org/iso/fr/support/faqs/faqs_widely_used_standards/widely_used_standards_other/currency_codes/currency_codes_list-1.htm
 *
 *
 * <ul>
 * * <li>premier caractère :       0111 1100 0000 0000 0000 0000 0000 0000 (0x7C000000)
 * <li>deuxième caractère :        0000 0011 1110 0000 0000 0000 0000 0000 (0x3E0000)
 * <li>trosième caractère :        0000 0000 0001 1111 0000 0000 0000 0000 (0x1F0000)
 * </ul>
 *
 * @author Vincent Calame
 */
public final class CurrencyInteger implements Serializable {

    private static final long serialVersionUID = 1L;
    public static final int EURO = -1766719488;
    public static final int USDOLLAR = -698089472;
    public static final int YEN = -1441202176;
    public static final int POUND = -1672478720;
    private static final int[] symbolArray = new int[4];

    static {
        symbolArray[0] = init(EURO, '€');
        symbolArray[1] = init(USDOLLAR, '$');
        symbolArray[2] = init(YEN, '¥');
        symbolArray[3] = init(POUND, '£');
    }

    private static int init(int ci, char symbol) {
        return (ci | ((int) symbol));
    }

    private int intValue;

    public CurrencyInteger(int intValue) {
        this.intValue = intValue;
    }

    public int intValue() {
        return intValue;
    }

    @Override
    public int hashCode() {
        return intValue;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof CurrencyInteger) {
            return (intValue == ((CurrencyInteger) obj).intValue());
        }
        return false;
    }

    @Override
    public String toString() {
        return toString(intValue);
    }

    public static int parse(String currencyIntegerString) throws CurrencyIntegerException {
        int length = currencyIntegerString.length();
        if (length == 1) {
            return parseChar(currencyIntegerString.charAt(0));
        }
        if (length != 3) {
            throw new CurrencyIntegerException();
        }
        int currencyInteger = 0x80000000;
        currencyInteger = putChar(currencyInteger, currencyIntegerString, 0);
        currencyInteger = putChar(currencyInteger, currencyIntegerString, 1);
        currencyInteger = putChar(currencyInteger, currencyIntegerString, 2);
        return currencyInteger;
    }

    public static int parseChar(char carac) throws CurrencyIntegerException {
        for (int i = 0; i < symbolArray.length; i++) {
            int symbol = symbolArray[i];
            if (testSymbol(symbol, carac)) {
                return getCurrencyInteger(symbol);
            }
        }
        throw new CurrencyIntegerException();
    }

    private static int putChar(int currencyInteger, String s, int index) throws CurrencyIntegerException {
        char carac = s.charAt(index);
        int cinqbits = toCinqbits(carac);
        switch (index) {
            case 0:
                return (currencyInteger | (cinqbits << 26));
            case 1:
                return (currencyInteger | (cinqbits << 21));
            case 2:
                return (currencyInteger | (cinqbits << 16));
            default:
                throw new IllegalArgumentException();
        }
    }

    private static int toCinqbits(char c) throws CurrencyIntegerException {
        if ((c >= 65) && (c <= 90)) {
            return (int) (c - 64);
        } else if ((c >= 97) && (c <= 122)) {
            return (int) (c - 96);
        }
        throw new CurrencyIntegerException();
    }

    static public String toString(int currencyInteger) {
        char[] caracs = new char[3];
        char carac1 = getChar(currencyInteger, 0);
        if (carac1 == '\00') {
            return "";
        }
        caracs[0] = carac1;
        char carac2 = getChar(currencyInteger, 1);
        if (carac2 == '\00') {
            return "";
        }
        caracs[1] = carac2;
        char carac3 = getChar(currencyInteger, 2);
        if (carac3 == '\00') {
            return "";
        }
        caracs[2] = carac3;
        return new String(caracs);
    }

    private static char getChar(int langInteger, int index) {
        switch (index) {
            case 0:
                return toChar((langInteger & 0x7C000000) >>> 26);
            case 1:
                return toChar((langInteger & 0x3E00000) >>> 21);
            case 2:
                return toChar((langInteger & 0x1F0000) >>> 16);
            default:
                return '\00';
        }
    }

    private static char toChar(int cinqbits) {
        if (cinqbits == 0) {
            return '\00';
        }
        return (char) (cinqbits + 64);
    }

    private static boolean testSymbol(int symbolInteger, char c) {
        int carac1 = (symbolInteger & 0xFFFF);
        return (c == carac1);
    }

    private static boolean testCurrency(int symbolInteger, int currencyInteger) {
        return (((symbolInteger) & (currencyInteger)) == currencyInteger);
    }

    private static int getCurrencyInteger(int symbolInteger) {
        return (symbolInteger & 0xFFFF0000);
    }

    private static char getSymbol(int symbolInteger) {
        return (char) (symbolInteger & 0xFFFF);
    }

    public static String toString(Decimal decimal, int currencyInteger, DecimalFormatSymbols symbols) {
        return toString(decimal, currencyInteger, symbols.getDecimalSeparator(), symbols.getGroupingSeparator());
    }

    public static String toString(Decimal decimal, int currencyInteger, char decimalSeparator, char groupingSeparator) {
        String nombreString = decimal.toString(decimalSeparator, groupingSeparator);
        for (int i = 0; i < symbolArray.length; i++) {
            int symbolInt = symbolArray[i];
            if (testCurrency(symbolInt, currencyInteger)) {
                char carac = getSymbol(symbolInt);
                if (currencyInteger == POUND) {
                    return carac + nombreString;
                } else {
                    return nombreString + " " + carac;
                }
            }
        }
        return nombreString + " " + toString(currencyInteger);
    }

    public static String toString(Decimal decimal, int currencyInteger) {
        return decimal.toString() + " " + toString(currencyInteger);
    }

}
