/* UtilLib - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import net.mapeadores.util.text.DateFormatBundle;
import java.util.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.text.ParseException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.io.Serializable;
import java.io.IOException;


/**
 * La classe DateInteger sert à stocker sur un entier des informations de datation.
 * La datation utilise un entier sur 28 bits
 * La valeur de datation est entier, soit une FFFF FFFF
 *Année :      1111 1111 1111 1100 0000 0000 0000
 *Semestre :   0000 0000 0000 0011 0000 0000 0000
 *Trimestre :  0000 0000 0000 0000 1110 0000 0000
 *Mois :       0000 0000 0000 0000 0001 1110 0000
 *Jour :       0000 0000 0000 0000 0000 0001 1111
 *<p>Cette valeur est toujours positive sauf pour la valeur NULL = 0x80000000
 *
 * @author  Vincent Calame
 */
public final class DateInteger implements Serializable {

    private static final long serialVersionUID = 1L;
    public static final short TYPE_ANNEE = 1;
    public static final short TYPE_MOIS = 4;
    public static final short TYPE_SEMESTRE = 2;
    public static final short TYPE_TRIMESTRE = 3;
    public static final short TYPE_JOUR = 5;
    public static final int NULL = 0x80000000;
    private static final SimpleDateFormat ISO_FORMAT = new SimpleDateFormat("yyyy-MM-dd");
    private int intValue;

    public DateInteger(int intValue) {
        this.intValue = intValue;
    }

    public int intValue() {
        return intValue;
    }

    @Override
    public int hashCode() {
        return intValue;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof DateInteger) {
            return (intValue == ((DateInteger) obj).intValue());
        }
        return false;
    }

    @Override
    public String toString() {
        return toString(intValue);
    }

    public static short getDateIntegerType(int dateInteger) {
        if (getJour(dateInteger) != 0) {
            return TYPE_JOUR;
        } else if (getMois(dateInteger) != 0) {
            return TYPE_MOIS;
        } else if (getTrimestre(dateInteger) != 0) {
            return TYPE_TRIMESTRE;
        } else if (getSemestre(dateInteger) != 0) {
            return TYPE_SEMESTRE;
        }
        return TYPE_ANNEE;
    }

    public static int getAnnee(int dateInteger) {
        return (dateInteger & 0xFFFC000) >>> 14;
    }

    public static int getSemestre(int dateInteger) {
        return (dateInteger & 0x3000) >>> 12;
    }

    public static int getTrimestre(int dateInteger) {
        return (dateInteger & 0xE00) >>> 9;
    }

    public static int getMois(int dateInteger) {
        return (dateInteger & 0x1E0) >>> 5;
    }

    public static int getJour(int dateInteger) {
        return dateInteger & 0x1F;
    }

    public static int troncateDateInteger(int dateInteger, short dateIntegerType) {
        switch (dateIntegerType) {
            case TYPE_ANNEE:
                return (dateInteger & 0xFFFC000);
            case TYPE_SEMESTRE:
                return (dateInteger & 0xFFFF000);
            case TYPE_TRIMESTRE:
                return (dateInteger & 0xFFFFE00);
            case TYPE_MOIS:
                return (dateInteger & 0xFFFFFE0);
            default:
                return dateInteger;
        }
    }

    public static boolean check(int dateInteger) {
        if (dateInteger == 0) {
            return false;
        }
        if ((dateInteger & 0xF0000000) != 0) {
            return false;
        }
        int annee = getAnnee(dateInteger);
        if ((annee < 1000) && (annee > 9999)) {
            return false;
        }
        if ((dateInteger & 0xFFFF) == 0) {
            return true;
        }
        int semestre = getSemestre(dateInteger);
        if ((semestre == 0) || (semestre == 3)) {
            return false;
        }
        int trimestre = getTrimestre(dateInteger);
        if ((trimestre == 0) || (trimestre > 4)) {
            return false;
        }
        if ((semestre == 1) && (trimestre > 2)) {
            return false;
        }
        if ((semestre == 2) && (trimestre < 3)) {
            return false;
        }
        int mois = getMois(dateInteger);
        if ((mois == 0) || (mois > 12)) {
            return false;
        }
        if ((trimestre == 1) && (mois > 3)) {
            return false;
        }
        if ((trimestre == 2) && ((mois < 4) || (mois > 6))) {
            return false;
        }
        if ((trimestre == 3) && ((mois < 7) || (mois > 9))) {
            return false;
        }
        if ((trimestre == 4) && (mois < 10)) {
            return false;
        }
        if (getJour(dateInteger) == 0) {
            return false;
        }
        return true;
    }

    public static String toString(int dateInteger) {
        StringBuilder buf = new StringBuilder();
        try {
            appendString(buf, dateInteger);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static void appendString(Appendable buf, int dateInteger) throws IOException {
        if (dateInteger < 1) {
            return;
        }
        int annee = getAnnee(dateInteger);
        buf.append(String.valueOf(annee));
        int semestre = getSemestre(dateInteger);
        if (semestre == 0) {
            return;
        }
        int trimestre = getTrimestre(dateInteger);
        if (trimestre == 0) {
            buf.append("-S");
            buf.append(String.valueOf(semestre));
            return;
        }
        int mois = getMois(dateInteger);
        if (mois == 0) {
            buf.append("-T");
            buf.append(String.valueOf(trimestre));
            return;
        }
        buf.append('-');
        if (mois < 10) {
            buf.append('0');
        }
        buf.append(String.valueOf(mois));
        int jour = getJour(dateInteger);
        if (jour == 0) {
            return;
        }
        buf.append('-');
        if (jour < 10) {
            buf.append('0');
        }
        buf.append(String.valueOf(jour));
    }

    /**
     * renvoie une chaîne permettant le classement
     */
    public static String toSortString(int dateInteger) {
        if (dateInteger < 1) {
            return "";
        }
        StringBuilder buf = new StringBuilder();
        try {
            appendSortString(dateInteger, buf);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    private static void appendSortString(int dateInteger, Appendable buf) throws IOException {
        int annee = getAnnee(dateInteger);
        buf.append(String.valueOf(annee));
        int semestre = getSemestre(dateInteger);
        if (semestre == 0) {
            return;
        }
        buf.append('S');
        buf.append(String.valueOf(semestre));
        int trimestre = getTrimestre(dateInteger);
        if (trimestre == 0) {
            return;
        }
        buf.append('T');
        buf.append(String.valueOf(trimestre));
        int mois = getMois(dateInteger);
        if (mois == 0) {
            return;
        }
        if (mois < 10) {
            buf.append('0');
        }
        buf.append(String.valueOf(mois));
        int jour = getJour(dateInteger);
        if (jour == 0) {
            return;
        }
        if (jour < 10) {
            buf.append('0');
        }
        buf.append(String.valueOf(jour));
    }

    public static int parse(String dateString) throws DateIntegerException {
        if (dateString == null) {
            throw new DateIntegerException("strg_datation is null");
        }
        int taille = dateString.length();
        if (taille == 4) {
            int annee = parseAnnee(dateString);
            return (annee << 14);
        } else if (taille == 7) {
            if (dateString.charAt(4) != '-') {
                throw new DateIntegerException("mauvais separateur apres l'annee");
            }
            int annee = parseAnnee(dateString);
            int datation = (annee << 14);
            char carac1 = dateString.charAt(5);
            char carac2 = dateString.charAt(6);
            if (carac1 == 'S') {
                if ((carac2 == '1') || (carac2 == '2')) {
                    return datation + ((carac2 - 48) << 12);
                } else {
                    throw new DateIntegerException("mauvaise valeur de semestre");
                }
            } else if (carac1 == 'T') {
                return datation + calculeTrimestre(carac2);
            } else if (carac1 == '0') {
                return datation + calculeMois(carac2, false);
            } else if (carac1 == '1') {
                return datation + calculeMois(carac2, true);
            } else {
                throw new DateIntegerException("mauvaise valeur de mois");
            }

        } else if (taille == 10) {
            if (dateString.charAt(4) != '-') {
                throw new DateIntegerException("mauvais separateur apres l'annee");
            }
            if (dateString.charAt(7) != '-') {
                throw new DateIntegerException("mauvais separateur apres le mois");
            }
            int annee = parseAnnee(dateString);
            int resultat = (annee << 14);
            char carac1 = dateString.charAt(5);
            char carac2 = dateString.charAt(6);
            if (carac1 == '0') {
                resultat = resultat + calculeMois(carac2, false);
            } else if (carac1 == '1') {
                resultat = resultat + calculeMois(carac2, true);
            } else {
                throw new DateIntegerException("mauvaise valeur de mois");
            }
            return resultat + calculeJour(dateString.charAt(8), dateString.charAt(9));
        }
        throw new DateIntegerException("taille de la chaine incorrecte");
    }

    private static int calculeTrimestre(char carac) throws DateIntegerException {
        int resultat = 0;
        int numero = carac - 48;
        if ((numero < 1) || (numero > 4)) {
            throw new DateIntegerException("mauvaise valeur de trimestre");
        }
        resultat = (numero << 9);
        if (numero < 3) {
            resultat = resultat + (1 << 12);
        } else {
            resultat = resultat + (2 << 12);
        }
        return resultat;
    }

    private static int calculeMois(char carac, boolean avecdizaine) throws DateIntegerException {
        int resultat = 0;
        if (avecdizaine) {
            if ((carac < '0') && (carac > '2')) {
                throw new DateIntegerException("mauvaise valeur de mois");
            }
        } else if ((carac < '1') || (carac > '9')) {
            throw new DateIntegerException("mauvaise valeur de mois");
        }
        int mois = carac - 48;
        if (avecdizaine) {
            mois = mois + 10;
            resultat = (mois << 5) + (2 << 12) + (4 << 9);
        } else {
            resultat = (mois << 5);
            if (mois < 4) {
                resultat = resultat + (1 << 12) + (1 << 9);
            } else if (mois < 7) {
                resultat = resultat + (1 << 12) + (2 << 9);
            } else {
                resultat = resultat + (2 << 12) + (3 << 9);
            }
        }
        return resultat;
    }

    private static int calculeJour(char carac1, char carac2) throws DateIntegerException {
        if ((carac1 < '0') && (carac1 > '3')) {
            throw new DateIntegerException("mauvaise valeur de jour");
        }
        if ((carac2 < '0') || (carac2 > '9')) {
            throw new DateIntegerException("mauvaise valeur de jour");
        }
        return (carac2 - 48) + ((carac1 - 48) * 10);
    }

    private static int parseAnnee(String chaine) throws DateIntegerException {
        int annee = 0;
        for (int i = 0; i < 4; i++) {
            char carac = chaine.charAt(i);
            if ((carac < '0') && (carac > '9')) {
                throw new DateIntegerException("mauvais caractere dans l'annee");
            }
            if ((i == 0) && (carac == '0')) {
                throw new DateIntegerException("annee inferieure a 1000");
            }
            annee = annee + ((int) Math.pow(10, (3 - i))) * (carac - 48);
        }
        return annee;
    }

    public static int fromAnnee(int annee) throws DateIntegerException {
        if ((annee < 1000) || (annee > 9999)) {
            throw new DateIntegerException("annee incorrecte");
        }
        return (annee << 14);
    }

    public static int fromSemestre(int annee, int semestre) throws DateIntegerException {
        if ((annee < 1000) || (annee > 9999)) {
            throw new DateIntegerException("annee incorrecte");
        }
        if ((semestre < 1) || (semestre > 2)) {
            throw new DateIntegerException("semestre incorrect");
        }
        int resultat = (annee << 14) + (semestre << 12);
        return resultat;
    }

    public static int fromTrimestre(int annee, int trimestre) throws DateIntegerException {
        if ((annee < 1000) || (annee > 9999)) {
            throw new DateIntegerException("annee incorrecte");
        }
        if ((trimestre < 1) || (trimestre > 4)) {
            throw new DateIntegerException("trimestre incorrect");
        }
        int resultat = (annee << 14) + (trimestre << 9);
        if (trimestre < 3) {
            resultat = resultat + (1 << 12);
        } else {
            resultat = resultat + (2 << 12);
        }
        return resultat;
    }

    public static int fromMois(int annee, int mois) throws DateIntegerException {
        if ((annee < 1000) || (annee > 9999)) {
            throw new DateIntegerException("annee incorrecte");
        }
        if ((mois < 1) || (mois > 12)) {
            throw new DateIntegerException("mois incorrect");
        }
        return fromInt(annee, mois, -1);
    }

    public static int fromJour(int annee, int mois, int jour) throws DateIntegerException {
        if ((annee < 1000) || (annee > 9999)) {
            throw new DateIntegerException("annee incorrecte");
        }
        if ((mois < 1) || (mois > 12)) {
            throw new DateIntegerException("mois incorrect");
        }
        if ((jour < 1) || (jour > 31)) {
            throw new DateIntegerException("jour incorrect");
        }
        return fromInt(annee, mois, jour);
    }

    private static int fromInt(int annee, int mois, int jour) {
        int resultat = (annee << 14);
        resultat = resultat + (mois << 5);
        if (jour != -1) {
            resultat = resultat + jour;
        }
        if (mois < 4) {
            resultat = resultat + (1 << 12) + (1 << 9);
        } else if (mois < 7) {
            resultat = resultat + (1 << 12) + (2 << 9);
        } else if (mois < 10) {
            resultat = resultat + (2 << 12) + (3 << 9);
        } else {
            resultat = resultat + (2 << 12) + (4 << 9);
        }
        return resultat;
    }

    public static GregorianCalendar toGregorianCalendar(int dateInteger) {
        GregorianCalendar gc = new GregorianCalendar();
        int annee = getAnnee(dateInteger);
        short type = getDateIntegerType(dateInteger);
        int mois = 1;
        int jour = 1;
        switch (type) {
            case TYPE_SEMESTRE:
                int semestre = getSemestre(dateInteger);
                if (semestre == 2) {
                    mois = 7;
                }
                break;
            case TYPE_TRIMESTRE:
                int trimestre = getTrimestre(dateInteger);
                if (trimestre == 2) {
                    mois = 4;
                } else if (trimestre == 3) {
                    mois = 7;
                } else if (trimestre == 4) {
                    mois = 10;
                }
                break;
            case TYPE_MOIS:
                mois = getMois(dateInteger);
                break;
            case TYPE_JOUR:
                mois = getMois(dateInteger);
                jour = getJour(dateInteger);
                break;
        }
        gc.set(annee, mois - 1, jour);
        return gc;
    }

    public static String toISOString(int dateInteger) {
        StringBuilder buf = new StringBuilder();
        try {
            appendCompleteString(buf, dateInteger, true, false);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    /**
     * Retourne sous la forme année-mois en forçant le mois. Si dernierMois est égal à true, c'est le dernier mois de la période considérée qui est pris en compte.
     */
    public static String toAnneeMoisString(int dateInteger, boolean dernierMois) {
        StringBuilder buf = new StringBuilder();
        try {
            appendCompleteString(buf, dateInteger, false, dernierMois);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static void appendCompleteString(Appendable buf, int dateInteger, boolean withDay, boolean lastDate) throws IOException {
        if (dateInteger == NULL) {
            return;
        }
        int annee = getAnnee(dateInteger);
        buf.append(String.valueOf(annee));
        buf.append('-');
        short type = getDateIntegerType(dateInteger);
        int mois = (lastDate) ? 12 : 1;
        int jour = -1;
        switch (type) {
            case TYPE_SEMESTRE:
                int semestre = getSemestre(dateInteger);
                if (lastDate) {
                    if (semestre == 1) {
                        mois = 6;
                    }
                } else {
                    if (semestre == 2) {
                        mois = 7;
                    }
                }
                break;
            case TYPE_TRIMESTRE:
                int trimestre = getTrimestre(dateInteger);
                if (lastDate) {
                    if (trimestre == 1) {
                        mois = 3;
                    } else if (trimestre == 2) {
                        mois = 6;
                    } else if (trimestre == 3) {
                        mois = 9;
                    }
                } else {
                    if (trimestre == 2) {
                        mois = 4;
                    } else if (trimestre == 3) {
                        mois = 7;
                    } else if (trimestre == 4) {
                        mois = 10;
                    }
                }
                break;
            case TYPE_MOIS:
                mois = getMois(dateInteger);
                break;
            case TYPE_JOUR:
                mois = getMois(dateInteger);
                if (withDay) {
                    jour = getJour(dateInteger);
                }
                break;
        }
        if (mois < 10) {
            buf.append("0");
        }
        buf.append(String.valueOf(mois));
        if (withDay) {
            if (jour == -1) {
                if (lastDate) {
                    jour = getDernierJour(mois, annee);
                } else {
                    jour = 1;
                }
            }
            buf.append("-");
            if (jour < 10) {
                buf.append("0");
            }
            buf.append(String.valueOf(jour));
        }
    }

    public static int fromGregorianCalendar(GregorianCalendar gregcalendar) {
        return fromInt(gregcalendar.get(Calendar.YEAR), gregcalendar.get(Calendar.MONTH) + 1, gregcalendar.get(Calendar.DAY_OF_MONTH));
    }

    public static int fromDate(Date date) {
        GregorianCalendar gc = new GregorianCalendar();
        gc.setTime(date);
        return fromGregorianCalendar(gc);
    }

    /**
     * Raccourci pour toGregorianCalendar().getTime()
     */
    public static Date toDate(int dateInteger) {
        return toGregorianCalendar(dateInteger).getTime();
    }

    public static int current() {
        return fromGregorianCalendar(new GregorianCalendar());
    }

    public static int compare(int dateInteger1, int dateInteger2) {
        if (dateInteger1 > dateInteger2) {
            return 1;
        } else if (dateInteger1 < dateInteger2) {
            return -1;
        } else {
            return 0;
        }
    }

    public static String getDateLibelle(int dateInteger, DateFormatBundle dateFormatBundle) {
        short type = getDateIntegerType(dateInteger);
        DateFormat dateformat = null;
        switch (type) {
            case DateInteger.TYPE_JOUR:
                dateformat = dateFormatBundle.getFormatJour();
                break;
            case DateInteger.TYPE_MOIS:
                dateformat = dateFormatBundle.getFormatMois();
                break;
            case DateInteger.TYPE_ANNEE:
                dateformat = dateFormatBundle.getFormatAnnee();
                break;
            case DateInteger.TYPE_SEMESTRE:
                int semestre = getSemestre(dateInteger);
                return dateFormatBundle.getLocalizedString("s" + String.valueOf(semestre)) + " " + String.valueOf(getAnnee(dateInteger));
            case DateInteger.TYPE_TRIMESTRE:
                int trimestre = getTrimestre(dateInteger);
                return dateFormatBundle.getLocalizedString("t" + String.valueOf(trimestre)) + " " + String.valueOf(getAnnee(dateInteger));
        }
        if (dateformat != null) {
            return dateformat.format(toDate(dateInteger));
        }
        return toString(dateInteger);
    }

    /**
     *
     *@throws IllegalArgumentException si s est nul ou vide
     */
    public static int parse(String s, DateFormatBundle dateFormatBundle) throws ParseException {
        if ((s == null) || (s.length() == 0)) {
            throw new IllegalArgumentException("strg_datation is null");
        }
        try {
            int dateInteger = parse(s);
            return dateInteger;
        } catch (DateIntegerException dee) {
            if (dateFormatBundle != null) {
                try {
                    Date date = dateFormatBundle.getParseFormat().parse(s);
                    return DateInteger.fromDate(date);
                } catch (ParseException e) {
                }
            }
            int idx = s.indexOf('/');
            if (idx > 0) {
                Date date = ISO_FORMAT.parse(s.substring(idx + 1) + "-" + s.substring(0, idx) + "-01");
                int datation = DateInteger.fromDate(date);
                return DateInteger.troncateDateInteger(datation, DateInteger.TYPE_MOIS);
            } else {
                throw new ParseException("unable to parse", 0);
            }
        }

    }

    public static short typeToShort(String typeString) {
        if (typeString.equals("jour")) {
            return TYPE_JOUR;
        }
        if (typeString.equals("mois")) {
            return TYPE_MOIS;
        }
        if (typeString.equals("trimestre")) {
            return TYPE_TRIMESTRE;
        }
        if (typeString.equals("semestre")) {
            return TYPE_SEMESTRE;
        }
        if (typeString.equals("annee")) {
            return TYPE_ANNEE;
        }
        throw new IllegalArgumentException("unknown type = " + typeString);
    }

    private static int getDernierJour(int mois, int annee) {
        switch (mois) {
            case 1:
                return 31;
            case 2:
                if (isBissextile(annee)) {
                    return 29;
                } else {
                    return 28;
                }
            case 3:
                return 31;
            case 4:
                return 30;
            case 5:
                return 31;
            case 6:
                return 30;
            case 7:
                return 31;
            case 8:
                return 31;
            case 9:
                return 30;
            case 10:
                return 31;
            case 11:
                return 30;
            case 12:
                return 31;
            default:
                throw new IllegalArgumentException("unknown mois");
        }
    }

    private static boolean isBissextile(int annee) {
        if ((annee % 4) != 0) {
            return false;
        }
        if ((annee % 400) == 0) {
            return true;
        }
        if ((annee % 100) == 0) {
            return false;
        }
        return true;
    }

}
