/* UtilLib - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.io.Serializable;


/**
 *La partie décimale n'a jamais plus de six chiffres. La partie entière est comprise entre 180 et -179
 *
 * @author Vincent Calame
 */
public final class DegreDecimal implements Serializable {

    private static final long serialVersionUID = 1L;
    private int partieEntiere;
    private int partieDecimale;
    private byte zeroLength;

    public DegreDecimal(int partieEntiere, byte zeroLength, int partieDecimale) {
        this.partieEntiere = modulo(partieEntiere);
        if (partieEntiere < 0) {
            if (partieDecimale > 0) {
                partieDecimale = -partieDecimale;
            }
        } else if (partieEntiere > 0) {
            if (partieDecimale < 0) {
                partieDecimale = -partieDecimale;
            }
        }
        if (zeroLength < 0) {
            zeroLength = (byte) (-zeroLength);
        }
        this.zeroLength = zeroLength;
        this.partieDecimale = partieDecimale;
        check();
    }

    public DegreDecimal(DegreSexagesimal degreSexagesimal) {
        int degre = degreSexagesimal.getDegre();
        int seconde = ((degreSexagesimal.getMinute()) * 60) + degreSexagesimal.getSeconde();
        int mm = Math.round((((float) seconde) * 2500) / 9);
        if (mm == 0) {
            this.zeroLength = 6;
        } else {
            String s = String.valueOf(mm);
            this.zeroLength = (byte) (6 - s.length());
        }
        if (degreSexagesimal.isPositif()) {
            this.partieEntiere = -degre;
            this.partieDecimale = -mm;
        } else {
            this.partieEntiere = degre;
            this.partieDecimale = mm;
        }
    }

    public DegreDecimal(Decimal decimal) {
        this.partieEntiere = modulo(decimal.getPartieEntiere());
        this.zeroLength = decimal.getZeroLength();
        this.partieDecimale = decimal.getPartieDecimale();
        check();
    }

    public boolean isPositif() {
        return ((partieEntiere >= 0) && (partieDecimale >= 0));
    }

    private int modulo(long partieEntiere) {
        long modulo = (partieEntiere) / 360;
        int result = (int) (partieEntiere - (modulo * 360));
        return result;
    }

    private void check() {
        checkDecimale();
        checkDegre();
    }

    private void checkDegre() {
        if (Math.abs(partieEntiere) < 180) {
            return;
        }
        if ((partieEntiere == 180) && (partieDecimale == 0)) {
            return;
        }
        Decimal newDecimal;
        if (partieEntiere > 180) {
            newDecimal = Decimal.add(new Decimal(-360, (byte) 0, 0), new Decimal(partieEntiere, zeroLength, partieDecimale));
        } else {
            newDecimal = Decimal.add(new Decimal(360, (byte) 0, 0), new Decimal(partieEntiere, zeroLength, partieDecimale));
        }
        this.partieEntiere = (int) newDecimal.getPartieEntiere();
        this.partieDecimale = newDecimal.getPartieDecimale();
        this.zeroLength = newDecimal.getZeroLength();
    }

    private void checkDecimale() {
        if (partieDecimale == 0) {
            zeroLength = 0;
            return;
        }
        if (zeroLength > 6) {
            zeroLength = 0;
            partieDecimale = 0;
        }
        int dec = Math.abs(partieDecimale);
        boolean negatif = (partieDecimale < 0);
        while (true) {
            if (dec % 10 == 0) {
                dec = dec / 10;
            } else {
                break;
            }
        }
        String s = String.valueOf(dec);
        if (zeroLength == 6) {
            if (s.charAt(0) > '5') {
                zeroLength = 5;
                partieDecimale = (negatif) ? -1 : 1;
            } else {
                zeroLength = 0;
                partieDecimale = 0;
            }
            return;
        }
        int p = 6 - zeroLength;
        if (s.length() <= p) {
            return;
        }
        dec = Integer.parseInt(s.substring(0, p));
        if (s.charAt(p) > '5') {
            boolean neufS = true;
            for (int i = 0; i < p; i++) {
                if (s.charAt(i) != '9') {
                    neufS = false;
                }
            }
            if (!neufS) {
                dec = dec + 1;
            } else if (zeroLength > 0) {
                zeroLength--;
                dec = 1;
            } else {
                dec = 0;
                if (partieEntiere > 0) {
                    partieEntiere = partieEntiere + 1;
                } else if (partieEntiere < 0) {
                    partieEntiere = partieEntiere - 1;
                } else {
                    if (negatif) {
                        partieEntiere = -1;
                    } else {
                        partieEntiere = 1;
                    }
                }
            }
        }
        partieDecimale = (negatif) ? -dec : dec;
    }

    public Decimal toDecimal() {
        return new Decimal(partieEntiere, zeroLength, partieDecimale);
    }

    @Override
    public String toString() {
        return Decimal.toString(partieEntiere, zeroLength, partieDecimale);
    }

    public int getPartieEntiere() {
        return partieEntiere;
    }

    public int getPartieDecimale() {
        return partieDecimale;
    }

    public byte getZeroLength() {
        return zeroLength;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof DegreDecimal)) {
            return false;
        }
        DegreDecimal other = (DegreDecimal) obj;
        if (other.partieEntiere != this.partieEntiere) {
            return false;
        }
        if (other.partieDecimale != this.partieDecimale) {
            return false;
        }
        if (other.zeroLength != this.zeroLength) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        return (int) partieEntiere;
    }

    public DegreSexagesimal toDegreSexagesimal() {
        return DegreSexagesimal.fromDegreDecimal(this);
    }

}
