/* UtilLib - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.util.AbstractList;
import java.util.RandomAccess;


/**
 *
 * @author Vincent Calame
 */
public class RangeList extends AbstractList<Range> implements RandomAccess {

    private int size = 0;
    private Range[] ranges = new Range[2];
    int globalMin = Range.MAXIMUM;
    int globalMax = Range.MINIMUM;

    public RangeList() {
    }

    public int size() {
        return size;
    }

    public Range get(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return ranges[index];
    }

    public int getGlobalMin() {
        return globalMin;
    }

    public int getGlobalMax() {
        return globalMax;
    }

    public boolean contains(int value) {
        if ((value < globalMin) || (value > globalMax)) {
            return false;
        }
        for (int i = 0; i < size; i++) {
            if (ranges[i].contains(value)) {
                return true;
            }
        }
        return false;
    }

    /**
     *Équivalent de (Range) get(index)
     *
     */
    public Range getRange(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return ranges[index];
    }

    /**
     *
     *@throws IllegalArgumentException si range est nul
     */
    public void addRange(Range range) {
        if (range == null) {
            throw new IllegalArgumentException("range is null");
        }
        if (size == 0) {
            ranges[0] = range;
            globalMin = range.min();
            globalMax = range.max();
            size = 1;
            return;
        }
        if (range.min() == (globalMax + 1)) {
            ranges[size - 1] = ranges[size - 1].union(range);
            globalMax = range.max();
        } else if (range.min() > globalMax) {
            checkLength();
            ranges[size] = range;
            size++;
            globalMax = range.max();
        } else if (range.max() == (globalMin - 1)) {
            ranges[0] = ranges[0].union(range);
            if (size > 1) {
                mergeFollowing(0);
            }
            globalMin = range.min();
        } else if (range.max() < globalMin) {
            checkLength();
            System.arraycopy(ranges, 0, ranges, 1, size);
            ranges[0] = range;
            globalMin = range.min();
            size++;
        } else {
            globalMin = Math.min(globalMin, range.min());
            globalMax = Math.max(globalMax, range.max());
            mergeRanges(range);
        }
    }

    private void mergeRanges(Range range) {
        int nearestIndex = -1;
        for (int i = 0; i < size; i++) {
            Range current = ranges[i];
            if (current.intersects(range) || current.isNeighbour(range)) {
                Range union = current.union(range);
                if (union.equals(current)) {
                    return;
                }
                ranges[i] = union;
                if (i < (size - 1)) {
                    mergeFollowing(i);
                }
                return;
            } else {
                if (current.max() < range.min()) {
                    nearestIndex = i;
                }
            }
        }
        nearestIndex = nearestIndex + 1;
        int nvlength = (size == ranges.length) ? size * 2 : ranges.length;
        Range[] temp = new Range[nvlength];
        System.arraycopy(ranges, 0, temp, 0, nearestIndex);
        temp[nearestIndex] = range;
        System.arraycopy(ranges, nearestIndex, temp, nearestIndex + 1, (size - nearestIndex));
        ranges = temp;
        size++;
    }

    private void mergeFollowing(int index) {
        int length = 0;
        Range start = ranges[index];
        for (int i = index + 1; i < size; i++) {
            Range current = ranges[i];
            if (current.intersects(start) || current.isNeighbour(start)) {
                length++;
                start = start.union(current);
            } else {
                break;
            }
        }
        if (length > 0) {
            ranges[index] = start;
            for (int i = index + length + 1; i < size; i++) {
                ranges[i - length] = ranges[i];
            }
            for (int i = size - length; i < size; i++) {
                ranges[i] = null;
            }
            size = size - length;
        }
    }

    private void checkLength() {
        if (size == ranges.length) {
            Range[] nv = new Range[size * 2];
            System.arraycopy(ranges, 0, nv, 0, size);
            ranges = nv;
        }
    }

}
