/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.io.Reader;
import java.io.Writer;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;


/**
 * Dans les gabarits, les variables sont entourées des caractères $,
 * doivent commencer par une lettre et comporter les caractères suivants : [A-Za-z0-9_]
 *
 * @author Vincent Calame
 */
public class Gabarit {

    static private final short TEXT = 0;
    static private final short VARIABLE_BEGIN = 1;
    static private final short VARIABLE_COURSE = 2;
    private ValueResolver valueResolver;

    public Gabarit(ValueResolver valueResolver) {
        this.valueResolver = valueResolver;
    }

    public void format(Reader gabaritReader, Appendable appendable) throws IOException {
        int character;
        ReadState readState = new ReadState(appendable);
        while ((character = gabaritReader.read()) != -1) {
            readState.read((char) character);
        }
        readState.end();
    }

    public String format(String gabaritString) {
        StringReader reader = new StringReader(gabaritString);
        try {
            return format(reader);
        } catch (IOException ioe) {
            // n'a jamais lieu !
        }
        return "";
    }

    public String format(Reader gabaritReader) throws IOException {
        StringWriter writer = new StringWriter();
        format(gabaritReader, writer);
        return writer.toString();
    }

    public String format(InputStream inputStream, String encoding) throws IOException {
        BufferedReader bufreader = new BufferedReader(new InputStreamReader(inputStream, encoding));
        return format(bufreader);
    }


    private class ReadState {

        private Appendable appendable;
        private short state = TEXT;
        private StringBuilder variableBuffer = null;

        ReadState(Appendable appendable) {
            this.appendable = appendable;
        }

        public void read(char c) throws IOException {
            switch (state) {
                case TEXT:
                    if (c == '$') {
                        state = VARIABLE_BEGIN;
                    } else {
                        appendable.append(c);
                    }
                    break;
                case VARIABLE_BEGIN:
                    if (testVariableFirstChar(c)) {
                        variableBuffer = new StringBuilder();
                        state = VARIABLE_COURSE;
                        variableBuffer.append(c);
                    } else {
                        state = TEXT;
                        appendable.append('$');
                        appendable.append(c);
                    }
                    break;
                case VARIABLE_COURSE:
                    if (testVariableChar(c)) {
                        variableBuffer.append(c);
                    } else if (c == '$') {
                        replace();
                    } else {
                        flush();
                        if (c == '$') {
                            state = VARIABLE_BEGIN;
                        } else {
                            appendable.append(c);
                        }
                    }
                    break;
            }
        }

        void flush() throws IOException {
            if (state != TEXT) {
                appendable.append('$');
                if (variableBuffer != null) {
                    appendable.append(variableBuffer.toString());
                }
            }
            state = TEXT;
            variableBuffer = null;
        }

        void replace() throws IOException {
            String key = variableBuffer.toString();
            String text = valueResolver.getValue(new InternalAccoladeArgument(key));
            if (text != null) {
                appendable.append(text);
            } else {
                appendable.append('$');
                appendable.append(key);
                appendable.append('$');
            }
            variableBuffer = null;
            state = TEXT;
        }

        void end() throws IOException {
            flush();
        }

        /*
         * Test du premier caractère : seuls autorisés : A-Za-z
         */
        boolean testVariableFirstChar(char c) {
            if ((c >= 'A') && (c <= 'Z')) {
                return true;
            }
            if ((c >= 'a') && (c <= 'z')) {
                return true;
            }
            return false;
        }

        /*
         * Ne sont acceptés que A-Z, a-z et '_'.
         */
        boolean testVariableChar(char c) {
            if (c == '_') {
                return true;
            }
            if ((c >= 'A') && (c <= 'Z')) {
                return true;
            }
            if ((c >= 'a') && (c <= 'z')) {
                return true;
            }
            if ((c >= '0') && (c <= '9')) {
                return true;
            }
            return false;
        }

    }


    private static class InternalAccoladeArgument implements AccoladeArgument {

        private String name;

        InternalAccoladeArgument(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public String getMode() {
            return null;
        }

    }

}
