/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.util.List;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.impl.InstructionParser;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class LibelleUtils {

    private LibelleUtils() {
    }

    public static boolean areEquals(Libelle lib1, Libelle lib2) {
        if (lib1 == null) {
            return (lib2 == null);
        } else if (lib2 == null) {
            return false;
        }
        if (lib1.getLangInteger() != lib2.getLangInteger()) {
            return false;
        }
        return lib1.getLibelleString().equals(lib2.getLibelleString());
    }

    /*
     * Retourne nul si vide.
     * si l'instruction ne contient aucun signe égal (=), retourne un libelleHolder vide mais avec la chaîne par défaut.
     * si l'instruction est incorrecte, retourne un libelleHolder vide avec un message d'erreur par défaut.
     * l'instruction doit être sous la forme fr="",en="", etc.
     * defaultString n'est utilisé que s'il n'y a pas d'erreur.
     */
    public static LibelleHolder parseInstruction(String s, String defaultString) {
        if (s == null) {
            return null;
        }
        s = s.trim();
        if (s.length() == 0) {
            return null;
        }
        if (s.indexOf('=') == -1) {
            return toEmptyLibelleHolder(s);
        }
        InstructionParseErrorHandler errorHandler = new InstructionParseErrorHandler();
        Instruction instruction = InstructionParser.parse(s, errorHandler);
        if (errorHandler.hasError()) {
            return toEmptyLibelleHolder(errorHandler.getErrorMessage());
        }
        if (instruction == null) {
            return toEmptyLibelleHolder(s);
        }
        ChangeLibelleHolderBuilder builder = new ChangeLibelleHolderBuilder(defaultString);
        int length = instruction.getLength();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            try {
                int langInteger = LangInteger.parse(argument.getKey());
                builder.putLibelle(langInteger, argument.getValue());
            } catch (LangIntegerException lie) {
            }
        }
        return builder.toChangeLibelleHolder();
    }

    public static LibelleHolder toLibelleHolder(List<Libelle> libelleList) {
        InternalLibelleHolder libelleHolder = new InternalLibelleHolder(null);
        for (Libelle libelle : libelleList) {
            libelleHolder.innerPutLibelle(libelle);
        }
        return libelleHolder;
    }

    public static LibelleHolder toEmptyLibelleHolder(CharSequence defaultLib) {
        CleanedString cleanedString = CleanedString.newInstance(defaultLib);
        if (cleanedString == null) {
            return new EmptyLibelleHolder("?");
        } else {
            return new EmptyLibelleHolder(cleanedString.toString());
        }
    }

    /**
     * Retourne nul si vide
     */
    public static Libelle readLib(Element element) throws LangIntegerException {
        int langInteger = LangInteger.parse(element.getAttribute("xml:lang"));
        CleanedString libString = DOMUtils.contentToCleanedString(element);
        if (libString == null) {
            return null;
        }
        return toLibelle(langInteger, libString);
    }

    public static Libelle toLibelle(int langInteger, CleanedString cleanedString) {
        if (cleanedString == null) {
            throw new IllegalArgumentException("cleanedString is null");
        }
        return new InternalLibelle(langInteger, cleanedString.toString());
    }


    private static class EmptyLibelleHolder implements LibelleHolder {

        private String defaultString;

        private EmptyLibelleHolder(String defaultString) {
            this.defaultString = defaultString;
        }

        public String getLibelleString(int langInteger, boolean defaut) {
            return defaultString;
        }

        public int getLibelleCount() {
            return 0;
        }

        public Libelle getLibelleByIndex(int i) {
            throw new IndexOutOfBoundsException();
        }

        public Libelle getLibelleByLang(int langInteger) {
            return null;
        }

    }


    public static class InstructionParseErrorHandler implements InstructionParser.ErrorHandler {

        private String errorMessage;

        public void invalidAsciiCharacterError(String part, int row, int col) {
            errorMessage = "#invalidAsciiCharacter: " + part + " (" + row + "," + col + ")#";
        }

        public void invalidEndCharacterError(String part, int row, int col) {
            errorMessage = "#invalidEndCharacter: " + part + " (" + row + "," + col + ")#";
        }

        public void invalidSeparatorCharacterError(String part, int row, int col) {
            errorMessage = "#invalidSeparatorCharacter: " + part + " (" + row + "," + col + ")#";
        }

        private boolean hasError() {
            return (errorMessage != null);
        }

        private String getErrorMessage() {
            return errorMessage;
        }

    }


    private static class InternalLibelleHolder extends DefaultLibelleHolder {

        public InternalLibelleHolder(String defaultLibelleString) {
            super(defaultLibelleString);
        }

    }


    private static class InternalLibelle implements Libelle {

        private int langInteger;
        private String libelleString;

        private InternalLibelle(int langInteger, String libelleString) {
            this.langInteger = langInteger;
            this.libelleString = libelleString;
        }

        public int getLangInteger() {
            return langInteger;
        }

        public String getLibelleString() {
            return libelleString;
        }

    }

}
