/* UtilLib - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.collation.map;

import net.mapeadores.util.text.collation.CollationUnit;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.TextConstants;
import java.util.Locale;
import java.util.Collection;
import java.util.Collections;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import java.text.RuleBasedCollator;
import java.text.Collator;


/**
 * Classe permettant de stocker des valeurs suivant des clés qui sont des chaînes traités par « collation ».
 * Lorsque une valeur est stocké suivant une clé, celle-ci subit d'abord une « collation » (dont le résultat s'appele collatedKey).
 * Autrement dit, deux chaînes key1 et key2 sont des clés identiques non pas si key1.equals(key2) mais si toCollatedString(key1).equals(toCollatedString(key2)) 
 *
 * @author  Vincent Calame
 */
public class SortedCollatedKeyMap implements CollatedKeyMap {

    private RuleBasedCollator collator;
    private Locale locale;
    private SortedMap<String, Object> valuesMap = new TreeMap<String, Object>();

    /**
     * Construit une instance de SortedCollatedKeyMap en prenant comme « collator » Collator.getInstance(locale)
     */
    public SortedCollatedKeyMap(Locale locale) {
        this.locale = locale;
        collator = (RuleBasedCollator) Collator.getInstance(locale);
        collator.setStrength(Collator.PRIMARY);
    }

    /**
     * Construit une instance de SortedCollatedKeyMap en prenant comme « collator » Collator.getInstance(LangInteger.getLocale(langInteger))
     */
    public SortedCollatedKeyMap(int langInteger) {
        this.locale = LangInteger.getLocale(langInteger);
        collator = (RuleBasedCollator) Collator.getInstance(locale);
        collator.setStrength(Collator.PRIMARY);
    }

    public Collection<Object> values() {
        return valuesMap.values();
    }

    /**
     * Retourne l'instance de Collator utilisée pour la création des « collatedKey » et la recherche.
     */
    public Collator getCollator() {
        return collator;
    }

    /**
     * Retourne l'objet Locale qui a servi à la construction de l'objet.
     */
    public Locale getLocale() {
        return locale;
    }

    /**
     * Retourne le nombre de valeurs stockées dans l'instance de SortedCollatedKeyMap.
     */
    public int size() {
        return valuesMap.size();
    }

    /**
     * Retourne la valeur stockée à la clé « s ». La chaîne s subit d'abord une « collation ».
     * Cette méthode est équivalente à getValueByCollatedKey(toCollateKey(key))
     */
    public Object getValue(String key) {
        String collatedKey = CollationUnit.collate(key, collator);
        return valuesMap.get(collatedKey);
    }

    /**
     * Retourne la valeur stockée à la clé déjà « collatée », collatedKey.
     */
    public Object getValueByCollatedKey(String collatedKey) {
        return valuesMap.get(collatedKey);
    }

    /**
     * Stocke la valeur à la clé Key. Équivalent de putValueByCollatedKey(toCollatedKey(key),value).
     */
    public void putValue(String key, Object value) {
        String collatedKey = CollationUnit.collate(key, collator).intern();
        valuesMap.put(collatedKey, value);
    }

    /**
     * Stocke la valeur à la clé déjà « collatée » collatedKey.
     */
    public void putValueByCollatedKey(String collatedKey, Object value) {
        valuesMap.put(collatedKey, value);
    }

    /**
     * Supprime la valeur référencée à la clé key.
     */
    public Object removeValue(String key) {
        String collatedKey = CollationUnit.collate(key, collator);
        return valuesMap.remove(collatedKey);
    }

    /**
     * Supprime la valeur à la clé déjà « collatée » collatedKey.
     */
    public Object removeValueByCollatedKey(String collatedKey) {
        return valuesMap.remove(collatedKey);
    }

    /**
     * Crèe un objet CollationUnit avec comme source s et en se basant sur le Collator de l'instance de SortedCollatedKeyMap
     * Équivalent de new CollationUnit(s,getCollator().
     */
    public CollationUnit collateString(String s) {
        return new CollationUnit(s, collator);
    }

    /**
     * Retourne la forme « collatée » d'une clé.
     */
    public String toCollatedKey(String key) {
        return CollationUnit.collate(key, collator);
    }

    /**
     * Effectue une recherche sur les clés de l'instance de SortedCollatedKeyMap et retourne le résultat sous la forme d'une instance de searchResultItemList.
     * Cette méthode vérifie la présence de searchText dans les clés suivant trois conditions : TextConstants.CONTAINS, TextConstants.ENDSWITH, TextConstants.STARTSWITH
     * searchText subit évidemment une « collation » pour que les conditions répondent aux règles du Collator.
     */
    public SearchResultUnitList search(String searchText, int type, ValueFilter valueFilter) {
        List<InternalSearchResultUnit> list = internalSearch(searchText, type, valueFilter);
        return new InternalSearchResultUnitList(list);
    }

    public void clear() {
        valuesMap.clear();
    }

    private List<InternalSearchResultUnit> internalSearch(String searchText, int type, ValueFilter valueFilter) {
        if (valueFilter == null) {
            valueFilter = CollationMapUtils.ALL_VALUEFILTER;
        }
        switch (type) {
            case TextConstants.CONTAINS:
                return getValuesContaining(searchText, valueFilter);
            case TextConstants.ENDSWITH:
                return getValuesEndingWith(searchText, valueFilter);
            case TextConstants.MATCHES:
                String collatedKey = CollationUnit.collate(searchText, collator);
                Object value = valuesMap.get(collatedKey);
                if ((value == null) || (!valueFilter.acceptValue(value))) {
                    return Collections.emptyList();
                } else {
                    return Collections.singletonList(new InternalSearchResultUnit(value, collatedKey, new SubstringPosition(0, collatedKey.length())));
                }
            case TextConstants.STARTSWITH:
                return getValuesStartingWith(searchText, valueFilter);
            default:
                throw new IllegalArgumentException("unknwon TextConstants type : " + type);
        }
    }

    /*
     * Méthode appelée par search(). 
     * Retourne la liste des valeurs dont la clé commence par s
     */
    private List<InternalSearchResultUnit> getValuesStartingWith(String s, ValueFilter valueFilter) {
        String searchkey = CollationUnit.collate(s, collator);
        List<InternalSearchResultUnit> searchResultItemList = new ArrayList<InternalSearchResultUnit>();
        int minlength = searchkey.length();
        SortedMap<String, Object> sorted = valuesMap.tailMap(searchkey);
        for (Map.Entry<String, Object> couple : sorted.entrySet()) {
            String currentkey = couple.getKey();
            if (currentkey.length() < minlength) {
                break;
            } else if (currentkey.startsWith(searchkey)) {
                Object value = couple.getValue();
                if (valueFilter.acceptValue(value)) {
                    InternalSearchResultUnit searchResultItem = new InternalSearchResultUnit(value, currentkey, new SubstringPosition(0, minlength));
                    searchResultItemList.add(searchResultItem);
                }
            } else {
                break;
            }
        }
        return searchResultItemList;
    }

    /*
     * Méthode appelée par search(). 
     * Retourne la liste des valeurs dont la clé termine par s
     */
    private List<InternalSearchResultUnit> getValuesEndingWith(String s, ValueFilter valueFilter) {
        String searchkey = CollationUnit.collate(s, collator);
        List<InternalSearchResultUnit> searchResultItemList = new ArrayList<InternalSearchResultUnit>();
        int minlength = searchkey.length();
        for (Map.Entry<String, Object> couple : valuesMap.entrySet()) {
            String currentkey = couple.getKey();
            if (currentkey.length() < minlength) {
                continue;
            } else if (currentkey.endsWith(searchkey)) {
                Object value = couple.getValue();
                if (valueFilter.acceptValue(value)) {
                    InternalSearchResultUnit searchResultItem = new InternalSearchResultUnit(value, currentkey, new SubstringPosition(currentkey.length() - minlength, minlength));
                    searchResultItemList.add(searchResultItem);
                }
            } else {
                continue;
            }
        }
        return searchResultItemList;
    }

    /*
     * Méthode appelée par search(). 
     * Retourne la liste des valeurs dont la clé contient s
     */
    private List<InternalSearchResultUnit> getValuesContaining(String s, ValueFilter valueFilter) {
        String searchkey = CollationUnit.collate(s, collator);
        List<InternalSearchResultUnit> searchResultItemList = new ArrayList<InternalSearchResultUnit>();
        int minlength = searchkey.length();
        for (Map.Entry<String, Object> couple : valuesMap.entrySet()) {
            String currentkey = couple.getKey();
            if (currentkey.length() < minlength) {
                continue;
            } else {
                int idx = currentkey.indexOf(searchkey);
                if (idx != -1) {
                    Object value = couple.getValue();
                    if (valueFilter.acceptValue(value)) {
                        InternalSearchResultUnit searchResultItem = new InternalSearchResultUnit(couple.getValue(), currentkey, new SubstringPosition(idx, minlength));
                        searchResultItemList.add(searchResultItem);
                    }
                }
            }
        }
        return searchResultItemList;
    }


    static class InternalSearchResultUnitList implements SearchResultUnitList {

        private List<InternalSearchResultUnit> list;

        private InternalSearchResultUnitList(List<InternalSearchResultUnit> list) {
            this.list = list;
        }

        public int getSearchResultUnitCount() {
            return list.size();
        }

        public SearchResultUnit getSearchResultUnit(int i) {
            return list.get(i);
        }

    }


    /**
     * La classe InternalSearchResultItem stocke les informations d'un résultat de recherche effectuée via la méthode
     */
    static class InternalSearchResultUnit implements SearchResultUnit {

        private Object value;
        private SubstringPosition collatedSearchTextPosition;
        private String collatedKey;

        /*
         * Construction du résultat de recherche.
         */
        private InternalSearchResultUnit(Object value, String collatedKey, SubstringPosition collatedSearchTextPosition) {
            this.value = value;
            this.collatedKey = collatedKey;
            this.collatedSearchTextPosition = collatedSearchTextPosition;
        }

        /**
         * Retourne la valeur trouvée par la recherche.
         */
        public Object getValue() {
            return value;
        }

        /**
         * Retourne la position du texte de recherche (après « collation ») au sein de la clé.
         *
         */
        public SubstringPosition getSearchTextPositionInCollatedKey() {
            return collatedSearchTextPosition;
        }

        /**
         * Retourne la clé à laquelle est stockée la valeur trouvée dans l'instance SortedCollatedKeyMap.
         */
        public String getCollatedKey() {
            return collatedKey;
        }

    }

}
