/* UtilLib - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.insertpattern.impl;

import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.api.InstructionErrorHandler;
import net.mapeadores.util.text.insertpattern.api.InsertionHandler;
import net.mapeadores.util.text.insertpattern.api.Parser;
import net.mapeadores.util.text.insertpattern.api.CharacterErrorException;
import net.mapeadores.util.text.StringUtils;
import java.util.List;
import java.util.ArrayList;
import java.util.Stack;
import java.io.Reader;
import java.io.LineNumberReader;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class InsertpatternParser implements Parser {

    final static int WAITING_FROM_ARGUMENT_END_STEP = 3;
    final static int WAITING_FROM_KEY_STEP = 4;
    final static int WAITING_FROM_VALUE_STEP = 5;
    final static int WAITING_FROM_EQUAL_STEP = 6;
    final static int KEY_STEP = 7;
    final static int QUOTED_KEY_STEP = 8;
    final static int VALUE_STEP = 9;
    final static int QUOTED_VALUE_STEP = 10;
    InsertionHandler insertionHandler;
    InstructionErrorHandler errorHandler;
    ZoneParser currentZoneParser;
    IfCounter ifCounter = new IfCounter();

    public InsertpatternParser() {
    }

    public void setInsertionHandler(InsertionHandler insertionHandler) {
        this.insertionHandler = insertionHandler;
    }

    public void setErrorHandler(InstructionErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
    }

    public void parse(Reader reader) throws IOException {
        if (insertionHandler == null) {
            throw new IllegalStateException("not set yet : documentHandler");
        }
        currentZoneParser = new LiteralParser(1);
        LineNumberReader bufreader = new LineNumberReader(reader);
        String line = null;
        while ((line = bufreader.readLine()) != null) {
            try {
                parseLine(line, bufreader.getLineNumber());
            } catch (CharacterErrorException cee) {
                return;
            }
        }
        currentZoneParser.flush();
    }

    private void parseLine(String line, int lineNumber) {
        currentZoneParser.newLine(lineNumber);
        int length = line.length();
        for (int i = 0; i < length; i++) {
            i = currentZoneParser.parseCharAt(line, i);
        }
    }


    private abstract class ZoneParser {

        int currentLineNumber;

        public ZoneParser(int currentLineNumber) {
            this.currentLineNumber = currentLineNumber;
        }

        public int getCurrentLineNumber() {
            return currentLineNumber;
        }

        public void newLine(int lineNumber) {
            this.currentLineNumber = lineNumber;
            checkNewLine();
        }

        protected abstract void checkNewLine();

        public abstract void flush();

        public abstract int parseCharAt(String line, int index);

    }

    private void throwCharacterError(char carac, String key, int col) {
        if (errorHandler != null) {
            errorHandler.characterError(carac, key, currentZoneParser.getCurrentLineNumber(), col);
        }
        throw new CharacterErrorException(carac, key, currentZoneParser.getCurrentLineNumber(), col);
    }


    private class InsertionParser extends ZoneParser {

        List instructionList = new ArrayList();
        InstructionParser instructionParser;

        InsertionParser(int currentLineNumber) {
            super(currentLineNumber);
        }

        protected void checkNewLine() {
            if (instructionParser != null) {
                instructionParser.checkNewLine();
            }
        }

        public int parseCharAt(String line, int index) {
            if (instructionParser != null) {
                return instructionParser.parseCharAt(line, index);
            } else {
                char carac = line.charAt(index);
                if (carac == '}') {
                    flush(true);
                } else if (!Character.isWhitespace(carac)) {
                    instructionParser = new InstructionParser();
                    return instructionParser.parseCharAt(line, index);
                }
            }
            return index;
        }

        public void flush() {
            flush(false);
        }

        void flush(boolean next) {
            int size = instructionList.size();
            Instruction[] instructionArray = new Instruction[size];
            for (int i = 0; i < size; i++) {
                instructionArray[i] = (Instruction) instructionList.get(i);
            }
            insertionHandler.doInsertion(instructionArray);
            if (next) {
                currentZoneParser = new LiteralParser(getCurrentLineNumber());
            }
        }

        void endInstruction(boolean insertionEnd) {
            Instruction instruction = instructionParser.toInstruction();
            if (instruction != null) {
                instructionList.add(instruction);
            }
            instructionParser = null;
            if (insertionEnd) {
                flush(true);
            }
        }


        private class InstructionParser {

            List argumentList = new ArrayList();
            ArgumentParser currentArgumentParser = null;
            //char argumentListEndCharacter = ';';

            InstructionParser() {
                /*if (unique) {
                argumentListEndCharacter = '}';
                } else {
                argumentListEndCharacter = ')';
                }*/
            }

            void checkNewLine() {
                if (currentArgumentParser != null) {
                    currentArgumentParser.testWhiteChar('\n');
                }
            }

            int parseCharAt(String line, int index) {
                if (currentArgumentParser == null) {
                    currentArgumentParser = new ArgumentParser();
                }
                return currentArgumentParser.parseCharAt(line, index);
            }

            void endArgumentList(boolean insertionEnd) {
                Argument arg = currentArgumentParser.toArgument();
                if ((arg.getKey().length() != 0) || (argumentList.size() > 0)) {
                    argumentList.add(arg);
                }
                endInstruction(insertionEnd);
            }

            Instruction toInstruction() {
                if (argumentList.size() == 0) {
                    return null;
                }
                return new ImplInstruction(argumentList);
            }

            void endArgument() {
                argumentList.add(currentArgumentParser.toArgument());
                currentArgumentParser = null;
            }


            private class ArgumentParser {

                int keyRow;
                int keyCol;
                int valueRow;
                int valueCol;
                StringBuffer keyBuffer = new StringBuffer();
                StringBuffer valueBuffer = new StringBuffer();
                boolean previousWhite = false;
                int step = WAITING_FROM_KEY_STEP;

                ArgumentParser() {
                }

                Argument toArgument() {
                    ImplArgument arg = new ImplArgument(keyBuffer.toString(), keyRow, keyCol);
                    if (valueCol > 0) {
                        arg.setValue(valueBuffer.toString(), valueRow, valueCol);
                    }
                    return arg;
                }

                void testWhiteChar(char carac) {
                    switch (step) {
                        case WAITING_FROM_ARGUMENT_END_STEP:
                        case WAITING_FROM_KEY_STEP:
                        case WAITING_FROM_EQUAL_STEP:
                            break;
                        case KEY_STEP:
                            step = WAITING_FROM_EQUAL_STEP;
                            break;
                        case QUOTED_KEY_STEP:
                            keyBuffer.append(carac);
                            break;
                        case VALUE_STEP:
                            previousWhite = true;
                            break;
                        case QUOTED_VALUE_STEP:
                            valueBuffer.append(carac);
                            break;
                    }

                }

                boolean testEnd(char carac) {
                    if (carac == ',') {
                        endArgument();
                        return true;
                    } else if ((carac == ';') || (carac == '}')) {
                        endArgumentList((carac == '}'));
                        return true;
                    }
                    return false;
                }

                int parseCharAt(String line, int index) {
                    char carac = line.charAt(index);
                    if (Character.isWhitespace(carac)) {
                        testWhiteChar(carac);
                        return index;
                    }
                    if ((step != QUOTED_KEY_STEP) && (step != QUOTED_VALUE_STEP)) {
                        if (testEnd(carac)) {
                            return index;
                        }
                    }
                    switch (step) {
                        case WAITING_FROM_ARGUMENT_END_STEP:
                            throwCharacterError(carac, InstructionErrorHandler.INVALID_END_CHARACTER, index);
                            break;
                        case WAITING_FROM_KEY_STEP:
                            keyRow = getCurrentLineNumber();
                            keyCol = index + 1;
                            if (carac == '\"') {
                                step = QUOTED_KEY_STEP;
                            } else {
                                appendAsciiChar(carac, index, keyBuffer);
                                step = KEY_STEP;
                            }
                            break;
                        case WAITING_FROM_EQUAL_STEP:
                            if (carac == '=') {
                                step = WAITING_FROM_VALUE_STEP;
                            } else {
                                throwCharacterError(carac, InstructionErrorHandler.INVALID_SEPARATOR_CHARACTER, index);
                            }
                            break;
                        case WAITING_FROM_VALUE_STEP:
                            valueRow = getCurrentLineNumber();
                            valueCol = index + 1;
                            if (carac == '\"') {
                                step = QUOTED_VALUE_STEP;
                            } else {
                                valueBuffer.append(carac);
                                step = VALUE_STEP;
                            }
                            break;
                        case KEY_STEP:
                            if (carac == '=') {
                                step = WAITING_FROM_VALUE_STEP;
                            } else {
                                appendAsciiChar(carac, index, keyBuffer);
                            }
                            break;
                        case QUOTED_KEY_STEP:
                            if (carac == '\"') {
                                if (keyBuffer.length() == 0) {
                                    throwCharacterError(carac, InstructionErrorHandler.INVALID_EMPTY_QUOTED_KEY_CHARACTER, index);
                                }
                                step = WAITING_FROM_EQUAL_STEP;
                            } else {
                                index = parseQuoteCharAt(carac, line, index, keyBuffer);
                            }
                            break;
                        case VALUE_STEP:
                            if (previousWhite) {
                                valueBuffer.append(' ');
                            }
                            valueBuffer.append(carac);
                            break;
                        case QUOTED_VALUE_STEP:
                            if (carac == '\"') {
                                step = WAITING_FROM_ARGUMENT_END_STEP;
                            } else {
                                index = parseQuoteCharAt(carac, line, index, valueBuffer);
                            }
                            break;
                    }
                    previousWhite = false;
                    return index;
                }

                void appendAsciiChar(char carac, int index, StringBuffer buf) {
                    if (StringUtils.isValidAsciiChar(carac)) {
                        buf.append(carac);
                    } else {
                        throwCharacterError(carac, InstructionErrorHandler.INVALID_ASCII_CHARACTER, index);
                    }
                }

                int parseQuoteCharAt(char carac, String line, int index, StringBuffer buf) {
                    if (carac == '\\') {
                        if (index < (line.length() - 1)) {
                            char next = line.charAt(index + 1);
                            if (next == 'n') {
                                buf.append('\n');
                            } else {
                                buf.append(next);
                            }
                            index = index + 1;
                        }
                    } else {
                        buf.append(carac);

                    }
                    return index;
                }

            }

        }

    }


    private class IfParser extends ZoneParser {

        StringBuffer buf = new StringBuffer();
        boolean newif;
        boolean unique;

        IfParser(int currentLineNumber, boolean newif, boolean unique) {
            super(currentLineNumber);
            this.newif = newif;
            this.unique = unique;
        }

        protected void checkNewLine() {
            if (getCurrentLineNumber() > 1) {
                buf.append('\n');
            }
        }

        public void flush() {
            String clause = buf.toString().trim();
            if (newif) {
                insertionHandler.startIf(clause, unique);
            } else {
                insertionHandler.startElseIf(clause);
            }
        }

        public int parseCharAt(String line, int index) {
            char carac = line.charAt(index);
            switch (carac) {
                case '\\':
                    if (index < (line.length() - 1)) {
                        char next = line.charAt(index + 1);
                        if ((next == '\\') || (next == '{')) {
                            buf.append(next);
                            index = index + 1;
                        } else {
                            buf.append('\\');
                            buf.append(next);
                        }
                        return index + 1;
                    } else {
                        buf.append('\\');
                        return index;
                    }
                case '}':
                    flush();
                    currentZoneParser = new LiteralParser(getCurrentLineNumber());
                    return index;

                default:
                    buf.append(carac);
                    return index;
            }
        }

    }


    private class LiteralParser extends ZoneParser {

        StringBuffer buf = new StringBuffer();

        LiteralParser(int currentLineNumber) {
            super(currentLineNumber);
        }

        protected void checkNewLine() {
            if (getCurrentLineNumber() > 1) {
                buf.append('\n');
            }
        }

        public void flush() {
            if (buf.length() > 0) {
                insertionHandler.appendString(buf.toString());
            }
            buf = null;
        }

        private void flushAndReuse() {
            if (buf.length() > 0) {
                insertionHandler.appendString(buf.toString());
                buf = new StringBuffer();
            }
        }

        public int parseCharAt(String line, int index) {
            char carac = line.charAt(index);
            switch (carac) {
                case '\\':
                    if (index < (line.length() - 1)) {
                        char next = line.charAt(index + 1);
                        if ((next == '\\') || (next == '{')) {
                            buf.append(next);
                            index = index + 1;
                        } else {
                            buf.append('\\');
                            buf.append(next);
                        }
                        return index + 1;
                    } else {
                        buf.append('\\');
                        return index;
                    }
                case '{':
                    return test(line, index);
                default:
                    buf.append(carac);
                    return index;
            }
        }

        int test(String line, int index) {
            if (index == (line.length() - 1)) {
                buf.append('{');
                return index;
            }
            char suivant = line.charAt(index + 1);
            if ((suivant != '\"') && (!StringUtils.isValidFirstAsciiChar(suivant))) {
                buf.append('{');
                return index;
            } else if (line.regionMatches(index, "{FI}", 0, 4)) {
                if (ifCounter.onIf()) {
                    flushAndReuse();
                    insertionHandler.endIf(ifCounter.isUnique(), ifCounter.onElse());
                    ifCounter.decrease();
                } else {
                    buf.append("{FI}");
                    errorHandler.warning(InstructionErrorHandler.WARNING_UNEXPECTED_FI, getCurrentLineNumber(), index + 1);
                }
                return index + 3;
            } else if (line.regionMatches(index, "{ELSE}", 0, 6)) {
                if (ifCounter.onMultiIf() && (!ifCounter.onElse())) {
                    flushAndReuse();
                    insertionHandler.startElse();
                    ifCounter.setOnElse();
                } else {
                    buf.append("{ELSE}");
                    errorHandler.warning(InstructionErrorHandler.WARNING_UNEXPECTED_ELSE, getCurrentLineNumber(), index + 1);
                }
                return index + 5;
            } else if (line.regionMatches(index, "{IF", 0, 3)) {
                boolean unique = false;
                int dif = 2;
                char carac = getCharAt(line, index + dif + 1);
                if (carac == 'F') {
                    unique = true;
                    dif = dif + 1;
                    carac = getCharAt(line, index + dif + 1);
                }
                if (carac == '}') {
                    flushAndReuse();
                    ifCounter.increase(unique);
                    insertionHandler.startIf("", unique);
                    return index + dif + 1;
                } else if ((carac == ' ') || (carac == '\t') || (carac == '\n')) {
                    flush();
                    ifCounter.increase(unique);
                    currentZoneParser = new IfParser(getCurrentLineNumber(), true, unique);
                    return index + dif;
                } else {
                    flush();
                    currentZoneParser = new InsertionParser(getCurrentLineNumber());
                    return index;
                }
            } else if (line.regionMatches(index, "{ELIF", 0, 5)) {
                if (!ifCounter.onMultiIf() || ifCounter.onElse()) {
                    buf.append("{ELIF");
                    errorHandler.warning(InstructionErrorHandler.WARNING_UNEXPECTED_ELIF, getCurrentLineNumber(), index + 1);
                    return index + 4;
                }
                char carac = getCharAt(line, index + 5);
                if (carac == '}') {
                    flushAndReuse();
                    insertionHandler.startElseIf("");
                    return index + 5;
                } else if ((carac == ' ') || (carac == '\t') || (carac == '\t')) {
                    flush();
                    currentZoneParser = new IfParser(getCurrentLineNumber(), false, false);
                    return index + 4;
                } else {
                    flush();
                    currentZoneParser = new InsertionParser(getCurrentLineNumber());
                    return index;
                }
            } else {
                flush();
                currentZoneParser = new InsertionParser(getCurrentLineNumber());
                return index;
            }
        }

    }

    private static char getCharAt(String line, int index) {
        if (index >= line.length()) {
            return '\n';
        } else {
            return line.charAt(index);
        }
    }


    private class IfCounter {

        int ifCount = 0;
        IfInfo currentIfInfo;
        Stack previousIfInfo = new Stack();

        void decrease() {
            if (ifCount > 0) {
                ifCount--;
                if (ifCount > 0) {
                    currentIfInfo = (IfInfo) previousIfInfo.pop();
                }
            }
        }

        void increase(boolean unique) {
            ifCount++;
            if (ifCount > 1) {
                previousIfInfo.add(currentIfInfo);
            }
            currentIfInfo = new IfInfo(unique);
        }

        boolean onIf() {
            return (ifCount != 0);
        }

        boolean onMultiIf() {
            return ((ifCount != 0) && (!currentIfInfo.isUnique()));
        }

        boolean onElse() {
            return currentIfInfo.isOnElse();
        }

        boolean isUnique() {
            return currentIfInfo.isUnique();
        }

        void setOnElse() {
            currentIfInfo.setOnElse();
        }


        private class IfInfo {

            boolean unique;
            boolean onElse = false;

            IfInfo(boolean unique) {
                this.unique = unique;
            }

            boolean isUnique() {
                return unique;
            }

            boolean isOnElse() {
                return onElse;
            }

            void setOnElse() {
                onElse = true;
            }

        }

    }

}
