/* UtilLib - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.insertpattern.impl;

import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.StringUtils;
import java.util.List;
import java.util.ArrayList;
import java.io.StringReader;
import java.io.LineNumberReader;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class InstructionParser {

    private final static int WAITING_FROM_ARGUMENT_END_STEP = 3;
    private final static int WAITING_FROM_KEY_STEP = 4;
    private final static int WAITING_FROM_VALUE_STEP = 5;
    private final static int WAITING_FROM_EQUAL_STEP = 6;
    private final static int KEY_STEP = 7;
    private final static int QUOTED_KEY_STEP = 8;
    private final static int VALUE_STEP = 9;
    private final static int QUOTED_VALUE_STEP = 10;
    private final static ErrorHandler DEFAULT_ERROR_HANDLER = new DefaultErrorHandler();
    private List<Argument> argumentList = new ArrayList<Argument>();
    private ArgumentParser currentArgumentParser;
    private int currentLineNumber = 0;
    private ErrorHandler currentErrorHandler;

    public InstructionParser() {
    }

    public static Instruction parse(String s, ErrorHandler errorHandler) {
        InstructionParser parser = new InstructionParser();
        if (errorHandler == null) {
            errorHandler = DEFAULT_ERROR_HANDLER;
        }
        return parser.parseInstruction(s, errorHandler);
    }

    private Instruction parseInstruction(String s, ErrorHandler errorHandler) {
        argumentList.clear();
        this.currentErrorHandler = errorHandler;
        currentLineNumber = 0;
        currentArgumentParser = new ArgumentParser();
        try {
            StringReader stringReader = new StringReader(s);
            LineNumberReader bufreader = new LineNumberReader(stringReader);
            String line = null;
            while ((line = bufreader.readLine()) != null) {
                try {
                    parseLine(line, bufreader.getLineNumber());
                } catch (IllegalArgumentException ise) {
                    break;
                }
            }
        } catch (IOException ioe) {
        }
        Argument arg = currentArgumentParser.toArgument();
        if ((arg.getKey().length() != 0) || (argumentList.size() > 0)) {
            argumentList.add(arg);
        }
        currentArgumentParser = null;
        if (argumentList.size() == 0) {
            return null;
        }
        return new ImplInstruction(argumentList);
    }


    public static interface ErrorHandler {

        public void invalidAsciiCharacterError(String part, int row, int col);

        public void invalidEndCharacterError(String part, int row, int col);

        public void invalidSeparatorCharacterError(String part, int row, int col);

    }


    public static class DefaultErrorHandler implements ErrorHandler {

        public void invalidAsciiCharacterError(String part, int row, int col) {
        }

        public void invalidEndCharacterError(String part, int row, int col) {
        }

        public void invalidSeparatorCharacterError(String part, int row, int col) {
        }

    }

    private void parseLine(String line, int lineNumber) {
        currentLineNumber = lineNumber;
        int length = line.length();
        for (int i = 0; i < length; i++) {
            i = currentArgumentParser.parseCharAt(line, i);
        }
    }

    private int getCurrentLineNumber() {
        return currentLineNumber;
    }

    private void endArgument() {
        argumentList.add(currentArgumentParser.toArgument());
        currentArgumentParser = new ArgumentParser();
    }


    private class ArgumentParser {

        int keyRow;
        int keyCol;
        int valueRow;
        int valueCol;
        StringBuilder keyBuffer = new StringBuilder();
        StringBuilder valueBuffer = new StringBuilder();
        boolean previousWhite = false;
        int step = WAITING_FROM_KEY_STEP;

        ArgumentParser() {
        }

        Argument toArgument() {
            ImplArgument arg = new ImplArgument(keyBuffer.toString(), keyRow, keyCol);
            if (valueCol > 0) {
                arg.setValue(valueBuffer.toString(), valueRow, valueCol);
            }
            return arg;
        }

        void testWhiteChar(char carac) {
            switch (step) {
                case WAITING_FROM_ARGUMENT_END_STEP:
                case WAITING_FROM_KEY_STEP:
                case WAITING_FROM_EQUAL_STEP:
                    break;
                case KEY_STEP:
                    step = WAITING_FROM_EQUAL_STEP;
                    break;
                case QUOTED_KEY_STEP:
                    keyBuffer.append(carac);
                    break;
                case VALUE_STEP:
                    previousWhite = true;
                    break;
                case QUOTED_VALUE_STEP:
                    valueBuffer.append(carac);
                    break;
            }

        }

        boolean testEnd(char carac) {
            if (carac == ',') {
                endArgument();
                return true;
            }
            return false;
        }

        int parseCharAt(String line, int index) {
            char carac = line.charAt(index);
            if (Character.isWhitespace(carac)) {
                testWhiteChar(carac);
                return index;
            }
            if ((step != QUOTED_KEY_STEP) && (step != QUOTED_VALUE_STEP)) {
                if (testEnd(carac)) {
                    return index;
                }
            }
            switch (step) {
                case WAITING_FROM_ARGUMENT_END_STEP:
                    valueBuffer.append(carac);
                    currentErrorHandler.invalidEndCharacterError(valueBuffer.toString(), getCurrentLineNumber(), index);
                    throw new IllegalArgumentException();
                case WAITING_FROM_KEY_STEP:
                    keyRow = getCurrentLineNumber();
                    keyCol = index + 1;
                    if (carac == '\"') {
                        step = QUOTED_KEY_STEP;
                    } else if (testAsciiChar(carac, index, keyBuffer)) {
                        step = KEY_STEP;
                    }
                    break;
                case WAITING_FROM_EQUAL_STEP:
                    if (carac == '=') {
                        step = WAITING_FROM_VALUE_STEP;
                    } else {
                        keyBuffer.append(carac);
                        currentErrorHandler.invalidSeparatorCharacterError(keyBuffer.toString(), getCurrentLineNumber(), index);
                        throw new IllegalArgumentException();
                    }
                    break;
                case WAITING_FROM_VALUE_STEP:
                    valueRow = getCurrentLineNumber();
                    valueCol = index + 1;
                    if (carac == '\"') {
                        step = QUOTED_VALUE_STEP;
                    } else if (testAsciiChar(carac, index, valueBuffer)) {
                        step = VALUE_STEP;
                    }
                    break;
                case KEY_STEP:
                    if (carac == '=') {
                        step = WAITING_FROM_VALUE_STEP;
                    } else {
                        testAsciiChar(carac, index, keyBuffer);
                    }
                    break;
                case QUOTED_KEY_STEP:
                    if (carac == '\"') {
                        step = WAITING_FROM_EQUAL_STEP;
                    } else {
                        index = parseQuoteCharAt(carac, line, index, keyBuffer);
                    }
                    break;
                case VALUE_STEP:
                    if (previousWhite) {
                        valueBuffer.append(' ');
                    }
                    testAsciiChar(carac, index, valueBuffer);
                    break;
                case QUOTED_VALUE_STEP:
                    if (carac == '\"') {
                        step = WAITING_FROM_ARGUMENT_END_STEP;
                    } else {
                        index = parseQuoteCharAt(carac, line, index, valueBuffer);
                    }
                    break;
            }
            previousWhite = false;
            return index;
        }

        boolean testAsciiChar(char carac, int index, StringBuilder buf) {
            if (testEnd(carac)) {
                return false;
            } else {
                if (StringUtils.isValidAsciiChar(carac)) {
                    buf.append(carac);
                    return true;
                } else {
                    buf.append(carac);
                    currentErrorHandler.invalidAsciiCharacterError(buf.toString(), getCurrentLineNumber(), index);
                    throw new IllegalArgumentException();
                }
            }
        }

        int parseQuoteCharAt(char carac, String line, int index, StringBuilder buf) {
            if (carac == '\\') {
                if (index < (line.length() - 1)) {
                    char next = line.charAt(index + 1);
                    if (next == 'n') {
                        buf.append('\n');
                    } else {
                        buf.append(next);
                    }
                    index = index + 1;
                }
            } else {
                buf.append(carac);

            }
            return index;
        }

    }

}

